Package: dataSDA
Type: Package
Title: Datasets and Basic Statistics for Symbolic Data Analysis
Version: 0.2.6
Date: 2026-06-11
Authors@R: c(
    person("Po-Wei", "Chen", email = "randy310162@gmail.com", role = "aut"),
    person("Chun-houh", "Chen", email = "cchen@stat.sinica.edu.tw", role = "aut"),
    person("Han-Ming", "Wu", email = "wuhm@g.nccu.edu.tw", role = "cre"))
Description: Provides benchmark datasets and foundational tools for Symbolic Data Analysis (SDA). The package includes functions for constructing symbolic data objects from classical data, converting among different interval-valued data formats, managing interval-valued, histogram-valued, modal-valued, and multi-valued data, and performing basic descriptive statistics. It is designed to support teaching, methodological research, and the development of SDA techniques.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0.0)
Suggests: testthat (>= 2.1.0), knitr, rmarkdown, ggInterval, ggplot2,
        MAINT.Data, e1071, symbolicDA
VignetteBuilder: knitr
Imports: magrittr, tidyr, dplyr, RSDA, HistDAWass, methods
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-12 01:01:50 UTC; hmwu
Author: Po-Wei Chen [aut],
  Chun-houh Chen [aut],
  Han-Ming Wu [cre]
Maintainer: Han-Ming Wu <wuhm@g.nccu.edu.tw>
Repository: CRAN
Date/Publication: 2026-06-12 07:10:28 UTC
Built: R 4.6.0; ; 2026-06-12 11:06:14 UTC; unix

Package: dataSDA
Type: Package
Title: Datasets and Basic Statistics for Symbolic Data Analysis
Version: 0.2.6
Date: 2026-06-11
Authors@R: c(
    person("Po-Wei", "Chen", email = "randy310162@gmail.com", role = "aut"),
    person("Chun-houh", "Chen", email = "cchen@stat.sinica.edu.tw", role = "aut"),
    person("Han-Ming", "Wu", email = "wuhm@g.nccu.edu.tw", role = "cre"))
Description: Provides benchmark datasets and foundational tools for Symbolic Data Analysis (SDA). The package includes functions for constructing symbolic data objects from classical data, converting among different interval-valued data formats, managing interval-valued, histogram-valued, modal-valued, and multi-valued data, and performing basic descriptive statistics. It is designed to support teaching, methodological research, and the development of SDA techniques.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0.0)
Suggests: testthat (>= 2.1.0), knitr, rmarkdown, ggInterval, ggplot2,
        MAINT.Data, e1071, symbolicDA
VignetteBuilder: knitr
Imports: magrittr, tidyr, dplyr, RSDA, HistDAWass, methods
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-06-12 01:01:50 UTC; hmwu
Author: Po-Wei Chen [aut],
  Chun-houh Chen [aut],
  Han-Ming Wu [cre]
Maintainer: Han-Ming Wu <wuhm@g.nccu.edu.tw>
Repository: CRAN
Date/Publication: 2026-06-12 07:10:28 UTC

# Generated by roxygen2: do not edit by hand

export(ARRAY_to_MM)
export(ARRAY_to_RSDA)
export(ARRAY_to_iGAP)
export(MM_to_ARRAY)
export(MM_to_RSDA)
export(MM_to_iGAP)
export(RSDA_format)
export(RSDA_to_ARRAY)
export(RSDA_to_MM)
export(RSDA_to_iGAP)
export(SODAS_to_ARRAY)
export(SODAS_to_MM)
export(SODAS_to_iGAP)
export(aggregate_to_symbolic)
export(check_zero_width_intervals)
export(clean_colnames)
export(hist_cor)
export(hist_cov)
export(hist_mean)
export(hist_var)
export(iGAP_to_ARRAY)
export(iGAP_to_MM)
export(iGAP_to_RSDA)
export(int_center)
export(int_containment)
export(int_convert_format)
export(int_cor)
export(int_cosine)
export(int_cov)
export(int_cv)
export(int_detect_format)
export(int_dice)
export(int_dispersion)
export(int_dist)
export(int_dist_all)
export(int_dist_matrix)
export(int_entropy)
export(int_granularity)
export(int_imprecision)
export(int_information_content)
export(int_iqr)
export(int_jaccard)
export(int_kurtosis)
export(int_list_conversions)
export(int_mad)
export(int_mean)
export(int_median)
export(int_midrange)
export(int_mode)
export(int_overlap)
export(int_overlap_coefficient)
export(int_pairwise_dist)
export(int_quantile)
export(int_radius)
export(int_range)
export(int_similarity_matrix)
export(int_skewness)
export(int_symmetry)
export(int_tailedness)
export(int_tanimoto)
export(int_trimmed_mean)
export(int_trimmed_var)
export(int_uniformity)
export(int_var)
export(int_width)
export(int_winsorized_mean)
export(int_winsorized_var)
export(read_symbolic_csv)
export(search_data)
export(set_variable_format)
export(to_all_interval_formats)
export(write_symbolic_csv)
import(HistDAWass)
importFrom(RSDA,SODAS.to.RSDA)
importFrom(dplyr,select)
importFrom(graphics,hist)
importFrom(magrittr,"%>%")
importFrom(methods,new)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cov)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,lm)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,read.table)
importFrom(utils,write.table)

# dataSDA

**Datasets and Basic Statistics for Symbolic Data Analysis**

[![R](https://img.shields.io/badge/R-%3E%3D%204.0.0-blue)](https://www.r-project.org/)
[![License: GPL v2](https://img.shields.io/badge/License-GPL%20v2-blue.svg)](https://www.gnu.org/licenses/old-licenses/gpl-2.0.html)
[![Version](https://img.shields.io/badge/version-0.2.6-green.svg)](https://github.com/hanmingwu1103/dataSDA/releases/tag/v0.2.6)

## Overview

`dataSDA` collects a diverse range of symbolic data and offers a comprehensive set of functions that facilitate the conversion of traditional data into the symbolic data format. It supports reading, writing, and conversion of symbolic data in diverse formats, as well as computing descriptive statistics of symbolic variables.

## Installation

### From GitHub

```r
# install.packages("devtools")
devtools::install_github("hanmingwu1103/dataSDA")
```

### From source

Download the latest release from the [Releases](https://github.com/hanmingwu1103/dataSDA/releases) page, then:

```r
# Source package (all platforms)
install.packages("dataSDA_0.2.6.tar.gz", repos = NULL, type = "source")

# Binary package (Windows)
install.packages("dataSDA_0.2.6.zip", repos = NULL, type = "win.binary")
```

## Features

### Descriptive Statistics

#### Interval-valued data (`int_*`)

Compute mean, variance, covariance, and correlation for interval-valued data with 8 methods: CM, VM, QM, SE, FV, EJD, GQ, SPT.

```r
library(dataSDA)
data(mushroom.int)

int_mean(mushroom.int, var_name = "Pileus.Cap.Width")
int_var(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"), method = c("CM", "FV", "EJD"))

int_cov(mushroom.int, var_name1 = "Pileus.Cap.Width",
        var_name2 = c("Stipe.Length", "Stipe.Thickness"),
        method = c("CM", "VM", "EJD", "GQ", "SPT"))
int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
        var_name2 = "Stipe.Length", method = "CM")
```

#### Histogram-valued data (`hist_*`)

Compute mean, variance, covariance, and correlation for histogram-valued data with methods BG and L2W (cov/cor also support BD, B).

```r
library(HistDAWass)

hist_mean(HistDAWass::BLOOD, var_name = "Cholesterol", method = "BG")
hist_var(HistDAWass::BLOOD, var_name = "Cholesterol", method = "L2W")

hist_cov(HistDAWass::BLOOD, var_name1 = "Cholesterol",
         var_name2 = "Hemoglobin", method = "BD")
hist_cor(HistDAWass::BLOOD, var_name1 = "Cholesterol",
         var_name2 = "Hemoglobin", method = "BG")
```

### Data Format Conversion

#### Interval format conversions

| Function | Description |
|---|---|
| `int_detect_format` | Detect the format of an interval-valued dataset |
| `int_convert_format` | Convert between interval formats |
| `int_list_conversions` | List all available format conversions |
| `to_all_interval_formats` | Convert intervals to all supported formats at once |

#### Other conversion functions

| Function | Description |
|---|---|
| `RSDA_format` | Convert conventional data to RSDA format |
| `set_variable_format` | One-hot encode set variables for RSDA format |
| `aggregate_to_symbolic` | Convert traditional data to symbolic data format |

### Interval Geometry

| Function | Description |
|---|---|
| `int_width` | Width of each interval |
| `int_radius` | Radius of each interval |
| `int_center` | Center point of each interval |
| `int_midrange` | Half-range of each interval |
| `int_overlap` | Overlap measure between two interval variables |
| `int_containment` | Check if one interval contains another |

### Interval Position and Scale

| Function | Description |
|---|---|
| `int_median` | Median of interval data |
| `int_quantile` | Quantiles of interval data |
| `int_range` | Range of interval data |
| `int_iqr` | Interquartile range |
| `int_mad` | Median absolute deviation |
| `int_mode` | Mode of interval data |

### Interval Shape

| Function | Description |
|---|---|
| `int_skewness` | Skewness of interval data |
| `int_kurtosis` | Kurtosis of interval data |
| `int_symmetry` | Symmetry coefficient |
| `int_tailedness` | Tailedness measure |

### Interval Distance and Similarity

| Function | Description |
|---|---|
| `int_dist` | Distance measures (GD, IY, L1, L2, CB, HD, EHD, WD, etc.) |
| `int_jaccard` | Jaccard similarity coefficient |
| `int_dice` | Dice similarity coefficient |
| `int_cosine` | Cosine similarity |
| `int_overlap_coefficient` | Overlap coefficient |
| `int_tanimoto` | Tanimoto coefficient |
| `int_similarity_matrix` | Pairwise similarity matrix |

### Interval Robust Statistics

| Function | Description |
|---|---|
| `int_trimmed_mean` | Trimmed mean |
| `int_winsorized_mean` | Winsorized mean |
| `int_trimmed_var` | Trimmed variance |
| `int_winsorized_var` | Winsorized variance |

### Interval Uncertainty and Variability

| Function | Description |
|---|---|
| `int_entropy` | Shannon entropy |
| `int_cv` | Coefficient of variation |
| `int_dispersion` | Dispersion index |
| `int_imprecision` | Imprecision based on interval width |
| `int_granularity` | Variability in interval sizes |
| `int_uniformity` | Uniformity of interval widths |
| `int_information_content` | Normalized entropy |

### Utilities

| Function | Description |
|---|---|
| `clean_colnames` | Clean column names of a data frame |
| `check_zero_width_intervals` | Flag zero-width intervals (`min == max`) in interval-valued data |
| `read_symbolic_csv` | Read symbolic data from CSV file |
| `write_symbolic_csv` | Write symbolic data to CSV file |
| `search_data` | Search available datasets by keyword or type |
| `aggregate_to_symbolic` | Convert traditional data to symbolic data format |

## Datasets

The package includes **114 built-in datasets** for symbolic data analysis:

### Interval-valued datasets (53 datasets, `.int`)

`abalone.int`, `acid_rain.int`, `age_cholesterol_weight.int`, `baseball.int`, `bats.int`, `blood_pressure.int`, `car.int`, `car_models.int`, `cardiological.int`, `cars.int`, `china_temp.int`, `china_temp_monthly.int`, `credit_card.int`, `ecoli_routes.int`, `employment.int`, `finance.int`, `freshwater_fish.int`, `fungi.int`, `genome_abundances.int`, `hdi_gender.int`, `horses.int`, `iris.int`, `judge1.int`, `judge2.int`, `judge3.int`, `lackinfo.int`, `lisbon_air_quality.int`, `loans_by_purpose.int`, `loans_by_risk.int`, `loans_by_risk_quantile.int`, `lynne1.int`, `mushroom.int`, `nycflights.int`, `ohtemp.int`, `oils.int`, `polish_voivodships.int`, `profession.int`, `prostate.int`, `soccer_bivar.int`, `synthetic_clusters.int`, `teams.int`, `temperature_city.int`, `tennis.int`, `trivial_intervals.int`, `uscrime.int`, `utsnow.int`, `veterinary.int`, `video1.int`, `video2.int`, `video3.int`, `water_flow.int`, `wine.int`, `world_cup.int`

### Histogram-valued datasets (25 datasets, `.hist`)

`age_pyramids.hist`, `airline_flights.hist`, `bird_color_taxonomy.hist`, `blood.hist`, `china_climate_month.hist`, `china_climate_season.hist`, `cholesterol.hist`, `county_income_gender.hist`, `cover_types.hist`, `exchange_rate_returns.hist`, `flights_detail.hist`, `french_agriculture.hist`, `glucose.hist`, `hardwood.hist`, `hematocrit.hist`, `hematocrit_hemoglobin.hist`, `hemoglobin.hist`, `hierarchy.hist`, `hospital.hist`, `iris_species.hist`, `lung_cancer.hist`, `ozone.hist`, `simulated.hist`, `state_income.hist`, `weight_age.hist`

### Mixed symbolic datasets (11 datasets, `.mix`)

`bird.mix`, `bird_species.mix`, `bird_species_extended.mix`, `census.mix`, `environment.mix`, `health_insurance.mix`, `joggers.mix`, `mtcars.mix`, `mushroom_fuzzy.mix`, `polish_cars.mix`, `town_services.mix`

### Interval time series datasets (9 datasets, `.its`)

`crude_oil_wti.its`, `djia.its`, `euro_usd.its`, `ibovespa.its`, `irish_wind.its`, `merval.its`, `petrobras.its`, `shanghai_stock.its`, `sp500.its`

### Modal-valued datasets (7 datasets, `.modal`)

`airline_flights2.modal`, `crime.modal`, `crime2.modal`, `fuel_consumption.modal`, `health_insurance2.modal`, `occupations.modal`, `occupations2.modal`

### Distribution-valued datasets (3 datasets, `.distr`)

`energy_consumption.distr`, `energy_usage.distr`, `household_characteristics.distr`

### iGAP format datasets (2 datasets, `.iGAP`)

`abalone.iGAP`, `face.iGAP`

### Other datasets

`bank_rates`, `hierarchy`, `mushroom.int.mm`

## Vignettes

- [Introduction to dataSDA](https://htmlpreview.github.io/?https://github.com/hanmingwu1103/dataSDA/blob/master/docs/dataSDA_intro.html)

## Dependencies

- **R** (>= 4.0.0)
- [RSDA](https://CRAN.R-project.org/package=RSDA), [HistDAWass](https://CRAN.R-project.org/package=HistDAWass), [dplyr](https://CRAN.R-project.org/package=dplyr), [tidyr](https://CRAN.R-project.org/package=tidyr), [magrittr](https://CRAN.R-project.org/package=magrittr), [methods](https://stat.ethz.ch/R-manual/R-devel/library/methods/html/00Index.html)

## Authors

- **Po-Wei Chen** (Author), **Chun-houh Chen** (Author)
- **Han-Ming Wu** (Creator, Maintainer) - [wuhm@g.nccu.edu.tw](mailto:wuhm@g.nccu.edu.tw)

## License

GPL (>= 2)

## Citation

Po-Wei Chen, Chun-houh Chen, Han-Ming Wu (2026), dataSDA: datasets and basic statistics for symbolic data analysis in R (v0.2.6). Journal of Applied Statistics.

## ----setup, include = FALSE---------------------------------------------------
not_cran <- identical(Sys.getenv("NOT_CRAN"), "true")

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  echo = TRUE,
  eval = not_cran,
  warning = FALSE,
  message = FALSE,
  out.width = "100%",
  fig.align = "center"
)
library(knitr)
library(dataSDA)
library(RSDA)
library(HistDAWass)

has_symbolicDA <- requireNamespace("symbolicDA", quietly = TRUE)
has_MAINT <- requireNamespace("MAINT.Data", quietly = TRUE)
has_e1071 <- requireNamespace("e1071", quietly = TRUE)
has_ggInterval <- requireNamespace("ggInterval", quietly = TRUE) && not_cran

## ----dataset-summary, echo = FALSE--------------------------------------------
# # Use installed package data dir; fall back to source tree when building locally
# data_dir <- system.file("data", package = "dataSDA")
# if (!nzchar(data_dir) || length(list.files(data_dir, pattern = "[.]rda$")) == 0) {
#   data_dir <- file.path("..", "data")
# }
# 
# data_files <- list.files(data_dir, pattern = "[.]rda$")
# data_names <- sub("[.]rda$", "", data_files)
# 
# classify_type <- function(name) {
#   if (grepl("[.]its$", name)) {
#     return("Interval Time Series (.its)")
#   }
#   if (grepl("[.]int[.]mm$", name)) {
#     return("Interval (.int.mm)")
#   }
#   if (grepl("[.]int$", name)) {
#     return("Interval (.int)")
#   }
#   if (grepl("[.]iGAP$", name)) {
#     return("Interval (.iGAP)")
#   }
#   if (grepl("[.]hist$", name)) {
#     return("Histogram (.hist)")
#   }
#   if (grepl("[.]distr$", name)) {
#     return("Distributional (.distr)")
#   }
#   if (grepl("[.]mix$", name)) {
#     return("Mixed (.mix)")
#   }
#   if (grepl("[.]modal$", name)) {
#     return("Modal (.modal)")
#   }
#   "Other"
# }
# 
# types <- sapply(data_names, classify_type, USE.NAMES = FALSE)
# type_tbl <- as.data.frame(table(Type = types), stringsAsFactors = FALSE)
# type_tbl <- type_tbl[order(-type_tbl$Freq), ]
# names(type_tbl)[2] <- "Datasets"
# #kable(type_tbl,
# #  row.names = FALSE,
# #  caption = "Dataset counts by type"
# #)

## -----------------------------------------------------------------------------
# data(mushroom.int)
# head(mushroom.int, 3)
# class(mushroom.int)

## -----------------------------------------------------------------------------
# data(abalone.int)
# head(abalone.int, 3)
# class(abalone.int)

## -----------------------------------------------------------------------------
# data(abalone.iGAP)
# head(abalone.iGAP, 3)
# class(abalone.iGAP)

## -----------------------------------------------------------------------------
# int_detect_format(mushroom.int)
# int_detect_format(abalone.int)
# int_detect_format(abalone.iGAP)

## -----------------------------------------------------------------------------
# int_list_conversions()

## -----------------------------------------------------------------------------
# # RSDA to MM
# mushroom.MM <- int_convert_format(mushroom.int, to = "MM")
# head(mushroom.MM, 3)

## -----------------------------------------------------------------------------
# # iGAP to MM
# abalone.MM <- int_convert_format(abalone.iGAP, to = "MM")
# head(abalone.MM, 3)

## -----------------------------------------------------------------------------
# # iGAP to RSDA
# data(face.iGAP)
# face.RSDA <- int_convert_format(face.iGAP, to = "RSDA")
# head(face.RSDA, 3)

## -----------------------------------------------------------------------------
# # RSDA to MM
# mushroom.MM <- RSDA_to_MM(mushroom.int, RSDA = TRUE)
# head(mushroom.MM, 3)

## -----------------------------------------------------------------------------
# # MM to iGAP
# mushroom.iGAP <- MM_to_iGAP(mushroom.MM)
# head(mushroom.iGAP, 3)

## -----------------------------------------------------------------------------
# # iGAP to MM
# data(face.iGAP)
# face.MM <- iGAP_to_MM(face.iGAP, location = 1:6)
# head(face.MM, 3)

## -----------------------------------------------------------------------------
# # MM to RSDA
# face.RSDA <- MM_to_RSDA(face.MM)
# head(face.RSDA, 3)
# class(face.RSDA)

## -----------------------------------------------------------------------------
# # iGAP to RSDA (direct, one-step)
# abalone.RSDA <- iGAP_to_RSDA(abalone.iGAP, location = 1:7)
# head(abalone.RSDA, 3)
# class(abalone.RSDA)

## -----------------------------------------------------------------------------
# # RSDA to iGAP
# mushroom.iGAP2 <- RSDA_to_iGAP(mushroom.int)
# head(mushroom.iGAP2, 3)

## -----------------------------------------------------------------------------
# data(mushroom.int.mm)
# head(mushroom.int.mm, 3)

## -----------------------------------------------------------------------------
# mushroom_set <- set_variable_format(
#   data = mushroom.int.mm, location = 8,
#   var = "Species"
# )
# head(mushroom_set, 3)

## -----------------------------------------------------------------------------
# mushroom_tmp <- RSDA_format(
#   data = mushroom_set,
#   sym_type1 = c("I", "I", "I", "S"),
#   location = c(25, 27, 29, 31),
#   sym_type2 = c("S"),
#   var = c("Species")
# )
# head(mushroom_tmp, 3)

## -----------------------------------------------------------------------------
# mushroom_clean <- clean_colnames(data = mushroom_tmp)
# head(mushroom_clean, 3)

## -----------------------------------------------------------------------------
# write_symbolic_csv(mushroom_clean, file = "mushroom_interval.csv")
# mushroom_int <- read_symbolic_csv(file = "mushroom_interval.csv")
# head(mushroom_int, 3)
# class(mushroom_int)

## ----include=FALSE------------------------------------------------------------
# file.remove("mushroom_interval.csv")

## -----------------------------------------------------------------------------
# BLOOD[1:3, 1:2]

## -----------------------------------------------------------------------------
# A1 <- c(50, 60, 70, 80, 90, 100, 110, 120)
# B1 <- c(0.00, 0.02, 0.08, 0.32, 0.62, 0.86, 0.92, 1.00)
# A2 <- c(50, 60, 70, 80, 90, 100, 110, 120)
# B2 <- c(0.00, 0.05, 0.12, 0.42, 0.68, 0.88, 0.94, 1.00)
# A3 <- c(50, 60, 70, 80, 90, 100, 110, 120)
# B3 <- c(0.00, 0.03, 0.24, 0.36, 0.75, 0.85, 0.98, 1.00)
# 
# ListOfWeight <- list(
#   distributionH(A1, B1),
#   distributionH(A2, B2),
#   distributionH(A3, B3)
# )
# 
# Weight <- methods::new("MatH",
#   nrows = 3, ncols = 1, ListOfDist = ListOfWeight,
#   names.rows = c("20s", "30s", "40s"),
#   names.cols = c("weight"), by.row = FALSE
# )
# Weight

## -----------------------------------------------------------------------------
# data(mushroom.int)
# var_name <- c("Stipe.Length", "Stipe.Thickness")
# int_mean(mushroom.int, var_name, method = c("CM", "FV", "EJD"))

## -----------------------------------------------------------------------------
# data(mushroom.int)
# var_name <- c("Pileus.Cap.Width", "Stipe.Length")
# method <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
# 
# mean_mat <- int_mean(mushroom.int, var_name, method)
# mean_mat
# 
# var_mat <- int_var(mushroom.int, var_name, method)
# var_mat

## ----fig.width=7, fig.height=8------------------------------------------------
# cols <- c("#4E79A7", "#F28E2B")
# par(mfrow = c(2, 1), mar = c(5, 4, 3, 6), las = 2, xpd = TRUE)
# 
# # --- Mean across eight methods ---
# bp <- barplot(t(mean_mat),
#   beside = TRUE, col = cols,
#   main = "Interval Mean by Method (mushroom.int)",
#   ylab = "Mean",
#   ylim = c(0, max(mean_mat) * 1.25)
# )
# legend("topright",
#   inset = c(-0.18, 0),
#   legend = colnames(mean_mat), fill = cols, bty = "n", cex = 0.85
# )
# 
# # --- Variance across eight methods ---
# bp <- barplot(t(var_mat),
#   beside = TRUE, col = cols,
#   main = "Interval Variance by Method (mushroom.int)",
#   ylab = "Variance",
#   ylim = c(0, max(var_mat) * 1.25)
# )
# legend("topright",
#   inset = c(-0.18, 0),
#   legend = colnames(var_mat), fill = cols, bty = "n", cex = 0.85
# )

## -----------------------------------------------------------------------------
# cov_list <- int_cov(mushroom.int, "Pileus.Cap.Width", "Stipe.Length", method)
# cor_list <- int_cor(mushroom.int, "Pileus.Cap.Width", "Stipe.Length", method)
# 
# # Collect scalar values into named vectors for display and plotting
# cov_vec <- sapply(cov_list, function(x) x[1, 1])
# cor_vec <- sapply(cor_list, function(x) x[1, 1])
# 
# data.frame(
#   Method = names(cov_vec), Covariance = round(cov_vec, 4),
#   Correlation = round(cor_vec, 4), row.names = NULL
# )

## ----fig.width=7, fig.height=8------------------------------------------------
# par(mfrow = c(2, 1), mar = c(5, 4, 3, 1), las = 2)
# 
# # --- Covariance across eight methods ---
# bar_cols <- c(
#   "#4E79A7", "#59A14F", "#F28E2B", "#E15759",
#   "#76B7B2", "#EDC948", "#B07AA1", "#FF9DA7"
# )
# bp <- barplot(cov_vec,
#   col = bar_cols, border = NA,
#   main = "Cov(Pileus.Cap.Width, Stipe.Length) by Method",
#   ylab = "Covariance",
#   ylim = c(0, max(cov_vec) * 1.25)
# )
# text(bp, cov_vec, labels = round(cov_vec, 2), pos = 3, cex = 0.8)
# 
# # --- Correlation across eight methods ---
# bp <- barplot(cor_vec,
#   col = bar_cols, border = NA,
#   main = "Cor(Pileus.Cap.Width, Stipe.Length) by Method",
#   ylab = "Correlation",
#   ylim = c(0, 1.15)
# )
# text(bp, cor_vec, labels = round(cor_vec, 2), pos = 3, cex = 0.8)
# abline(h = 1, lty = 2, col = "grey50")

## -----------------------------------------------------------------------------
# data(mushroom.int)
# 
# # Width = upper - lower
# head(int_width(mushroom.int, "Stipe.Length"))
# 
# # Radius = width / 2
# head(int_radius(mushroom.int, "Stipe.Length"))
# 
# # Center = (lower + upper) / 2
# head(int_center(mushroom.int, "Stipe.Length"))
# 
# # Midrange
# head(int_midrange(mushroom.int, "Stipe.Length"))

## -----------------------------------------------------------------------------
# # Overlap between two interval variables
# head(int_overlap(mushroom.int, "Stipe.Length", "Stipe.Thickness"))
# 
# # Containment: proportion of var_name2 contained within var_name1
# head(int_containment(mushroom.int, "Stipe.Length", "Stipe.Thickness"))

## -----------------------------------------------------------------------------
# data(mushroom.int)
# 
# # Median (default method = "CM")
# int_median(mushroom.int, "Stipe.Length")
# 
# # Quantiles
# int_quantile(mushroom.int, "Stipe.Length", probs = c(0.25, 0.5, 0.75))
# 
# # Compare median across methods
# int_median(mushroom.int, "Stipe.Length", method = c("CM", "FV"))

## -----------------------------------------------------------------------------
# # Range (max - min)
# int_range(mushroom.int, "Stipe.Length")
# 
# # Interquartile range (Q3 - Q1)
# int_iqr(mushroom.int, "Stipe.Length")
# 
# # Median absolute deviation
# int_mad(mushroom.int, "Stipe.Length")
# 
# # Mode (histogram-based estimation)
# int_mode(mushroom.int, "Stipe.Length")

## -----------------------------------------------------------------------------
# data(mushroom.int)
# 
# # Compare standard mean vs trimmed mean (10% trim)
# int_mean(mushroom.int, "Stipe.Length", method = "CM")
# int_trimmed_mean(mushroom.int, "Stipe.Length", trim = 0.1, method = "CM")
# 
# # Winsorized mean: extreme values are replaced (not removed)
# int_winsorized_mean(mushroom.int, "Stipe.Length", trim = 0.1, method = "CM")

## -----------------------------------------------------------------------------
# int_var(mushroom.int, "Stipe.Length", method = "CM")
# int_trimmed_var(mushroom.int, "Stipe.Length", trim = 0.1, method = "CM")
# int_winsorized_var(mushroom.int, "Stipe.Length", trim = 0.1, method = "CM")

## -----------------------------------------------------------------------------
# data(mushroom.int)
# 
# # Skewness: asymmetry of the distribution
# int_skewness(mushroom.int, "Stipe.Length", method = "CM")
# 
# # Kurtosis: tail heaviness
# int_kurtosis(mushroom.int, "Stipe.Length", method = "CM")
# 
# # Symmetry coefficient
# int_symmetry(mushroom.int, "Stipe.Length", method = "CM")
# 
# # Tailedness (related to kurtosis)
# int_tailedness(mushroom.int, "Stipe.Length", method = "CM")

## -----------------------------------------------------------------------------
# data(mushroom.int)
# 
# int_jaccard(mushroom.int, "Stipe.Length", "Stipe.Thickness")
# int_dice(mushroom.int, "Stipe.Length", "Stipe.Thickness")
# int_cosine(mushroom.int, "Stipe.Length", "Stipe.Thickness")
# int_overlap_coefficient(mushroom.int, "Stipe.Length", "Stipe.Thickness")

## -----------------------------------------------------------------------------
# int_tanimoto(mushroom.int, "Stipe.Length", "Stipe.Thickness")

## -----------------------------------------------------------------------------
# int_similarity_matrix(mushroom.int, method = "jaccard")

## -----------------------------------------------------------------------------
# data(mushroom.int)
# 
# # Shannon entropy (higher = more uncertainty)
# int_entropy(mushroom.int, "Stipe.Length", method = "CM")
# 
# # Coefficient of variation (SD / mean)
# int_cv(mushroom.int, "Stipe.Length", method = "CM")
# 
# # Dispersion index
# int_dispersion(mushroom.int, "Stipe.Length", method = "CM")

## -----------------------------------------------------------------------------
# # Imprecision: based on interval widths
# int_imprecision(mushroom.int, "Stipe.Length")
# 
# # Granularity: variability in interval sizes
# int_granularity(mushroom.int, "Stipe.Length")
# 
# # Uniformity: inverse of granularity (higher = more uniform)
# int_uniformity(mushroom.int, "Stipe.Length")
# 
# # Normalized information content (between 0 and 1)
# int_information_content(mushroom.int, "Stipe.Length", method = "CM")

## -----------------------------------------------------------------------------
# data(car.int)
# car_num <- car.int[, 2:5]
# head(car_num, 3)

## -----------------------------------------------------------------------------
# # Euclidean distance between observations
# int_dist(car_num, method = "euclidean")

## -----------------------------------------------------------------------------
# # Return as a full matrix
# dm <- int_dist_matrix(car_num, method = "hausdorff")
# dm[1:5, 1:5]

## -----------------------------------------------------------------------------
# int_pairwise_dist(car_num, "Price", "Max_Velocity", method = "euclidean")

## -----------------------------------------------------------------------------
# all_dists <- int_dist_all(car_num)
# names(all_dists)

## -----------------------------------------------------------------------------
# all_methods <- c("BG", "BD", "B", "L2W")
# var_names <- c("Cholesterol", "Hemoglobin")
# 
# # Compute mean for each variable and method
# mean_mat <- sapply(all_methods, function(m) {
#   sapply(var_names, function(v) hist_mean(BLOOD, v, method = m))
# })
# rownames(mean_mat) <- var_names
# mean_mat
# 
# # Compute variance for each variable and method
# var_mat <- sapply(all_methods, function(m) {
#   sapply(var_names, function(v) hist_var(BLOOD, v, method = m))
# })
# rownames(var_mat) <- var_names
# var_mat

## ----fig.width=7, fig.height=8------------------------------------------------
# bar_cols <- c("#4E79A7", "#59A14F", "#F28E2B", "#E15759")
# par(mfrow = c(2, 2), mar = c(4, 5, 3, 1), las = 1)
# 
# # --- Mean: Cholesterol ---
# bp <- barplot(mean_mat["Cholesterol", ],
#   col = bar_cols, border = NA,
#   main = "Mean of Cholesterol", ylab = "Mean",
#   ylim = c(0, max(mean_mat["Cholesterol", ]) * 1.15)
# )
# text(bp, mean_mat["Cholesterol", ],
#   labels = round(mean_mat["Cholesterol", ], 2), pos = 3, cex = 0.8
# )
# 
# # --- Mean: Hemoglobin ---
# bp <- barplot(mean_mat["Hemoglobin", ],
#   col = bar_cols, border = NA,
#   main = "Mean of Hemoglobin", ylab = "Mean",
#   ylim = c(0, max(mean_mat["Hemoglobin", ]) * 1.15)
# )
# text(bp, mean_mat["Hemoglobin", ],
#   labels = round(mean_mat["Hemoglobin", ], 2), pos = 3, cex = 0.8
# )
# 
# # --- Variance: Cholesterol ---
# bp <- barplot(var_mat["Cholesterol", ],
#   col = bar_cols, border = NA,
#   main = "Variance of Cholesterol", ylab = "Variance",
#   ylim = c(0, max(var_mat["Cholesterol", ]) * 1.25)
# )
# text(bp, var_mat["Cholesterol", ],
#   labels = round(var_mat["Cholesterol", ], 1), pos = 3, cex = 0.8
# )
# 
# # --- Variance: Hemoglobin ---
# bp <- barplot(var_mat["Hemoglobin", ],
#   col = bar_cols, border = NA,
#   main = "Variance of Hemoglobin", ylab = "Variance",
#   ylim = c(0, max(var_mat["Hemoglobin", ]) * 1.25)
# )
# text(bp, var_mat["Hemoglobin", ],
#   labels = round(var_mat["Hemoglobin", ], 4), pos = 3, cex = 0.8
# )

## -----------------------------------------------------------------------------
# cov_vec <- sapply(all_methods, function(m) {
#   hist_cov(BLOOD, "Cholesterol", "Hemoglobin", method = m)
# })
# cor_vec <- sapply(all_methods, function(m) {
#   hist_cor(BLOOD, "Cholesterol", "Hemoglobin", method = m)
# })
# 
# data.frame(
#   Method = all_methods,
#   Covariance = round(cov_vec, 4),
#   Correlation = round(cor_vec, 4),
#   row.names = NULL
# )

## ----fig.width=7, fig.height=4------------------------------------------------
# par(mfrow = c(1, 2), mar = c(4, 5, 3, 1), las = 1)
# 
# # --- Covariance ---
# bp <- barplot(cov_vec,
#   col = bar_cols, border = NA,
#   main = "Cov(Cholesterol, Hemoglobin)",
#   ylab = "Covariance",
#   ylim = c(min(cov_vec) * 1.35, 0)
# )
# text(bp, cov_vec, labels = round(cov_vec, 2), pos = 1, cex = 0.8)
# 
# # --- Correlation ---
# bp <- barplot(cor_vec,
#   col = bar_cols, border = NA,
#   main = "Cor(Cholesterol, Hemoglobin)",
#   ylab = "Correlation",
#   ylim = c(min(cor_vec) * 1.4, 0)
# )
# text(bp, cor_vec, labels = round(cor_vec, 2), pos = 1, cex = 0.8)
# abline(h = -1, lty = 2, col = "grey50")

## ----eda-aggregate------------------------------------------------------------
# set.seed(42)
# iris_int <- aggregate_to_symbolic(
#   iris,
#   type = "int",
#   group_by = "kmeans",
#   stratify_var = "Species",
#   K = 10,
#   zero_width = "remove"
# )
# iris_int

## ----eda-indeximage, eval = has_ggInterval, fig.width = 8, fig.height = 5-----
# library(ggInterval)
# library(ggplot2)
# ggInterval_indexImage(iris_int[, 2:5], plotAll = TRUE, full_strip = FALSE,
#                       column_condition = FALSE) +
#   scale_colour_distiller(palette = "Spectral")
# 
# 

## ----eda-pca, eval = has_ggInterval, fig.width = 6, fig.height = 5------------
# ggInterval_PCA(iris_int[, 2:5])

## ----eda-radar, eval = has_ggInterval, fig.width = 7, fig.height = 7----------
# data(environment.mix)
# ggInterval_radarplot(environment.mix,
#   plotPartial = c(4, 6),
#   showLegend = FALSE, addText = FALSE
# )

## ----eda-ts, fig.width = 12, fig.height = 4-----------------------------------
# library(ggplot2)
# data(irish_wind.its)
# wind_sub <- irish_wind.its[1:12, ]
# 
# # Reshape to long format
# stations <- c("BIR", "DUB", "KIL", "SHA", "VAL")
# wind_long <- do.call(rbind, lapply(stations, function(st) {
#   data.frame(
#     month_num = seq_len(12),
#     Station   = st,
#     lower     = wind_sub[[paste0(st, "_l")]],
#     upper     = wind_sub[[paste0(st, "_u")]],
#     mid       = (wind_sub[[paste0(st, "_l")]] + wind_sub[[paste0(st, "_u")]]) / 2
#   )
# }))
# wind_long$Station <- factor(wind_long$Station, levels = stations)
# 
# # Dodge bars for each station within each month
# n_st <- length(stations)
# bar_w <- 0.6 / n_st
# wind_long$st_idx <- as.numeric(wind_long$Station)
# wind_long$x <- wind_long$month_num +
#   (wind_long$st_idx - (n_st + 1) / 2) * bar_w
# 
# ggplot(wind_long) +
#   geom_rect(
#     aes(
#       xmin = x - bar_w / 2, xmax = x + bar_w / 2,
#       ymin = lower, ymax = upper, fill = Station
#     ),
#     alpha = 0.4, color = NA
#   ) +
#   geom_line(aes(x = x, y = mid, color = Station, group = Station),
#     linewidth = 0.5
#   ) +
#   geom_point(aes(x = x, y = mid, color = Station), size = 1) +
#   scale_x_continuous(breaks = 1:12, labels = month.abb) +
#   labs(
#     title = "Irish Wind Speed Intervals (1961)",
#     x = "Month", y = "Wind Speed (knots)"
#   ) +
#   theme_grey(base_size = 12)

## ----clust-int-helpers, include = FALSE---------------------------------------
# # Helper: extract interval-only columns as symbolic_tbl
# .get_interval_cols <- function(x) {
#   int_cols <- sapply(x, function(col) inherits(col, "symbolic_interval"))
#   if (sum(int_cols) == 0) {
#     return(x)
#   }
#   out <- x[, int_cols, drop = FALSE]
#   class(out) <- c("symbolic_tbl", class(out))
#   out
# }
# 
# # Helper: convert symbolic_tbl to 3D array [n, p, 2] for symbolicDA
# .to_3d_array <- function(x) {
#   n <- nrow(x)
#   p <- ncol(x)
#   arr <- array(0, dim = c(n, p, 2))
#   for (j in seq_len(p)) {
#     cv <- unclass(x[[j]])
#     arr[, j, 1] <- Re(cv)
#     arr[, j, 2] <- Im(cv)
#   }
#   arr
# }
# 
# # Helper: compute clustering quality (1 - WSS/TSS) from distance matrix
# .clust_quality <- function(d, cl) {
#   d <- as.matrix(d)
#   n <- nrow(d)
#   TSS <- sum(d^2) / (2 * n)
#   WSS <- 0
#   for (k in unique(cl)) {
#     idx <- which(cl == k)
#     nk <- length(idx)
#     if (nk > 1) WSS <- WSS + sum(d[idx, idx]^2) / (2 * nk)
#   }
#   1 - WSS / TSS
# }
# 
# # Helper: find optimal k via n-adaptive elbow method with 2-step lookahead
# .find_optimal_k <- function(qualities, n) {
#   ks <- as.integer(names(qualities))
#   qs <- qualities
#   valid <- !is.na(qs)
#   if (sum(valid) < 2) {
#     return(ks[which(valid)[1]])
#   }
#   valid_ks <- ks[valid]
#   valid_qs <- qs[valid]
#   gains <- diff(valid_qs)
#   max_gain <- max(gains, na.rm = TRUE)
#   if (max_gain <= 0) {
#     return(valid_ks[1])
#   }
#   threshold <- max_gain / (1 + n / 100)
#   for (i in seq_along(gains)) {
#     if (gains[i] < threshold) {
#       look <- (i + 1):min(i + 2, length(gains))
#       look <- look[look >= i + 1 & look <= length(gains)]
#       ahead <- gains[look]
#       if (length(ahead) > 0 && any(!is.na(ahead) & ahead >= threshold)) next
#       return(valid_ks[i])
#     }
#   }
#   valid_ks[length(valid_ks)]
# }

## ----clust-int, eval = has_symbolicDA && not_cran, results = "hide"-----------
# library(symbolicDA)
# 
# set.seed(123)
# 
# datasets_clust_int <- list(
#   list(name = "face.iGAP", data = "face.iGAP"),
#   list(name = "prostate.int", data = "prostate.int"),
#   list(name = "nycflights.int", data = "nycflights.int"),
#   list(name = "china_temp.int", data = "china_temp.int"),
#   list(name = "lisbon_air_quality.int", data = "lisbon_air_quality.int")
# )
# 
# clust_int_results <- do.call(rbind, lapply(datasets_clust_int, function(ds) {
#   tryCatch(
#     {
#       data(list = ds$data)
#       x <- get(ds$data)
#       if (!inherits(x, "symbolic_tbl")) {
#         x <- tryCatch(int_convert_format(x, to = "RSDA"), error = function(e) x)
#         for (i in seq_along(x)) {
#           if (is.complex(x[[i]]) && !inherits(x[[i]], "symbolic_interval")) {
#             class(x[[i]]) <- c("symbolic_interval", "vctrs_vctr")
#           }
#         }
#         if (!inherits(x, "symbolic_tbl")) {
#           class(x) <- c("symbolic_tbl", class(x))
#         }
#       }
#       x_int <- .get_interval_cols(x)
#       n <- nrow(x_int)
#       p <- ncol(x_int)
#       k_max <- min(n - 1, 10, max(3, floor(n / 5)))
# 
#       d <- int_dist_matrix(x_int, method = "hausdorff")
#       so <- simple2SO(.to_3d_array(x_int))
# 
#       km_qs <- dc_qs <- sc_qs <- setNames(
#         rep(NA_real_, k_max - 1),
#         as.character(2:k_max)
#       )
#       for (k in 2:k_max) {
#         set.seed(123)
#         km_qs[as.character(k)] <- tryCatch(
#           {
#             res <- sym.kmeans(x_int, k = k)
#             1 - res$tot.withinss / res$totss
#           },
#           error = function(e) NA
#         )
# 
#         set.seed(123)
#         dc_qs[as.character(k)] <- tryCatch(
#           {
#             cl <- DClust(d, cl = k, iter = 100)
#             .clust_quality(d, cl)
#           },
#           error = function(e) NA
#         )
# 
#         set.seed(123)
#         sc_qs[as.character(k)] <- tryCatch(
#           {
#             cl <- SClust(so, cl = k, iter = 100)
#             .clust_quality(d, cl)
#           },
#           error = function(e) NA
#         )
#       }
# 
#       km_k <- .find_optimal_k(km_qs, n)
#       km_q <- km_qs[as.character(km_k)]
#       dc_k <- .find_optimal_k(dc_qs, n)
#       dc_q <- dc_qs[as.character(dc_k)]
#       sc_k <- .find_optimal_k(sc_qs, n)
#       sc_q <- sc_qs[as.character(sc_k)]
# 
#       data.frame(
#         Dataset = ds$name, n = n, p = p,
#         sym.kmeans = sprintf("%.4f (%d)", km_q, km_k),
#         DClust = sprintf("%.4f (%d)", dc_q, dc_k),
#         SClust = sprintf("%.4f (%d)", sc_q, sc_k)
#       )
#     },
#     error = function(e) NULL
#   )
# }))

## ----clust-int-table, eval = has_symbolicDA && not_cran-----------------------
# kable(clust_int_results,
#   row.names = FALSE,
#   caption = "Table 4: Interval clustering quality (1 - WSS/TSS) with optimal k in parentheses"
# )

## ----clust-hist-helpers, include = FALSE--------------------------------------
# # Helper: parse a dataSDA histogram string into a HistDAWass distributionH
# # Format: "{[lo, hi), prob; [lo, hi], prob; ...}"
# .parse_hist_to_distH <- function(s) {
#   s <- trimws(sub("^\\{", "", sub("\\}$", "", s)))
#   bins <- trimws(strsplit(s, ";")[[1]])
#   xs <- numeric(0)
#   ps <- numeric(0)
#   for (b in bins) {
#     b_clean <- gsub("\\[|\\]|\\(|\\)", "", b) # strip brackets
#     parts <- as.numeric(trimws(strsplit(b_clean, ",")[[1]]))
#     lo <- parts[1]
#     hi <- parts[2]
#     p <- parts[3]
#     if (length(xs) == 0) xs <- lo
#     xs <- c(xs, hi)
#     ps <- c(ps, p)
#   }
#   cp <- c(0, cumsum(ps))
#   cp[length(cp)] <- 1 # ensure exact 1
#   distributionH(xs, cp)
# }
# 
# # Helper: convert a dataSDA histogram data frame to a HistDAWass MatH object
# # Keeps histogram columns with >50% non-NA, then drops incomplete rows
# .dataSDA_hist_to_MatH <- function(df) {
#   df <- as.data.frame(df)
#   hist_cols <- names(df)[sapply(df, is.character)]
#   hist_cols <- hist_cols[sapply(hist_cols, function(cn) {
#     any(grepl("^\\{\\[", na.omit(df[[cn]])))
#   })]
#   # Keep only columns where >50% of values are non-NA (handles conditional vars)
#   hist_cols <- hist_cols[sapply(hist_cols, function(cn) {
#     mean(!is.na(df[[cn]])) > 0.5
#   })]
#   # Drop rows with remaining NAs
#   complete <- complete.cases(df[, hist_cols, drop = FALSE])
#   df <- df[complete, , drop = FALSE]
#   n <- nrow(df)
#   p <- length(hist_cols)
#   dists <- vector("list", n * p)
#   for (j in seq_along(hist_cols)) {
#     for (i in seq_len(n)) {
#       dists[[(j - 1) * n + i]] <- .parse_hist_to_distH(df[[hist_cols[j]]][i])
#     }
#   }
#   rn <- if (!is.null(rownames(df))) rownames(df) else paste0("I", seq_len(n))
#   methods::new("MatH",
#     nrows = n, ncols = p,
#     ListOfDist = dists,
#     names.rows = rn,
#     names.cols = hist_cols,
#     by.row = FALSE
#   )
# }

## ----clust-hist, results = "hide"---------------------------------------------
# set.seed(123)
# 
# datasets_clust_hist <- list(
#   list(name = "age_pyramids.hist"),
#   list(name = "ozone.hist"),
#   list(name = "china_climate_season.hist"),
#   list(name = "french_agriculture.hist"),
#   list(name = "flights_detail.hist")
# )
# 
# clust_hist_results <- do.call(rbind, lapply(datasets_clust_hist, function(ds) {
#   tryCatch(
#     {
#       data(list = ds$name, package = "dataSDA")
#       raw <- get(ds$name)
#       x <- .dataSDA_hist_to_MatH(raw)
#       n <- nrow(x@M)
#       p <- ncol(x@M)
#       k_max <- min(n - 1, 10, max(3, floor(n / 5)))
# 
#       # Precompute Wasserstein distance matrix and hclust tree (shared across k)
#       dm <- WH_MAT_DIST(x)
#       set.seed(123)
#       hc <- WH_hclust(x, simplify = TRUE)
# 
#       km_qs <- fc_qs <- hc_qs <- setNames(
#         rep(NA_real_, k_max - 1),
#         as.character(2:k_max)
#       )
#       for (k in 2:k_max) {
#         set.seed(123)
#         km_qs[as.character(k)] <- tryCatch(
#           {
#             res <- WH_kmeans(x, k = k)
#             res$quality
#           },
#           error = function(e) NA
#         )
# 
#         set.seed(123)
#         fc_qs[as.character(k)] <- tryCatch(
#           {
#             res <- WH_fcmeans(x, k = k)
#             res$quality
#           },
#           error = function(e) NA
#         )
# 
#         set.seed(123)
#         hc_qs[as.character(k)] <- tryCatch(
#           {
#             cl <- cutree(hc, k = k)
#             .clust_quality(dm, cl)
#           },
#           error = function(e) NA
#         )
#       }
# 
#       km_k <- .find_optimal_k(km_qs, n)
#       km_q <- km_qs[as.character(km_k)]
#       fc_k <- .find_optimal_k(fc_qs, n)
#       fc_q <- fc_qs[as.character(fc_k)]
#       hc_k <- .find_optimal_k(hc_qs, n)
#       hc_q <- hc_qs[as.character(hc_k)]
# 
#       data.frame(
#         Dataset = ds$name, n = n, p = p,
#         WH_kmeans = sprintf("%.4f (%d)", km_q, km_k),
#         WH_fcmeans = sprintf("%.4f (%d)", fc_q, fc_k),
#         WH_hclust = sprintf("%.4f (%d)", hc_q, hc_k)
#       )
#     },
#     error = function(e) NULL
#   )
# }))

## ----clust-hist-table---------------------------------------------------------
# kable(clust_hist_results,
#   row.names = FALSE,
#   caption = "Table 5: Histogram clustering quality (1 - WSS/TSS) with optimal k in parentheses"
# )

## ----class-helpers, include = FALSE-------------------------------------------
# # Helper: extract class labels from symbolic_set or character/factor column
# .get_class_labels <- function(x, col) {
#   cls <- x[[col]]
#   if (inherits(cls, "symbolic_set")) {
#     factor(vapply(cls, function(v) paste(v, collapse = ","), character(1)))
#   } else {
#     factor(cls)
#   }
# }
# 
# # Helper: build IData from interval columns of a symbolic_tbl
# .build_IData <- function(x) {
#   int_cols <- sapply(x, function(col) inherits(col, "symbolic_interval"))
#   df <- data.frame(row.names = seq_len(nrow(x)))
#   for (v in names(x)[int_cols]) {
#     cv <- unclass(x[[v]])
#     df[[paste0(v, "_l")]] <- Re(cv)
#     df[[paste0(v, "_u")]] <- Im(cv)
#   }
#   MAINT.Data::IData(df)
# }

## ----class-int, eval = has_MAINT && has_e1071 && not_cran, results = "hide"----
# library(MAINT.Data)
# library(e1071)
# 
# datasets_class <- list(
#   list(
#     name = "cars.int", data = "cars.int",
#     class_col = "class",
#     class_desc = "class: Utilitarian(7), Berlina(8), Sportive(8), Luxury(4)"
#   ),
#   list(
#     name = "china_temp.int", data = "china_temp.int",
#     class_col = "GeoReg",
#     class_desc = "GeoReg: 6 regions"
#   ),
#   list(
#     name = "mushroom.int", data = "mushroom.int",
#     class_col = "Edibility",
#     class_desc = "Edibility: T(4), U(2), Y(17)"
#   ),
#   list(
#     name = "ohtemp.int", data = "ohtemp.int",
#     class_col = "STATE",
#     class_desc = "STATE: 10 groups"
#   ),
#   list(
#     name = "wine.int", data = "wine.int",
#     class_col = "class",
#     class_desc = "class: 1(21), 2(12)"
#   )
# )
# 
# class_results <- do.call(rbind, lapply(datasets_class, function(ds) {
#   tryCatch(
#     {
#       data(list = ds$data)
#       x <- get(ds$data)
#       grp <- .get_class_labels(x, ds$class_col)
# 
#       idata <- .build_IData(x)
# 
#       int_cols <- sapply(x, function(col) inherits(col, "symbolic_interval"))
#       svm_df <- data.frame(row.names = seq_len(nrow(x)))
#       for (v in names(x)[int_cols]) {
#         cv <- unclass(x[[v]])
#         svm_df[[paste0(v, "_l")]] <- Re(cv)
#         svm_df[[paste0(v, "_u")]] <- Im(cv)
#       }
# 
#       set.seed(123)
#       lda_acc <- tryCatch(
#         {
#           res <- MAINT.Data::lda(idata, grouping = grp)
#           pred <- predict(res, idata)
#           mean(pred$class == grp)
#         },
#         error = function(e) NA
#       )
# 
#       set.seed(123)
#       qda_acc <- tryCatch(
#         {
#           res <- MAINT.Data::qda(idata, grouping = grp)
#           pred <- predict(res, idata)
#           mean(pred$class == grp)
#         },
#         error = function(e) NA
#       )
# 
#       set.seed(123)
#       svm_acc <- tryCatch(
#         {
#           svm_df$class <- grp
#           res <- svm(class ~ ., data = svm_df, kernel = "radial")
#           pred <- predict(res, svm_df)
#           mean(pred == grp)
#         },
#         error = function(e) NA
#       )
# 
#       data.frame(
#         Dataset = ds$name, Response = ds$class_desc,
#         LDA = lda_acc, QDA = qda_acc, SVM = svm_acc
#       )
#     },
#     error = function(e) NULL
#   )
# }))

## ----class-int-table, eval = has_MAINT && has_e1071 && not_cran---------------
# kable(class_results,
#   digits = 4, row.names = FALSE,
#   caption = "Table 6: Classification accuracy (resubstitution)"
# )

## ----reg-int, results = "hide"------------------------------------------------
# datasets_reg <- list(
#   list(
#     name = "abalone.iGAP", data = "abalone.iGAP",
#     response = "Length", n_x = 6
#   ),
#   list(
#     name = "cardiological.int", data = "cardiological.int",
#     response = "pulse", n_x = 4
#   ),
#   list(
#     name = "nycflights.int", data = "nycflights.int",
#     response = "distance", n_x = 3
#   ),
#   list(
#     name = "oils.int", data = "oils.int",
#     response = "specific_gravity", n_x = 3
#   ),
#   list(
#     name = "prostate.int", data = "prostate.int",
#     response = "lpsa", n_x = 8
#   )
# )
# 
# reg_results <- do.call(rbind, lapply(datasets_reg, function(ds) {
#   tryCatch(
#     {
#       data(list = ds$data)
#       x <- get(ds$data)
# 
#       if (!inherits(x, "symbolic_tbl")) {
#         x2 <- tryCatch(int_convert_format(x, to = "RSDA"), error = function(e) NULL)
#         if (!is.null(x2)) {
#           x <- x2
#           for (i in seq_along(x)) {
#             if (is.complex(x[[i]]) && !inherits(x[[i]], "symbolic_interval")) {
#               class(x[[i]]) <- c("symbolic_interval", "vctrs_vctr")
#             }
#           }
#           if (!inherits(x, "symbolic_tbl")) {
#             class(x) <- c("symbolic_tbl", class(x))
#           }
#         } else {
#           cn <- colnames(x)
#           l_cols <- grep("_l$", cn, value = TRUE)
#           vars <- sub("_l$", "", l_cols)
#           out <- data.frame(row.names = seq_len(nrow(x)))
#           for (v in vars) {
#             lv <- x[[paste0(v, "_l")]]
#             uv <- x[[paste0(v, "_u")]]
#             si <- complex(real = lv, imaginary = uv)
#             class(si) <- c("symbolic_interval", "vctrs_vctr")
#             out[[v]] <- si
#           }
#           class(out) <- c("symbolic_tbl", class(out))
#           x <- out
#         }
#       }
#       x_int <- .get_interval_cols(x)
# 
#       fml <- as.formula(paste(ds$response, "~ ."))
#       nc <- data.frame(row.names = seq_len(nrow(x_int)))
#       for (v in names(x_int)) {
#         cv <- unclass(x_int[[v]])
#         nc[[v]] <- (Re(cv) + Im(cv)) / 2
#       }
#       actual <- nc[[ds$response]]
#       resp_idx <- which(names(x_int) == ds$response)
#       .r2 <- function(a, p) 1 - sum((a - p)^2) / sum((a - mean(a))^2)
# 
#       set.seed(123)
#       lm_r2 <- tryCatch(
#         {
#           res <- sym.lm(fml, sym.data = x_int, method = "cm")
#           summary(res)$r.squared
#         },
#         error = function(e) NA
#       )
# 
#       set.seed(123)
#       glm_r2 <- tryCatch(
#         {
#           res <- sym.glm(sym.data = x_int, response = resp_idx, method = "cm")
#           pred <- as.numeric(predict(res,
#             newx = as.matrix(nc[, -resp_idx]),
#             s = "lambda.min"
#           ))
#           .r2(actual, pred)
#         },
#         error = function(e) NA
#       )
# 
#       set.seed(123)
#       rf_r2 <- tryCatch(
#         {
#           res <- sym.rf(fml, sym.data = x_int, method = "cm")
#           tail(res$rsq, 1)
#         },
#         error = function(e) NA
#       )
# 
#       set.seed(123)
#       rt_r2 <- tryCatch(
#         {
#           res <- sym.rt(fml, sym.data = x_int, method = "cm")
#           .r2(actual, predict(res))
#         },
#         error = function(e) NA
#       )
# 
#       set.seed(123)
#       nnet_r2 <- tryCatch(
#         {
#           res <- sym.nnet(fml, sym.data = x_int, method = "cm")
#           pred_sc <- as.numeric(res$net.result[[1]])
#           pred <- pred_sc * res$data_c_sds[resp_idx] + res$data_c_means[resp_idx]
#           .r2(actual, pred)
#         },
#         error = function(e) NA
#       )
# 
#       data.frame(
#         Dataset = ds$name, Response = ds$response, p = ds$n_x,
#         sym.lm = lm_r2, sym.glm = glm_r2, sym.rf = rf_r2,
#         sym.rt = rt_r2, sym.nnet = nnet_r2
#       )
#     },
#     error = function(e) NULL
#   )
# }))

## ----reg-int-table------------------------------------------------------------
# kable(reg_results,
#   digits = 4, row.names = FALSE,
#   caption = "Table 7: Regression R-squared"
# )

#' @name interval_shape
#' @title Distribution Shape Measures for Interval Data
#' @description Functions to compute shape statistics (skewness, kurtosis) for interval-valued data.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name the variable name or the column location (multiple variables are allowed).
#' @param method methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.
#' @param ... additional parameters
#' @return A numeric matrix
#' @details
#' These functions measure distribution shape:
#' \itemize{
#'   \item \code{int_skewness}: Measure of asymmetry (skewness)
#'   \item \code{int_kurtosis}: Measure of tail heaviness (kurtosis)
#'   \item \code{int_symmetry}: Symmetry coefficient
#'   \item \code{int_tailedness}: Tailedness measure (alias for excess kurtosis)
#' }
#' 
#' Skewness interpretation:
#' \itemize{
#'   \item = 0: Symmetric distribution
#'   \item > 0: Right-skewed (positive skew)
#'   \item < 0: Left-skewed (negative skew)
#' }
#' 
#' Kurtosis interpretation (excess kurtosis):
#' \itemize{
#'   \item = 0: Normal distribution (mesokurtic)
#'   \item > 0: Heavy tails (leptokurtic)
#'   \item < 0: Light tails (platykurtic)
#' }
#' @author Han-Ming Wu
#' @seealso int_mean int_var int_skewness int_kurtosis
#' @examples
#' data(mushroom.int)
#' 
#' # Calculate skewness
#' int_skewness(mushroom.int, var_name = "Pileus.Cap.Width")
#' int_skewness(mushroom.int, var_name = 2:3, method = c("CM", "EJD"))
#' 
#' # Calculate kurtosis
#' int_kurtosis(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))
#' 
#' # Check symmetry
#' int_symmetry(mushroom.int, var_name = 2:4, method = "CM")
#'
#' # Check tailedness
#' int_tailedness(mushroom.int, var_name = "Pileus.Cap.Width", method = "CM")
#' @importFrom stats sd
#' @export
int_skewness <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_skewness")
  .check_var_name(var_name, x, "int_skewness")
  .check_interval_method(method, "int_skewness")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  skew_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  
  # Compute skewness: E[(X - μ)^3] / σ^3
  compute_skewness <- function(X_tmp) {
    if (length(var_name) == 1) {
      m <- mean(X_tmp)
      s <- sd(X_tmp)
      if (s == 0) return(0)
      skew <- sum((X_tmp - m)^3) / (n * s^3)
    } else {
      skew <- numeric(ncol(X_tmp))
      for (j in 1:ncol(X_tmp)) {
        m <- mean(X_tmp[, j])
        s <- sd(X_tmp[, j])
        if (s == 0) {
          skew[j] <- 0
        } else {
          skew[j] <- sum((X_tmp[, j] - m)^3) / (n * s^3)
        }
      }
    }
    skew
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    skew_tmp[idx, ] <- compute_skewness(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    skew_tmp[6, ] <- skew_tmp[7, ] <- skew_tmp[8, ] <- compute_skewness(X_tmp)
  }
  
  skew_output <- matrix(skew_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(skew_output) <- colnames(x)[var_name]
  } else {
    colnames(skew_output) <- var_name
  }
  rownames(skew_output) <- options[at]
  
  skew_output
}


#' @rdname interval_shape
#' @export
int_kurtosis <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_kurtosis")
  .check_var_name(var_name, x, "int_kurtosis")
  .check_interval_method(method, "int_kurtosis")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  kurt_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  
  # Compute excess kurtosis: E[(X - μ)^4] / σ^4 - 3
  compute_kurtosis <- function(X_tmp) {
    if (length(var_name) == 1) {
      m <- mean(X_tmp)
      s <- sd(X_tmp)
      if (s == 0) return(0)
      kurt <- sum((X_tmp - m)^4) / (n * s^4) - 3
    } else {
      kurt <- numeric(ncol(X_tmp))
      for (j in 1:ncol(X_tmp)) {
        m <- mean(X_tmp[, j])
        s <- sd(X_tmp[, j])
        if (s == 0) {
          kurt[j] <- 0
        } else {
          kurt[j] <- sum((X_tmp[, j] - m)^4) / (n * s^4) - 3
        }
      }
    }
    kurt
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    kurt_tmp[idx, ] <- compute_kurtosis(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    kurt_tmp[6, ] <- kurt_tmp[7, ] <- kurt_tmp[8, ] <- compute_kurtosis(X_tmp)
  }
  
  kurt_output <- matrix(kurt_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(kurt_output) <- colnames(x)[var_name]
  } else {
    colnames(kurt_output) <- var_name
  }
  rownames(kurt_output) <- options[at]
  
  kurt_output
}


#' @rdname interval_shape
#' @export
int_symmetry <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_symmetry")
  .check_var_name(var_name, x, "int_symmetry")
  .check_interval_method(method, "int_symmetry")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  symm_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  
  # Compute symmetry coefficient: 1 - |skewness|
  # Returns 1 for perfect symmetry, 0 for highly skewed
  compute_symmetry <- function(X_tmp) {
    if (length(var_name) == 1) {
      m <- mean(X_tmp)
      s <- sd(X_tmp)
      if (s == 0) return(1)
      skew <- abs(sum((X_tmp - m)^3) / (n * s^3))
      symm <- exp(-skew)  # Exponential decay with skewness
    } else {
      symm <- numeric(ncol(X_tmp))
      for (j in 1:ncol(X_tmp)) {
        m <- mean(X_tmp[, j])
        s <- sd(X_tmp[, j])
        if (s == 0) {
          symm[j] <- 1
        } else {
          skew <- abs(sum((X_tmp[, j] - m)^3) / (n * s^3))
          symm[j] <- exp(-skew)
        }
      }
    }
    symm
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    symm_tmp[idx, ] <- compute_symmetry(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    symm_tmp[6, ] <- symm_tmp[7, ] <- symm_tmp[8, ] <- compute_symmetry(X_tmp)
  }
  
  symm_output <- matrix(symm_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(symm_output) <- colnames(x)[var_name]
  } else {
    colnames(symm_output) <- var_name
  }
  rownames(symm_output) <- options[at]
  
  symm_output
}


#' @rdname interval_shape
#' @export
int_tailedness <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_tailedness")
  .check_var_name(var_name, x, "int_tailedness")
  .check_interval_method(method, "int_tailedness")
  
  # Tailedness is simply kurtosis (excess kurtosis already computed)
  # This function provides a more intuitive name
  int_kurtosis(x, var_name, method, ...)
}

#' @name interval_distance
#' @title Distance Measures for Interval Data
#' @description Functions to compute various distance measures between interval-valued observations.
#' @param x interval-valued data with symbolic_tbl class, or an array of dimension [n, p, 2]
#' @param method distance method: "GD", "IY", "L1", "L2", "CB", "HD", "EHD", "nEHD", "snEHD", "TD", "WD", "euclidean", "hausdorff", "manhattan", "city_block", "minkowski", "wasserstein", "ichino", "de_carvalho"
#' @param gamma parameter for the Ichino-Yaguchi distance, 0 <= gamma <= 0.5 (default: 0.5)
#' @param q parameter for the Ichino-Yaguchi distance (Minkowski exponent) (default: 1)
#' @param p power parameter for Minkowski distance (default: 2)
#' @param var_name1 first variable name or column location
#' @param var_name2 second variable name or column location
#' @param ... additional parameters
#' @return A distance matrix (class 'dist') or numeric vector
#' @details
#' Available distance methods:
#' \itemize{
#'   \item \code{GD}: Gowda-Diday distance (Gowda & Diday, 1991)
#'   \item \code{IY}: Ichino-Yaguchi distance (Ichino, 1988)
#'   \item \code{L1}: L1 (midpoint Manhattan) distance
#'   \item \code{L2}: L2 (Euclidean midpoint) distance
#'   \item \code{CB}: City-Block distance (Souza & de Carvalho, 2004)
#'   \item \code{HD}: Hausdorff distance (Chavent & Lechevallier, 2002)
#'   \item \code{EHD}: Euclidean Hausdorff distance
#'   \item \code{nEHD}: Normalized Euclidean Hausdorff distance
#'   \item \code{snEHD}: Span Normalized Euclidean Hausdorff distance
#'   \item \code{TD}: Tran-Duckstein distance (Tran & Duckstein, 2002)
#'   \item \code{WD}: L2-Wasserstein distance (Verde & Irpino, 2008)
#'   \item \code{euclidean}: Euclidean distance on interval centers (same as L2)
#'   \item \code{hausdorff}: Hausdorff distance (same as HD)
#'   \item \code{manhattan}: Manhattan distance (same as L1)
#'   \item \code{city_block}: City-block distance (same as CB)
#'   \item \code{minkowski}: Minkowski distance with parameter p
#'   \item \code{wasserstein}: Wasserstein distance (same as WD)
#'   \item \code{ichino}: Ichino-Yaguchi distance (simplified version)
#'   \item \code{de_carvalho}: De Carvalho distance
#' }
#' @references
#' Gowda, K. C., & Diday, E. (1991). Symbolic clustering using a new dissimilarity measure. 
#' \emph{Pattern Recognition}, 24(6), 567-578.
#' 
#' Ichino, M. (1988). General metrics for mixed features. 
#' \emph{Systems and Computers in Japan}, 19(2), 37-50.
#' 
#' Chavent, M., & Lechevallier, Y. (2002). Dynamical clustering of interval data. 
#' In \emph{Classification, Clustering and Data Analysis} (pp. 53-60). Springer.
#' 
#' Tran, L., & Duckstein, L. (2002). Comparison of fuzzy numbers using a fuzzy distance measure.
#' \emph{Fuzzy Sets and Systems}, 130, 331-341.
#' 
#' Verde, R., & Irpino, A. (2008). A new interval data distance based on the Wasserstein metric.
#' 
#' Kao, C.-H. et al. (2014). Exploratory data analysis of interval-valued symbolic data with
#' matrix visualization. \emph{Computational Statistics & Data Analysis}, 79, 14--29.
#' @author Han-Ming Wu
#' @seealso int_dist_matrix int_dist_all int_pairwise_dist
#' @examples
#' # Using symbolic_tbl format
#' data(mushroom.int)
#' d1 <- int_dist(mushroom.int[, 3:4], method = "euclidean")
#' d2 <- int_dist(mushroom.int[, 3:4], method = "hausdorff")
#' d3 <- int_dist(mushroom.int[, 3:4], method = "GD")
#' 
#' # Using array format: 4 concepts, 3 variables
#' x <- array(NA, dim = c(4, 3, 2))
#' x[,,1] <- matrix(c(1,2,3,4, 5,6,7,8, 9,10,11,12), nrow=4)
#' x[,,2] <- matrix(c(3,5,6,7, 8,9,10,12, 13,15,16,18), nrow=4)
#' d4 <- int_dist(x, method = "snEHD")
#' d5 <- int_dist(x, method = "IY", gamma = 0.3)
#' @importFrom stats as.dist
#' @export
int_dist <- function(x, method = "euclidean", gamma = 0.5, q = 1, p = 2, ...) {
  
  # Check if x is symbolic_tbl or array
  is_symbolic <- inherits(x, "symbolic_tbl")
  
  if (is_symbolic) {
    .check_symbolic_tbl(x, "int_dist")
    # Filter to interval (symbolic_interval) columns only
    int_cols <- sapply(x, inherits, "symbolic_interval")
    if (!any(int_cols)) {
      stop("int_dist: no interval (symbolic_interval) columns found in 'x'.", call. = FALSE)
    }
    idata <- symbolic_tbl_to_idata(x[, int_cols, drop = FALSE])
    x_array <- idata
  } else {
    if (is.null(x)) {
      stop("int_dist: 'x' must not be NULL.", call. = FALSE)
    }
    if (!is.array(x) || length(dim(x)) != 3 || dim(x)[3] != 2) {
      stop("int_dist: 'x' must be an array of dimension [n, p, 2] or a symbolic_tbl object.", 
           call. = FALSE)
    }
    x_array <- x
  }
  
  # Method aliases mapping
  method_aliases <- c(
    euclidean = "L2",
    manhattan = "L1",
    hausdorff = "HD",
    city_block = "CB",
    wasserstein = "WD"
  )
  
  if (method %in% names(method_aliases)) {
    method <- method_aliases[method]
  }
  
  # Validate method
  valid_methods <- c("GD", "IY", "L1", "L2", "CB", "HD", "EHD", "nEHD", "snEHD",
                     "TD", "WD", "minkowski", "ichino", "de_carvalho")
  
  if (!method %in% valid_methods) {
    stop("int_dist: 'method' must be one of: ",
         paste(c(valid_methods, names(method_aliases)), collapse = ", "), ".", 
         call. = FALSE)
  }
  
  n <- dim(x_array)[1]
  n_vars <- dim(x_array)[2]
  
  a <- x_array[, , 1, drop = FALSE]
  b <- x_array[, , 2, drop = FALSE]
  dim(a) <- c(n, n_vars)
  dim(b) <- c(n, n_vars)
  
  if (any(a > b, na.rm = TRUE)) {
    stop("int_dist: all lower bounds must be <= upper bounds.", call. = FALSE)
  }
  
  mid <- (a + b) / 2
  rad <- (b - a) / 2
  len <- b - a
  
  D <- matrix(0, nrow = n, ncol = n)
  
  # =========================================================================
  # Compute distances based on method
  # =========================================================================
  
  if (method == "GD") {
    global_max <- apply(b, 2, max, na.rm = TRUE)
    global_min <- apply(a, 2, min, na.rm = TRUE)
    global_range <- global_max - global_min
    
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d_ij <- 0
        for (r in 1:n_vars) {
          denom1 <- ifelse(global_range[r] == 0, 1, global_range[r])
          denom23 <- max(b[i, r], b[j, r]) - min(a[i, r], a[j, r])
          denom23 <- ifelse(denom23 == 0, 1, denom23)
          
          Ir <- abs(max(a[i, r], a[j, r]) - min(b[i, r], b[j, r]))
          
          comp1 <- abs(a[i, r] - a[j, r]) / denom1
          comp2 <- abs(len[i, r] - len[j, r]) / denom23
          comp3 <- (len[i, r] + len[j, r] - 2 * Ir) / denom23
          
          d_ij <- d_ij + comp1 + comp2 + comp3
        }
        D[i, j] <- D[j, i] <- d_ij
      }
    }
  }
  
  else if (method == "IY") {
    if (gamma < 0 || gamma > 0.5) {
      stop("int_dist: 'gamma' must be between 0 and 0.5.", call. = FALSE)
    }
    
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d_ij <- 0
        for (r in 1:n_vars) {
          union_len <- max(b[i, r], b[j, r]) - min(a[i, r], a[j, r])
          inter_len <- max(0, min(b[i, r], b[j, r]) - max(a[i, r], a[j, r]))
          
          d_r <- (union_len - inter_len) + gamma * (2 * inter_len - len[i, r] - len[j, r])
          
          if (q == 1) {
            d_ij <- d_ij + d_r
          } else {
            d_ij <- d_ij + d_r^q
          }
        }
        
        D[i, j] <- D[j, i] <- ifelse(q == 1, d_ij, d_ij^(1 / q))
      }
    }
  }
  
  else if (method == "L1") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        D[i, j] <- D[j, i] <- sum(abs(mid[i, ] - mid[j, ]))
      }
    }
  }
  
  else if (method == "L2") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        D[i, j] <- D[j, i] <- sqrt(sum((mid[i, ] - mid[j, ])^2))
      }
    }
  }
  
  else if (method == "CB") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        D[i, j] <- D[j, i] <- sum(abs(a[i, ] - a[j, ]) + abs(b[i, ] - b[j, ]))
      }
    }
  }
  
  else if (method == "HD") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        D[i, j] <- D[j, i] <- sum(pmax(abs(a[i, ] - a[j, ]), abs(b[i, ] - b[j, ])))
      }
    }
  }
  
  else if (method == "EHD") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d_r <- pmax(abs(a[i, ] - a[j, ]), abs(b[i, ] - b[j, ]))
        D[i, j] <- D[j, i] <- sqrt(sum(d_r^2))
      }
    }
  }
  
  else if (method == "nEHD") {
    H2 <- numeric(n_vars)
    for (r in 1:n_vars) {
      ss <- 0
      for (i in 1:n) {
        for (j in 1:n) {
          d_r <- max(abs(a[i, r] - a[j, r]), abs(b[i, r] - b[j, r]))
          ss <- ss + d_r^2
        }
      }
      H2[r] <- ss / (2 * n^2)
    }
    H2[H2 == 0] <- 1
    
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d_r <- pmax(abs(a[i, ] - a[j, ]), abs(b[i, ] - b[j, ]))
        D[i, j] <- D[j, i] <- sqrt(sum((d_r^2) / H2))
      }
    }
  }
  
  else if (method == "snEHD") {
    R_r <- apply(b, 2, max, na.rm = TRUE) - apply(a, 2, min, na.rm = TRUE)
    R_r[R_r == 0] <- 1
    
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d_r <- pmax(abs(a[i, ] - a[j, ]), abs(b[i, ] - b[j, ]))
        D[i, j] <- D[j, i] <- sqrt(sum((d_r / R_r)^2))
      }
    }
  }
  
  else if (method == "TD") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d2 <- sum((mid[i, ] - mid[j, ])^2 + (1 / 3) * (rad[i, ]^2 + rad[j, ]^2))
        D[i, j] <- D[j, i] <- sqrt(d2)
      }
    }
  }
  
  else if (method == "WD") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d2 <- sum((mid[i, ] - mid[j, ])^2 + (1 / 3) * (rad[i, ] - rad[j, ])^2)
        D[i, j] <- D[j, i] <- sqrt(d2)
      }
    }
  }
  
  else if (method == "minkowski") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        D[i, j] <- D[j, i] <- (sum(abs(mid[i, ] - mid[j, ])^p))^(1/p)
      }
    }
  }
  
  else if (method == "ichino") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d <- 0
        for (r in 1:n_vars) {
          d <- d + (mid[i, r] - mid[j, r])^2 + (rad[i, r] - rad[j, r])^2 / 3
        }
        D[i, j] <- D[j, i] <- sqrt(d)
      }
    }
  }
  
  else if (method == "de_carvalho") {
    for (i in 1:(n - 1)) {
      for (j in (i + 1):n) {
        d_pos <- sum((mid[i, ] - mid[j, ])^2)
        d_span <- sum((len[i, ] - len[j, ])^2)
        D[i, j] <- D[j, i] <- sqrt(d_pos + d_span)
      }
    }
  }
  
  # Set row/column names
  if (is_symbolic) {
    rownames(D) <- rownames(x)
    colnames(D) <- rownames(x)
  } else if (!is.null(rownames(x_array))) {
    rownames(D) <- rownames(x_array)
    colnames(D) <- rownames(x_array)
  }
  
  dist_obj <- as.dist(D)
  attr(dist_obj, "method") <- method
  return(dist_obj)
}


#' @rdname interval_distance
#' @export
int_dist_matrix <- function(x, method = "euclidean", gamma = 0.5, q = 1, p = 2, ...) {
  dist_obj <- int_dist(x, method = method, gamma = gamma, q = q, p = p, ...)
  as.matrix(dist_obj)
}


#' @rdname interval_distance
#' @export
int_pairwise_dist <- function(x, var_name1, var_name2, method = "euclidean", ...) {
  .check_symbolic_tbl(x, "int_pairwise_dist")
  .check_var_name(var_name1, x, "int_pairwise_dist")
  .check_var_name(var_name2, x, "int_pairwise_dist")

  # Filter to interval columns only for proper numeric conversion
  int_cols <- sapply(x, inherits, "symbolic_interval")
  idata <- symbolic_tbl_to_idata(x[, int_cols, drop = FALSE])
  n <- nrow(idata)
  
  data1 <- idata[, var_name1, , drop = FALSE]
  data2 <- idata[, var_name2, , drop = FALSE]
  
  dist_vec <- numeric(n)
  
  for (i in 1:n) {
    mini_array <- array(0, dim = c(2, ncol(data1), 2))
    mini_array[1, , ] <- data1[i, , ]
    mini_array[2, , ] <- data2[i, , ]
    
    d <- int_dist(mini_array, method = method, ...)
    dist_vec[i] <- as.matrix(d)[1, 2]
  }
  
  names(dist_vec) <- rownames(x)
  return(dist_vec)
}


#' @rdname interval_distance
#' @description \code{int_dist_all} computes all available distance measures at once.
#' @export
int_dist_all <- function(x, gamma = 0.5, q = 1) {
  methods <- c("GD", "IY", "L1", "L2", "CB", "HD", "EHD", "nEHD", "snEHD",
               "TD", "WD")
  
  result <- lapply(methods, function(m) {
    int_dist(x, method = m, gamma = gamma, q = q)
  })
  
  names(result) <- methods
  return(result)
}

# Internal helper functions for histogram_stats.R
# These are NOT exported.

# Extract mean matrix from MatH object (replaces 5 duplicate definitions)
.MatH_mean <- function(object) {
  nr <- nrow(object@M)
  nc <- ncol(object@M)
  MAT <- matrix(NA, nr, nc,
                dimnames = list(rownames(object@M), colnames(object@M)))
  for (i in 1:nr) {
    for (j in 1:nc) {
      if (length(object@M[i, j][[1]]@x) > 0) {
        MAT[i, j] <- object@M[i, j][[1]]@m
      }
    }
  }
  MAT
}

# Extract sd matrix from MatH object (replaces 5 duplicate definitions)
.MatH_sd <- function(object) {
  nr <- nrow(object@M)
  nc <- ncol(object@M)
  MAT <- matrix(NA, nr, nc,
                dimnames = list(rownames(object@M), colnames(object@M)))
  for (i in 1:nr) {
    for (j in 1:nc) {
      if (length(object@M[i, j][[1]]@x) > 0) {
        MAT[i, j] <- object@M[i, j][[1]]@s
      }
    }
  }
  MAT
}

# Sign function for histogram BD covariance: per-bin sign.
# Returns a vector of -1/1, one per bin, based on whether
# the bin midpoint is <= or > the grand mean (hmean).
.hist_Gj <- function(a, b, hmean) {
  ifelse((a + b) / 2 <= hmean, -1, 1)
}

# Q value computation (replaces 2 duplicate definitions)
.hist_Qj <- function(a, b, hmean) {
  (a - hmean)^2 + (a - hmean) * (b - hmean) + (b - hmean)^2
}

# QQ outer product computation (replaces 2 duplicate definitions)
.hist_QQ <- function(a, b, hmean1, hmean2) {
  outer((a - hmean1), (b - hmean2))
}

# Extract p variables for two locations (replaces 3 duplicate definitions)
.hist_get_pvars <- function(object, i, loc1, loc2) {
  object1 <- object@M[i, loc1][[1]]
  object2 <- object@M[i, loc2][[1]]
  p1 <- object1@p[2:length(object1@p)]
  p2 <- object1@p[1:(length(object1@p) - 1)]
  p3 <- object2@p[2:length(object2@p)]
  p4 <- object2@p[1:(length(object2@p) - 1)]
  pvar1 <- p1 - p2
  pvar2 <- p3 - p4
  list(pvar1 = pvar1, pvar2 = pvar2)
}

# Compute G and Q values for BD method.
# G1, G2 are per-bin sign vectors; Q1, Q2 are per-bin quadratic form vectors.
.hist_get_GQ <- function(object, i, loc1, loc2, var1, var2) {
  object1 <- object@M[i, loc1][[1]]
  object2 <- object@M[i, loc2][[1]]
  lenx1 <- length(object1@x)
  lenx2 <- length(object2@x)
  hmean1 <- hist_mean_BG(object, var1)
  hmean2 <- hist_mean_BG(object, var2)
  Q1 <- .hist_Qj(object1@x[1:(lenx1 - 1)], object1@x[2:lenx1], hmean1)
  Q2 <- .hist_Qj(object2@x[1:(lenx2 - 1)], object2@x[2:lenx2], hmean2)
  G1 <- .hist_Gj(object1@x[1:(lenx1 - 1)], object1@x[2:lenx1], hmean1)
  G2 <- .hist_Gj(object2@x[1:(lenx2 - 1)], object2@x[2:lenx2], hmean2)
  list(Q1 = Q1, Q2 = Q2, G1 = G1, G2 = G2)
}

# Compute QQ matrices for B method (replaces 2 duplicate definitions)
.hist_get_QQ_vals <- function(object, i, loc1, loc2, var1, var2) {
  object1 <- object@M[i, loc1][[1]]
  object2 <- object@M[i, loc2][[1]]
  lenx1 <- length(object1@x)
  lenx2 <- length(object2@x)
  hmean1 <- hist_mean_BG(object, var1)
  hmean2 <- hist_mean_BG(object, var2)
  Q1 <- .hist_QQ(object1@x[2:lenx1], object2@x[2:lenx2], hmean1, hmean2)
  Q2 <- .hist_QQ(object1@x[2:lenx1], object2@x[1:(lenx2 - 1)], hmean1, hmean2)
  Q3 <- .hist_QQ(object1@x[1:(lenx1 - 1)], object2@x[2:lenx2], hmean1, hmean2)
  Q4 <- .hist_QQ(object1@x[1:(lenx1 - 1)], object2@x[1:(lenx2 - 1)], hmean1, hmean2)
  list(Q1 = Q1, Q2 = Q2, Q3 = Q3, Q4 = Q4)
}

# ============================================================================
# Interval Data Utilities
# ============================================================================
# Internal transformation helpers (used by interval_stats.R) and exported
# format-preparation functions (RSDA_format, set_variable_format).
# ============================================================================


#' Internal Utility Functions for Interval Data
#'
#' @name interval_utils
#' @title Internal Utility Functions for Interval Data
#' @description Internal functions for interval data transformation.
#'   These are used by the exported interval statistics functions
#'   (\code{\link{int_mean}}, \code{\link{int_var}}, \code{\link{int_cov}},
#'   \code{\link{int_cor}}) and are not intended to be called directly.
#' @keywords internal
NULL


# Convert a symbolic_tbl to a 3D array [n, p, 2] (min/max).
symbolic_tbl_to_idata <- function(symbolic_tbl){
  idata <- array(0, dim = c(dim(symbolic_tbl), 2))
  p <- dim(symbolic_tbl)[2]
  for(j in 1:p){
    idata[,j,1:2] <- as.matrix(as.data.frame(c(symbolic_tbl[,j])))
  }
  dimnames(idata) <- list(row.names(symbolic_tbl),
                          colnames(symbolic_tbl),
                          c("min", "max"))
  idata
}


# Center Method (CM): midpoints (a + b) / 2.
Interval_to_Center <- function(idata){
  n <- dim(idata)[[1]]
  p <- dim(idata)[[2]]
  XC <- (idata[,,1]+idata[,,2])/2
  XC
}


# Midrange: half-ranges (b - a) / 2.
Interval_to_Midrange <- function(idata){
  n <- dim(idata)[[1]]
  p <- dim(idata)[[2]]
  XR <- (idata[,,2]-idata[,,1])/2
  XR
}


# Vertices Method (VM): all 2^p vertex combinations per concept.
Interval_to_Vertices <- function(idata) {
  n <- dim(idata)[[1]]
  p <- dim(idata)[[2]]
  XV <- matrix(0, nrow = n * 2^p, ncol = p)

  C.code <- FALSE
  if (C.code == FALSE) {
    cc <- 1
    index <- matrix(0, nrow = n * 2^p, ncol = 2)
    for (i in 1:n) {
      for (j in 1:(2^p)) {
        jj <- (j - 1)

        for (k in p:1) {
          if (jj %% 2 == 0) {
            XV[j + (i - 1) * 2^p, k] <- idata[i, k, 1]
          }
          else{
            XV[j + (i - 1) * 2^p, k] <- idata[i, k, 2]
          }
          jj <- jj %/% 2
        }
        index[cc, ] <- c(i, j + (i - 1) * 2^p)
        cc <- cc + 1
      }
    }
  }

  rownames(XV) <- paste0(rep(dimnames(idata)[[1]], each = (2^p)),
                         "_", rep(1:(2^p), n))
  colnames(XV) <- dimnames(idata)[[2]]
  XV
}


# Quantiles Method (QM): m+1 equally spaced quantiles per concept.
Interval_to_Quantiles <- function(idata, m = 4) {
  n <- dim(idata)[[1]]
  p <- dim(idata)[[2]]
  XQ <- matrix(0, nrow = n * (m + 1), ncol = p)
  cc <- 1
  index <- matrix(0, nrow = n * (m + 1), ncol = 2)
  for (i in 1:n) {
    aij <- idata[i, , 1]
    bij <- idata[i, , 2]

    for (k in 0:m) {
      XQ[cc, ] <- aij + (bij - aij) * k / m
      index[cc, ] <- c(i, cc)
      cc <- cc + 1
    }
  }

  rownames(XQ) <- paste0(rep(dimnames(idata)[[1]], each = (m+1)),
                         "_", rep(1:(m+1), n))
  colnames(XQ) <- dimnames(idata)[[2]]
  XQ
}


# Set Expansion (SE): endpoints only (quantiles with m = 1).
Interval_to_SE <- function(idata) {
  m <- 1
  n <- dim(idata)[[1]]
  XSE <- Interval_to_Quantiles(idata, m)
  rownames(XSE) <- paste0(rep(dimnames(idata)[[1]], each = (m+1)),
                         "_", rep(1:(m+1), n))
  colnames(XSE) <- dimnames(idata)[[2]]
  XSE
}


# Fitted Values (FV): linear regression fitted values of max on min.
Interval_to_FV <- function(idata) {
  n <- dim(idata)[1]
  p <- dim(idata)[2]
  XFV <- matrix(0, ncol = p, nrow = n)
  for (j in 1:p) {
    x <- idata[, j, 1]
    y <- idata[, j, 2]
    my.lm <- stats::lm(y ~ x)
    XFV[, j]  <- my.lm$fitted.values
  }

  rownames(XFV) <- dimnames(idata)[[1]]
  colnames(XFV) <- dimnames(idata)[[2]]
  XFV
}


# Unified dispatch for CM/VM/QM/SE/FV interval transforms.
# Returns a named list with entries for each active standard method.
# at = logical vector of length 8, options = c("CM","VM","QM","SE","FV","EJD","GQ","SPT")
.get_interval_transforms <- function(idata, at) {
  result <- list()
  if (at[1]) result$CM <- Interval_to_Center(idata)
  if (at[2]) result$VM <- Interval_to_Vertices(idata)
  if (at[3]) result$QM <- Interval_to_Quantiles(idata)
  if (at[4]) result$SE <- Interval_to_SE(idata)
  if (at[5]) result$FV <- Interval_to_FV(idata)
  result
}


# --- Diagnostic functions -----------------------------------------------------

#' Check for Zero-Width Intervals
#'
#' @name check_zero_width_intervals
#' @aliases check_zero_width_intervals
#' @description Checks whether interval-valued data contains any zero-width
#' intervals (\code{min == max}). Such degenerate intervals break downstream
#' tools that divide by interval width (e.g.
#' \code{ggInterval::ggInterval_indexImage()}). The input may be supplied either
#' in MM format (a \code{data.frame} with paired \code{_min}/\code{_max} columns)
#' or in RSDA format (a \code{symbolic_tbl} with \code{symbolic_interval}
#' columns).
#' @usage check_zero_width_intervals(data, tol = 0, warn = TRUE)
#' @param data Interval-valued data, in one of two accepted formats:
#' \itemize{
#'   \item a \code{data.frame} in MM format with paired \code{_min}/\code{_max}
#'   columns, or
#'   \item a \code{symbolic_tbl} (RSDA format) with \code{symbolic_interval}
#'   columns. Non-interval columns (e.g. set/modal variables) are ignored.
#' }
#' @param tol Non-negative numeric tolerance. An interval is flagged when
#' \code{abs(max - min) <= tol}. Defaults to \code{0} (exact \code{min == max}).
#' @param warn Logical; if \code{TRUE} (default) a single warning naming the
#' affected variables is emitted when zero-width intervals are found.
#' @returns Invisibly, a logical scalar: \code{TRUE} if any zero-width interval
#' is present, otherwise \code{FALSE}. The returned value carries two attributes:
#' \code{"flagged"}, a logical \code{[n, p]} matrix marking each zero-width cell
#' (rows = concepts, columns = interval variables), and \code{"variables"}, the
#' names of variables containing at least one zero-width interval.
#' @examples
#' # MM format (paired _min/_max columns)
#' data(mushroom.int.mm)
#' check_zero_width_intervals(mushroom.int.mm)
#'
#' # RSDA format (symbolic_tbl)
#' data(mushroom.int)
#' check_zero_width_intervals(mushroom.int)
#' @export
check_zero_width_intervals <- function(data, tol = 0, warn = TRUE){
  .check_data_frame(data, "check_zero_width_intervals")
  if (!is.numeric(tol) || length(tol) != 1 || is.na(tol) || tol < 0) {
    stop("check_zero_width_intervals: 'tol' must be a single non-negative number.",
         call. = FALSE)
  }
  .check_logical(warn, "warn", "check_zero_width_intervals")

  if (inherits(data, "symbolic_tbl")) {
    flagged <- .zero_width_flagged_symbolic_tbl(data, tol)
  } else {
    flagged <- .zero_width_flagged_mm(data, tol)
  }

  base_names <- colnames(flagged)
  affected_vars <- base_names[apply(flagged, 2L, any, na.rm = TRUE)]
  any_flagged <- length(affected_vars) > 0

  if (any_flagged && warn) {
    warning("check_zero_width_intervals: zero-width intervals (min == max) ",
            "found in variable(s): ", paste(affected_vars, collapse = ", "), ".",
            call. = FALSE)
  }

  attr(any_flagged, "flagged") <- flagged
  attr(any_flagged, "variables") <- affected_vars
  invisible(any_flagged)
}


# Build the [n, p] zero-width flag matrix from MM-format data
# (paired _min/_max columns).
.zero_width_flagged_mm <- function(data, tol) {
  col_names <- names(data)
  min_cols <- grep("_[Mm]in$", col_names, value = TRUE)
  if (length(min_cols) == 0) {
    stop("check_zero_width_intervals: no '_min' columns found in 'data'.",
         call. = FALSE)
  }

  base_names <- sub("_[Mm]in$", "", min_cols)
  p <- length(base_names)
  flagged <- matrix(FALSE, nrow = nrow(data), ncol = p,
                    dimnames = list(rownames(data), base_names))

  for (j in seq_len(p)) {
    max_col <- col_names[grepl(paste0("^", base_names[j], "_[Mm]ax$"), col_names)]
    if (length(max_col) != 1) {
      stop("check_zero_width_intervals: no matching '_max' column for '",
           min_cols[j], "'.", call. = FALSE)
    }
    lo <- as.numeric(data[[min_cols[j]]])
    hi <- as.numeric(data[[max_col]])
    flagged[, j] <- abs(hi - lo) <= tol
  }
  flagged
}


# Build the [n, p] zero-width flag matrix from RSDA-format data
# (symbolic_tbl). Only symbolic_interval / complex columns are considered.
.zero_width_flagged_symbolic_tbl <- function(data, tol) {
  int_cols <- vapply(data, function(col) {
    inherits(col, "symbolic_interval") || mode(col) == "complex"
  }, logical(1))
  if (!any(int_cols)) {
    stop("check_zero_width_intervals: no interval (symbolic_interval) columns ",
         "found in 'data'.", call. = FALSE)
  }

  idata <- symbolic_tbl_to_idata(data[, int_cols, drop = FALSE])
  lo <- idata[, , 1]
  hi <- idata[, , 2]
  flagged <- abs(hi - lo) <= tol
  dim(flagged) <- dim(idata)[1:2]
  dimnames(flagged) <- dimnames(idata)[1:2]
  flagged
}


# --- Format preparation functions ---------------------------------------------

#' clean_colnames
#'
#' @name clean_colnames
#' @aliases clean_colnames
#' @description This function is used to clean up variable names to conform to the RSDA format.
#' @usage clean_colnames(data)
#' @param data The conventional data.
#' @returns Data after cleaning variable names.
#' @examples
#' data(mushroom.int.mm)
#' mushroom.clean <- clean_colnames(data = mushroom.int.mm)
#' @export

clean_colnames <- function(data){
  .check_data_frame(data, "clean_colnames")
  colnames(data) <- gsub("_min|_max|_Min|_Max|.min|.max|.Min|.Max",
                         '', colnames(data))
  return(data)
}


#' RSDA Format
#'
#' @name RSDA_format
#' @aliases RSDA_format
#' @description This function changes the format of the data to conform to RSDA format.
#' @usage RSDA_format(data, sym_type1 = NULL, location = NULL, sym_type2 = NULL, var = NULL)
#' @param data A conventional data.
#' @param sym_type1 The labels I means an interval variable and $S means set variable.
#' @param location The location of the sym_type in the data.
#' @param sym_type2 The labels I means an interval variable and $S means set variable.
#' @param var The name of the symbolic variable in the data.
#' @returns Return a dataframe with a label added to the previous column of symbolic variable.
#' @examples
#' data("mushroom.int.mm")
#' mushroom.set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
#' mushroom.tmp <- RSDA_format(data = mushroom.set, sym_type1 = c("I", "S"),
#'                             location = c(25, 31), sym_type2 = c("S", "I", "I"),
#'                             var = c("Species", "Stipe.Length_min", "Stipe.Thickness_min"))
#' @export
RSDA_format <- function(data, sym_type1 = NULL, location = NULL,
                        sym_type2 = NULL, var = NULL){
  .check_data_frame(data, "RSDA_format")
  if (!is.null(sym_type1) && !is.character(sym_type1)) {
    stop("RSDA_format: 'sym_type1' must be a character vector.", call. = FALSE)
  }
  if (!is.null(location) && !is.numeric(location)) {
    stop("RSDA_format: 'location' must be numeric.", call. = FALSE)
  }
  if (!is.null(sym_type2) && !is.character(sym_type2)) {
    stop("RSDA_format: 'sym_type2' must be a character vector.", call. = FALSE)
  }
  if (!is.null(var) && !is.character(var)) {
    stop("RSDA_format: 'var' must be a character vector.", call. = FALSE)
  }
  nc <- ncol(data)
  nr <- nrow(data)
  if (is.null(sym_type1) != TRUE && is.null(sym_type2) == TRUE){
    if(length(sym_type1) != length(location)){
      stop("RSDA_format: length of 'sym_type1' (", length(sym_type1),
           ") must equal length of 'location' (", length(location), ").", call. = FALSE)
    }
    data.rep <- .insert_sym_labels(data, location, sym_type1, nr, nc)
  }
  if (is.null(sym_type1) == TRUE && is.null(sym_type2) != TRUE){
    location_fun <- function(x){
      return(x %in% var)
    }
    location_var <- which(apply(matrix(colnames(data), nrow = 1), 1, location_fun))
    if(length(sym_type2) != length(location_var)){
      stop("RSDA_format: length of 'sym_type2' (", length(sym_type2),
           ") must equal number of matched variables (", length(location_var), ").", call. = FALSE)
    }
    data.rep <- .insert_sym_labels(data, location_var, sym_type2, nr, nc)
  }
  if (is.null(sym_type1) != TRUE && is.null(sym_type2) != TRUE){
    location_fun <- function(x){
      return(x %in% var)
    }
    location_var <- which(apply(matrix(colnames(data), nrow = 1), 1, location_fun))
    if(length(sym_type1) != length(location)){
      stop("RSDA_format: length of 'sym_type1' (", length(sym_type1),
           ") must equal length of 'location' (", length(location), ").", call. = FALSE)
    }
    if(length(sym_type2) != length(location_var)){
      stop("RSDA_format: length of 'sym_type2' (", length(sym_type2),
           ") must equal number of matched variables (", length(location_var), ").", call. = FALSE)
    }
    location_sort <- sort(c(location, location_var), index.return = TRUE)
    location_merge <- location_sort$x
    location_index <- location_sort$ix
    sym_type_merge <- c(sym_type1, sym_type2)
    sym_type <- sym_type_merge[location_index]
    data.rep <- .insert_sym_labels(data, location_merge, sym_type, nr, nc)
  }
  return(data.rep)
}

# Internal helper: insert sym_type labels and rebuild data.rep
.insert_sym_labels <- function(data, locations, sym_types, nr, nc) {
  n <- length(locations)
  lc <- c(locations, nc)
  data.rep <- rep(NA, nr)
  gap <- NULL
  for (i in 1:n) {
    gap[i] <- lc[(i + 1)] - lc[i]
    gap.data <- data[, lc[i]:(lc[i] + gap[i] - 1)]
    rep.money <- rep(paste0("$", sym_types[i]), nr)
    data.rep <- cbind(data.rep, rep.money, gap.data)
  }
  if (locations[n] == nc){
    data.rep <- data.rep[, -c(1, length(data.rep))]
  } else {
    data.rep <- data.rep[, -1]
    data.rep <- cbind(data.rep, data[, nc])
  }
  if (length(locations) == 1){
    if (locations[1] != 1){
      data.rep <- cbind(data[, 1:locations[1] - 1], data.rep)
    }
  } else {
    if (locations[1] != 1){
      if (locations[1] == 2){
        data.rep <- cbind(data[, 1], data.rep)
        names(data.rep)[1] <- names(data)[1]
      } else {
        data.rep[, 1:locations[1] - 1] <- data[, 1:locations[1] - 1]
      }
    }
  }
  index <- lc[1:n] + c(1:n) - 1
  var.name <- lc[1:n] + c(1:n)
  names(data.rep)[index] <- c(paste0("$", sym_types))
  names(data.rep)[var.name] <- names(data)[locations]
  names(data.rep)[ncol(data.rep)] <- names(data)[nc]
  data.rep
}


#' Set Variable Format
#'
#' @name set_variable_format
#' @aliases set_variable_format
#' @description This function changes the format of the set variables
#' in the data to conform to the RSDA format.
#' @usage set_variable_format(data, location = NULL, var = NULL)
#' @param data A conventional data.
#' @param location The location of the set variable in the data.
#' @param var The name of the set variable in the data.
#' @returns Return a dataframe in which a set variable is converted to one-hot encoding.
#' @examples
#' data("mushroom.int.mm")
#' mushroom.set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
#' @export
set_variable_format <- function(data, location = NULL, var = NULL){
  .check_data_frame(data, "set_variable_format")
  if (!is.null(location)) {
    .check_location(location, ncol(data), "set_variable_format")
  }
  if (!is.null(var)) {
    if (!is.character(var)) {
      stop("set_variable_format: 'var' must be a character string.", call. = FALSE)
    }
    missing_vars <- setdiff(var, colnames(data))
    if (length(missing_vars) > 0) {
      stop("set_variable_format: variable(s) not found in data: ",
           paste(missing_vars, collapse = ", "), ".", call. = FALSE)
    }
  }
  if (is.null(location) != TRUE){
    data <- .one_hot_at(data, location)
  }
  if (is.null(var) != TRUE){
    location_var <- which(colnames(data) == var)
    data <- .one_hot_at(data, location_var)
  }
  return(data)
}

# Internal helper: one-hot encode the column at col_index
.one_hot_at <- function(data, col_index) {
  data.set <- data
  nr <- nrow(data)
  nc <- ncol(data)
  y <- data[, col_index]
  set_table <- data.frame(matrix(0, nr, length(unique(y)) + 1))
  for (i in 1:length(unique(y))){
    set_table[, 1] <- y
    set_table[, i + 1] <- y %in% unique(y)[i]*1
    names(set_table)[i + 1] <- unique(y)[i]
  }
  set_table[, 1] <- rep(length(unique(y)), nr)
  if (col_index == ncol(data)){
    data <- cbind(data[, 1:col_index], set_table)
  } else{
    data <- cbind(data[, 1:col_index], set_table, data[,(col_index + 1):nc])
  }
  names(data)[col_index + 1] <- names(data.set)[col_index]
  data <- data[, -col_index]
  data
}

test_that("set_variable_format works with location parameter", {
  data(mushroom.int.mm)
  result <- set_variable_format(data = mushroom.int.mm, location = 8)
  expect_s3_class(result, "data.frame")
})

test_that("set_variable_format works with var parameter", {
  data(mushroom.int.mm)
  result <- set_variable_format(data = mushroom.int.mm, var = "Species")
  expect_s3_class(result, "data.frame")
})

test_that("set_variable_format preserves row count with location", {
  data(mushroom.int.mm)
  result <- set_variable_format(data = mushroom.int.mm, location = 8)
  expect_equal(nrow(result), nrow(mushroom.int.mm))
})

test_that("set_variable_format preserves row count with var", {
  data(mushroom.int.mm)
  result <- set_variable_format(data = mushroom.int.mm, var = "Species")
  expect_equal(nrow(result), nrow(mushroom.int.mm))
})

test_that("set_variable_format creates one-hot columns with 0/1 values", {
  data(mushroom.int.mm)
  n_orig <- ncol(mushroom.int.mm)
  n_species <- length(unique(mushroom.int.mm$Species))
  result <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  species_vals <- unique(mushroom.int.mm$Species)
  for (sp in species_vals) {
    if (sp %in% names(result)) {
      vals <- unique(result[[sp]])
      expect_true(all(vals %in% c(0, 1)),
                  info = paste("Column", sp, "should be 0/1"))
    }
  }
})

test_that("set_variable_format with both location and var", {
  data(mushroom.int.mm)
  result <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), nrow(mushroom.int.mm))
})

test_that("set_variable_format increases column count", {
  data(mushroom.int.mm)
  n_orig <- ncol(mushroom.int.mm)
  result <- set_variable_format(data = mushroom.int.mm, location = 8)
  expect_true(ncol(result) > n_orig)
})

test_that("set_variable_format documentation example works", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  expect_s3_class(mushroom.int.mm_set, "data.frame")
  expect_true(nrow(mushroom.int.mm_set) > 0)
  expect_true(ncol(mushroom.int.mm_set) > 0)
})

# Tests for the zero_width handling in aggregate_to_symbolic(type = "int").

# A categorical grouping in which group "a" is constant on v1 (-> zero-width
# interval) while group "b" is not.
make_zw_df <- function() {
  data.frame(
    g  = rep(c("a", "b"), each = 5),
    v1 = c(rep(7, 5), 1:5),
    v2 = c(1:5, 6:10)
  )
}

test_that("zero_width = 'keep' (default) leaves zero-width intervals untouched", {
  df <- make_zw_df()
  # The default is "keep": no concepts dropped, no modification, no warning.
  expect_silent(
    res <- aggregate_to_symbolic(df, type = "int", group_by = "g")
  )
  expect_equal(nrow(res), 2L)
  expect_setequal(as.character(res$g), c("a", "b"))
  # group "a" on v1 is constant (== 7) -> a zero-width interval is retained
  a_v1 <- unclass(res$v1)[as.character(res$g) == "a"]
  expect_equal(Re(a_v1), 7)
  expect_equal(Im(a_v1), 7)
  expect_true(any(check_zero_width_intervals(res, warn = FALSE)))
})

test_that("explicit zero_width = 'keep' matches the default behaviour", {
  df <- make_zw_df()
  expect_silent(
    res <- aggregate_to_symbolic(df, type = "int", group_by = "g",
                                 zero_width = "keep")
  )
  expect_equal(nrow(res), 2L)
})

test_that("zero_width = 'remove' drops concepts with zero-width intervals", {
  df <- make_zw_df()
  res <- suppressWarnings(
    aggregate_to_symbolic(df, type = "int", group_by = "g",
                          zero_width = "remove")
  )
  expect_equal(nrow(res), 1L)
  expect_equal(as.character(res$g), "b")
  expect_false(any(check_zero_width_intervals(res, warn = FALSE)))
})

test_that("zero_width = 'remove' warns naming removed concept and variable", {
  df <- make_zw_df()
  expect_warning(
    aggregate_to_symbolic(df, type = "int", group_by = "g", zero_width = "remove"),
    "removed 1 concept.*a.*v1"
  )
})

test_that("zero_width = 'remove' errors when every concept is degenerate", {
  df <- data.frame(g = rep(c("a", "b"), each = 3), v1 = rep(c(2, 5), each = 3))
  expect_error(
    suppressWarnings(
      aggregate_to_symbolic(df, type = "int", group_by = "g", zero_width = "remove")
    ),
    "every concept contains a zero-width interval"
  )
})

test_that("zero_width = 'adjust' widens zero-width upper endpoints by epsilon", {
  df <- make_zw_df()
  res <- suppressWarnings(
    aggregate_to_symbolic(df, type = "int", group_by = "g", zero_width = "adjust")
  )
  expect_equal(nrow(res), 2L)
  expect_false(any(check_zero_width_intervals(res, warn = FALSE)))

  # group "a" on v1: min == max == 7, upper endpoint bumped by epsilon
  a_v1 <- unclass(res$v1)[as.character(res$g) == "a"]
  expect_equal(Re(a_v1), 7)
  expect_equal(Im(a_v1), 7 + 1e-07)
})

test_that("zero_width = 'adjust' honours a custom epsilon", {
  df <- make_zw_df()
  res <- suppressWarnings(
    aggregate_to_symbolic(df, type = "int", group_by = "g",
                          zero_width = "adjust", epsilon = 1e-03)
  )
  a_v1 <- unclass(res$v1)[as.character(res$g) == "a"]
  expect_equal(Im(a_v1), 7 + 1e-03)
})

test_that("zero_width = 'adjust' warns about the adjustment", {
  df <- make_zw_df()
  expect_warning(
    aggregate_to_symbolic(df, type = "int", group_by = "g", zero_width = "adjust"),
    "adjusted by epsilon"
  )
})

test_that("zero_width = 'regenerate' errors on deterministic grouping", {
  df <- make_zw_df()
  expect_error(
    aggregate_to_symbolic(df, type = "int", group_by = "g",
                          zero_width = "regenerate"),
    "deterministic"
  )
})

test_that("zero_width = 'regenerate' succeeds for stochastic resampling", {
  set.seed(42)
  res <- aggregate_to_symbolic(iris[, 1:4], type = "int",
                               group_by = "resampling", K = 5, nK = 30,
                               zero_width = "regenerate")
  expect_equal(nrow(res), 5L)
  expect_false(any(check_zero_width_intervals(res, warn = FALSE)))
})

test_that("no warning and no rows dropped when there are no zero-width intervals", {
  set.seed(1)
  expect_silent(
    res <- aggregate_to_symbolic(iris[, 1:4], type = "int",
                                 group_by = "kmeans", K = 3)
  )
  expect_equal(nrow(res), 3L)
})

test_that("epsilon must be a single positive number", {
  df <- make_zw_df()
  expect_error(
    aggregate_to_symbolic(df, type = "int", group_by = "g", epsilon = -1),
    "'epsilon' must be a single positive number"
  )
  expect_error(
    aggregate_to_symbolic(df, type = "int", group_by = "g", epsilon = c(1, 2)),
    "'epsilon' must be a single positive number"
  )
})

test_that("invalid zero_width option is rejected", {
  df <- make_zw_df()
  expect_error(
    aggregate_to_symbolic(df, type = "int", group_by = "g", zero_width = "nope")
  )
})

test_that("zero_width is ignored for type = 'hist'", {
  h <- aggregate_to_symbolic(iris, type = "hist", group_by = "Species",
                             bins = 5, zero_width = "remove")
  expect_s4_class(h, "MatH")
  expect_equal(dim(h@M), c(3L, 4L))
})

test_that("zero_width handling applies within stratified aggregation", {
  df <- data.frame(
    s  = rep(c("x", "y"), each = 6),
    g  = rep(c("a", "b"), 6),
    v1 = c(rep(2, 3), 4:6, 7:9, rep(3, 3)),
    v2 = 1:12
  )
  res <- suppressWarnings(
    aggregate_to_symbolic(df, type = "int", group_by = "g",
                          stratify_var = "s", zero_width = "adjust")
  )
  expect_false(any(check_zero_width_intervals(res, warn = FALSE)))
  expect_setequal(as.character(res$g), c("x.a", "x.b", "y.a", "y.b"))
})

test_that("clean_colnames removes _min and _max suffixes", {
  df <- data.frame(a_min = 1, a_max = 2, b_min = 3, b_max = 4)
  result <- clean_colnames(df)
  expect_equal(names(result), c("a", "a", "b", "b"))
})

test_that("clean_colnames removes .min and .max suffixes", {
  df <- data.frame(a.min = 1, a.max = 2)
  result <- clean_colnames(df)
  expect_equal(names(result), c("a", "a"))
})

test_that("clean_colnames removes _Min and _Max suffixes", {
  df <- data.frame(a_Min = 1, a_Max = 2)
  result <- clean_colnames(df)
  expect_equal(names(result), c("a", "a"))
})

test_that("clean_colnames removes .Min and .Max suffixes", {
  df <- data.frame(a.Min = 1, a.Max = 2)
  result <- clean_colnames(df)
  expect_equal(names(result), c("a", "a"))
})

test_that("clean_colnames keeps columns without suffixes unchanged", {
  df <- data.frame(x = 1, y = 2, z = 3)
  result <- clean_colnames(df)
  expect_equal(names(result), c("x", "y", "z"))
})

test_that("clean_colnames handles mixed suffixes", {
  df <- data.frame(a_min = 1, a_max = 2, b = 3, c.Min = 4, c.Max = 5)
  result <- clean_colnames(df)
  expect_equal(names(result), c("a", "a", "b", "c", "c"))
})

test_that("clean_colnames returns a data.frame", {
  df <- data.frame(a_min = 1, a_max = 2)
  result <- clean_colnames(df)
  expect_s3_class(result, "data.frame")
})

test_that("clean_colnames works with mushroom.int.mm dataset", {
  data(mushroom.int.mm)
  result <- clean_colnames(mushroom.int.mm)
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), nrow(mushroom.int.mm))
})

test_that("RSDA_format with sym_type1 and location only", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("I"),
                        location = c(25))
  expect_s3_class(result, "data.frame")
  expect_true(any(grepl("\\$I", names(result))))
})

test_that("RSDA_format with sym_type2 and var only", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type2 = c("I"),
                        var = c("Stipe.Length_min"))
  expect_s3_class(result, "data.frame")
  expect_true(any(grepl("\\$I", names(result))))
})

test_that("RSDA_format with both sym_type1 and sym_type2", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("I", "S"),
                        location = c(25, 31), sym_type2 = c("S", "I", "I"),
                        var = c("Species", "Stipe.Length_min", "Stipe.Thickness_min"))
  expect_s3_class(result, "data.frame")
})

test_that("RSDA_format inserts $I labels", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("I"),
                        location = c(25))
  dollar_cols <- grep("^\\$", names(result), value = TRUE)
  expect_true(length(dollar_cols) > 0)
  expect_true("$I" %in% names(result))
})

test_that("RSDA_format inserts $S labels", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("S"),
                        location = c(31))
  dollar_cols <- grep("^\\$", names(result), value = TRUE)
  expect_true(length(dollar_cols) > 0)
  expect_true("$S" %in% names(result))
})

test_that("RSDA_format errors for length mismatch (sym_type1)", {
  data(mushroom.int.mm)
  expect_error(
    RSDA_format(data = mushroom.int.mm, sym_type1 = c("I", "S"), location = c(1)),
    "length of 'sym_type1'"
  )
})

test_that("RSDA_format errors for length mismatch (sym_type2)", {
  data(mushroom.int.mm)
  expect_error(
    RSDA_format(data = mushroom.int.mm, sym_type2 = c("I", "S"),
                var = c("Pileus.Cap.Width_min")),
    "length of 'sym_type2'"
  )
})

test_that("RSDA_format preserves row count", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("I"),
                        location = c(25))
  expect_equal(nrow(result), nrow(mushroom.int.mm_set))
})

test_that("RSDA_format adds columns for labels", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  n_orig <- ncol(mushroom.int.mm_set)
  result <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("I"),
                        location = c(25))
  expect_true(ncol(result) > n_orig)
})

test_that("RSDA_format full pipeline from documentation example", {
  data(mushroom.int.mm)
  mushroom.int.mm_set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
  mushroom_tmp <- RSDA_format(data = mushroom.int.mm_set, sym_type1 = c("I", "S"),
                              location = c(25, 31), sym_type2 = c("S", "I", "I"),
                              var = c("Species", "Stipe.Length_min", "Stipe.Thickness_min"))
  expect_s3_class(mushroom_tmp, "data.frame")
  expect_true(nrow(mushroom_tmp) > 0)
  expect_true(ncol(mushroom_tmp) > 0)
})

test_that("check_zero_width_intervals detects zero-width intervals", {
  df <- data.frame(A_min = c(1, 2, 3), A_max = c(1, 5, 3),
                   B_min = c(0, 0, 0), B_max = c(2, 4, 6))
  result <- suppressWarnings(check_zero_width_intervals(df, warn = FALSE))
  expect_true(result)
  expect_equal(attr(result, "variables"), "A")
  expect_equal(unname(attr(result, "flagged")[, "A"]), c(TRUE, FALSE, TRUE))
  expect_equal(unname(attr(result, "flagged")[, "B"]), c(FALSE, FALSE, FALSE))
})

test_that("check_zero_width_intervals returns FALSE when all intervals have width", {
  df <- data.frame(A_min = c(1, 2), A_max = c(3, 5))
  result <- check_zero_width_intervals(df)
  expect_false(result)
  expect_length(attr(result, "variables"), 0)
  expect_false(any(attr(result, "flagged")))
})

test_that("check_zero_width_intervals warns by default when found", {
  df <- data.frame(A_min = c(1, 2), A_max = c(1, 5))
  expect_warning(check_zero_width_intervals(df),
                 "zero-width intervals.*A")
})

test_that("check_zero_width_intervals is silent with warn = FALSE", {
  df <- data.frame(A_min = c(1, 2), A_max = c(1, 5))
  expect_silent(check_zero_width_intervals(df, warn = FALSE))
})

test_that("check_zero_width_intervals honours tol for near-zero widths", {
  df <- data.frame(A_min = c(1, 2), A_max = c(1.0005, 5))
  expect_false(check_zero_width_intervals(df, tol = 0, warn = FALSE))
  expect_true(check_zero_width_intervals(df, tol = 0.001, warn = FALSE))
})

test_that("check_zero_width_intervals handles _Min/_Max casing", {
  df <- data.frame(A_Min = c(1, 2), A_Max = c(1, 2))
  result <- check_zero_width_intervals(df, warn = FALSE)
  expect_true(result)
  expect_equal(attr(result, "variables"), "A")
})

test_that("check_zero_width_intervals returns invisibly", {
  df <- data.frame(A_min = c(1, 2), A_max = c(3, 5))
  expect_invisible(check_zero_width_intervals(df))
})

test_that("check_zero_width_intervals rejects non-data.frame", {
  expect_error(check_zero_width_intervals(NULL), "must not be NULL")
  expect_error(check_zero_width_intervals("text"), "must be a data.frame")
})

test_that("check_zero_width_intervals rejects bad tol", {
  df <- data.frame(A_min = 1, A_max = 2)
  expect_error(check_zero_width_intervals(df, tol = -1), "non-negative")
  expect_error(check_zero_width_intervals(df, tol = "a"), "non-negative")
  expect_error(check_zero_width_intervals(df, tol = c(1, 2)), "non-negative")
})

test_that("check_zero_width_intervals rejects non-logical warn", {
  df <- data.frame(A_min = 1, A_max = 2)
  expect_error(check_zero_width_intervals(df, warn = "yes"), "must be TRUE or FALSE")
})

test_that("check_zero_width_intervals errors when no _min columns", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_error(check_zero_width_intervals(df), "no '_min' columns")
})

test_that("check_zero_width_intervals errors on unmatched _max column", {
  df <- data.frame(A_min = c(1, 2), B_max = c(3, 5))
  expect_error(check_zero_width_intervals(df), "no matching '_max' column")
})

test_that("check_zero_width_intervals works with mushroom.int.mm dataset", {
  data(mushroom.int.mm)
  result <- check_zero_width_intervals(mushroom.int.mm, warn = FALSE)
  expect_type(result, "logical")
  expect_length(result, 1)
})

# --- symbolic_tbl (RSDA format) input ----------------------------------------

test_that("check_zero_width_intervals accepts a symbolic_tbl", {
  x <- array(0, dim = c(3, 2, 2),
             dimnames = list(c("r1", "r2", "r3"), c("A", "B"), c("min", "max")))
  x[, , 1] <- cbind(A = c(1, 2, 3), B = c(0, 0, 0))   # mins
  x[, , 2] <- cbind(A = c(1, 5, 3), B = c(2, 4, 6))   # maxs (A zero-width in r1, r3)
  st <- ARRAY_to_RSDA(x)

  result <- check_zero_width_intervals(st, warn = FALSE)
  expect_true(result)
  expect_equal(attr(result, "variables"), "A")
  expect_equal(unname(attr(result, "flagged")[, "A"]), c(TRUE, FALSE, TRUE))
  expect_equal(unname(attr(result, "flagged")[, "B"]), c(FALSE, FALSE, FALSE))
})

test_that("check_zero_width_intervals returns FALSE for a clean symbolic_tbl", {
  x <- array(0, dim = c(2, 1, 2),
             dimnames = list(c("r1", "r2"), "A", c("min", "max")))
  x[, , 1] <- c(1, 2)
  x[, , 2] <- c(3, 5)
  st <- ARRAY_to_RSDA(x)

  result <- check_zero_width_intervals(st)
  expect_false(result)
  expect_length(attr(result, "variables"), 0)
  expect_false(any(attr(result, "flagged")))
})

test_that("check_zero_width_intervals warns on a zero-width symbolic_tbl", {
  x <- array(0, dim = c(2, 1, 2),
             dimnames = list(c("r1", "r2"), "A", c("min", "max")))
  x[, , 1] <- c(1, 2)
  x[, , 2] <- c(1, 5)   # r1 zero-width
  st <- ARRAY_to_RSDA(x)
  expect_warning(check_zero_width_intervals(st), "zero-width intervals.*A")
})

test_that("check_zero_width_intervals honours tol for a symbolic_tbl", {
  x <- array(0, dim = c(2, 1, 2),
             dimnames = list(c("r1", "r2"), "A", c("min", "max")))
  x[, , 1] <- c(1, 2)
  x[, , 2] <- c(1.0005, 5)
  st <- ARRAY_to_RSDA(x)
  expect_false(check_zero_width_intervals(st, tol = 0, warn = FALSE))
  expect_true(check_zero_width_intervals(st, tol = 0.001, warn = FALSE))
})

test_that("check_zero_width_intervals works with mushroom.int symbolic_tbl", {
  data(mushroom.int)
  result <- check_zero_width_intervals(mushroom.int, warn = FALSE)
  expect_type(result, "logical")
  expect_length(result, 1)
})

options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
#' @name interval_stats
#' @title Statistics for Interval Data
#' @description Functions to compute the mean, variance, covariance, and correlation of interval-valued data.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name the variable name or the column location (multiple variables are allowed).
#' @param var_name1 the variable name or the column location (multiple variables are allowed).
#' @param var_name2 the variable name or the column location (multiple variables are allowed).
#' @param method methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.
#' @param ... additional parameters
#' @return A numeric matrix for \code{int_mean} and \code{int_var} (methods x variables);
#'   a named list of covariance/correlation matrices for \code{int_cov} and \code{int_cor}
#'   (one matrix per method).
#' @details
#' Available methods (applicable to all four functions):
#' \itemize{
#'   \item \code{CM}: Center Method — uses midpoints (a + b) / 2
#'   \item \code{VM}: Vertices Method — uses all 2^p vertex combinations
#'   \item \code{QM}: Quantiles Method — uses equally spaced quantile points
#'   \item \code{SE}: Set Expansion — uses endpoints only (quantiles with m = 1)
#'   \item \code{FV}: Fitted Values — uses linear regression fitted values
#'   \item \code{EJD}: Empirical Joint Distribution
#'   \item \code{GQ}: Symbolic Covariance method (Billard and Diday, 2006)
#'   \item \code{SPT}: Total Sum of Products (Billard, 2008)
#' }
#' @author Han-Ming Wu
#' @seealso int_mean int_var int_cov int_cor
#' @examples
#' data(mushroom.int)
#' int_mean(mushroom.int, var_name = "Pileus.Cap.Width")
#' int_mean(mushroom.int, var_name = 2:3)
#'
#' var_name <- c("Stipe.Length", "Stipe.Thickness")
#' method <- c("CM", "FV", "EJD")
#' int_mean(mushroom.int, var_name, method)
#' int_var(mushroom.int, var_name, method)
#'
#' var_name1 <- "Pileus.Cap.Width"
#' var_name2 <- c("Stipe.Length", "Stipe.Thickness")
#' method <- c("CM", "VM", "EJD", "GQ", "SPT")
#' int_cov(mushroom.int, var_name1, var_name2, method)
#' int_cor(mushroom.int, var_name1, var_name2, method)
#' @importFrom stats var cov lm
#' @export
int_mean <- function(x, var_name, method = "CM", ...){
  .check_symbolic_tbl(x, "int_mean")
  .check_var_name(var_name, x, "int_mean")
  .check_interval_method(method, "int_mean")

  at <- options %in% method
  mean_tmp <- matrix(0, nrow = length(options),
                        ncol = length(var_name))
  idata <- symbolic_tbl_to_idata(x[, var_name])

  compute_mean <- function(X_tmp){
    ifelse(length(var_name) == 1,
           x <- mean(X_tmp),
           x <- colMeans(X_tmp))
    x
  }

  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    mean_tmp[idx, ] <- compute_mean(transforms[[nm]])
  }

  if(at[6] | at[7] | at[8]){ # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    mean_tmp[6, ] <- mean_tmp[7, ] <- mean_tmp[8, ] <- compute_mean(X_tmp)
  }

  mean_output <- matrix(mean_tmp[at, ],
                           nrow = length(method),
                           ncol = length(var_name))

  if(is.numeric(var_name)){
    colnames(mean_output) <- colnames(x)[var_name]
  }else{
    colnames(mean_output) <- var_name
  }
  rownames(mean_output) <- options[at]

  mean_output
}



#' @rdname interval_stats
#' @export
int_var <- function(x, var_name, method = "CM", ...){
  .check_symbolic_tbl(x, "int_var")
  .check_var_name(var_name, x, "int_var")
  .check_interval_method(method, "int_var")

  at <- options %in% method
  var_tmp <- matrix(0, nrow = length(options),
                     ncol = length(var_name))
  idata <- symbolic_tbl_to_idata(x[, var_name])

  n <- nrow(idata)
  p <- ncol(idata)
  compute_var <- function(X_tmp){
    ifelse(length(var_name) == 1,
           x <- stats::var(X_tmp),
           x <- apply(X_tmp, 2, stats::var))
    x
  }

  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    var_tmp[idx, ] <- compute_var(transforms[[nm]])
  }

  if(at[6] | at[7] | at[8]){ # EJD, GQ, SPT
    ans <- numeric(length(var_name))
    names(ans) <- var_name
    for(i in var_name){
      a <- sum(idata[, i,2]^2 + idata[, i,1]*idata[, i,2]+idata[,i,1]^2)
      b <- (sum(idata[, i,1] + idata[, i,2]))^2
      ans[i] <- a/(3*n)-b/(4*n^2)
    }
    if(at[6]) var_tmp[6, ] <- ans
    if(at[7]) var_tmp[7, ] <- ans
    if(at[8]) var_tmp[8, ] <- ans
  }

  var_output <- matrix(var_tmp[at, ],
                        nrow = length(method),
                        ncol = length(var_name))

  if(is.numeric(var_name)){
    colnames(var_output) <- colnames(x)[var_name]
  }else{
    colnames(var_output) <- var_name
  }
  rownames(var_output) <- options[at]

  var_output
}


#' @rdname interval_stats
#' @export
int_cov <- function(x, var_name1, var_name2, method = "CM", ...){
  .check_symbolic_tbl(x, "int_cov")
  .check_var_name(var_name1, x, "int_cov")
  .check_var_name(var_name2, x, "int_cov")
  .check_interval_method(method, "int_cov")

  var_name <- c(var_name1, var_name2)
  at <- options %in% method
  cov_tmp <- new.env()
  cov_tmp <- as.list(cov_tmp)
  idata <- symbolic_tbl_to_idata(x[, var_name])

  n <- nrow(idata)
  p <- ncol(idata)
  compute_cov <- function(X_tmp){
    ans <- as.matrix(stats::cov(X_tmp[, var_name1],
                                X_tmp[, var_name2]))
    if(length(var_name1) == 1){
      rownames(ans) <- var_name1
    }
    if(length(var_name2) == 1){
      colnames(ans) <- var_name2
    }
    ans
  }

  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    cov_tmp[[nm]] <- compute_cov(transforms[[nm]])
  }

  if(at[6]){ # EJD
    ans <- matrix(0, nrow = length(var_name1), ncol = length(var_name2))
    rownames(ans) <- var_name1
    colnames(ans) <- var_name2
    for(i in var_name1){
      for(j in var_name2){
        a <- sum(idata[, i,1] + idata[, i,2])
        b <- sum(idata[, j,1] + idata[, j,2])
        c <- sum((idata[, i,1] + idata[, i,2])*(idata[, j,1] + idata[, j,2]))
        ans[i, j] <- c/(4*n)-(a*b)/(4*n^2)
      }
      cov_tmp$EJD <- ans
    }
  }
  if(at[7] | at[8]){ # GQ or SPT
      xbaru <- (idata[, ,1] + idata[, ,2])/2
      xbar <- colMeans(xbaru)
      xbar
  }

  if(at[7]){ # GQ
    Gu = matrix(-1, n, p)

    for (j in 1:p){
      for (u in 1:n){
        if (xbaru[u,j] > xbar[j])
          Gu[u,j] = 1
      }
    }

    colnames(Gu) <- var_name

    Qu = matrix(0, n, p)
    for (j in 1:p){
      for (u in 1:n){
        Qu[u,j] = (idata[u,j,1] - xbar[j])^2 +
          (idata[u,j,1] - xbar[j])*(idata[u,j,2] - xbar[j]) +
          (idata[u,j,2] - xbar[j])^2
      }
    }
    colnames(Qu) <- var_name
    ans <- matrix(0, nrow = length(var_name1), ncol = length(var_name2))
    rownames(ans) <- var_name1
    colnames(ans) <- var_name2

    for(i in var_name1){
      for(j in var_name2){
        ans[i,j] <- sum((Gu[,i]*Gu[,j]*sqrt(Qu[,i]*Qu[,j])))/(3*n)
      }
    }
    cov_tmp$GQ <- ans
  }

  if(at[8]){ # SPT
    ans <- matrix(0, nrow = length(var_name1), ncol = length(var_name2))
    rownames(ans) <- var_name1
    colnames(ans) <- var_name2

    for(i in var_name1){
      for(j in var_name2){
      a2 <- (idata[, i,1] - xbar[i])*(idata[, j,1] - xbar[j])
      ab <- (idata[, i,1] - xbar[i])*(idata[, j,2] - xbar[j]) +
        (idata[, i,2] - xbar[i])*(idata[, j,1] - xbar[j])
      b2 <- (idata[, i,2] - xbar[i])*(idata[, j,2] - xbar[j])
      ans[i, j] <- sum(2*a2 + ab + 2*b2)/(6*n)
      }
    }
    cov_tmp$SPT <- ans
  }

  cov_output <- cov_tmp

  cov_output
}



#' @rdname interval_stats
#' @export
int_cor <- function(x, var_name1, var_name2, method = "CM", ...){
  .check_symbolic_tbl(x, "int_cor")
  .check_var_name(var_name1, x, "int_cor")
  .check_var_name(var_name2, x, "int_cor")
  .check_interval_method(method, "int_cor")

  var_1 <- int_var(x, var_name1, method)
  var_2 <- int_var(x, var_name2, method)
  cov_12 <- int_cov(x, var_name1, var_name2, method)

  cor_output <- cov_12
  for(k in 1:length(method)){
    for(i in var_name1){
      for(j in var_name2){
        cor_output[[k]][i, j] <- cov_12[[k]][i, j]/sqrt(var_1[k,i]*var_2[k,j])
      }
    }
  }
  cor_output
}

# ---- int_mean ----

test_that("int_mean with default CM method works", {
  data(mushroom.int)
  result <- int_mean(mushroom.int, var_name = "Pileus.Cap.Width")
  expect_true(is.matrix(result))
  expect_equal(nrow(result), 1)
  expect_equal(ncol(result), 1)
})

test_that("int_mean with numeric var_name works", {
  data(mushroom.int)
  result <- int_mean(mushroom.int, var_name = 2:3)
  expect_true(is.matrix(result))
  expect_equal(ncol(result), 2)
})

test_that("int_mean with multiple methods works", {
  data(mushroom.int)
  methods <- c("CM", "FV", "EJD")
  result <- int_mean(mushroom.int, var_name = "Pileus.Cap.Width", method = methods)
  expect_equal(nrow(result), length(methods))
})

test_that("int_mean with multiple variables and methods works", {
  data(mushroom.int)
  var_name <- c("Stipe.Length", "Stipe.Thickness")
  methods <- c("CM", "FV", "EJD")
  result <- int_mean(mushroom.int, var_name, methods)
  expect_equal(nrow(result), length(methods))
  expect_equal(ncol(result), length(var_name))
})

test_that("int_mean all 8 methods produce results", {
  data(mushroom.int)
  all_methods <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  result <- int_mean(mushroom.int, var_name = "Pileus.Cap.Width", method = all_methods)
  expect_equal(nrow(result), 8)
  expect_true(all(is.finite(result)))
})

test_that("int_mean returns named matrix", {
  data(mushroom.int)
  result <- int_mean(mushroom.int, var_name = "Pileus.Cap.Width", method = "CM")
  expect_equal(rownames(result), "CM")
  expect_equal(colnames(result), "Pileus.Cap.Width")
})

# ---- int_var ----

test_that("int_var with default CM method works", {
  data(mushroom.int)
  result <- int_var(mushroom.int, var_name = "Pileus.Cap.Width")
  expect_true(is.matrix(result))
  expect_true(all(result >= 0))
})

test_that("int_var with multiple methods works", {
  data(mushroom.int)
  methods <- c("CM", "VM", "QM")
  result <- int_var(mushroom.int, var_name = "Pileus.Cap.Width", method = methods)
  expect_equal(nrow(result), length(methods))
  expect_true(all(result >= 0))
})

test_that("int_var variance is non-negative for all methods", {
  data(mushroom.int)
  all_methods <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  result <- int_var(mushroom.int, var_name = "Pileus.Cap.Width", method = all_methods)
  expect_true(all(result >= 0))
})

test_that("int_var with multiple variables works", {
  data(mushroom.int)
  var_name <- c("Stipe.Length", "Stipe.Thickness")
  result <- int_var(mushroom.int, var_name, method = "CM")
  expect_equal(ncol(result), 2)
  expect_true(all(result >= 0))
})

test_that("int_var returns named matrix", {
  data(mushroom.int)
  result <- int_var(mushroom.int, var_name = "Pileus.Cap.Width", method = "CM")
  expect_equal(rownames(result), "CM")
  expect_equal(colnames(result), "Pileus.Cap.Width")
})

test_that("int_var with numeric var_name works", {
  data(mushroom.int)
  result <- int_var(mushroom.int, var_name = 2, method = "CM")
  expect_true(is.matrix(result))
  expect_true(all(result >= 0))
})

# ---- int_cov ----

test_that("int_cov with default CM method works", {
  data(mushroom.int)
  result <- int_cov(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length")
  expect_true(is.list(result))
  expect_true("CM" %in% names(result))
})

test_that("int_cov returns a list with method names", {
  data(mushroom.int)
  methods <- c("CM", "VM", "EJD", "GQ", "SPT")
  result <- int_cov(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = methods)
  expect_true(is.list(result))
  expect_equal(length(result), length(methods))
})

test_that("int_cov with multiple var_name2 works", {
  data(mushroom.int)
  result <- int_cov(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = c("Stipe.Length", "Stipe.Thickness"))
  expect_true(is.list(result))
  expect_equal(ncol(result$CM), 2)
})

test_that("int_cov each element is a matrix", {
  data(mushroom.int)
  result <- int_cov(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = "CM")
  expect_true(is.matrix(result$CM))
})

test_that("int_cov with all 8 methods works", {
  data(mushroom.int)
  all_methods <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  result <- int_cov(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = all_methods)
  expect_equal(length(result), 8)
})

# ---- int_cor ----

test_that("int_cor with default CM method works", {
  data(mushroom.int)
  result <- int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length")
  expect_true(is.list(result))
})

test_that("int_cor values are in [-1, 1]", {
  data(mushroom.int)
  methods <- c("CM", "VM")
  result <- int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = methods)
  for (m in names(result)) {
    vals <- as.numeric(result[[m]])
    expect_true(all(vals >= -1 & vals <= 1),
                info = paste("Correlation out of range for method", m))
  }
})

test_that("int_cor with multiple var_name2 works", {
  data(mushroom.int)
  result <- int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = c("Stipe.Length", "Stipe.Thickness"),
                    method = "CM")
  expect_true(is.list(result))
  expect_equal(ncol(result$CM), 2)
})

test_that("int_cor EJD method values in [-1, 1]", {
  data(mushroom.int)
  result <- int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = "EJD")
  vals <- as.numeric(result$EJD)
  expect_true(all(vals >= -1 & vals <= 1))
})

test_that("int_cor GQ method values in [-1, 1]", {
  data(mushroom.int)
  result <- int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = "GQ")
  vals <- as.numeric(result$GQ)
  expect_true(all(vals >= -1 & vals <= 1))
})

test_that("int_cor SPT method values in [-1, 1]", {
  data(mushroom.int)
  result <- int_cor(mushroom.int, var_name1 = "Pileus.Cap.Width",
                    var_name2 = "Stipe.Length", method = "SPT")
  vals <- as.numeric(result$SPT)
  expect_true(all(vals >= -1 & vals <= 1))
})

#' @name interval_position
#' @title Position and Scale Measures for Interval Data
#' @description Functions to compute position and scale statistics for interval-valued data.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name the variable name or the column location (multiple variables are allowed).
#' @param method methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.
#' @param probs numeric vector of probabilities with values in [0,1].
#' @param breaks number of histogram breaks for mode estimation (default: 30).
#' @param ... additional parameters
#' @return A numeric matrix or value
#' @details
#' These functions provide position and scale measures:
#' \itemize{
#'   \item \code{int_median}: Median of interval data
#'   \item \code{int_quantile}: Quantiles of interval data
#'   \item \code{int_range}: Range (max - min) of interval data
#'   \item \code{int_iqr}: Interquartile range (Q3 - Q1)
#'   \item \code{int_mad}: Median absolute deviation
#'   \item \code{int_mode}: Mode of interval data (estimated via histogram)
#' }
#' @author Han-Ming Wu
#' @seealso int_mean int_var int_median int_quantile
#' @examples
#' data(mushroom.int)
#' 
#' # Calculate median
#' int_median(mushroom.int, var_name = "Pileus.Cap.Width")
#' int_median(mushroom.int, var_name = 2:3, method = c("CM", "EJD"))
#' 
#' # Calculate quantiles
#' int_quantile(mushroom.int, var_name = 2, probs = c(0.25, 0.5, 0.75))
#' 
#' # Calculate interquartile range
#' int_iqr(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))
#' 
#' # Calculate range
#' int_range(mushroom.int, var_name = "Pileus.Cap.Width")
#'
#' # Calculate MAD
#' int_mad(mushroom.int, var_name = 2:3, method = "CM")
#'
#' # Estimate mode
#' int_mode(mushroom.int, var_name = "Stipe.Length", method = "CM")
#' @importFrom stats quantile median mad
#' @importFrom graphics hist
#' @export
int_median <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_median")
  .check_var_name(var_name, x, "int_median")
  .check_interval_method(method, "int_median")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  median_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  compute_median <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- stats::median(X_tmp)
    } else {
      x <- apply(X_tmp, 2, stats::median)
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    median_tmp[idx, ] <- compute_median(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    median_tmp[6, ] <- median_tmp[7, ] <- median_tmp[8, ] <- compute_median(X_tmp)
  }
  
  median_output <- matrix(median_tmp[at, ], 
                          nrow = length(method), 
                          ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(median_output) <- colnames(x)[var_name]
  } else {
    colnames(median_output) <- var_name
  }
  rownames(median_output) <- options[at]
  
  median_output
}


#' @rdname interval_position
#' @export
int_quantile <- function(x, var_name, probs = c(0.25, 0.5, 0.75), method = "CM", ...) {
  .check_symbolic_tbl(x, "int_quantile")
  .check_var_name(var_name, x, "int_quantile")
  .check_interval_method(method, "int_quantile")
  
  if (any(probs < 0 | probs > 1)) {
    stop("probs must be between 0 and 1", call. = FALSE)
  }
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n_probs <- length(probs)
  
  # Initialize output list
  quantile_output <- vector("list", length(method))
  names(quantile_output) <- options[at]
  
  compute_quantile <- function(X_tmp) {
    q_mat <- matrix(0, nrow = n_probs, ncol = length(var_name))
    if (length(var_name) == 1) {
      q_mat[, 1] <- stats::quantile(X_tmp, probs = probs)
    } else {
      for (j in seq_along(var_name)) {
        q_mat[, j] <- stats::quantile(X_tmp[, j], probs = probs)
      }
    }
    
    if (is.numeric(var_name)) {
      colnames(q_mat) <- colnames(x)[var_name]
    } else {
      colnames(q_mat) <- var_name
    }
    rownames(q_mat) <- paste0(probs * 100, "%")
    q_mat
  }
  
  transforms <- .get_interval_transforms(idata, at)
  k <- 1
  for (nm in names(transforms)) {
    quantile_output[[k]] <- compute_quantile(transforms[[nm]])
    k <- k + 1
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    q_result <- compute_quantile(X_tmp)
    if (at[6]) quantile_output[["EJD"]] <- q_result
    if (at[7]) quantile_output[["GQ"]] <- q_result
    if (at[8]) quantile_output[["SPT"]] <- q_result
  }
  
  quantile_output
}


#' @rdname interval_position
#' @export
int_range <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_range")
  .check_var_name(var_name, x, "int_range")
  .check_interval_method(method, "int_range")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  range_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  compute_range <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- diff(range(X_tmp))
    } else {
      x <- apply(X_tmp, 2, function(col) diff(range(col)))
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    range_tmp[idx, ] <- compute_range(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    range_tmp[6, ] <- range_tmp[7, ] <- range_tmp[8, ] <- compute_range(X_tmp)
  }
  
  range_output <- matrix(range_tmp[at, ], 
                         nrow = length(method), 
                         ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(range_output) <- colnames(x)[var_name]
  } else {
    colnames(range_output) <- var_name
  }
  rownames(range_output) <- options[at]
  
  range_output
}


#' @rdname interval_position
#' @export
int_iqr <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_iqr")
  .check_var_name(var_name, x, "int_iqr")
  .check_interval_method(method, "int_iqr")
  
  # Get Q1 and Q3
  quantiles <- int_quantile(x, var_name, probs = c(0.25, 0.75), method = method)
  
  # Calculate IQR = Q3 - Q1
  iqr_output <- lapply(quantiles, function(q_mat) {
    iqr_vec <- q_mat[2, ] - q_mat[1, ]
    matrix(iqr_vec, nrow = 1, dimnames = list("IQR", colnames(q_mat)))
  })
  
  # Convert list to matrix if only one method
  if (length(method) == 1) {
    iqr_output <- iqr_output[[1]]
  }
  
  iqr_output
}


#' @rdname interval_position
#' @export
int_mad <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_mad")
  .check_var_name(var_name, x, "int_mad")
  .check_interval_method(method, "int_mad")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  mad_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  compute_mad <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- stats::mad(X_tmp, constant = 1)
    } else {
      x <- apply(X_tmp, 2, stats::mad, constant = 1)
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    mad_tmp[idx, ] <- compute_mad(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    mad_tmp[6, ] <- mad_tmp[7, ] <- mad_tmp[8, ] <- compute_mad(X_tmp)
  }
  
  mad_output <- matrix(mad_tmp[at, ], 
                       nrow = length(method), 
                       ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(mad_output) <- colnames(x)[var_name]
  } else {
    colnames(mad_output) <- var_name
  }
  rownames(mad_output) <- options[at]
  
  mad_output
}


#' @rdname interval_position
#' @export
int_mode <- function(x, var_name, method = "CM", breaks = 30, ...) {
  .check_symbolic_tbl(x, "int_mode")
  .check_var_name(var_name, x, "int_mode")
  .check_interval_method(method, "int_mode")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  mode_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Function to estimate mode using histogram
  estimate_mode <- function(x) {
    if (length(unique(x)) == 1) return(x[1])
    h <- hist(x, breaks = breaks, plot = FALSE)
    h$mids[which.max(h$counts)]
  }
  
  compute_mode <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- estimate_mode(X_tmp)
    } else {
      x <- apply(X_tmp, 2, estimate_mode)
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    mode_tmp[idx, ] <- compute_mode(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    mode_tmp[6, ] <- mode_tmp[7, ] <- mode_tmp[8, ] <- compute_mode(X_tmp)
  }
  
  mode_output <- matrix(mode_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(mode_output) <- colnames(x)[var_name]
  } else {
    colnames(mode_output) <- var_name
  }
  rownames(mode_output) <- options[at]
  
  mode_output
}

#' @name interval_geometry
#' @title Geometric Properties of Interval Data
#' @description Functions to compute geometric characteristics of interval-valued data.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name the variable name or the column location (multiple variables are allowed).
#' @param var_name1 the first variable name or column location.
#' @param var_name2 the second variable name or column location.
#' @param ... additional parameters
#' @return A numeric matrix or value
#' @details
#' These functions compute basic geometric properties:
#' \itemize{
#'   \item \code{int_width}: Width of each interval (upper - lower)
#'   \item \code{int_radius}: Radius of each interval (width / 2)
#'   \item \code{int_center}: Center point of each interval ((lower + upper) / 2)
#'   \item \code{int_overlap}: Overlap measure between two interval variables
#'   \item \code{int_containment}: Check if one interval contains another
#'   \item \code{int_midrange}: Half-range of each interval ((upper - lower) / 2)
#' }
#' @author Han-Ming Wu
#' @seealso int_width int_radius int_center int_overlap
#' @examples
#' data(mushroom.int)
#' 
#' # Calculate interval widths
#' int_width(mushroom.int, var_name = "Pileus.Cap.Width")
#' int_width(mushroom.int, var_name = 2:3)
#' 
#' # Calculate interval radius
#' int_radius(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))
#' 
#' # Get interval centers
#' int_center(mushroom.int, var_name = 2:4)
#' 
#' # Measure overlap between two variables
#' int_overlap(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")
#'
#' # Check containment
#' int_containment(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")
#'
#' # Calculate midrange
#' int_midrange(mushroom.int, var_name = 2:3)
#' @export
int_width <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_width")
  .check_var_name(var_name, x, "int_width")
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  p <- ncol(idata)
  
  # Calculate width: upper - lower
  width_mat <- idata[, , 2] - idata[, , 1]
  
  # Ensure matrix format
  if (length(var_name) == 1) {
    width_mat <- matrix(width_mat, ncol = 1)
  }
  
  rownames(width_mat) <- rownames(x)
  if (is.numeric(var_name)) {
    colnames(width_mat) <- colnames(x)[var_name]
  } else {
    colnames(width_mat) <- var_name
  }
  
  return(width_mat)
}


#' @rdname interval_geometry
#' @export
int_radius <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_radius")
  .check_var_name(var_name, x, "int_radius")
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Calculate radius: (upper - lower) / 2
  radius_mat <- (idata[, , 2] - idata[, , 1]) / 2
  
  # Ensure matrix format
  if (length(var_name) == 1) {
    radius_mat <- matrix(radius_mat, ncol = 1)
  }
  
  rownames(radius_mat) <- rownames(x)
  if (is.numeric(var_name)) {
    colnames(radius_mat) <- colnames(x)[var_name]
  } else {
    colnames(radius_mat) <- var_name
  }
  
  return(radius_mat)
}


#' @rdname interval_geometry
#' @export
int_center <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_center")
  .check_var_name(var_name, x, "int_center")
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Use internal function to get centers
  center_mat <- Interval_to_Center(idata)
  
  # Ensure matrix format
  if (length(var_name) == 1) {
    center_mat <- matrix(center_mat, ncol = 1)
  }
  
  rownames(center_mat) <- rownames(x)
  if (is.numeric(var_name)) {
    colnames(center_mat) <- colnames(x)[var_name]
  } else {
    colnames(center_mat) <- var_name
  }
  
  return(center_mat)
}


#' @rdname interval_geometry
#' @export
int_overlap <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_overlap")
  .check_var_name(var_name1, x, "int_overlap")
  .check_var_name(var_name2, x, "int_overlap")

  # Resolve names
  if (is.numeric(var_name1)) {
    name1 <- colnames(x)[var_name1]
  } else {
    name1 <- var_name1
  }
  if (is.numeric(var_name2)) {
    name2 <- colnames(x)[var_name2]
  } else {
    name2 <- var_name2
  }

  all_vars <- unique(c(name1, name2))
  idata <- symbolic_tbl_to_idata(x[, all_vars])
  n <- nrow(idata)

  data1 <- idata[, name1, , drop = FALSE]
  data2 <- idata[, name2, , drop = FALSE]
  
  # Calculate overlap for each observation
  overlap_mat <- matrix(0, nrow = n, ncol = length(name1) * length(name2))
  col_idx <- 1
  col_names <- character(length(name1) * length(name2))
  
  for (i in seq_along(name1)) {
    for (j in seq_along(name2)) {
      for (k in 1:n) {
        # Overlap length
        overlap_start <- max(data1[k, i, 1], data2[k, j, 1])
        overlap_end <- min(data1[k, i, 2], data2[k, j, 2])
        overlap_length <- max(0, overlap_end - overlap_start)
        
        # Normalize by union length
        union_start <- min(data1[k, i, 1], data2[k, j, 1])
        union_end <- max(data1[k, i, 2], data2[k, j, 2])
        union_length <- union_end - union_start
        
        if (union_length > 0) {
          overlap_mat[k, col_idx] <- overlap_length / union_length
        } else {
          overlap_mat[k, col_idx] <- 0
        }
      }
      col_names[col_idx] <- paste0(name1[i], "_", name2[j])
      col_idx <- col_idx + 1
    }
  }
  
  rownames(overlap_mat) <- rownames(x)
  colnames(overlap_mat) <- col_names
  
  return(overlap_mat)
}


#' @rdname interval_geometry
#' @export
int_containment <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_containment")
  .check_var_name(var_name1, x, "int_containment")
  .check_var_name(var_name2, x, "int_containment")

  # Resolve names
  if (is.numeric(var_name1)) {
    name1 <- colnames(x)[var_name1]
  } else {
    name1 <- var_name1
  }
  if (is.numeric(var_name2)) {
    name2 <- colnames(x)[var_name2]
  } else {
    name2 <- var_name2
  }

  all_vars <- unique(c(name1, name2))
  idata <- symbolic_tbl_to_idata(x[, all_vars])
  n <- nrow(idata)

  data1 <- idata[, name1, , drop = FALSE]
  data2 <- idata[, name2, , drop = FALSE]
  
  # Check if var1 is contained in var2
  containment_mat <- matrix(FALSE, nrow = n, ncol = length(name1) * length(name2))
  col_idx <- 1
  col_names <- character(length(name1) * length(name2))
  
  for (i in seq_along(name1)) {
    for (j in seq_along(name2)) {
      for (k in 1:n) {
        # var1 contained in var2 if: var2_lower <= var1_lower AND var1_upper <= var2_upper
        containment_mat[k, col_idx] <- (data2[k, j, 1] <= data1[k, i, 1]) && 
                                        (data1[k, i, 2] <= data2[k, j, 2])
      }
      col_names[col_idx] <- paste0(name1[i], "_in_", name2[j])
      col_idx <- col_idx + 1
    }
  }
  
  rownames(containment_mat) <- rownames(x)
  colnames(containment_mat) <- col_names
  
  return(containment_mat)
}


#' @rdname interval_geometry
#' @export
int_midrange <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_midrange")
  .check_var_name(var_name, x, "int_midrange")
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Use internal function
  midrange_mat <- Interval_to_Midrange(idata)
  
  # Ensure matrix format
  if (length(var_name) == 1) {
    midrange_mat <- matrix(midrange_mat, ncol = 1)
  }
  
  rownames(midrange_mat) <- rownames(x)
  if (is.numeric(var_name)) {
    colnames(midrange_mat) <- colnames(x)[var_name]
  } else {
    colnames(midrange_mat) <- var_name
  }
  
  return(midrange_mat)
}

#' @name interval_uncertainty
#' @title Uncertainty and Variability Measures for Interval Data
#' @description Functions to compute uncertainty and variability measures for interval-valued data.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name the variable name or the column location (multiple variables are allowed).
#' @param method methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.
#' @param base logarithm base for entropy calculation (default: 2)
#' @param ... additional parameters
#' @return A numeric matrix or value
#' @details
#' These functions measure uncertainty and variability:
#' \itemize{
#'   \item \code{int_entropy}: Shannon entropy (information content)
#'   \item \code{int_cv}: Coefficient of variation (CV = SD / Mean)
#'   \item \code{int_dispersion}: General dispersion index
#'   \item \code{int_imprecision}: Imprecision based on interval width
#'   \item \code{int_granularity}: Variability in interval sizes
#'   \item \code{int_uniformity}: Uniformity of interval widths (inverse of granularity)
#'   \item \code{int_information_content}: Normalized entropy (entropy / log2(n))
#' }
#' @author Han-Ming Wu
#' @seealso int_var int_entropy int_cv
#' @examples
#' data(mushroom.int)
#' 
#' # Calculate entropy
#' int_entropy(mushroom.int, var_name = "Pileus.Cap.Width")
#' 
#' # Coefficient of variation
#' int_cv(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"), method = c("CM", "EJD"))
#' 
#' # Measure imprecision
#' int_imprecision(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))
#' 
#' # Dispersion index
#' int_dispersion(mushroom.int, var_name = "Pileus.Cap.Width", method = "CM")
#'
#' # Check data granularity
#' int_granularity(mushroom.int, var_name = 2:4)
#'
#' # Check uniformity
#' int_uniformity(mushroom.int, var_name = 2:3)
#'
#' # Information content
#' int_information_content(mushroom.int, var_name = "Stipe.Length", method = "CM")
#' @importFrom graphics hist
#' @importFrom stats sd median
#' @export
int_entropy <- function(x, var_name, method = "CM", base = 2, ...) {
  .check_symbolic_tbl(x, "int_entropy")
  .check_var_name(var_name, x, "int_entropy")
  .check_interval_method(method, "int_entropy")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  entropy_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Compute Shannon entropy using histogram bins
  compute_entropy <- function(X_tmp, breaks = 30) {
    if (length(var_name) == 1) {
      h <- hist(X_tmp, breaks = breaks, plot = FALSE)
      p <- h$counts / sum(h$counts)
      p <- p[p > 0]  # Remove zero probabilities
      H <- -sum(p * log(p, base = base))
    } else {
      H <- numeric(ncol(X_tmp))
      for (j in 1:ncol(X_tmp)) {
        h <- hist(X_tmp[, j], breaks = breaks, plot = FALSE)
        p <- h$counts / sum(h$counts)
        p <- p[p > 0]
        H[j] <- -sum(p * log(p, base = base))
      }
    }
    H
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    entropy_tmp[idx, ] <- compute_entropy(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    entropy_tmp[6, ] <- entropy_tmp[7, ] <- entropy_tmp[8, ] <- compute_entropy(X_tmp)
  }
  
  entropy_output <- matrix(entropy_tmp[at, ], 
                           nrow = length(method), 
                           ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(entropy_output) <- colnames(x)[var_name]
  } else {
    colnames(entropy_output) <- var_name
  }
  rownames(entropy_output) <- options[at]
  
  entropy_output
}


#' @rdname interval_uncertainty
#' @export
int_cv <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_cv")
  .check_var_name(var_name, x, "int_cv")
  .check_interval_method(method, "int_cv")
  
  # Calculate CV = SD / Mean
  mean_val <- int_mean(x, var_name, method)
  var_val <- int_var(x, var_name, method)
  sd_val <- sqrt(var_val)
  
  # Handle division by zero
  cv_output <- sd_val / mean_val
  cv_output[is.infinite(cv_output)] <- NA
  cv_output[is.nan(cv_output)] <- NA
  
  cv_output
}


#' @rdname interval_uncertainty
#' @export
int_dispersion <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_dispersion")
  .check_var_name(var_name, x, "int_dispersion")
  .check_interval_method(method, "int_dispersion")
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  disp_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  
  # Dispersion index: Average absolute deviation from median
  compute_dispersion <- function(X_tmp) {
    if (length(var_name) == 1) {
      med <- median(X_tmp)
      disp <- mean(abs(X_tmp - med))
    } else {
      disp <- numeric(ncol(X_tmp))
      for (j in 1:ncol(X_tmp)) {
        med <- median(X_tmp[, j])
        disp[j] <- mean(abs(X_tmp[, j] - med))
      }
    }
    disp
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    disp_tmp[idx, ] <- compute_dispersion(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    disp_tmp[6, ] <- disp_tmp[7, ] <- disp_tmp[8, ] <- compute_dispersion(X_tmp)
  }
  
  disp_output <- matrix(disp_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(disp_output) <- colnames(x)[var_name]
  } else {
    colnames(disp_output) <- var_name
  }
  rownames(disp_output) <- options[at]
  
  disp_output
}


#' @rdname interval_uncertainty
#' @export
int_imprecision <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_imprecision")
  .check_var_name(var_name, x, "int_imprecision")
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Imprecision = Average interval width / Average interval center
  # Normalized measure of interval size
  widths <- idata[, , 2] - idata[, , 1]
  centers <- (idata[, , 1] + idata[, , 2]) / 2
  
  if (length(var_name) == 1) {
    widths <- as.vector(widths)
    centers <- as.vector(centers)
    avg_width <- mean(widths)
    avg_center <- mean(abs(centers))
    if (avg_center == 0) {
      imprecision <- avg_width
    } else {
      imprecision <- avg_width / avg_center
    }
    imprecision <- matrix(imprecision, nrow = 1, ncol = 1)
  } else {
    imprecision <- numeric(length(var_name))
    for (j in seq_along(var_name)) {
      avg_width <- mean(widths[, j])
      avg_center <- mean(abs(centers[, j]))
      if (avg_center == 0) {
        imprecision[j] <- avg_width
      } else {
        imprecision[j] <- avg_width / avg_center
      }
    }
    imprecision <- matrix(imprecision, nrow = 1)
  }
  
  if (is.numeric(var_name)) {
    colnames(imprecision) <- colnames(x)[var_name]
  } else {
    colnames(imprecision) <- var_name
  }
  rownames(imprecision) <- "Imprecision"
  
  imprecision
}


#' @rdname interval_uncertainty
#' @export
int_granularity <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_granularity")
  .check_var_name(var_name, x, "int_granularity")
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Granularity = CV of interval widths
  # Measures how variable the interval sizes are
  widths <- idata[, , 2] - idata[, , 1]
  
  if (length(var_name) == 1) {
    widths <- as.vector(widths)
    mean_width <- mean(widths)
    sd_width <- sd(widths)
    if (mean_width == 0) {
      granularity <- 0
    } else {
      granularity <- sd_width / mean_width
    }
    granularity <- matrix(granularity, nrow = 1, ncol = 1)
  } else {
    granularity <- numeric(length(var_name))
    for (j in seq_along(var_name)) {
      mean_width <- mean(widths[, j])
      sd_width <- sd(widths[, j])
      if (mean_width == 0) {
        granularity[j] <- 0
      } else {
        granularity[j] <- sd_width / mean_width
      }
    }
    granularity <- matrix(granularity, nrow = 1)
  }
  
  if (is.numeric(var_name)) {
    colnames(granularity) <- colnames(x)[var_name]
  } else {
    colnames(granularity) <- var_name
  }
  rownames(granularity) <- "Granularity"
  
  granularity
}


#' @rdname interval_uncertainty
#' @export
int_uniformity <- function(x, var_name, ...) {
  .check_symbolic_tbl(x, "int_uniformity")
  .check_var_name(var_name, x, "int_uniformity")
  
  # Uniformity = 1 - Granularity
  # Returns 1 for uniform interval widths, lower for varying widths
  gran <- int_granularity(x, var_name, ...)
  uniformity <- 1 / (1 + gran)  # Bounded between 0 and 1
  
  rownames(uniformity) <- "Uniformity"
  uniformity
}


#' @rdname interval_uncertainty
#' @export
int_information_content <- function(x, var_name, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_information_content")
  .check_var_name(var_name, x, "int_information_content")
  
  # Information content = Entropy / log2(n)
  # Normalized entropy (between 0 and 1)
  n <- nrow(x)
  entropy <- int_entropy(x, var_name, method, base = 2, ...)
  
  max_entropy <- log2(n)
  if (max_entropy == 0) {
    info_content <- entropy * 0
  } else {
    info_content <- entropy / max_entropy
  }
  
  info_content
}

## ===========================================================================
## read_symbolic_csv() / write_symbolic_csv() - Symbolic CSV I/O
## ===========================================================================

#' Read a Symbolic Data CSV File
#'
#' @name read_symbolic_csv
#' @aliases read_symbolic_csv
#' @description
#' Reads an external CSV file containing symbolic data, automatically detects
#' whether the data is interval-valued (min/max pairs or comma-separated),
#' histogram-valued, modal-valued, or another symbolic type, and returns an
#' appropriate R object.
#'
#' @param file Path to the CSV file to read.
#' @param sep Field separator character. Default \code{","}.
#' @param header Logical; does the first row contain column names?
#'   Default \code{TRUE}.
#' @param row.names Column number or character string giving row names.
#'   Passed to \code{\link[utils]{read.table}}.  Default \code{NULL} (automatic).
#' @param stringsAsFactors Logical; should character columns be converted to
#'   factors? Default \code{FALSE}.
#' @param na.strings Character vector of strings to interpret as \code{NA}.
#'   Default \code{c("", "NA")}.
#' @param symbolic_type Optional character string to override automatic type
#'   detection.  One of \code{"interval"}, \code{"histogram"}, \code{"modal"},
#'   or \code{"other"}.  When \code{NULL} (the default) the type is detected
#'   automatically.
#' @param \dots Additional arguments passed to \code{\link[utils]{read.table}}.
#'
#' @details
#' The detection heuristic works as follows:
#' \enumerate{
#'   \item \strong{Interval (MM)}: If the file contains paired
#'     \code{_min}/\code{_max} columns the data is returned as-is (MM format).
#'   \item \strong{Interval (iGAP)}: If one or more character columns contain
#'     comma-separated numeric pairs (e.g., \code{"1.2,3.4"}) they are
#'     expanded into \code{_min}/\code{_max} column pairs and the result is
#'     returned in MM format.
#'   \item \strong{Histogram / Modal}: If columns follow a \code{VarName(bin)}
#'     naming pattern (e.g., \code{Crime(violent)}) and the proportions within
#'     each variable group sum to approximately 1, the data is classified as
#'     histogram or modal.  It is returned as a plain \code{data.frame}.
#'   \item \strong{Other}: If none of the above patterns match, the data is
#'     returned as a plain \code{data.frame}.
#' }
#'
#' @returns A \code{data.frame}.  Interval data is returned in MM format
#'   (paired \code{_min}/\code{_max} columns).  All other symbolic types are
#'   returned as plain data frames.
#'
#' @seealso \code{\link{write_symbolic_csv}}, \code{\link{int_detect_format}},
#'   \code{\link{int_convert_format}}
#'
#' @importFrom utils read.table
#' @examples
#' # Write then read back an interval dataset
#' data(mushroom.int.mm)
#' tmp <- tempfile(fileext = ".csv")
#' write_symbolic_csv(mushroom.int.mm, tmp)
#' df <- read_symbolic_csv(tmp)
#' head(df)
#'
#' # Write then read back a histogram dataset
#' data(airline_flights.hist)
#' tmp2 <- tempfile(fileext = ".csv")
#' write_symbolic_csv(airline_flights.hist, tmp2)
#' df2 <- read_symbolic_csv(tmp2)
#' head(df2)
#' @export

read_symbolic_csv <- function(file,
                              sep = ",",
                              header = TRUE,
                              row.names = NULL,
                              stringsAsFactors = FALSE,
                              na.strings = c("", "NA"),
                              symbolic_type = NULL,
                              ...) {
  .check_file_path(file, "read_symbolic_csv")
  .check_file_exists(file, "read_symbolic_csv")
  if (!is.null(symbolic_type)) {
    valid_types <- c("interval", "histogram", "modal", "other")
    symbolic_type <- tolower(symbolic_type)
    if (!symbolic_type %in% valid_types) {
      stop("read_symbolic_csv: 'symbolic_type' must be one of: ",
           paste(valid_types, collapse = ", "), ".", call. = FALSE)
    }
  }

  # First pass: read with check.names = FALSE to preserve column names
  # like "Var(bin)".  Auto-detect row names when the header line has one
  # fewer field than the data lines (standard write.table / write.csv
  # behaviour) -- unless the caller specified row.names explicitly.
  df <- utils::read.table(file, sep = sep, header = header,
                          row.names = row.names,
                          stringsAsFactors = stringsAsFactors,
                          check.names = FALSE,
                          na.strings = na.strings,
                          quote = "\"",
                          comment.char = "",
                          ...)

  # When col.names = NA was used during writing, the first column header is

  # blank and the first data column holds the original row names.  Promote
  # that column to proper row names so the round-trip is transparent.
  if (is.null(row.names) && ncol(df) > 1 && nchar(names(df)[1]) == 0) {
    rownames(df) <- df[[1]]
    df <- df[, -1, drop = FALSE]
  }

  detected <- if (!is.null(symbolic_type)) symbolic_type
              else .detect_symbolic_csv_type(df)

  if (detected == "interval_igap") {
    df <- .convert_igap_to_mm(df)
    detected <- "interval"
  }

  attr(df, "symbolic_type") <- detected
  df
}

# ---- internal: detect symbolic CSV type from a data.frame ----
.detect_symbolic_csv_type <- function(df) {
  col_names <- names(df)

  # Check MM format: paired _min/_max columns
  min_cols <- grep("_min$|_Min$", col_names, value = TRUE)
  max_cols <- grep("_max$|_Max$", col_names, value = TRUE)
  if (length(min_cols) > 0 && length(max_cols) > 0) {
    min_bases <- sub("_(min|Min)$", "", min_cols)
    max_bases <- sub("_(max|Max)$", "", max_cols)
    if (length(intersect(min_bases, max_bases)) > 0) {
      return("interval")
    }
  }

  # Check iGAP format: character columns with comma-separated numeric pairs
  if (nrow(df) > 0) {
    char_cols <- which(vapply(df, is.character, logical(1)))
    for (ci in char_cols) {
      vals <- trimws(df[[ci]])
      vals <- vals[!is.na(vals) & nchar(vals) > 0]
      if (length(vals) == 0) next
      sample_vals <- utils::head(vals, 20)
      if (all(grepl("^-?[0-9.]+\\s*,\\s*-?[0-9.]+$", sample_vals))) {
        return("interval_igap")
      }
    }
  }

  # Check histogram / modal: VarName(bin) column pattern
  paren_cols <- grep("\\(.*\\)$", col_names, value = TRUE)
  if (length(paren_cols) >= 2) {
    # Extract variable groups
    groups <- sub("\\(.*\\)$", "", paren_cols)
    if (length(unique(groups)) >= 1) {
      return("histogram")
    }
  }

  "other"
}

# ---- internal: expand iGAP columns to MM _min/_max pairs ----
.convert_igap_to_mm <- function(df) {
  if (nrow(df) == 0) return(df)
  result <- list()
  for (j in seq_along(df)) {
    col <- df[[j]]
    nm <- names(df)[j]
    if (is.character(col) &&
        any(grepl(",", col[!is.na(col)])) &&
        all(grepl("^\\s*-?[0-9.]+\\s*,\\s*-?[0-9.]+\\s*$",
                  trimws(col[!is.na(col) & nchar(trimws(col)) > 0])))) {
      parts <- strsplit(trimws(col), "\\s*,\\s*")
      mins <- vapply(parts, function(p) as.numeric(p[1]), numeric(1))
      maxs <- vapply(parts, function(p) as.numeric(p[2]), numeric(1))
      result[[paste0(nm, "_min")]] <- mins
      result[[paste0(nm, "_max")]] <- maxs
    } else {
      result[[nm]] <- col
    }
  }
  as.data.frame(result, stringsAsFactors = FALSE)
}


#' Write Symbolic Data to a CSV File
#'
#' @name write_symbolic_csv
#' @aliases write_symbolic_csv
#' @description
#' Writes a symbolic data object (interval, histogram, modal, or any
#' data frame) to a CSV file.  Interval data stored in RSDA format
#' (\code{symbolic_tbl} with complex columns) is automatically converted to
#' MM format (paired \code{_min}/\code{_max} columns) before writing.
#'
#' @param x A \code{data.frame}, \code{symbolic_tbl}, or other tabular object
#'   containing symbolic data.
#' @param file Path to the output CSV file.
#' @param sep Field separator character. Default \code{","}.
#' @param row.names Logical or character.  If \code{TRUE} (the default),
#'   row names are written as the first column.
#' @param na Character string to use for missing values.  Default \code{"NA"}.
#' @param quote Logical; should character and factor columns be quoted?
#'   Default \code{TRUE}.
#' @param \dots Additional arguments passed to \code{\link[utils]{write.table}}.
#'
#' @details
#' \code{write_symbolic_csv} handles every tabular symbolic type stored in
#' \pkg{dataSDA}:
#' \itemize{
#'   \item \strong{Interval (RSDA)}: \code{symbolic_tbl} objects with complex
#'     interval columns are converted to MM format before writing.
#'   \item \strong{Interval (MM)}: Data frames with \code{_min}/\code{_max}
#'     columns are written directly.
#'   \item \strong{Histogram / Modal / Other}: Plain data frames are written
#'     directly.
#' }
#' The output is a standard CSV that can be read back with
#' \code{\link{read_symbolic_csv}}.
#'
#' @returns Invisibly returns the data frame that was written (after any
#'   conversion).
#'
#' @seealso \code{\link{read_symbolic_csv}}
#'
#' @importFrom utils write.table
#' @examples
#' # Interval data (RSDA symbolic_tbl)
#' data(mushroom.int)
#' tmp <- tempfile(fileext = ".csv")
#' write_symbolic_csv(mushroom.int, tmp)
#' cat(readLines(tmp, n = 3), sep = "\n")
#'
#' # Histogram data
#' data(airline_flights.hist)
#' tmp2 <- tempfile(fileext = ".csv")
#' write_symbolic_csv(airline_flights.hist, tmp2)
#' cat(readLines(tmp2, n = 3), sep = "\n")
#' @export

write_symbolic_csv <- function(x, file,
                               sep = ",",
                               row.names = TRUE,
                               na = "NA",
                               quote = TRUE,
                               ...) {
  if (is.null(x)) {
    stop("write_symbolic_csv: 'x' must not be NULL.", call. = FALSE)
  }
  if (!is.data.frame(x)) {
    stop("write_symbolic_csv: 'x' must be a data.frame or symbolic_tbl, not ",
         class(x)[1], ".", call. = FALSE)
  }
  .check_file_path(file, "write_symbolic_csv")

  dir_path <- dirname(file)
  if (nchar(dir_path) > 0 && dir_path != "." && !dir.exists(dir_path)) {
    stop("write_symbolic_csv: directory does not exist: '", dir_path, "'.",
         call. = FALSE)
  }

  # Convert RSDA symbolic_tbl (complex interval columns) to MM format
  out <- .symbolic_to_writable(x)

  # Use col.names = NA when row.names are written so that the header line

  # gets a blank first field — the standard CSV convention that lets
  # read.table auto-detect row names on read-back.
  col_names <- if (isTRUE(row.names)) NA else TRUE
  utils::write.table(out, file = file, sep = sep, row.names = row.names,
                     col.names = col_names, na = na, quote = quote, ...)
  invisible(out)
}

# ---- internal: convert symbolic_tbl to a plain data.frame for writing ----
.symbolic_to_writable <- function(x) {
  if (!inherits(x, "symbolic_tbl")) return(as.data.frame(x))

  # Identify complex (interval) columns
  complex_idx <- which(vapply(x, mode, character(1)) == "complex")
  if (length(complex_idx) == 0) return(as.data.frame(x))

  # Build column list expanding complex columns to _min/_max pairs.
  # Use as.complex() to strip the symbolic_interval vctrs class so that
  # Re()/Im() return plain numeric vectors (data.frame() on symbolic_interval
  # auto-expands to .min/.max which is not wanted here).
  cols <- list()
  for (j in seq_along(x)) {
    nm <- names(x)[j]
    col <- x[[j]]
    if (j %in% complex_idx) {
      raw <- as.complex(col)
      cols[[paste0(nm, "_min")]] <- Re(raw)
      cols[[paste0(nm, "_max")]] <- Im(raw)
    } else {
      cols[[nm]] <- as.vector(col)
    }
  }
  out <- data.frame(cols, stringsAsFactors = FALSE, check.names = FALSE)
  if (!is.null(row.names(x))) row.names(out) <- row.names(x)
  out
}


## ===========================================================================
## search_data() - Search and filter the dataSDA dataset catalog
## ===========================================================================

#' Search Datasets
#'
#' @name search_data
#' @aliases search_data
#' @description Search and filter the dataSDA dataset catalog by metadata
#' criteria including sample size, number of variables, subject area,
#' symbolic format, analytical tasks, keywords, and book reference.
#'
#' @usage search_data(...)
#'
#' @param ... Filter expressions. Each argument is a comparison expression
#' evaluated against the dataset metadata. Supported columns:
#' \describe{
#'   \item{\code{n}}{Sample size (numeric). Operators: \code{==, >, <, >=, <=}.}
#'   \item{\code{p}}{Number of variables (numeric). Operators: \code{==, >, <, >=, <=}.}
#'   \item{\code{subject}}{Subject area (character). Case-insensitive partial match with \code{==}.
#'     Areas: Agriculture, Automotive, Biology, Biometrics, Botany, Chemistry, Climate,
#'     Criminology, Demographics, Digital media, Economics, Education, Energy,
#'     Engineering, Environment, Finance, Food science, Forestry, Genomics,
#'     Healthcare, Marine biology, Medical, Methodology, Public services,
#'     Socioeconomics, Sociology, Sports, Transportation, Zoology.}
#'   \item{\code{type}}{Symbolic format (character). Exact match with \code{==}.
#'     Types correspond to the dataset name suffix:
#'     \code{"int"} (interval), \code{"hist"} (histogram),
#'     \code{"mix"} (mixed), \code{"distr"} (distribution),
#'     \code{"its"} (interval time series), \code{"modal"} (modal),
#'     \code{"iGAP"} (interval in iGAP format).}
#'   \item{\code{task}}{Analytical tasks (character). Case-insensitive partial match with \code{==}.
#'     Tasks: Clustering, Classification, Regression, PCA, Descriptive statistics,
#'     Discriminant analysis, Visualization, Spatial analysis, Time series, Aggregation.}
#'   \item{\code{tag}}{Keywords (character). Case-insensitive partial match with \code{==}.
#'     Use \code{tag == "all"} to list all datasets.}
#'   \item{\code{book}}{Book reference short name (character). Case-insensitive partial match
#'     with \code{==}. Available books:
#'     \code{SDA_2006} (Billard & Diday, 2006),
#'     \code{CMD_2020} (Billard & Diday, 2020),
#'     \code{SODAS_2008} (Diday & Noirhomme-Fraiture, 2008).}
#' }
#'
#' @details
#' For character columns (\code{subject}, \code{type}, \code{task}, \code{tag},
#' \code{book}), the \code{==} operator performs a case-insensitive substring
#' match (using \code{grepl}). The \code{type} column uses short suffix-based
#' labels that match the dataset name suffix (e.g., \code{type == "int"}
#' matches all \code{.int} datasets).
#'
#' For numeric columns (\code{n}, \code{p}), standard comparison operators
#' are used with exact semantics.
#'
#' When no arguments are provided, or when \code{tag == "all"} is used,
#' all datasets are returned.
#'
#' @returns A data frame with one row per matching dataset and the following
#' columns: \code{name}, \code{n}, \code{p}, \code{subject}, \code{type},
#' \code{task}, \code{tag}, \code{book}.
#'
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester.
#'
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley.
#'
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley.
#'
#' @examples
#' # List all datasets
#' search_data()
#'
#' # Filter by symbolic format (suffix-based)
#' search_data(type == "hist")
#'
#' # Filter by analytical task and size
#' search_data(task == "Regression", n > 10)
#'
#' # Filter by book reference
#' search_data(book == "SDA_2006")
#'
#' # Combine multiple filters
#' search_data(type == "int", task == "Clustering", subject == "Biology")
#'
#' # Filter by size range
#' search_data(n >= 20, n <= 100, p < 10)
#'
#' @export

search_data <- function(...) {
  meta <- .dataset_registry()

  dots <- substitute(list(...))
  if (length(dots) <= 1L) return(meta)

  for (i in 2L:length(dots)) {
    expr <- dots[[i]]

    # Special case: tag == "all"
    if (is.call(expr) && length(expr) == 3L) {
      lhs_name <- as.character(expr[[2]])
      rhs_val <- eval(expr[[3]], parent.frame())
      if (lhs_name == "tag" && identical(tolower(as.character(rhs_val)), "all")) {
        next
      }
    }

    meta <- .apply_search_filter(meta, expr, parent.frame())
  }

  meta
}


# Internal: apply a single filter expression to the metadata data frame
.apply_search_filter <- function(data, expr, envir) {
  if (!is.call(expr)) {
    stop("search_data: each argument must be a comparison expression ",
         "(e.g., n > 10, type == \"interval\").", call. = FALSE)
  }

  op  <- as.character(expr[[1]])
  lhs <- as.character(expr[[2]])
  rhs <- eval(expr[[3]], envir)

  valid_cols <- names(data)
  if (!lhs %in% valid_cols) {
    stop("search_data: unknown column '", lhs, "'. Available columns: ",
         paste(valid_cols, collapse = ", "), ".", call. = FALSE)
  }

  col <- data[[lhs]]

  if (is.numeric(col)) {
    rhs <- as.numeric(rhs)
    result <- switch(op,
      "==" = col == rhs,
      "!=" = col != rhs,
      ">"  = col >  rhs,
      "<"  = col <  rhs,
      ">=" = col >= rhs,
      "<=" = col <= rhs,
      stop("search_data: unsupported operator '", op,
           "' for numeric column '", lhs, "'.", call. = FALSE)
    )
  } else {
    rhs <- as.character(rhs)
    if (op == "==") {
      result <- grepl(rhs, col, ignore.case = TRUE)
    } else if (op == "!=") {
      result <- !grepl(rhs, col, ignore.case = TRUE)
    } else {
      stop("search_data: only '==' and '!=' are supported for text column '",
           lhs, "'.", call. = FALSE)
    }
  }

  data[result & !is.na(result), ]
}


# Internal: build and cache the dataset metadata registry
.dataset_registry <- function() {
  if (!is.null(.registry_cache$meta)) return(.registry_cache$meta)

  meta <- data.frame(
    name = c(
      # --- Section 1: External R packages (1-7) ---
      "lackinfo.int",
      "ohtemp.int",
      "soccer_bivar.int",
      "cars.int",
      "china_temp.int",
      "loans_by_purpose.int",
      "nycflights.int",
      # --- Section 2-3: Billard & Diday SDA_2006 core (8-29) ---
      "mushroom.int.mm",
      "mushroom.int",
      "age_cholesterol_weight.int",
      "airline_flights.hist",
      "airline_flights2.modal",
      "baseball.int",
      "bird.mix",
      "blood_pressure.int",
      "car.int",
      "crime.modal",
      "crime2.modal",
      "finance.int",
      "fuel_consumption.modal",
      "health_insurance.mix",
      "health_insurance2.modal",
      "hierarchy",
      "hierarchy.int",
      "horses.int",
      "occupations.modal",
      "occupations2.modal",
      "profession.int",
      "veterinary.int",
      # --- Section 4: Special symbolic types (30-48) ---
      "abalone.iGAP",
      "abalone.int",
      "face.iGAP",
      "oils.int",
      "teams.int",
      "tennis.int",
      "bats.int",
      "credit_card.int",
      "energy_consumption.distr",
      "trivial_intervals.int",
      "bird_species.mix",
      "temperature_city.int",
      "bird_species_extended.mix",
      "employment.int",
      "town_services.mix",
      "world_cup.int",
      "mushroom_fuzzy.mix",
      "bank_rates",
      "lung_cancer.hist",
      # --- Section 5: SDA_2006 continued (49-51) ---
      "acid_rain.int",
      "weight_age.hist",
      "hospital.hist",
      # --- Section 6: Various sources (52-62) ---
      "freshwater_fish.int",
      "fungi.int",
      "iris.int",
      "water_flow.int",
      "wine.int",
      "car_models.int",
      "hdi_gender.int",
      "cardiological.int",
      "prostate.int",
      "uscrime.int",
      "hardwood.hist",
      # --- Section 7: Clustering/histogram datasets (63-88) ---
      "synthetic_clusters.int",
      "environment.mix",
      "cholesterol.hist",
      "hemoglobin.hist",
      "hematocrit.hist",
      "hematocrit_hemoglobin.hist",
      "energy_usage.distr",
      "genome_abundances.int",
      "china_temp_monthly.int",
      "ecoli_routes.int",
      "loans_by_risk.int",
      "polish_voivodships.int",
      "iris_species.hist",
      "flights_detail.hist",
      "cover_types.hist",
      "glucose.hist",
      "state_income.hist",
      "simulated.hist",
      "age_pyramids.hist",
      "ozone.hist",
      "french_agriculture.hist",
      "household_characteristics.distr",
      "county_income_gender.hist",
      "joggers.mix",
      "census.mix",
      "mtcars.mix",
      # --- Section 8: Additional datasets (89-105) ---
      "utsnow.int",
      "lynne1.int",
      "loans_by_risk_quantile.int",
      "judge1.int",
      "judge2.int",
      "judge3.int",
      "video1.int",
      "video2.int",
      "video3.int",
      "lisbon_air_quality.int",
      "polish_cars.mix",
      "blood.hist",
      "china_climate_month.hist",
      "china_climate_season.hist",
      "exchange_rate_returns.hist",
      "hierarchy.hist",
      "bird_color_taxonomy.hist",
      # --- Section 9: Interval time series (106-114) ---
      "sp500.its",
      "djia.its",
      "ibovespa.its",
      "crude_oil_wti.its",
      "merval.its",
      "petrobras.its",
      "euro_usd.its",
      "shanghai_stock.its",
      "irish_wind.its"
    ),

    n = c(
      # 1-7
      50L, 161L, 20L, 27L, 899L, 14L, 142L,
      # 8-29
      23L, 23L, 7L, 16L, 16L, 19L, 20L, 15L, 8L, 15L, 15L, 14L, 10L,
      51L, 6L, 20L, 20L, 8L, 9L, 9L, 15L, 10L,
      # 30-48
      24L, 24L, 27L, 8L, 5L, 4L, 21L, 6L, 5L, 5L, 3L, 6L, 3L,
      12L, 3L, 2L, 4L, 4L, 2L,
      # 49-51
      2L, 7L, 15L,
      # 52-62
      12L, 55L, 30L, 316L, 33L, 33L, 183L, 44L, 97L, 46L, 5L,
      # 63-88
      125L, 14L, 14L, 14L, 14L, 10L, 10L, 14L, 15L, 9L, 35L, 18L,
      3L, 16L, 7L, 4L, 6L, 5L, 229L, 84L, 22L, 12L, 12L, 10L, 10L, 5L,
      # 89-105
      415L, 10L, 35L, 6L, 6L, 6L, 10L, 10L, 10L, 1096L, 30L, 14L,
      60L, 60L, 108L, 10L, 20L,
      # 106-114
      504L, 504L, 3216L, 2261L, 248L, 503L, 520L, 970L, 216L
    ),

    p = c(
      # 1-7
      8L, 7L, 3L, 5L, 5L, 4L, 5L,
      # 8-29
      5L, 5L, 4L, 17L, 6L, 3L, 2L, 3L, 5L, 7L, 3L, 7L, 3L,
      30L, 6L, 6L, 6L, 7L, 11L, 4L, 4L, 3L,
      # 30-48
      7L, 14L, 6L, 9L, 7L, 7L, 9L, 11L, 3L, 6L, 5L, 13L, 6L,
      20L, 8L, 8L, 9L, 6L, 2L,
      # 49-51
      5L, 1L, 1L,
      # 52-62
      14L, 6L, 5L, 48L, 10L, 9L, 6L, 5L, 9L, 102L, 4L,
      # 63-88
      7L, 17L, 3L, 3L, 3L, 2L, 2L, 11L, 13L, 5L, 5L, 9L,
      5L, 5L, 4L, 1L, 4L, 2L, 3L, 4L, 4L, 3L, 4L, 2L, 6L, 11L,
      # 89-105
      5L, 4L, 4L, 4L, 4L, 4L, 5L, 5L, 5L, 8L, 12L, 3L,
      168L, 56L, 1L, 7L, 4L,
      # 106-114
      3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 11L
    ),

    subject = c(
      # 1-7
      "Education", "Climate", "Sports", "Automotive", "Climate",
      "Finance", "Transportation",
      # 8-29
      "Biology", "Biology", "Medical", "Transportation", "Transportation",
      "Sports", "Zoology", "Medical", "Automotive", "Criminology",
      "Criminology", "Finance", "Energy", "Medical", "Medical",
      "Methodology", "Methodology", "Zoology", "Sociology", "Sociology",
      "Sociology", "Zoology",
      # 30-48
      "Marine biology", "Marine biology", "Biometrics", "Chemistry",
      "Sports", "Sports", "Zoology", "Finance", "Energy", "Methodology",
      "Zoology", "Climate", "Zoology", "Economics", "Public services",
      "Sports", "Biology", "Finance", "Medical",
      # 49-51
      "Environment", "Medical", "Healthcare",
      # 52-62
      "Biology", "Biology", "Botany", "Engineering", "Food science",
      "Automotive", "Socioeconomics", "Medical", "Medical", "Criminology",
      "Forestry",
      # 63-88
      "Methodology", "Environment", "Medical", "Medical", "Medical",
      "Medical", "Energy", "Genomics", "Climate", "Biology", "Finance",
      "Socioeconomics", "Botany", "Transportation", "Forestry", "Medical",
      "Economics", "Methodology", "Demographics", "Environment",
      "Agriculture", "Socioeconomics", "Economics", "Sports",
      "Demographics", "Automotive",
      # 89-105
      "Climate", "Medical", "Finance", "Methodology", "Methodology",
      "Methodology", "Digital media", "Digital media", "Digital media",
      "Environment", "Automotive", "Medical", "Climate", "Climate",
      "Finance", "Methodology", "Zoology",
      # 106-114
      "Finance", "Finance", "Finance", "Energy", "Finance",
      "Finance", "Finance", "Finance", "Climate"
    ),

    type = c(
      # 1-7
      "int", "int", "int", "int", "int", "int", "int",
      # 8-29
      "int", "int", "int", "hist", "modal",
      "int", "mix", "int", "int", "modal", "modal",
      "int", "modal", "mix", "modal",
      "int", "int", "int", "modal", "modal",
      "int", "int",
      # 30-48
      "iGAP", "int", "iGAP", "int",
      "int", "int", "int", "int", "distr",
      "int", "mix", "int",
      "mix", "int",
      "mix", "int", "mix",
      "modal", "hist",
      # 49-51
      "int", "hist", "hist",
      # 52-62
      "int", "int", "int", "int", "int",
      "int", "int", "int", "int", "int",
      "hist",
      # 63-88
      "int", "mix", "hist", "hist",
      "hist", "hist", "distr", "int", "int",
      "int", "int", "int", "hist", "hist",
      "hist", "hist", "hist", "hist", "hist",
      "hist", "hist", "distr", "hist",
      "mix", "mix",
      "mix",
      # 89-105
      "int", "int", "int", "int", "int",
      "int", "int", "int", "int", "int",
      "mix", "hist", "hist",
      "hist", "hist",
      "mix",
      "mix",
      # 106-114
      "its", "its", "its", "its", "its",
      "its", "its", "its", "its"
    ),

    task = c(
      # 1-7
      "Descriptive statistics, Regression",
      "Regression, Spatial analysis",
      "Regression",
      "Classification",
      "Clustering",
      "Descriptive statistics, Clustering",
      "Regression, Descriptive statistics",
      # 8-29
      "Clustering, Descriptive statistics",
      "Clustering, Descriptive statistics",
      "Descriptive statistics, Regression",
      "Clustering, Descriptive statistics",
      "Clustering, Descriptive statistics",
      "Descriptive statistics, Clustering",
      "Descriptive statistics",
      "Descriptive statistics, Regression",
      "Descriptive statistics, Clustering",
      "Clustering, Descriptive statistics",
      "Clustering, Descriptive statistics",
      "PCA",
      "Regression",
      "Descriptive statistics, Aggregation",
      "Clustering, Descriptive statistics",
      "Aggregation, Descriptive statistics",
      "Descriptive statistics, Regression",
      "Clustering",
      "Descriptive statistics, Clustering",
      "Descriptive statistics, Clustering",
      "Descriptive statistics, Classification",
      "Descriptive statistics, Clustering",
      # 30-48
      "Clustering, Visualization",
      "Clustering, Visualization",
      "Classification, Visualization",
      "Clustering",
      "PCA",
      "Clustering",
      "Clustering, Visualization",
      "Descriptive statistics",
      "Descriptive statistics",
      "PCA",
      "Descriptive statistics",
      "Clustering",
      "Descriptive statistics",
      "Discriminant analysis, Classification",
      "Descriptive statistics",
      "Descriptive statistics",
      "Descriptive statistics",
      "Descriptive statistics",
      "Descriptive statistics",
      # 49-51
      "Descriptive statistics",
      "Descriptive statistics",
      "Descriptive statistics, Clustering",
      # 52-62
      "Clustering",
      "Clustering",
      "Clustering",
      "Clustering",
      "Clustering",
      "Clustering, Classification",
      "Classification",
      "Descriptive statistics, Clustering",
      "Regression",
      "Regression, Clustering",
      "Clustering, Descriptive statistics",
      # 63-88
      "Clustering",
      "Descriptive statistics, Clustering",
      "Descriptive statistics",
      "Descriptive statistics",
      "Descriptive statistics",
      "Regression",
      "Descriptive statistics",
      "Clustering, Descriptive statistics",
      "Clustering",
      "Clustering",
      "Classification, Clustering",
      "Clustering",
      "Clustering, Descriptive statistics",
      "Clustering",
      "Clustering, Classification",
      "Descriptive statistics",
      "Clustering",
      "Clustering",
      "Clustering, Descriptive statistics",
      "Regression, Clustering",
      "Regression, Clustering",
      "Clustering, Descriptive statistics",
      "Clustering, Descriptive statistics",
      "Clustering",
      "Clustering",
      "Descriptive statistics, Clustering",
      # 89-105
      "Regression, Spatial analysis",
      "Descriptive statistics, Regression",
      "Classification, Clustering",
      "PCA",
      "PCA",
      "PCA",
      "PCA",
      "PCA",
      "PCA",
      "Regression, Time series",
      "Clustering, Descriptive statistics",
      "Descriptive statistics, Clustering",
      "Clustering",
      "Clustering",
      "Time series, Descriptive statistics",
      "Descriptive statistics",
      "Clustering, Descriptive statistics",
      # 106-114
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression",
      "Time series, Regression"
    ),

    tag = c(
      # 1-7
      "datasets interval",
      "datasets interval",
      "datasets interval regression",
      "datasets interval classification",
      "datasets interval clustering",
      "datasets interval",
      "datasets interval",
      # 8-29
      "datasets interval SDA_2006",
      "datasets interval SDA_2006",
      "datasets interval SDA_2006",
      "datasets histogram SDA_2006",
      "datasets modal SDA_2006",
      "datasets interval SDA_2006",
      "datasets interval SDA_2006",
      "datasets interval SDA_2006",
      "datasets interval SDA_2006",
      "datasets modal SDA_2006",
      "datasets modal SDA_2006",
      "datasets interval PCA SDA_2006",
      "datasets modal regression SDA_2006",
      "datasets mixed symbolic SDA_2006",
      "datasets modal SDA_2006",
      "datasets hierarchical SDA_2006",
      "datasets interval SDA_2006",
      "datasets interval clustering SDA_2006",
      "datasets modal SDA_2006",
      "datasets modal SDA_2006",
      "datasets interval SDA_2006",
      "datasets interval SDA_2006",
      # 30-48
      "datasets interval iGAP",
      "datasets interval",
      "datasets interval iGAP",
      "datasets interval clustering",
      "datasets interval PCA SDA_2006",
      "datasets interval clustering SDA_2006",
      "datasets interval clustering visualization",
      "datasets interval SDA_2006",
      "datasets distribution SDA_2006",
      "datasets interval PCA SDA_2006",
      "datasets mixed interval categorical SODAS_2008",
      "datasets interval clustering distance",
      "datasets mixed interval histogram categorical SDA_2006",
      "datasets interval discriminant SODAS_2008",
      "datasets mixed interval modal multi-valued SODAS_2008",
      "datasets interval SODAS_2008",
      "datasets fuzzy symbolic SODAS_2008",
      "datasets symbolic model SDA_2006",
      "datasets histogram SDA_2006",
      # 49-51
      "datasets interval SDA_2006",
      "datasets histogram SDA_2006",
      "datasets histogram SDA_2006",
      # 52-62
      "datasets interval clustering",
      "datasets interval clustering",
      "datasets interval clustering",
      "datasets interval clustering",
      "datasets interval clustering",
      "datasets interval clustering",
      "datasets interval ordinal",
      "datasets interval",
      "datasets interval medical",
      "datasets interval crime",
      "datasets histogram",
      # 63-88
      "datasets interval clustering synthetic",
      "datasets mixed interval modal",
      "datasets histogram medical SDA_2006",
      "datasets histogram medical SDA_2006",
      "datasets histogram medical SDA_2006",
      "datasets histogram medical regression SDA_2006",
      "datasets distribution SDA_2006",
      "datasets interval genomics CMD_2020",
      "datasets interval temperature climate CMD_2020",
      "datasets interval biology CMD_2020",
      "datasets interval finance",
      "datasets interval socioeconomic",
      "datasets histogram iris CMD_2020",
      "datasets histogram flights CMD_2020",
      "datasets histogram forestry CMD_2020",
      "datasets histogram medical CMD_2020",
      "datasets histogram income CMD_2020",
      "datasets histogram simulated CMD_2020",
      "datasets histogram demographics",
      "datasets histogram weather environment",
      "datasets histogram agriculture economics",
      "datasets distribution household CMD_2020",
      "datasets histogram income gender CMD_2020",
      "datasets mixed interval histogram CMD_2020",
      "datasets mixed interval histogram distribution CMD_2020",
      "datasets mixed interval modal",
      # 89-105
      "datasets interval",
      "datasets interval SDA_2006",
      "datasets interval",
      "datasets interval",
      "datasets interval",
      "datasets interval",
      "datasets interval",
      "datasets interval",
      "datasets interval",
      "datasets interval",
      "datasets mixed interval multinomial",
      "datasets histogram",
      "datasets histogram",
      "datasets histogram",
      "datasets histogram",
      "datasets mixed histogram interval SDA_2006",
      "datasets mixed histogram distribution SDA_2006",
      # 106-114
      "datasets its time series finance",
      "datasets its time series finance",
      "datasets its time series finance",
      "datasets its time series energy",
      "datasets its time series finance",
      "datasets its time series finance",
      "datasets its time series finance",
      "datasets its time series finance",
      "datasets its time series climate"
    ),

    book = c(
      # 1-7: External packages
      NA, NA, NA, NA, NA, NA, NA,
      # 8-29: SDA_2006 core
      "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006",
      "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006",
      "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006",
      "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006",
      "SDA_2006", "SDA_2006",
      # 30-48: Special types
      NA, NA, NA, NA,
      "SDA_2006", "SDA_2006",   # teams, tennis
      NA,                        # bats
      "SDA_2006", "SDA_2006", "SDA_2006",  # credit_card, energy_consumption, trivial
      "SODAS_2008",              # bird_species.mix
      NA,                        # temperature_city
      "SDA_2006",                # bird_species_extended
      "SODAS_2008",              # employment
      "SODAS_2008",              # town_services
      "SODAS_2008",              # world_cup
      "SODAS_2008",              # mushroom_fuzzy.mix
      "SDA_2006", "SDA_2006",   # bank_rates, lung_cancer
      # 49-51
      "SDA_2006", "SDA_2006", "SDA_2006",  # acid_rain, weight_age, hospital
      # 52-62: Various sources
      NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA,
      # 63-88
      NA, NA,                    # synthetic_clusters, environment
      "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006", "SDA_2006",
      # cholesterol, hemoglobin, hematocrit, hematocrit_hemoglobin, energy_usage
      "CMD_2020", "CMD_2020", "CMD_2020",  # genome, china_temp_monthly, ecoli
      NA, NA,                    # loans_by_risk, polish_voivodships
      "CMD_2020", "CMD_2020", "CMD_2020", "CMD_2020",
      # iris_species, flights, cover_types, glucose
      "CMD_2020", "CMD_2020",   # state_income, simulated
      NA, NA, NA,               # age_pyramids, ozone, french_agriculture
      "CMD_2020", "CMD_2020",   # household, county_income_gender
      "CMD_2020", "CMD_2020",   # joggers, census
      NA,                        # mtcars
      # 89-105
      NA,                        # utsnow
      "SDA_2006",                # lynne1
      NA, NA, NA, NA, NA, NA, NA,  # loans_quant, judge1-3, video1-3
      NA, NA, NA,               # lisbon, polish_cars, blood
      NA, NA, NA,               # china_month, china_season, exchange_rate
      "SDA_2006", "SDA_2006",   # hierarchy.hist, bird_color_taxonomy
      # 106-114: ITS
      NA, NA, NA, NA, NA, NA, NA, NA, NA
    ),

    stringsAsFactors = FALSE
  )

  rownames(meta) <- NULL
  .registry_cache$meta <- meta
  meta
}

# Environment for caching the registry (avoids rebuilding each call)
.registry_cache <- new.env(parent = emptyenv())


## ===========================================================================
## aggregate_to_symbolic() - Aggregate tabular data to symbolic data
## ===========================================================================

#' Aggregate Tabular Data to Symbolic Data
#'
#' @name aggregate_to_symbolic
#' @aliases aggregate_to_symbolic
#' @description Aggregate tabular numerical data (n by p) into interval-valued
#' or histogram-valued symbolic data (K by p) based on a grouping mechanism.
#'
#' @usage aggregate_to_symbolic(x, type = "int", group_by = "kmeans",
#'   stratify_var = NULL, K = 5, interval = "range",
#'   quantile_probs = c(0.05, 0.95), bins = 10, nK = NULL,
#'   zero_width = c("keep", "remove", "regenerate", "adjust"), epsilon = 1e-07)
#'
#' @param x A data.frame with n rows and p columns. May contain non-numeric
#'   columns used for grouping or stratification; only numeric columns are
#'   aggregated.
#' @param type Output symbolic type: \code{"int"} for interval data or
#'   \code{"hist"} for histogram data.
#' @param group_by Grouping mechanism. One of:
#'   \describe{
#'     \item{\code{"kmeans"}}{Partition the data into \code{K} groups using
#'       k-means clustering.}
#'     \item{\code{"hclust"}}{Partition the data into \code{K} groups using
#'       hierarchical clustering.}
#'     \item{\code{"resampling"}}{Generate \code{K} concepts by randomly
#'       sampling \code{nK} observations with replacement, repeated \code{K}
#'       times.}
#'     \item{A column name or column index}{Use the specified categorical
#'       variable to define groups.}
#'   }
#' @param stratify_var Optional column name or index for a stratification
#'   variable. When provided, grouping and aggregation are performed
#'   independently within each level. Default is \code{NULL}.
#' @param K Number of groups for clustering (\code{group_by = "kmeans"} or
#'   \code{"hclust"}) or resampling (\code{group_by = "resampling"}).
#'   Ignored when \code{group_by} is a variable. Default is 5.
#' @param interval Interval construction method when \code{type = "int"}:
#'   \code{"range"} uses min/max; \code{"quantile"} uses quantiles given by
#'   \code{quantile_probs}. Default is \code{"range"}.
#' @param quantile_probs Numeric vector of length 2 giving the lower and upper
#'   quantile probabilities for \code{interval = "quantile"}.
#'   Default is \code{c(0.05, 0.95)}.
#' @param bins Number of histogram bins when \code{type = "hist"}.
#'   Default is 10.
#' @param nK Number of observations to sample per group when
#'   \code{group_by = "resampling"}. Default is \code{floor(n / K)}.
#' @param zero_width How to handle zero-width intervals (\code{min == max})
#'   produced when \code{type = "int"}. Such degenerate intervals break
#'   downstream tools that divide by interval width (e.g.
#'   \code{ggInterval::ggInterval_indexImage()}). One of:
#'   \describe{
#'     \item{\code{"keep"}}{(default) Leave the aggregated output unchanged;
#'       zero-width intervals are returned as-is and no action is taken. Use
#'       \code{\link{check_zero_width_intervals}} to screen the result.}
#'     \item{\code{"remove"}}{Drop every concept (row) that contains
#'       at least one zero-width interval.}
#'     \item{\code{"regenerate"}}{Re-run the aggregation (re-clustering or
#'       re-sampling) until no zero-width interval remains. Only effective for
#'       stochastic \code{group_by} (\code{"kmeans"}, \code{"resampling"});
#'       for deterministic grouping (a variable or \code{"hclust"}) the result
#'       cannot change, so an error is raised suggesting another option.}
#'     \item{\code{"adjust"}}{Add a small amount \code{epsilon} to the upper
#'       endpoint of each zero-width interval.}
#'   }
#'   Ignored when \code{type = "hist"}.
#' @param epsilon Positive amount added to the upper endpoint of each
#'   zero-width interval when \code{zero_width = "adjust"}. Default is
#'   \code{1e-07}.
#'
#' @returns
#' \itemize{
#'   \item For \code{type = "int"}: a \code{symbolic_tbl} (RSDA format) with
#'     a label column followed by \code{symbolic_interval} columns for each
#'     numeric variable (K rows, 1 + p columns).
#'   \item For \code{type = "hist"}: a \code{\link[HistDAWass]{MatH}} object
#'     (K rows by p columns of histogram-valued data).
#' }
#'
#' @details
#' The function aggregates classical tabular data into symbolic data by:
#' \enumerate{
#'   \item Partitioning observations into groups via \code{group_by}
#'     (clustering, resampling, or a categorical variable).
#'   \item Within each group, summarizing each numeric variable as an
#'     interval (min/max or quantiles) or a histogram.
#' }
#'
#' When \code{stratify_var} is provided, grouping and aggregation are performed
#' within each level of the stratification variable. Label values are prefixed
#' by the stratum name (e.g., \code{"setosa.cluster_1"}).
#'
#' For \code{type = "hist"}, bin boundaries are computed from the global data
#' range to ensure comparability across groups.
#'
#' Non-numeric columns (other than those used for grouping or stratification)
#' are silently excluded from aggregation.
#'
#' @examples
#' # Group by a categorical variable -> interval data
#' res1 <- aggregate_to_symbolic(iris, type = "int", group_by = "Species")
#' res1
#'
#' # K-means clustering -> interval data
#' res2 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
#'                                group_by = "kmeans", K = 3)
#'
#' # Quantile-based intervals
#' res3 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
#'                                group_by = "kmeans", K = 3,
#'                                interval = "quantile",
#'                                quantile_probs = c(0.1, 0.9))
#'
#' # Resampling -> interval data
#' set.seed(42)
#' res4 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
#'                                group_by = "resampling", K = 5, nK = 30)
#'
#' # Histogram aggregation
#' res5 <- aggregate_to_symbolic(iris, type = "hist",
#'                                group_by = "Species", bins = 5)
#'
#' # Hierarchical clustering -> interval data
#' res6 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
#'                                group_by = "hclust", K = 3)
#'
#' # Stratified aggregation
#' res7 <- aggregate_to_symbolic(iris, type = "int",
#'                                group_by = "kmeans", K = 2,
#'                                stratify_var = "Species")
#'
#' @importFrom stats kmeans hclust dist cutree complete.cases quantile
#' @importFrom graphics hist
#' @importFrom methods new
#' @export

aggregate_to_symbolic <- function(x, type = "int", group_by = "kmeans",
                                  stratify_var = NULL, K = 5,
                                  interval = "range",
                                  quantile_probs = c(0.05, 0.95),
                                  bins = 10, nK = NULL,
                                  zero_width = c("keep", "remove",
                                                 "regenerate", "adjust"),
                                  epsilon = 1e-07) {
  fn <- "aggregate_to_symbolic"

  # --- Input validation ---
  if (!is.data.frame(x))
    stop(fn, ": 'x' must be a data.frame.", call. = FALSE)
  if (nrow(x) == 0L)
    stop(fn, ": 'x' must have at least one row.", call. = FALSE)
  type <- match.arg(type, c("int", "hist"))
  interval <- match.arg(interval, c("range", "quantile"))
  zero_width <- match.arg(zero_width)

  if (!is.numeric(epsilon) || length(epsilon) != 1L || is.na(epsilon) ||
      epsilon <= 0)
    stop(fn, ": 'epsilon' must be a single positive number.", call. = FALSE)

  if (!is.numeric(K) || length(K) != 1L || K < 1)
    stop(fn, ": 'K' must be a positive integer.", call. = FALSE)
  K <- as.integer(K)

  if (!is.numeric(bins) || length(bins) != 1L || bins < 1)
    stop(fn, ": 'bins' must be a positive integer.", call. = FALSE)
  bins <- as.integer(bins)

  if (type == "int" && interval == "quantile") {
    if (!is.numeric(quantile_probs) || length(quantile_probs) != 2L ||
        any(quantile_probs < 0) || any(quantile_probs > 1) ||
        quantile_probs[1] >= quantile_probs[2])
      stop(fn, ": 'quantile_probs' must be a length-2 numeric vector ",
           "with 0 <= probs[1] < probs[2] <= 1.", call. = FALSE)
  }

  # --- Resolve stratify_var ---
  stratify_col <- NULL
  if (!is.null(stratify_var))
    stratify_col <- .resolve_column(x, stratify_var, fn, "stratify_var")

  # --- Resolve group_by ---
  group_col <- NULL
  if (is.character(group_by) && length(group_by) == 1L) {
    if (group_by %in% c("kmeans", "hclust", "resampling")) {
      group_mode <- group_by
    } else {
      group_col <- .resolve_column(x, group_by, fn, "group_by")
      group_mode <- "variable"
    }
  } else if (is.numeric(group_by) && length(group_by) == 1L) {
    group_col <- .resolve_column(x, group_by, fn, "group_by")
    group_mode <- "variable"
  } else {
    stop(fn, ": 'group_by' must be 'kmeans', 'hclust', 'resampling', ",
         "or a column name/index.", call. = FALSE)
  }

  if (!is.null(stratify_col) && !is.null(group_col) && stratify_col == group_col)
    stop(fn, ": 'stratify_var' and 'group_by' cannot refer to the same column.",
         call. = FALSE)

  # --- Identify numeric columns (exclude grouping/stratification cols) ---
  exclude_cols <- c(stratify_col, group_col)
  num_cols <- setdiff(names(x), exclude_cols)
  num_cols <- num_cols[vapply(x[num_cols], is.numeric, logical(1))]
  if (length(num_cols) == 0L)
    stop(fn, ": no numeric columns found for aggregation.", call. = FALSE)

  # --- Default nK for resampling ---
  if (group_mode == "resampling") {
    if (is.null(nK)) nK <- max(2L, floor(nrow(x) / K))
    if (!is.numeric(nK) || length(nK) != 1L || nK < 1)
      stop(fn, ": 'nK' must be a positive integer.", call. = FALSE)
    nK <- as.integer(nK)
  }

  # --- Label column name for type = "int" ---
  if (group_mode == "variable") {
    label_name <- group_col
  } else if (group_mode %in% c("kmeans", "hclust")) {
    label_name <- "cluster"
  } else {
    label_name <- "sample"
  }

  # --- Precompute global histogram breaks ---
  global_breaks <- NULL
  if (type == "hist") {
    global_breaks <- lapply(num_cols, function(col) {
      vals <- x[[col]][!is.na(x[[col]])]
      r <- range(vals)
      if (r[1] == r[2]) r <- r + c(-0.5, 0.5)
      seq(r[1], r[2], length.out = bins + 1L)
    })
    names(global_breaks) <- num_cols
  }

  # --- Dispatch ---
  if (is.null(stratify_col)) {
    .agg_dispatch(x, num_cols, type, group_mode, group_col,
                  K, interval, quantile_probs, global_breaks, nK,
                  label_name, zero_width, epsilon)
  } else {
    strata <- split(x, x[[stratify_col]])
    parts <- lapply(names(strata), function(s) {
      res <- .agg_dispatch(strata[[s]], num_cols, type, group_mode, group_col,
                           K, interval, quantile_probs, global_breaks, nK,
                           label_name, zero_width, epsilon)
      if (type == "int") {
        res[[label_name]] <- paste0(s, ".", res[[label_name]])
      } else {
        rownames(res@M) <- paste0(s, ".", rownames(res@M))
      }
      res
    })
    if (type == "int") {
      .agg_rbind_symbolic_tbl(parts)
    } else {
      .agg_combine_MatH(parts, num_cols)
    }
  }
}


# --- Internal helpers for aggregate_to_symbolic ----------------------------

# Resolve a column reference (name or integer index) to a column name
.resolve_column <- function(data, col_ref, fn_name, arg_name) {
  if (is.character(col_ref) && length(col_ref) == 1L) {
    if (!col_ref %in% names(data))
      stop(fn_name, ": '", arg_name, "' column '", col_ref,
           "' not found in data.", call. = FALSE)
    return(col_ref)
  }
  if (is.numeric(col_ref) && length(col_ref) == 1L) {
    idx <- as.integer(col_ref)
    if (idx < 1L || idx > ncol(data))
      stop(fn_name, ": '", arg_name, "' index must be between 1 and ",
           ncol(data), ".", call. = FALSE)
    return(names(data)[idx])
  }
  stop(fn_name, ": '", arg_name, "' must be a column name or index.",
       call. = FALSE)
}

# Dispatch to resampling or group-based aggregation
.agg_dispatch <- function(data, num_cols, type, group_mode, group_col,
                          K, interval, quantile_probs, global_breaks, nK,
                          label_name, zero_width = "keep", epsilon = 1e-07) {
  # --- Histogram path (zero-width handling does not apply) ---
  if (type == "hist") {
    if (group_mode == "resampling") {
      return(.agg_resampling_hist(data, num_cols, K, nK, global_breaks))
    }
    groups <- .agg_assign_groups(data, num_cols, group_mode, group_col, K)
    return(.agg_to_hist(data, num_cols, groups, global_breaks))
  }

  # --- Interval path: build min/max matrices, then handle zero-width ---
  build <- function() {
    .agg_int_matrices(data, num_cols, group_mode, group_col, K,
                      interval, quantile_probs, nK)
  }
  if (zero_width == "regenerate") {
    m <- .agg_regenerate(build, group_mode)
  } else {
    m <- .agg_apply_zero_width(build(), num_cols, zero_width, epsilon)
  }
  .agg_make_symbolic_tbl(m$lo, m$hi, num_cols, m$labels, label_name)
}

# Build interval min/max matrices (grouped or resampling), returning a list
# with components lo, hi (K-by-p matrices) and labels (length-K concept names).
.agg_int_matrices <- function(data, num_cols, group_mode, group_col, K,
                              interval, quantile_probs, nK) {
  if (group_mode == "resampling") {
    return(.agg_int_matrices_resampling(data, num_cols, K, nK,
                                        interval, quantile_probs))
  }
  groups <- .agg_assign_groups(data, num_cols, group_mode, group_col, K)
  .agg_int_matrices_grouped(data, num_cols, groups, interval, quantile_probs)
}

# Detect, report, and handle zero-width intervals (min == max) in the
# interval min/max matrices according to 'zero_width'. Returns the (possibly
# modified) list of matrices/labels. A zero-width interval breaks downstream
# tools that divide by interval width (e.g. ggInterval::ggInterval_indexImage).
.agg_apply_zero_width <- function(m, num_cols, zero_width, epsilon) {
  # "keep": leave the output untouched (zero-width intervals are returned as-is)
  if (zero_width == "keep") return(m)
  degenerate <- m$hi == m$lo
  if (!any(degenerate, na.rm = TRUE)) return(m)
  degenerate[is.na(degenerate)] <- FALSE

  affected_vars <- num_cols[apply(degenerate, 2L, any)]

  if (zero_width == "adjust") {
    m$hi[degenerate] <- m$hi[degenerate] + epsilon
    warning("aggregate_to_symbolic: zero-width intervals (min == max) in ",
            "variable(s): ", paste(affected_vars, collapse = ", "),
            "; upper endpoints adjusted by epsilon = ", format(epsilon, scientific = TRUE),
            ".", call. = FALSE)
    return(m)
  }

  # zero_width == "remove": drop concepts (rows) with any zero-width interval
  row_has_zw <- apply(degenerate, 1L, any)
  if (all(row_has_zw)) {
    stop("aggregate_to_symbolic: every concept contains a zero-width interval; ",
         "nothing remains after removal. Try zero_width = 'adjust' or ",
         "'regenerate', or adjust the grouping parameters.", call. = FALSE)
  }
  removed <- m$labels[row_has_zw]
  keep <- !row_has_zw
  m$lo <- m$lo[keep, , drop = FALSE]
  m$hi <- m$hi[keep, , drop = FALSE]
  m$labels <- m$labels[keep]
  warning("aggregate_to_symbolic: removed ", sum(row_has_zw), " concept(s) ",
          "with zero-width intervals (", paste(removed, collapse = ", "),
          ") arising from variable(s): ",
          paste(affected_vars, collapse = ", "), ".", call. = FALSE)
  m
}

# Re-run interval aggregation until no zero-width interval remains.
# Deterministic grouping ("variable"/"hclust") cannot change between runs,
# so it is attempted only once before failing with a helpful message.
.agg_regenerate <- function(build, group_mode, max_attempts = 100L) {
  deterministic <- group_mode %in% c("variable", "hclust")
  attempts <- if (deterministic) 1L else max_attempts
  for (i in seq_len(attempts)) {
    m <- build()
    if (!any(m$hi == m$lo, na.rm = TRUE)) return(m)
  }
  extra <- if (deterministic) {
    paste0(" ('", group_mode, "' grouping is deterministic, so regeneration ",
           "cannot change the result)")
  } else ""
  stop("aggregate_to_symbolic: could not produce interval data free of ",
       "zero-width intervals after ", attempts, " regeneration attempt(s)",
       extra, ". Try zero_width = 'adjust' or 'remove', or adjust 'K' / ",
       "'quantile_probs'.", call. = FALSE)
}

# Assign group labels via categorical variable or clustering
.agg_assign_groups <- function(data, num_cols, group_mode, group_col, K) {
  if (group_mode == "variable") {
    return(as.character(data[[group_col]]))
  }

  # Clustering (group_mode is "kmeans" or "hclust")
  num_data <- data[, num_cols, drop = FALSE]
  cc <- complete.cases(num_data)
  num_complete <- num_data[cc, , drop = FALSE]

  if (nrow(num_complete) < K)
    stop("aggregate_to_symbolic: fewer complete observations (",
         nrow(num_complete), ") than K (", K, ").", call. = FALSE)

  if (group_mode == "kmeans") {
    cl <- kmeans(num_complete, centers = K, nstart = 10)
    labels <- cl$cluster
  } else {
    d <- dist(num_complete)
    hc <- hclust(d)
    labels <- cutree(hc, k = K)
  }

  groups <- rep(NA_character_, nrow(data))
  groups[cc] <- paste0("cluster_", labels)
  groups
}

# Build interval min/max matrices for group-based aggregation.
# Returns list(lo, hi, labels); zero-width handling is applied by the caller.
.agg_int_matrices_grouped <- function(data, num_cols, groups, interval,
                                      quantile_probs) {
  valid <- !is.na(groups)
  data <- data[valid, , drop = FALSE]
  groups <- groups[valid]
  group_levels <- unique(groups)
  K <- length(group_levels)
  p <- length(num_cols)

  lo_mat <- matrix(NA_real_, nrow = K, ncol = p)
  hi_mat <- matrix(NA_real_, nrow = K, ncol = p)

  for (j in seq_len(p)) {
    by_group <- split(data[[num_cols[j]]], groups)
    if (interval == "range") {
      lo_mat[, j] <- vapply(by_group, min, numeric(1), na.rm = TRUE)[group_levels]
      hi_mat[, j] <- vapply(by_group, max, numeric(1), na.rm = TRUE)[group_levels]
    } else {
      lo_mat[, j] <- vapply(by_group, quantile, numeric(1),
                             probs = quantile_probs[1],
                             na.rm = TRUE)[group_levels]
      hi_mat[, j] <- vapply(by_group, quantile, numeric(1),
                             probs = quantile_probs[2],
                             na.rm = TRUE)[group_levels]
    }
  }

  list(lo = lo_mat, hi = hi_mat, labels = group_levels)
}

# Aggregate to histogram data (MatH from HistDAWass)
.agg_to_hist <- function(data, num_cols, groups, global_breaks) {
  valid <- !is.na(groups)
  data <- data[valid, , drop = FALSE]
  groups <- groups[valid]
  group_levels <- unique(groups)
  K <- length(group_levels)
  p <- length(num_cols)

  dist_list <- vector("list", K * p)
  idx <- 1L
  for (i in seq_len(K)) {
    mask <- groups == group_levels[i]
    for (j in seq_len(p)) {
      vals <- data[[num_cols[j]]][mask]
      vals <- vals[!is.na(vals)]
      dist_list[[idx]] <- .agg_vals_to_distH(vals, global_breaks[[num_cols[j]]])
      idx <- idx + 1L
    }
  }

  HistDAWass::MatH(x = dist_list, nrows = K, ncols = p,
                   rownames = as.character(group_levels),
                   varnames = num_cols)
}

# Build interval min/max matrices for resampling aggregation: draw K samples
# of nK rows each. Returns list(lo, hi, labels); zero-width handling is applied
# by the caller (so "regenerate" can redraw fresh samples).
.agg_int_matrices_resampling <- function(data, num_cols, K, nK,
                                         interval, quantile_probs) {
  n <- nrow(data)
  p <- length(num_cols)
  row_labels <- paste0("sample_", seq_len(K))

  lo_mat <- matrix(NA_real_, nrow = K, ncol = p)
  hi_mat <- matrix(NA_real_, nrow = K, ncol = p)
  for (k in seq_len(K)) {
    idx <- sample(n, nK, replace = TRUE)
    for (j in seq_len(p)) {
      vals <- data[[num_cols[j]]][idx]
      if (interval == "range") {
        lo_mat[k, j] <- min(vals, na.rm = TRUE)
        hi_mat[k, j] <- max(vals, na.rm = TRUE)
      } else {
        lo_mat[k, j] <- quantile(vals, probs = quantile_probs[1],
                                 na.rm = TRUE)
        hi_mat[k, j] <- quantile(vals, probs = quantile_probs[2],
                                 na.rm = TRUE)
      }
    }
  }
  list(lo = lo_mat, hi = hi_mat, labels = row_labels)
}

# Resampling-based histogram aggregation: draw K samples of nK rows each.
.agg_resampling_hist <- function(data, num_cols, K, nK, global_breaks) {
  n <- nrow(data)
  p <- length(num_cols)
  row_labels <- paste0("sample_", seq_len(K))

  dist_list <- vector("list", K * p)
  idx <- 1L
  for (k in seq_len(K)) {
    sample_idx <- sample(n, nK, replace = TRUE)
    for (j in seq_len(p)) {
      vals <- data[[num_cols[j]]][sample_idx]
      vals <- vals[!is.na(vals)]
      dist_list[[idx]] <- .agg_vals_to_distH(vals, global_breaks[[num_cols[j]]])
      idx <- idx + 1L
    }
  }

  HistDAWass::MatH(x = dist_list, nrows = K, ncols = p,
                   rownames = row_labels, varnames = num_cols)
}

# Convert numeric values to a distributionH using precomputed breaks
.agg_vals_to_distH <- function(vals, breaks) {
  if (length(vals) < 2L) {
    if (length(vals) == 1L)
      return(new("distributionH", x = c(vals, vals), p = c(0, 1)))
    return(new("distributionH",
               x = breaks[c(1, length(breaks))], p = c(0, 1)))
  }
  h <- hist(vals, breaks = breaks, plot = FALSE)
  total <- sum(h$counts)
  cum_probs <- c(0, cumsum(h$counts) / total)
  new("distributionH", x = h$breaks, p = cum_probs)
}

# Create a symbolic_interval vector (same representation as RSDA)
.new_symbolic_interval <- function(min, max) {
  structure(complex(real = min, imaginary = max),
            class = c("symbolic_interval", "vctrs_vctr"))
}

# Build a symbolic_tbl from min/max matrices
.agg_make_symbolic_tbl <- function(lo_mat, hi_mat, num_cols, group_labels,
                                   label_name) {
  K <- nrow(lo_mat)
  p <- ncol(lo_mat)

  col_list <- vector("list", 1L + p)
  col_names <- c(label_name, num_cols)
  names(col_list) <- col_names

  col_list[[1L]] <- group_labels
  for (j in seq_len(p)) {
    col_list[[j + 1L]] <- .new_symbolic_interval(lo_mat[, j], hi_mat[, j])
  }

  structure(col_list,
            class = c("symbolic_tbl", "tbl_df", "tbl", "data.frame"),
            row.names = seq_len(K),
            names = col_names,
            concept = as.character(seq_len(K)))
}

# Row-bind multiple symbolic_tbl objects
.agg_rbind_symbolic_tbl <- function(tbl_list) {
  col_names <- names(tbl_list[[1]])
  K_total <- sum(vapply(tbl_list, nrow, integer(1)))

  col_list <- vector("list", length(col_names))
  names(col_list) <- col_names

  for (cn in col_names) {
    pieces <- lapply(tbl_list, function(tbl) tbl[[cn]])
    if (inherits(pieces[[1]], "symbolic_interval")) {
      combined <- unlist(lapply(pieces, unclass))
      col_list[[cn]] <- .new_symbolic_interval(Re(combined), Im(combined))
    } else {
      col_list[[cn]] <- unlist(pieces)
    }
  }

  structure(col_list,
            class = c("symbolic_tbl", "tbl_df", "tbl", "data.frame"),
            row.names = seq_len(K_total),
            names = col_names,
            concept = as.character(seq_len(K_total)))
}

# Combine multiple MatH objects (from stratified aggregation)
.agg_combine_MatH <- function(math_list, num_cols) {
  p <- length(num_cols)
  all_rows <- unlist(lapply(math_list, function(m) rownames(m@M)))
  K_total <- length(all_rows)

  dist_list <- vector("list", K_total * p)
  idx <- 1L
  for (m in math_list) {
    for (i in seq_len(nrow(m@M))) {
      for (j in seq_len(p)) {
        dist_list[[idx]] <- m@M[i, j][[1]]
        idx <- idx + 1L
      }
    }
  }

  HistDAWass::MatH(x = dist_list, nrows = K_total, ncols = p,
                   rownames = all_rows, varnames = num_cols)
}

# ============================================================================
# Interval Format Conversions
# ============================================================================
# Conversions between RSDA, MM, iGAP, and SODAS interval data formats.
# Organized by target format: first to MM, then to iGAP.
# ============================================================================


# --- Format Detection --------------------------------------------------------

#' Detect Interval Data Format
#'
#' @name int_detect_format
#' @aliases int_detect_format
#' @description Automatically detect the format of interval data.
#' @usage int_detect_format(x)
#' @param x interval data in unknown format
#' @returns A character string indicating the detected format: "RSDA", "MM", "iGAP", "ARRAY", "SODAS", or "unknown"
#' @details
#' Detection rules:
#' \itemize{
#'   \item \code{RSDA}: has class "symbolic_tbl" and contains complex columns
#'   \item \code{MM}: data.frame with paired "_min" and "_max" columns
#'   \item \code{iGAP}: data.frame with columns containing comma-separated values (e.g., "1.2,3.4")
#'   \item \code{ARRAY}: a 3-dimensional array with \code{dim[3] = 2} (min/max slices)
#'   \item \code{SODAS}: character string ending with ".xml" (file path)
#'   \item \code{SDS}: alias for SODAS
#' }
#' @examples
#' data(mushroom.int)
#' int_detect_format(mushroom.int)  # Should return "RSDA"
#'
#' data(abalone.iGAP)
#' int_detect_format(abalone.iGAP)  # Should return "iGAP"
#'
#' # ARRAY format
#' x <- array(1:24, dim = c(4, 3, 2))
#' int_detect_format(x)  # Should return "ARRAY"
#' @export
int_detect_format <- function(x) {
  
  # Check for NULL
  if (is.null(x)) {
    return("unknown")
  }
  
  # Check for SODAS/SDS (XML file path)
  if (is.character(x) && length(x) == 1) {
    if (grepl("\\.xml$", tolower(x))) {
      return("SODAS")
    }
  }

  # Check for ARRAY format (3D array with dim[3] = 2)
  if (is.array(x) && length(dim(x)) == 3 && dim(x)[3] == 2) {
    return("ARRAY")
  }

  # Check for RSDA format (symbolic_tbl with complex columns)
  if (inherits(x, "symbolic_tbl")) {
    # Check if it has complex mode columns (interval columns)
    has_complex <- any(sapply(x, mode) == "complex")
    if (has_complex) {
      return("RSDA")
    }
  }
  
  # Check for data.frame formats (MM or iGAP)
  if (is.data.frame(x)) {
    col_names <- names(x)
    
    # Check for MM format (paired _min/_max columns)
    has_min <- any(grepl("_min|_Min", col_names))
    has_max <- any(grepl("_max|_Max", col_names))
    
    if (has_min && has_max) {
      # Further verify: count min/max pairs
      min_cols <- grep("_min|_Min", col_names, value = TRUE)
      max_cols <- grep("_max|_Max", col_names, value = TRUE)
      
      if (length(min_cols) == length(max_cols) && length(min_cols) > 0) {
        return("MM")
      }
    }
    
    # Check for iGAP format (comma-separated values in character columns)
    # Sample a few rows to check for comma-separated pattern
    if (ncol(x) > 0 && nrow(x) > 0) {
      # Check first few character columns
      char_cols <- which(sapply(x, is.character))
      
      if (length(char_cols) > 0) {
        # Check if values contain comma pattern like "1.2,3.4"
        sample_size <- min(10, nrow(x))
        sample_rows <- head(x, sample_size)
        
        has_comma_pattern <- FALSE
        for (col_idx in char_cols) {
          values <- as.character(sample_rows[[col_idx]])
          # Check if values match interval pattern: "number,number"
          pattern_match <- grepl("^[0-9.-]+,[0-9.-]+$", values)
          if (sum(pattern_match, na.rm = TRUE) > 0) {
            has_comma_pattern <- TRUE
            break
          }
        }
        
        if (has_comma_pattern) {
          return("iGAP")
        }
      }
    }
  }
  
  return("unknown")
}


#' List Available Format Conversions
#'
#' @name int_list_conversions
#' @aliases int_list_conversions
#' @description List all available format conversion functions.
#' @usage int_list_conversions(from = NULL, to = NULL)
#' @param from source format (optional): "RSDA", "MM", "iGAP", "ARRAY", "SODAS"
#' @param to target format (optional): "RSDA", "MM", "iGAP", "ARRAY", "SODAS"
#' @returns A data.frame showing available conversions
#' @examples
#' # List all conversions
#' int_list_conversions()
#' 
#' # List conversions from RSDA
#' int_list_conversions(from = "RSDA")
#' 
#' # List conversions to MM
#' int_list_conversions(to = "MM")
#' @export
int_list_conversions <- function(from = NULL, to = NULL) {
  
  # Define all available conversions
  conversions <- data.frame(
    from = c("RSDA", "RSDA", "RSDA",
             "MM", "MM", "MM",
             "iGAP", "iGAP", "iGAP",
             "ARRAY", "ARRAY", "ARRAY",
             "SODAS", "SODAS", "SODAS"),
    to = c("MM", "iGAP", "ARRAY",
           "iGAP", "RSDA", "ARRAY",
           "MM", "RSDA", "ARRAY",
           "RSDA", "MM", "iGAP",
           "MM", "iGAP", "ARRAY"),
    function_name = c("RSDA_to_MM", "RSDA_to_iGAP", "RSDA_to_ARRAY",
                      "MM_to_iGAP", "MM_to_RSDA", "MM_to_ARRAY",
                      "iGAP_to_MM", "iGAP_to_RSDA", "iGAP_to_ARRAY",
                      "ARRAY_to_RSDA", "ARRAY_to_MM", "ARRAY_to_iGAP",
                      "SODAS_to_MM", "SODAS_to_iGAP", "SODAS_to_ARRAY"),
    stringsAsFactors = FALSE
  )
  
  # Filter by 'from' if specified (case-insensitive)
  if (!is.null(from)) {
    from <- toupper(from)
    conversions <- conversions[toupper(conversions$from) == from, ]
  }

  # Filter by 'to' if specified (case-insensitive)
  if (!is.null(to)) {
    to <- toupper(to)
    conversions <- conversions[toupper(conversions$to) == to, ]
  }
  
  return(conversions)
}


#' Convert Interval Data Format
#'
#' @name int_convert_format
#' @aliases int_convert_format
#' @description Automatically detect the format of interval data and convert it to the target format.
#' @usage int_convert_format(x, to = "MM", from = NULL, ...)
#' @param x interval data in one of the supported formats
#' @param to target format: "MM", "iGAP", "RSDA", "ARRAY", "SODAS" (default: "MM")
#' @param from source format (optional): "MM", "iGAP", "RSDA", "ARRAY", "SODAS". If NULL, will auto-detect.
#' @param ... additional parameters passed to specific conversion functions
#' @returns Interval data in the target format
#' @details
#' This function provides a unified interface for all interval format conversions.
#' It automatically detects the source format (unless specified) and applies the
#' appropriate conversion function.
#'
#' Supported conversions:
#' \itemize{
#'   \item RSDA ??? MM, iGAP, ARRAY
#'   \item MM ??? iGAP, RSDA, ARRAY
#'   \item iGAP ??? MM, RSDA, ARRAY
#'   \item ARRAY ??? RSDA, MM, iGAP
#'   \item SODAS ??? MM, iGAP, ARRAY
#' }
#' @author Han-Ming Wu
#' @seealso int_detect_format int_list_conversions RSDA_to_MM RSDA_to_ARRAY
#'   MM_to_RSDA MM_to_ARRAY ARRAY_to_RSDA ARRAY_to_MM ARRAY_to_iGAP
#'   iGAP_to_MM iGAP_to_RSDA iGAP_to_ARRAY MM_to_iGAP
#' @importFrom utils capture.output head
#' @examples
#' # Auto-detect and convert to MM
#' data(mushroom.int)
#' data_mm <- int_convert_format(mushroom.int, to = "MM")
#'
#' # Explicitly specify source format
#' data(abalone.iGAP)
#' data_mm <- int_convert_format(abalone.iGAP, from = "iGAP", to = "MM")
#'
#' # Convert MM to iGAP
#' data_igap <- int_convert_format(data_mm, to = "iGAP")
#'
#'  # Convert multiple datasets to MM
#' datasets <- list(mushroom.int, abalone.int, car.int)
#' mm_datasets <- lapply(datasets, int_convert_format, to = "MM")
#'
#' # Check what conversions are available
#' int_list_conversions()
#' @export
int_convert_format <- function(x, to = "MM", from = NULL, ...) {
  
  # Normalize target format
  to <- toupper(to)
  valid_targets <- c("MM", "IGAP", "RSDA", "ARRAY", "SODAS", "SDS")
  
  if (!to %in% valid_targets) {
    stop("int_convert_format: 'to' must be one of: ", 
         paste(valid_targets, collapse = ", "), call. = FALSE)
  }
  
  # Normalize aliases
  if (to == "SDS") to <- "SODAS"
  
  # Detect source format if not specified
  if (is.null(from)) {
    detected <- int_detect_format(x)
    message("Detected source format: ", detected)
    from <- toupper(detected)
  } else {
    from <- toupper(from)
  }
  if (from == "SDS") from <- "SODAS"
  
  # Check if format was detected
  if (from == "UNKNOWN") {
    stop("int_convert_format: Could not detect source format. ",
         "Please specify 'from' parameter explicitly.", call. = FALSE)
  }
  
  # Check if conversion is needed
  if (from == to) {
    message("Source and target formats are the same. Returning original data.")
    return(x)
  }
  
  # Determine conversion path and execute
  conversion_key <- paste(from, to, sep = "_to_")
  
  result <- switch(conversion_key,
    # To MM format
    "RSDA_to_MM" = {
      RSDA_to_MM(x, ...)
    },
    "IGAP_to_MM" = {
      # Need to provide location parameter for iGAP_to_MM
      args <- list(...)
      if (is.null(args$location)) {
        # Auto-detect interval columns (character columns with comma pattern)
        if (is.data.frame(x)) {
          char_cols <- which(sapply(x, is.character))
          if (length(char_cols) > 0) {
            # Check which columns have comma-separated values
            interval_cols <- c()
            for (col_idx in char_cols) {
              values <- as.character(x[[col_idx]])
              if (any(grepl(",", values), na.rm = TRUE)) {
                interval_cols <- c(interval_cols, col_idx)
              }
            }
            if (length(interval_cols) > 0) {
              message("Auto-detected interval columns: ", paste(interval_cols, collapse = ", "))
              iGAP_to_MM(x, location = interval_cols)
            } else {
              stop("int_convert_format: No interval columns detected in iGAP data. ",
                   "Please specify 'location' parameter.", call. = FALSE)
            }
          } else {
            stop("int_convert_format: No character columns found in iGAP data.", 
                 call. = FALSE)
          }
        } else {
          stop("int_convert_format: iGAP data must be a data.frame.", call. = FALSE)
        }
      } else {
        iGAP_to_MM(x, location = args$location)
      }
    },
    "SODAS_to_MM" = {
      SODAS_to_MM(x)
    },
    
    # To iGAP format
    "RSDA_to_IGAP" = {
      RSDA_to_iGAP(x)
    },
    "MM_to_IGAP" = {
      MM_to_iGAP(x)
    },
    "SODAS_to_IGAP" = {
      SODAS_to_iGAP(x)
    },

    # To RSDA format
    "MM_to_RSDA" = {
      MM_to_RSDA(x)
    },
    "ARRAY_to_RSDA" = {
      ARRAY_to_RSDA(x)
    },
    "IGAP_to_RSDA" = {
      args <- list(...)
      if (is.null(args$location)) {
        # Auto-detect interval columns (character columns with comma pattern)
        if (is.data.frame(x)) {
          char_cols <- which(sapply(x, is.character))
          if (length(char_cols) > 0) {
            interval_cols <- c()
            for (col_idx in char_cols) {
              values <- as.character(x[[col_idx]])
              if (any(grepl(",", values), na.rm = TRUE)) {
                interval_cols <- c(interval_cols, col_idx)
              }
            }
            if (length(interval_cols) > 0) {
              message("Auto-detected interval columns: ", paste(interval_cols, collapse = ", "))
              iGAP_to_RSDA(x, location = interval_cols)
            } else {
              stop("int_convert_format: No interval columns detected in iGAP data. ",
                   "Please specify 'location' parameter.", call. = FALSE)
            }
          } else {
            stop("int_convert_format: No character columns found in iGAP data.",
                 call. = FALSE)
          }
        } else {
          stop("int_convert_format: iGAP data must be a data.frame.", call. = FALSE)
        }
      } else {
        iGAP_to_RSDA(x, location = args$location)
      }
    },

    # To ARRAY format
    "RSDA_to_ARRAY" = {
      RSDA_to_ARRAY(x)
    },
    "MM_to_ARRAY" = {
      MM_to_ARRAY(x)
    },
    "IGAP_to_ARRAY" = {
      args <- list(...)
      if (is.null(args$location)) {
        if (is.data.frame(x)) {
          char_cols <- which(sapply(x, is.character))
          interval_cols <- c()
          for (col_idx in char_cols) {
            values <- as.character(x[[col_idx]])
            if (any(grepl(",", values), na.rm = TRUE))
              interval_cols <- c(interval_cols, col_idx)
          }
          if (length(interval_cols) > 0) {
            message("Auto-detected interval columns: ", paste(interval_cols, collapse = ", "))
            iGAP_to_ARRAY(x, location = interval_cols)
          } else {
            stop("int_convert_format: No interval columns detected in iGAP data. ",
                 "Please specify 'location' parameter.", call. = FALSE)
          }
        } else {
          stop("int_convert_format: iGAP data must be a data.frame.", call. = FALSE)
        }
      } else {
        iGAP_to_ARRAY(x, location = args$location)
      }
    },
    "SODAS_to_ARRAY" = {
      SODAS_to_ARRAY(x)
    },

    # From ARRAY to other formats
    "ARRAY_to_MM" = {
      ARRAY_to_MM(x)
    },
    "ARRAY_to_IGAP" = {
      ARRAY_to_iGAP(x)
    },

    # Unsupported conversion
    {
      # Check if indirect conversion is possible via MM
      if (from != "MM" && to == "IGAP") {
        message("No direct conversion from ", from, " to ", to, ". ",
                "Converting via MM format...")
        # Two-step conversion: from -> MM -> iGAP
        temp_mm <- int_convert_format(x, to = "MM", from = from, ...)
        int_convert_format(temp_mm, to = "IGAP", from = "MM")
      } else {
        stop("int_convert_format: Conversion from ", from, " to ", to, 
             " is not supported.\n",
             "Available conversions:\n",
             paste(capture.output(print(int_list_conversions())), collapse = "\n"),
             call. = FALSE)
      }
    }
  )
  
  return(result)
}


# --- Conversions to MM format ------------------------------------------------

#' RSDA to MM
#'
#' @name RSDA_to_MM
#' @aliases RSDA_to_MM
#' @description To convert RSDA format interval dataframe to MM format.
#' @usage RSDA_to_MM(data, RSDA = TRUE)
#' @param data The RSDA format with interval dataframe.
#' @param RSDA Whether to load the RSDA package.
#' @returns Return a dataframe with the MM format.
#' @examples
#' data(mushroom.int)
#' RSDA_to_MM(mushroom.int, RSDA = FALSE)
#' @export
RSDA_to_MM <- function(data, RSDA = TRUE){
  if (is.null(data)) {
    stop("RSDA_to_MM: 'data' must not be NULL.", call. = FALSE)
  }
  if (!inherits(data, "symbolic_tbl") && !is.data.frame(data)) {
    stop("RSDA_to_MM: 'data' must be a data.frame or symbolic_tbl, not ",
         class(data)[1], ".", call. = FALSE)
  }
  .check_logical(RSDA, "RSDA", "RSDA_to_MM")
  num_int <- 0
  num_chr <- 0
  chr <- c()
  int <- c()
  index <- c()
  for (i in 1:ncol(data)){
    if (sapply(data, mode)[i] == 'complex'){
      num_int <- num_int + 1
      int <- c(int, i)
    } else{
      num_chr <- num_chr + 1
      chr <- c(chr, i)
    }
    index <- append(chr, int)
  }
  num <- num_chr + 2 * num_int
  df <- as.data.frame(matrix(nrow = nrow(data), ncol = num))
  gsubfun <- function(x){
    x <- gsub("[{.*}]", "", x)
  }
  if (RSDA == TRUE){
    if (length(chr) != 0){
      for (i in 1:length(chr)){
        if (index[i] == 1){
          df[index[i]] <- .process_chr_col(data, index[i], gsubfun, nrow(data), TRUE)
          names(df)[index[i]] <- colnames(data)[index[i]]
        } else{
          j <- 2 * (index[i] - index[i - 1])
          df[j] <- .process_chr_col(data, index[i], gsubfun, nrow(data), TRUE)
          names(df)[j] <- colnames(data)[index[i]]
        }
      }
      df <- .process_int_cols(data, df, int, length(chr), index, FALSE)
    } else{
      df <- .process_int_cols(data, df, int, 0, index, FALSE)
    }
  } else{
    for (i in 1:length(chr)){
      if (index[i] == 1){
        df[index[i]] <- .process_chr_col(data, index[i], gsubfun, nrow(data), FALSE)
        names(df)[index[i]] <- colnames(data)[index[i]]
      } else{
        j <- 2 * (index[i] - index[i - 1])
        df[j] <- .process_chr_col(data, index[i], gsubfun, nrow(data), FALSE)
        names(df)[j] <- colnames(data)[index[i]]
      }
    }
    df <- .process_int_cols(data, df, int, length(chr), index, TRUE)
    for (i in 1:length(df)){
      if (sapply(df, class)[i] != 'character'){
        attributes(df[[i]])$class <- 'numeric'
      }
    }
  }
  return(df)
}

# Internal helper: process a character column (strip {.*} formatting)
.process_chr_col <- function(data, col_idx, gsubfun, nrow_data, use_format) {
  if (use_format) {
    col_data <- data.frame(data[col_idx])
    A <- lapply(format(col_data)[[1]], gsubfun)
    df1 <- as.data.frame(matrix(nrow = nrow_data, ncol = 1))
    for (k in 1:nrow_data){
      df1[k, 1] <- A[[k]]
    }
    df1[[1]]
  } else {
    data.frame(data[col_idx])[[1]]
  }
}

# Internal helper: process interval columns into min/max pairs
.process_int_cols <- function(data, df, int_indices, chr_len, index, use_Re_Im) {
  x <- 0
  for (i in 1:length(int_indices)) {
    col_pos <- index[chr_len + i]
    if (use_Re_Im) {
      df[col_pos + x] <- lapply(data.frame(data[[col_pos]]), Re)
      df[col_pos + x + 1] <- lapply(data.frame(data[[col_pos]]), Im)
    } else {
      df[col_pos + x] <- data.frame(data[[col_pos]])[1]
      df[col_pos + x + 1] <- data.frame(data[[col_pos]])[2]
    }
    names(df)[col_pos + x] <- paste(names(data[col_pos]), '_min', sep = '')
    names(df)[col_pos + x + 1] <- paste(names(data[col_pos]), '_max', sep = '')
    x <- x + 1
  }
  df
}


#' iGAP to MM
#'
#' @name iGAP_to_MM
#' @aliases iGAP_to_MM
#' @description To convert iGAP format to MM format.
#' @usage iGAP_to_MM(data, location = NULL)
#' @param data The dataframe with the iGAP format.
#' @param location The location of the symbolic variable in the data.
#' @returns Return a dataframe with the MM format.
#' @importFrom tidyr separate
#' @importFrom magrittr %>%
#' @examples
#' data(abalone.iGAP)
#' abalone <- iGAP_to_MM(abalone.iGAP, 1:7)
#' @export

iGAP_to_MM <- function(data, location = NULL){
  .check_data_frame(data, "iGAP_to_MM")
  .check_location(location, ncol(data), "iGAP_to_MM")
  location <- sort(location)
  x <- 0
  for (i in location){
    y <- i + x
    data <- data %>%
      tidyr::separate(names(data)[y], c(paste(names(data)[y], '_min', sep = ''),
                                 paste(names(data)[y], '_max', sep = '')), ",")
    x <- x + 1
  }
  return(data)
}


#' SODAS to MM
#'
#' @name SODAS_to_MM
#' @aliases SODAS_to_MM
#' @description To convert SODAS format interval dataframe to the MM format.
#' @usage SODAS_to_MM(XMLPath)
#' @param XMLPath Disk path where the SODAS *.XML file is.
#' @returns Return a dataframe with the MM format.
#' @importFrom RSDA SODAS.to.RSDA
#' @examples
#' \dontrun{
#' # Read from a SODAS XML file:
#' abalone <- SODAS_to_MM("C:/Users/user/AppData/abalone.xml")
#' }
#' @export

SODAS_to_MM <- function(XMLPath){
  .check_file_path(XMLPath, "SODAS_to_MM")
  .check_file_exists(XMLPath, "SODAS_to_MM")
  data <- RSDA::SODAS.to.RSDA(XMLPath)
  df <- RSDA_to_MM(data, RSDA = TRUE)
  return(df)
}


# --- Conversions to iGAP format ----------------------------------------------

#' MM to iGAP
#'
#' @name MM_to_iGAP
#' @aliases MM_to_iGAP
#' @description To convert MM format to iGAP format.
#' @usage MM_to_iGAP(data)
#' @param data The dataframe with the MM format.
#' @returns Return a dataframe with the iGAP format.
#' @importFrom dplyr select
#' @importFrom tidyr unite
#' @importFrom magrittr %>%
#' @examples
#' data(face.iGAP)
#' face <- iGAP_to_MM(face.iGAP, 1:6)
#' MM_to_iGAP(face)
#' @export

MM_to_iGAP <- function(data){
  .check_data_frame(data, "MM_to_iGAP")
  if (!any(grepl("_min|_max|_Min|_Max", names(data)))) {
    warning("MM_to_iGAP: no _min/_max columns detected in 'data'. ",
            "Result may not be meaningful.", call. = FALSE)
  }
  data1 <- clean_colnames(data)
  cols <- unique(names(data1))
  df <- cbind(do.call(cbind, lapply(cols,
                                    function(x){tidyr::unite(data, x, grep(x, names(data), value = TRUE),
                                                            sep = ',', remove = TRUE)} %>% dplyr::select(x))
  ))
  names(df) <- cols
  return(df)
}


#' RSDA to iGAP
#'
#' @name RSDA_to_iGAP
#' @aliases RSDA_to_iGAP
#' @description To convert RSDA format interval dataframe to iGAP format.
#' @usage RSDA_to_iGAP(data)
#' @param data The RSDA format with interval dataframe.
#' @returns Return a dataframe with the iGAP format.
#' @examples
#' data(mushroom.int)
#' RSDA_to_iGAP(mushroom.int)
#' @export

RSDA_to_iGAP <- function(data){
  if (is.null(data)) {
    stop("RSDA_to_iGAP: 'data' must not be NULL.", call. = FALSE)
  }
  if (!inherits(data, "symbolic_tbl")) {
    stop("RSDA_to_iGAP: 'data' must be a symbolic_tbl object, not ",
         class(data)[1], ".", call. = FALSE)
  }
  df <- RSDA_to_MM(data, RSDA = TRUE)
  df.iGAP <- MM_to_iGAP(df)
  return(df.iGAP)
}


#' SODAS to iGAP
#'
#' @name SODAS_to_iGAP
#' @aliases SODAS_to_iGAP
#' @description To convert SODAS format interval dataframe to the iGAP format.
#' @usage SODAS_to_iGAP(XMLPath)
#' @param XMLPath Disk path where the SODAS *.XML file is.
#' @returns Return a dataframe with the iGAP format.
#' @importFrom RSDA SODAS.to.RSDA
#' @examples
#' \dontrun{
#' # Read from a SODAS XML file:
#' abalone <- SODAS_to_iGAP("C:/Users/user/AppData/abalone.xml")
#' }
#' @export

SODAS_to_iGAP <- function(XMLPath){
  .check_file_path(XMLPath, "SODAS_to_iGAP")
  .check_file_exists(XMLPath, "SODAS_to_iGAP")
  data <- RSDA::SODAS.to.RSDA(XMLPath)
  df <- RSDA_to_iGAP(data)
  return(df)
}


# --- Conversions to RSDA format ----------------------------------------------

#' MM to RSDA
#'
#' @name MM_to_RSDA
#' @aliases MM_to_RSDA
#' @description To convert MM format interval dataframe to RSDA format (symbolic_tbl).
#' @usage MM_to_RSDA(data)
#' @param data The dataframe with the MM format (paired _min/_max columns).
#' @returns Return a symbolic_tbl dataframe with complex-encoded interval columns.
#' @examples
#' data(mushroom.int)
#' mm <- RSDA_to_MM(mushroom.int, RSDA = FALSE)
#' rsda <- MM_to_RSDA(mm)
#' @export
MM_to_RSDA <- function(data){
  .check_data_frame(data, "MM_to_RSDA")
  col_names <- names(data)
  if (!any(grepl("_min|_max|_Min|_Max", col_names))) {
    warning("MM_to_RSDA: no _min/_max columns detected in 'data'. ",
            "Result may not be meaningful.", call. = FALSE)
  }

  # Find _min columns and extract base names
  min_cols <- grep("_[Mm]in$", col_names, value = TRUE)
  base_names <- sub("_[Mm]in$", "", min_cols)

  # Build the result data.frame column by column in original order
  result_cols <- list()
  result_names <- c()
  used <- character(0)

  for (col in col_names) {
    if (col %in% used) next

    # Check if this is a _min column
    if (grepl("_[Mm]in$", col)) {
      base <- sub("_[Mm]in$", "", col)
      max_col <- col_names[grepl(paste0("^", base, "_[Mm]ax$"), col_names)]
      if (length(max_col) == 1) {
        # Combine into complex column: min + max*i
        mins <- as.numeric(data[[col]])
        maxs <- as.numeric(data[[max_col]])
        result_cols[[length(result_cols) + 1]] <- mins + maxs * 1i
        result_names <- c(result_names, base)
        used <- c(used, col, max_col)
      } else {
        # No matching _max, keep as-is
        result_cols[[length(result_cols) + 1]] <- data[[col]]
        result_names <- c(result_names, col)
        used <- c(used, col)
      }
    } else if (grepl("_[Mm]ax$", col)) {
      # _max column without matching _min already processed -- skip if used
      next
    } else {
      # Non-interval column, keep as-is
      result_cols[[length(result_cols) + 1]] <- data[[col]]
      result_names <- c(result_names, col)
      used <- c(used, col)
    }
  }

  result <- as.data.frame(result_cols, stringsAsFactors = FALSE)
  names(result) <- result_names
  rownames(result) <- rownames(data)
  class(result) <- c("symbolic_tbl", "data.frame")
  return(result)
}


#' iGAP to RSDA
#'
#' @name iGAP_to_RSDA
#' @aliases iGAP_to_RSDA
#' @description To convert iGAP format interval dataframe to RSDA format (symbolic_tbl).
#' @usage iGAP_to_RSDA(data, location = NULL)
#' @param data The dataframe with the iGAP format.
#' @param location The location of the symbolic variable in the data.
#' @returns Return a symbolic_tbl dataframe with complex-encoded interval columns.
#' @examples
#' data(abalone.iGAP)
#' rsda <- iGAP_to_RSDA(abalone.iGAP, 1:7)
#' @export
iGAP_to_RSDA <- function(data, location = NULL){
  .check_data_frame(data, "iGAP_to_RSDA")
  .check_location(location, ncol(data), "iGAP_to_RSDA")
  mm <- iGAP_to_MM(data, location)
  result <- MM_to_RSDA(mm)
  return(result)
}


# --- Conversions to/from ARRAY format ----------------------------------------

# Internal validator for ARRAY format
.check_array_format <- function(data, fn_name) {
  if (is.null(data))
    stop(fn_name, ": 'data' must not be NULL.", call. = FALSE)
  if (!is.array(data) || length(dim(data)) != 3 || dim(data)[3] != 2)
    stop(fn_name, ": 'data' must be a 3-dimensional array with dim[3] = 2 ",
         "(i.e., [n, p, 2]).", call. = FALSE)
}


#' RSDA to ARRAY
#'
#' @name RSDA_to_ARRAY
#' @aliases RSDA_to_ARRAY
#' @description Convert RSDA format (symbolic_tbl) to a 3-dimensional array
#' \code{[n, p, 2]} where slice \code{[,,1]} contains the minima and
#' slice \code{[,,2]} contains the maxima.
#' @usage RSDA_to_ARRAY(data)
#' @param data A symbolic_tbl with interval columns.
#' @returns A numeric array of dimension \code{[n, p, 2]} with dimnames.
#' Only interval (symbolic_interval) columns are included.
#' @examples
#' data(mushroom.int)
#' arr <- RSDA_to_ARRAY(mushroom.int)
#' dim(arr)  # [23, 3, 2]
#' @export
RSDA_to_ARRAY <- function(data) {
  if (is.null(data))
    stop("RSDA_to_ARRAY: 'data' must not be NULL.", call. = FALSE)
  if (!inherits(data, "symbolic_tbl"))
    stop("RSDA_to_ARRAY: 'data' must be a symbolic_tbl object, not ",
         class(data)[1], ".", call. = FALSE)

  # Filter to interval columns only
  int_cols <- sapply(data, function(col) {
    inherits(col, "symbolic_interval") || mode(col) == "complex"
  })
  if (!any(int_cols))
    stop("RSDA_to_ARRAY: no interval columns found in 'data'.", call. = FALSE)

  symbolic_tbl_to_idata(data[, int_cols, drop = FALSE])
}


#' ARRAY to RSDA
#'
#' @name ARRAY_to_RSDA
#' @aliases ARRAY_to_RSDA
#' @description Convert a 3-dimensional array \code{[n, p, 2]} to RSDA format
#' (symbolic_tbl with symbolic_interval columns).
#' @usage ARRAY_to_RSDA(data)
#' @param data A numeric array of dimension \code{[n, p, 2]} where
#' \code{[,,1]} stores minima and \code{[,,2]} stores maxima.
#' @returns A symbolic_tbl with p symbolic_interval columns.
#' @examples
#' x <- array(NA, dim = c(4, 3, 2))
#' x[,,1] <- matrix(c(1,2,3,4, 5,6,7,8, 9,10,11,12), nrow = 4)
#' x[,,2] <- matrix(c(3,5,6,7, 8,9,10,12, 13,15,16,18), nrow = 4)
#' dimnames(x) <- list(paste0("obs_", 1:4), c("V1","V2","V3"), c("min","max"))
#' rsda <- ARRAY_to_RSDA(x)
#' rsda
#' @export
ARRAY_to_RSDA <- function(data) {
  .check_array_format(data, "ARRAY_to_RSDA")

  n <- dim(data)[1]
  p <- dim(data)[2]

  var_names <- dimnames(data)[[2]]
  if (is.null(var_names)) var_names <- paste0("V", seq_len(p))
  row_names <- dimnames(data)[[1]]
  if (is.null(row_names)) row_names <- as.character(seq_len(n))

  col_list <- vector("list", p)
  names(col_list) <- var_names
  for (j in seq_len(p)) {
    col_list[[j]] <- .new_symbolic_interval(data[, j, 1], data[, j, 2])
  }

  structure(col_list,
            class = c("symbolic_tbl", "tbl_df", "tbl", "data.frame"),
            row.names = seq_len(n),
            names = var_names,
            concept = row_names)
}


#' MM to ARRAY
#'
#' @name MM_to_ARRAY
#' @aliases MM_to_ARRAY
#' @description Convert MM format (paired \code{_min}/\code{_max} columns) to a
#' 3-dimensional array \code{[n, p, 2]}.
#' @usage MM_to_ARRAY(data)
#' @param data A data.frame in MM format with paired \code{_min} and \code{_max}
#' columns.
#' @returns A numeric array of dimension \code{[n, p, 2]} with dimnames.
#' Non-interval columns are excluded.
#' @examples
#' data(mushroom.int)
#' mm <- RSDA_to_MM(mushroom.int, RSDA = FALSE)
#' arr <- MM_to_ARRAY(mm)
#' dim(arr)
#' @export
MM_to_ARRAY <- function(data) {
  .check_data_frame(data, "MM_to_ARRAY")
  col_names <- names(data)

  min_cols <- grep("_[Mm]in$", col_names, value = TRUE)
  if (length(min_cols) == 0)
    stop("MM_to_ARRAY: no '_min' columns found in 'data'.", call. = FALSE)

  base_names <- sub("_[Mm]in$", "", min_cols)
  n <- nrow(data)
  p <- length(base_names)

  result <- array(NA_real_, dim = c(n, p, 2))
  var_names <- character(p)

  for (j in seq_len(p)) {
    max_col <- col_names[grepl(paste0("^", base_names[j], "_[Mm]ax$"), col_names)]
    if (length(max_col) != 1)
      stop("MM_to_ARRAY: no matching '_max' column for '", min_cols[j], "'.",
           call. = FALSE)
    result[, j, 1] <- as.numeric(data[[min_cols[j]]])
    result[, j, 2] <- as.numeric(data[[max_col]])
    var_names[j] <- base_names[j]
  }

  dimnames(result) <- list(rownames(data), var_names, c("min", "max"))
  result
}


#' ARRAY to MM
#'
#' @name ARRAY_to_MM
#' @aliases ARRAY_to_MM
#' @description Convert a 3-dimensional array \code{[n, p, 2]} to MM format
#' (data.frame with paired \code{_min}/\code{_max} columns).
#' @usage ARRAY_to_MM(data)
#' @param data A numeric array of dimension \code{[n, p, 2]} where
#' \code{[,,1]} stores minima and \code{[,,2]} stores maxima.
#' @returns A data.frame with \code{2p} columns (paired \code{_min}/\code{_max}).
#' @examples
#' x <- array(NA, dim = c(4, 3, 2))
#' x[,,1] <- matrix(c(1,2,3,4, 5,6,7,8, 9,10,11,12), nrow = 4)
#' x[,,2] <- matrix(c(3,5,6,7, 8,9,10,12, 13,15,16,18), nrow = 4)
#' dimnames(x) <- list(paste0("obs_", 1:4), c("V1","V2","V3"), c("min","max"))
#' mm <- ARRAY_to_MM(x)
#' mm
#' @export
ARRAY_to_MM <- function(data) {
  .check_array_format(data, "ARRAY_to_MM")

  n <- dim(data)[1]
  p <- dim(data)[2]

  var_names <- dimnames(data)[[2]]
  if (is.null(var_names)) var_names <- paste0("V", seq_len(p))
  row_names <- dimnames(data)[[1]]

  col_names <- character(2 * p)
  result <- data.frame(matrix(NA_real_, nrow = n, ncol = 2 * p))
  for (j in seq_len(p)) {
    result[, 2 * j - 1] <- data[, j, 1]
    result[, 2 * j]     <- data[, j, 2]
    col_names[2 * j - 1] <- paste0(var_names[j], "_min")
    col_names[2 * j]     <- paste0(var_names[j], "_max")
  }
  names(result) <- col_names
  if (!is.null(row_names)) rownames(result) <- row_names
  result
}


#' iGAP to ARRAY
#'
#' @name iGAP_to_ARRAY
#' @aliases iGAP_to_ARRAY
#' @description Convert iGAP format to a 3-dimensional array \code{[n, p, 2]}.
#' @usage iGAP_to_ARRAY(data, location = NULL)
#' @param data A data.frame in iGAP format.
#' @param location Integer vector specifying which columns contain
#' comma-separated interval values.
#' @returns A numeric array of dimension \code{[n, p, 2]} with dimnames.
#' @importFrom tidyr separate
#' @importFrom magrittr %>%
#' @examples
#' data(abalone.iGAP)
#' arr <- iGAP_to_ARRAY(abalone.iGAP, 1:7)
#' dim(arr)
#' @export
iGAP_to_ARRAY <- function(data, location = NULL) {
  .check_data_frame(data, "iGAP_to_ARRAY")
  .check_location(location, ncol(data), "iGAP_to_ARRAY")
  mm <- iGAP_to_MM(data, location)
  MM_to_ARRAY(mm)
}


#' ARRAY to iGAP
#'
#' @name ARRAY_to_iGAP
#' @aliases ARRAY_to_iGAP
#' @description Convert a 3-dimensional array \code{[n, p, 2]} to iGAP format
#' (data.frame with comma-separated interval values).
#' @usage ARRAY_to_iGAP(data)
#' @param data A numeric array of dimension \code{[n, p, 2]} where
#' \code{[,,1]} stores minima and \code{[,,2]} stores maxima.
#' @returns A data.frame in iGAP format with comma-separated \code{"min,max"}
#' values.
#' @importFrom dplyr select
#' @importFrom tidyr unite
#' @importFrom magrittr %>%
#' @examples
#' x <- array(NA, dim = c(4, 3, 2))
#' x[,,1] <- matrix(c(1,2,3,4, 5,6,7,8, 9,10,11,12), nrow = 4)
#' x[,,2] <- matrix(c(3,5,6,7, 8,9,10,12, 13,15,16,18), nrow = 4)
#' dimnames(x) <- list(paste0("obs_", 1:4), c("V1","V2","V3"), c("min","max"))
#' igap <- ARRAY_to_iGAP(x)
#' igap
#' @export
ARRAY_to_iGAP <- function(data) {
  .check_array_format(data, "ARRAY_to_iGAP")
  mm <- ARRAY_to_MM(data)
  MM_to_iGAP(mm)
}


#' Convert Interval Data to All Supported Formats
#'
#' @name to_all_interval_formats
#' @aliases to_all_interval_formats
#' @description Convert interval data from any recognized format to all six
#' supported interval data formats and return the results as a named list.
#' This is useful for inspecting and comparing how the same interval data is
#' represented across different formats.
#' @usage to_all_interval_formats(x, ...)
#' @param x Interval data in one of the supported formats:
#' \code{"RSDA"}, \code{"MM"}, \code{"iGAP"}, \code{"ARRAY"},
#' \code{"SODAS"}, or \code{"SDS"}.
#' @param ... Additional arguments passed to conversion functions (e.g.,
#' \code{location} for iGAP input).
#' @returns A named list with six slots:
#' \describe{
#'   \item{\code{RSDA}}{A \code{symbolic_tbl} with complex-encoded
#'     \code{symbolic_interval} columns.}
#'   \item{\code{MM}}{A \code{data.frame} with paired \code{_min}/\code{_max}
#'     columns.}
#'   \item{\code{iGAP}}{A \code{data.frame} with comma-separated
#'     \code{"min,max"} character values.}
#'   \item{\code{ARRAY}}{A three-dimensional numeric \code{array} of dimension
#'     \code{[n, p, 2]} where \code{[,,1]} stores minima and \code{[,,2]}
#'     stores maxima.}
#'   \item{\code{SODAS}}{\code{NULL} unless the input is a SODAS XML file path,
#'     in which case it stores the original path.}
#'   \item{\code{SDS}}{\code{NULL} unless the input is a SODAS/SDS XML file path
#'     (alias for SODAS).}
#' }
#' @details
#' Six interval data formats are supported in this package.  Each format
#' stores the same information -- lower and upper bounds for every variable of
#' every observation -- but differs in its structure and origin:
#'
#' \describe{
#'   \item{\strong{RSDA}}{
#'     A \code{symbolic_tbl} object (class
#'     \code{c("symbolic_tbl", "tbl_df", "tbl", "data.frame")}) where each
#'     interval variable is a complex column (\code{symbolic_interval}):
#'     \code{Re()} gives the minimum and \code{Im()} gives the maximum.
#'     This is the native format of the \pkg{RSDA} package
#'     (Billard & Diday, 2006; Rodriguez, 2024).
#'   }
#'   \item{\strong{MM (Min-Max)}}{
#'     A plain \code{data.frame} where each interval variable is represented by
#'     two numeric columns named \code{<var>_min} and \code{<var>_max}.
#'     This is a widely used general-purpose representation.
#'   }
#'   \item{\strong{iGAP}}{
#'     A \code{data.frame} where each interval variable is stored as a character
#'     column with comma-separated values \code{"min,max"}.
#'     This is the format used by the \pkg{iGAP} software (Correia, 2009).
#'   }
#'   \item{\strong{ARRAY}}{
#'     A three-dimensional numeric \code{array} of size \code{[n, p, 2]}.
#'     The first slice \code{[,,1]} contains all minima and the second slice
#'     \code{[,,2]} contains all maxima.  Dimnames encode observation labels,
#'     variable names, and \code{c("min", "max")}.  This format is convenient
#'     for matrix-based computations.
#'   }
#'   \item{\strong{SODAS}}{
#'     An XML file on disk produced by the SODAS software (Diday & Noirhomme,
#'     2008).  In R, SODAS data is referenced by its file path and read via
#'     \code{RSDA::SODAS.to.RSDA()}.  Since SODAS is a file-based format, it
#'     cannot be generated from in-memory data.
#'   }
#'   \item{\strong{SDS}}{
#'     An alias for SODAS.  Both refer to the same XML-based format.
#'   }
#' }
#' @references
#' Billard, L. and Diday, E. (2006).
#' \emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
#' Wiley.
#'
#' Rodriguez, O. (2024).
#' \emph{RSDA: R to Symbolic Data Analysis}.
#' R package, \url{https://CRAN.R-project.org/package=RSDA}.
#'
#' Correia, M. (2009).
#' \emph{Interval GARCH and Aggregation of Predictions}.
#'
#' Diday, E. and Noirhomme-Fraiture, M. (2008).
#' \emph{Symbolic Data Analysis and the SODAS Software}.
#' Wiley.
#' @author Han-Ming Wu
#' @seealso \code{\link{int_detect_format}}, \code{\link{int_convert_format}},
#'   \code{\link{int_list_conversions}}
#' @examples
#' data(car.int)
#' result <- to_all_interval_formats(car.int)
#' names(result)
#'
#' # RSDA format (symbolic_tbl)
#' result$RSDA
#'
#' # MM format (data.frame with _min/_max columns)
#' head(result$MM)
#'
#' # iGAP format (data.frame with comma-separated values)
#' head(result$iGAP)
#'
#' # ARRAY format (3D array)
#' dim(result$ARRAY)
#' result$ARRAY[1:3, , 1]  # minima
#' result$ARRAY[1:3, , 2]  # maxima
#'
#' # SODAS/SDS slots are NULL (file-based format)
#' result$SODAS
#' result$SDS
#' @export
to_all_interval_formats <- function(x, ...) {
  # Detect source format
  src <- int_detect_format(x)
  if (src == "unknown")
    stop("to_all_interval_formats: could not detect the format of 'x'. ",
         "Supported formats: RSDA, MM, iGAP, ARRAY, SODAS/SDS.", call. = FALSE)

  src_upper <- toupper(src)
  if (src_upper == "SDS") src_upper <- "SODAS"

  # Initialize result list
  result <- list(RSDA = NULL, MM = NULL, iGAP = NULL,
                 ARRAY = NULL, SODAS = NULL, SDS = NULL)

  # Place original data in the matching slot
  switch(src_upper,
    "RSDA"  = { result$RSDA  <- x },
    "MM"    = { result$MM    <- x },
    "IGAP"  = { result$iGAP  <- x },
    "ARRAY" = { result$ARRAY <- x },
    "SODAS" = { result$SODAS <- x; result$SDS <- x }
  )

  # Convert to each in-memory format that is not yet filled
  targets <- c(RSDA = "RSDA", MM = "MM", iGAP = "IGAP", ARRAY = "ARRAY")

  for (slot_name in names(targets)) {
    if (!is.null(result[[slot_name]])) next
    result[[slot_name]] <- tryCatch(
      suppressMessages(int_convert_format(x, to = targets[[slot_name]],
                                          from = src_upper, ...)),
      error = function(e) {
        warning("to_all_interval_formats: conversion to ", slot_name,
                " failed: ", conditionMessage(e), call. = FALSE)
        NULL
      }
    )
  }

  # SODAS/SDS: file-based format, cannot be produced from in-memory data
  if (src_upper != "SODAS") {
    message("Note: SODAS/SDS is a file-based XML format and cannot be ",
            "generated from in-memory data. The SODAS and SDS slots are NULL.")
  }

  result
}


#' SODAS to ARRAY
#'
#' @name SODAS_to_ARRAY
#' @aliases SODAS_to_ARRAY
#' @description Convert SODAS format (XML file) to a 3-dimensional array
#' \code{[n, p, 2]}.
#' @usage SODAS_to_ARRAY(XMLPath)
#' @param XMLPath Disk path where the SODAS \code{*.XML} file is.
#' @returns A numeric array of dimension \code{[n, p, 2]} with dimnames.
#' @importFrom RSDA SODAS.to.RSDA
#' @examples
#' \dontrun{
#' arr <- SODAS_to_ARRAY("C:/Users/user/AppData/abalone.xml")
#' }
#' @export
SODAS_to_ARRAY <- function(XMLPath) {
  .check_file_path(XMLPath, "SODAS_to_ARRAY")
  .check_file_exists(XMLPath, "SODAS_to_ARRAY")
  rsda <- RSDA::SODAS.to.RSDA(XMLPath)
  RSDA_to_ARRAY(rsda)
}

test_that("RSDA_to_MM converts mushroom.int with RSDA=FALSE", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  expect_s3_class(result, "data.frame")
})

test_that("RSDA_to_MM output has _min and _max columns", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  col_names <- names(result)
  has_min <- any(grepl("_min$", col_names))
  has_max <- any(grepl("_max$", col_names))
  expect_true(has_min)
  expect_true(has_max)
})

test_that("RSDA_to_MM preserves row count", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  expect_equal(nrow(result), nrow(mushroom.int))
})

test_that("RSDA_to_MM min columns have numeric values", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  min_cols <- grep("_min$", names(result), value = TRUE)
  for (col in min_cols) {
    expect_true(is.numeric(result[[col]]),
                info = paste(col, "should be numeric"))
  }
})

test_that("RSDA_to_MM max columns have numeric values", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  max_cols <- grep("_max$", names(result), value = TRUE)
  for (col in max_cols) {
    expect_true(is.numeric(result[[col]]),
                info = paste(col, "should be numeric"))
  }
})

test_that("RSDA_to_MM mushroom.int has correct column count", {
  data(mushroom.int)
  n_complex <- sum(sapply(mushroom.int, mode) == "complex")
  n_other <- ncol(mushroom.int) - n_complex
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  expected_cols <- n_other + 2 * n_complex
  expect_equal(ncol(result), expected_cols)
})

test_that("RSDA_to_MM creates paired min/max columns", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  min_cols <- grep("_min$", names(result), value = TRUE)
  max_cols <- grep("_max$", names(result), value = TRUE)
  min_bases <- sub("_min$", "", min_cols)
  max_bases <- sub("_max$", "", max_cols)
  expect_equal(min_bases, max_bases)
})

test_that("RSDA_to_MM output contains no NA in interval columns", {
  data(mushroom.int)
  result <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  min_cols <- grep("_min$", names(result), value = TRUE)
  max_cols <- grep("_max$", names(result), value = TRUE)
  for (col in c(min_cols, max_cols)) {
    expect_false(any(is.na(result[[col]])),
                 info = paste(col, "should have no NAs"))
  }
})

# ---------- clean_colnames ----------
test_that("clean_colnames rejects non-data.frame", {
  expect_error(clean_colnames(NULL), "must not be NULL")
  expect_error(clean_colnames("text"), "must be a data.frame")
  expect_error(clean_colnames(1:5), "must be a data.frame")
})

# ---------- iGAP_to_MM ----------
test_that("iGAP_to_MM rejects non-data.frame", {
  expect_error(iGAP_to_MM(NULL, 1), "must not be NULL")
  expect_error(iGAP_to_MM("text", 1), "must be a data.frame")
})

test_that("iGAP_to_MM rejects bad location", {
  data(abalone.iGAP)
  expect_error(iGAP_to_MM(abalone.iGAP, NULL), "must not be NULL")
  expect_error(iGAP_to_MM(abalone.iGAP, "a"), "must be numeric")
  expect_error(iGAP_to_MM(abalone.iGAP, 0), "must be between 1 and")
  expect_error(iGAP_to_MM(abalone.iGAP, 999), "must be between 1 and")
})

# ---------- MM_to_iGAP ----------
test_that("MM_to_iGAP rejects non-data.frame", {
  expect_error(MM_to_iGAP(NULL), "must not be NULL")
  expect_error(MM_to_iGAP(42), "must be a data.frame")
})

test_that("MM_to_iGAP warns when no _min/_max columns", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_warning(MM_to_iGAP(df), "no _min/_max columns")
})

# ---------- RSDA_to_MM ----------
test_that("RSDA_to_MM rejects NULL", {
  expect_error(RSDA_to_MM(NULL), "must not be NULL")
})

test_that("RSDA_to_MM rejects wrong type", {
  expect_error(RSDA_to_MM("text"), "must be a data.frame or symbolic_tbl")
  expect_error(RSDA_to_MM(1:5), "must be a data.frame or symbolic_tbl")
})

test_that("RSDA_to_MM rejects bad RSDA", {
  data(mushroom.int)
  expect_error(RSDA_to_MM(mushroom.int, RSDA = "yes"), "must be TRUE or FALSE")
  expect_error(RSDA_to_MM(mushroom.int, RSDA = NA), "must be TRUE or FALSE")
})

# ---------- RSDA_to_iGAP ----------
test_that("RSDA_to_iGAP rejects NULL", {
  expect_error(RSDA_to_iGAP(NULL), "must not be NULL")
})

test_that("RSDA_to_iGAP rejects non-symbolic_tbl", {
  expect_error(RSDA_to_iGAP(data.frame(a = 1)), "must be a symbolic_tbl")
})

# ---------- RSDA_format ----------
test_that("RSDA_format rejects non-data.frame", {
  expect_error(RSDA_format(NULL), "must not be NULL")
  expect_error(RSDA_format("text"), "must be a data.frame")
})

test_that("RSDA_format rejects wrong param types", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_error(RSDA_format(df, sym_type1 = 123), "must be a character")
  expect_error(RSDA_format(df, location = "a"), "must be numeric")
  expect_error(RSDA_format(df, sym_type2 = 123), "must be a character")
  expect_error(RSDA_format(df, sym_type2 = "I", var = 123), "must be a character")
})

# ---------- set_variable_format ----------
test_that("set_variable_format rejects non-data.frame", {
  expect_error(set_variable_format(NULL, location = 1), "must not be NULL")
  expect_error(set_variable_format(42, location = 1), "must be a data.frame")
})

test_that("set_variable_format rejects bad location", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  expect_error(set_variable_format(df, location = 0), "must be between 1 and")
  expect_error(set_variable_format(df, location = 99), "must be between 1 and")
})

test_that("set_variable_format rejects missing var", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  expect_error(set_variable_format(df, var = "nonexistent"), "not found in data")
})


# ---------- int_mean / int_var ----------
test_that("int_mean rejects non-symbolic_tbl", {
  expect_error(int_mean(NULL, "x"), "must not be NULL")
  expect_error(int_mean(data.frame(a = 1), "a"), "must be a symbolic_tbl")
})

test_that("int_mean rejects missing var_name", {
  data(mushroom.int)
  expect_error(int_mean(mushroom.int, "NoSuchVar"), "not found in data")
})

test_that("int_mean warns on invalid method", {
  data(mushroom.int)
  expect_warning(int_mean(mushroom.int, "Pileus.Cap.Width", method = "INVALID"),
                 "unknown method")
})

test_that("int_var rejects non-symbolic_tbl", {
  expect_error(int_var(NULL, "x"), "must not be NULL")
})

test_that("int_var rejects missing var_name", {
  data(mushroom.int)
  expect_error(int_var(mushroom.int, "NoSuchVar"), "not found in data")
})

# ---------- int_cov / int_cor ----------
test_that("int_cov rejects non-symbolic_tbl", {
  expect_error(int_cov(NULL, "x", "y"), "must not be NULL")
})

test_that("int_cov rejects missing var_name", {
  data(mushroom.int)
  expect_error(int_cov(mushroom.int, "NoSuchVar", "Pileus.Cap.Width"), "not found in data")
  expect_error(int_cov(mushroom.int, "Pileus.Cap.Width", "NoSuchVar"), "not found in data")
})

test_that("int_cor rejects non-symbolic_tbl", {
  expect_error(int_cor(NULL, "x", "y"), "must not be NULL")
})

# ---------- hist_mean / hist_var ----------
test_that("hist_mean rejects non-MatH", {
  expect_error(hist_mean(NULL, "x"), "must not be NULL")
  expect_error(hist_mean(data.frame(a = 1), "a"), "must be a MatH object")
})

test_that("hist_mean rejects missing var_name", {
  blood <- HistDAWass::BLOOD
  expect_error(hist_mean(blood, "NoSuchVar"), "not found in data")
})

test_that("hist_mean warns on invalid method", {
  blood <- HistDAWass::BLOOD
  var1 <- colnames(blood@M)[1]
  expect_warning(
    tryCatch(hist_mean(blood, var1, method = "INVALID"), error = function(e) NULL),
    "unknown method"
  )
})

test_that("hist_var rejects non-MatH", {
  expect_error(hist_var(NULL, "x"), "must not be NULL")
})

# ---------- hist_cov / hist_cor ----------
test_that("hist_cov rejects non-MatH", {
  expect_error(hist_cov(NULL, "x", "y"), "must not be NULL")
})

test_that("hist_cov rejects missing var_name", {
  blood <- HistDAWass::BLOOD
  var1 <- colnames(blood@M)[1]
  expect_error(hist_cov(blood, "NoSuchVar", var1), "not found in data")
  expect_error(hist_cov(blood, var1, "NoSuchVar"), "not found in data")
})

test_that("hist_cor rejects non-MatH", {
  expect_error(hist_cor(NULL, "x", "y"), "must not be NULL")
})

# ---------- MM_to_RSDA ----------
test_that("MM_to_RSDA rejects non-data.frame", {
  expect_error(MM_to_RSDA(NULL), "must not be NULL")
  expect_error(MM_to_RSDA("text"), "must be a data.frame")
  expect_error(MM_to_RSDA(1:5), "must be a data.frame")
})

test_that("MM_to_RSDA warns when no _min/_max columns", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_warning(MM_to_RSDA(df), "no _min/_max columns")
})

# ---------- iGAP_to_RSDA ----------
test_that("iGAP_to_RSDA rejects non-data.frame", {
  expect_error(iGAP_to_RSDA(NULL, 1), "must not be NULL")
  expect_error(iGAP_to_RSDA("text", 1), "must be a data.frame")
})

test_that("iGAP_to_RSDA rejects bad location", {
  data(abalone.iGAP)
  expect_error(iGAP_to_RSDA(abalone.iGAP, NULL), "must not be NULL")
  expect_error(iGAP_to_RSDA(abalone.iGAP, "a"), "must be numeric")
  expect_error(iGAP_to_RSDA(abalone.iGAP, 0), "must be between 1 and")
  expect_error(iGAP_to_RSDA(abalone.iGAP, 999), "must be between 1 and")
})

# ---------- SODAS_to_MM / SODAS_to_iGAP ----------
test_that("SODAS_to_MM rejects bad path", {
  expect_error(SODAS_to_MM(NULL), "must be a non-empty character")
  expect_error(SODAS_to_MM(""), "must be a non-empty character")
  expect_error(SODAS_to_MM("no_such_file.xml"), "file not found")
})

test_that("SODAS_to_iGAP rejects bad path", {
  expect_error(SODAS_to_iGAP(NULL), "must be a non-empty character")
  expect_error(SODAS_to_iGAP(""), "must be a non-empty character")
  expect_error(SODAS_to_iGAP("no_such_file.xml"), "file not found")
})

test_that("RSDA_to_iGAP converts mushroom.int", {
  skip_if_not_installed("RSDA")
  data(mushroom.int)
  result <- RSDA_to_iGAP(mushroom.int)
  expect_s3_class(result, "data.frame")
})

test_that("RSDA_to_iGAP output contains comma-separated values", {
  skip_if_not_installed("RSDA")
  data(mushroom.int)
  result <- RSDA_to_iGAP(mushroom.int)
  has_comma <- any(sapply(result, function(col) any(grepl(",", col))))
  expect_true(has_comma)
})

test_that("RSDA_to_iGAP preserves row count", {
  skip_if_not_installed("RSDA")
  data(mushroom.int)
  result <- RSDA_to_iGAP(mushroom.int)
  expect_equal(nrow(result), nrow(mushroom.int))
})

test_that("RSDA_to_iGAP is consistent with RSDA_to_MM + MM_to_iGAP", {
  skip_if_not_installed("RSDA")
  data(mushroom.int)
  result1 <- RSDA_to_iGAP(mushroom.int)
  mm <- RSDA_to_MM(mushroom.int, RSDA = TRUE)
  result2 <- MM_to_iGAP(mm)
  expect_equal(result1, result2)
})

test_that("RSDA_to_iGAP output is data.frame with correct dimensions", {
  skip_if_not_installed("RSDA")
  data(mushroom.int)
  result <- RSDA_to_iGAP(mushroom.int)
  expect_true(ncol(result) > 0)
  expect_true(nrow(result) > 0)
})

test_that("iGAP_to_MM converts abalone.iGAP correctly", {
  data(abalone.iGAP)
  result <- iGAP_to_MM(abalone.iGAP, 1:7)
  expect_s3_class(result, "data.frame")
})

test_that("iGAP_to_MM output has _min and _max columns", {
  data(abalone.iGAP)
  result <- iGAP_to_MM(abalone.iGAP, 1:7)
  col_names <- names(result)
  has_min <- any(grepl("_min$", col_names))
  has_max <- any(grepl("_max$", col_names))
  expect_true(has_min)
  expect_true(has_max)
})

test_that("iGAP_to_MM doubles the interval columns", {
  data(abalone.iGAP)
  n_orig <- ncol(abalone.iGAP)
  n_interval <- 7
  result <- iGAP_to_MM(abalone.iGAP, 1:7)
  expected_cols <- n_orig + n_interval
  expect_equal(ncol(result), expected_cols)
})

test_that("iGAP_to_MM preserves row count", {
  data(abalone.iGAP)
  result <- iGAP_to_MM(abalone.iGAP, 1:7)
  expect_equal(nrow(result), nrow(abalone.iGAP))
})

test_that("iGAP_to_MM converts face.iGAP correctly", {
  data(face.iGAP)
  result <- iGAP_to_MM(face.iGAP, 1:6)
  expect_s3_class(result, "data.frame")
  expected_cols <- ncol(face.iGAP) + 6
  expect_equal(ncol(result), expected_cols)
})

test_that("iGAP_to_MM _min values <= _max values for Abalone", {
  data(abalone.iGAP)
  result <- iGAP_to_MM(abalone.iGAP, 1:7)
  min_cols <- grep("_min$", names(result), value = TRUE)
  max_cols <- grep("_max$", names(result), value = TRUE)
  for (i in seq_along(min_cols)) {
    mins <- as.numeric(trimws(result[[min_cols[i]]]))
    maxs <- as.numeric(trimws(result[[max_cols[i]]]))
    expect_true(all(mins <= maxs, na.rm = TRUE),
                info = paste("Failed for", min_cols[i]))
  }
})

test_that("iGAP_to_MM _min values <= _max values for Face", {
  data(face.iGAP)
  result <- iGAP_to_MM(face.iGAP, 1:6)
  min_cols <- grep("_min$", names(result), value = TRUE)
  max_cols <- grep("_max$", names(result), value = TRUE)
  for (i in seq_along(min_cols)) {
    mins <- as.numeric(trimws(result[[min_cols[i]]]))
    maxs <- as.numeric(trimws(result[[max_cols[i]]]))
    expect_true(all(mins <= maxs, na.rm = TRUE),
                info = paste("Failed for", min_cols[i]))
  }
})

test_that("iGAP_to_MM with single location works", {
  data(abalone.iGAP)
  result <- iGAP_to_MM(abalone.iGAP, 1)
  expect_s3_class(result, "data.frame")
  expect_equal(ncol(result), ncol(abalone.iGAP) + 1)
})

test_that("iGAP_to_MM column names contain original variable names", {
  data(face.iGAP)
  orig_names <- names(face.iGAP)[1:6]
  result <- iGAP_to_MM(face.iGAP, 1:6)
  for (nm in orig_names) {
    expect_true(any(grepl(nm, names(result))),
                info = paste("Missing column derived from", nm))
  }
})

test_that("iGAP_to_MM output values are character after separate", {
  data(face.iGAP)
  result <- iGAP_to_MM(face.iGAP, 1:6)
  min_cols <- grep("_min$", names(result), value = TRUE)
  for (col in min_cols) {
    expect_true(is.character(result[[col]]) || is.numeric(result[[col]]))
  }
})

# ============================================================================
# Tests for int_detect_format, int_list_conversions, int_convert_format
# ============================================================================

# ---------- int_detect_format ------------------------------------------------

test_that("int_detect_format detects RSDA format", {
  data(mushroom.int)
  expect_equal(int_detect_format(mushroom.int), "RSDA")
})

test_that("int_detect_format detects RSDA for symbolic_tbl datasets", {
  data(bird.mix)
  # bird.mix is symbolic_tbl with complex columns
  if (inherits(bird.mix, "symbolic_tbl") && any(sapply(bird.mix, mode) == "complex")) {
    expect_equal(int_detect_format(bird.mix), "RSDA")
  }
  data(car.int)
  if (inherits(car.int, "symbolic_tbl") && any(sapply(car.int, mode) == "complex")) {
    expect_equal(int_detect_format(car.int), "RSDA")
  }
})

test_that("int_detect_format detects iGAP format", {
  data(abalone.iGAP)
  expect_equal(int_detect_format(abalone.iGAP), "iGAP")
})

test_that("int_detect_format detects iGAP for face.iGAP", {
  data(face.iGAP)
  expect_equal(int_detect_format(face.iGAP), "iGAP")
})

test_that("int_detect_format detects MM format", {
  data(mushroom.int)
  mm <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  expect_equal(int_detect_format(mm), "MM")
})

test_that("int_detect_format detects MM from iGAP conversion", {
  data(abalone.iGAP)
  mm <- iGAP_to_MM(abalone.iGAP, 1:7)
  expect_equal(int_detect_format(mm), "MM")
})

test_that("int_detect_format detects MM for _min/_max data.frames", {
  # abalone.int is stored as plain data.frame with _min/_max columns
  data(abalone.int)
  expect_equal(int_detect_format(abalone.int), "MM")
})

test_that("int_detect_format detects SODAS for XML paths", {
  expect_equal(int_detect_format("data/example.xml"), "SODAS")
  expect_equal(int_detect_format("C:/path/to/file.XML"), "SODAS")
  expect_equal(int_detect_format("test.Xml"), "SODAS")
})

test_that("int_detect_format returns unknown for NULL", {
  expect_equal(int_detect_format(NULL), "unknown")
})

test_that("int_detect_format returns unknown for plain data.frame", {
  df <- data.frame(a = 1:3, b = letters[1:3])
  expect_equal(int_detect_format(df), "unknown")
})

test_that("int_detect_format returns unknown for non-data objects", {
  expect_equal(int_detect_format(1:10), "unknown")
  expect_equal(int_detect_format("hello"), "unknown")
  expect_equal(int_detect_format(matrix(1:4, 2, 2)), "unknown")
})

test_that("int_detect_format returns unknown for non-XML string", {
  expect_equal(int_detect_format("not_xml.csv"), "unknown")
  expect_equal(int_detect_format("file.txt"), "unknown")
})

# ---------- int_list_conversions ---------------------------------------------

test_that("int_list_conversions returns all 15 conversions", {
  result <- int_list_conversions()
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 15)
  expect_true(all(c("from", "to", "function_name") %in% names(result)))
})

test_that("int_list_conversions filters by from", {
  rsda <- int_list_conversions(from = "RSDA")
  expect_true(all(toupper(rsda$from) == "RSDA"))
  expect_equal(nrow(rsda), 3)

  sodas <- int_list_conversions(from = "SODAS")
  expect_true(all(toupper(sodas$from) == "SODAS"))
  expect_equal(nrow(sodas), 3)

  mm <- int_list_conversions(from = "MM")
  expect_true(all(toupper(mm$from) == "MM"))
  expect_equal(nrow(mm), 3)
})

test_that("int_list_conversions filters by to", {
  to_mm <- int_list_conversions(to = "MM")
  expect_true(all(toupper(to_mm$to) == "MM"))
  expect_equal(nrow(to_mm), 4)

  to_igap <- int_list_conversions(to = "iGAP")
  expect_true(all(toupper(to_igap$to) == "IGAP"))
  expect_equal(nrow(to_igap), 4)

  to_rsda <- int_list_conversions(to = "RSDA")
  expect_true(all(toupper(to_rsda$to) == "RSDA"))
  expect_equal(nrow(to_rsda), 3)
})

test_that("int_list_conversions filters by from and to", {
  result <- int_list_conversions(from = "RSDA", to = "MM")
  expect_equal(nrow(result), 1)
  expect_equal(result$function_name, "RSDA_to_MM")
})

test_that("int_list_conversions finds MM to RSDA conversion", {
  result <- int_list_conversions(from = "MM", to = "RSDA")
  expect_equal(nrow(result), 1)
  expect_equal(result$function_name, "MM_to_RSDA")
})

test_that("int_list_conversions is case-insensitive for from", {
  result <- int_list_conversions(from = "rsda")
  expect_equal(nrow(result), 3)
})

test_that("int_list_conversions is case-insensitive for to", {
  result <- int_list_conversions(to = "mm")
  expect_equal(nrow(result), 4)
})

test_that("int_list_conversions handles iGAP case variations", {
  r1 <- int_list_conversions(from = "iGAP")
  r2 <- int_list_conversions(from = "IGAP")
  r3 <- int_list_conversions(from = "igap")
  expect_equal(nrow(r1), nrow(r2))
  expect_equal(nrow(r2), nrow(r3))
})

test_that("int_list_conversions function_name column matches real functions", {
  result <- int_list_conversions()
  for (fn in result$function_name) {
    expect_true(is.function(get(fn)),
                info = paste(fn, "should be an exported function"))
  }
})

# ---------- int_convert_format -----------------------------------------------

test_that("int_convert_format converts RSDA to MM with auto-detect", {
  data(mushroom.int)
  result <- suppressMessages(int_convert_format(mushroom.int, to = "MM"))
  expect_s3_class(result, "data.frame")
  expect_true(any(grepl("_min$", names(result))))
  expect_true(any(grepl("_max$", names(result))))
  expect_equal(nrow(result), nrow(mushroom.int))
})

test_that("int_convert_format converts RSDA to iGAP with auto-detect", {
  data(mushroom.int)
  result <- suppressMessages(int_convert_format(mushroom.int, to = "iGAP"))
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), nrow(mushroom.int))
})

test_that("int_convert_format converts iGAP to MM with auto-detect", {
  data(abalone.iGAP)
  result <- suppressMessages(int_convert_format(abalone.iGAP, to = "MM"))
  expect_s3_class(result, "data.frame")
  expect_true(any(grepl("_min$", names(result))))
  expect_equal(nrow(result), nrow(abalone.iGAP))
})

test_that("int_convert_format converts MM to iGAP with auto-detect", {
  data(mushroom.int)
  mm <- suppressMessages(int_convert_format(mushroom.int, to = "MM"))
  result <- suppressMessages(int_convert_format(mm, to = "iGAP"))
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), nrow(mm))
})

test_that("int_convert_format works with explicit from parameter", {
  data(mushroom.int)
  result <- suppressMessages(int_convert_format(mushroom.int, to = "MM", from = "RSDA"))
  expect_s3_class(result, "data.frame")
  expect_true(any(grepl("_min$", names(result))))
})

test_that("int_convert_format returns original when from == to", {
  data(mushroom.int)
  result <- suppressMessages(int_convert_format(mushroom.int, to = "RSDA", from = "RSDA"))
  expect_identical(result, mushroom.int)
})

test_that("int_convert_format errors on invalid target format", {
  data(mushroom.int)
  expect_error(int_convert_format(mushroom.int, to = "INVALID"), "must be one of")
})

test_that("int_convert_format errors on unknown source format", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_error(suppressMessages(int_convert_format(df, to = "MM")),
               "Could not detect source format")
})

test_that("int_convert_format converts MM to RSDA with auto-detect", {
  data(mushroom.int)
  mm <- suppressMessages(int_convert_format(mushroom.int, to = "MM"))
  result <- suppressMessages(int_convert_format(mm, to = "RSDA"))
  expect_s3_class(result, "symbolic_tbl")
  expect_equal(nrow(result), nrow(mushroom.int))
})

test_that("int_convert_format prints detection message", {
  data(mushroom.int)
  expect_message(int_convert_format(mushroom.int, to = "MM"),
                 "Detected source format")
})

test_that("int_convert_format same-format message", {
  data(mushroom.int)
  expect_message(int_convert_format(mushroom.int, to = "RSDA", from = "RSDA"),
                 "same")
})

test_that("int_convert_format handles iGAP case in to parameter", {
  data(mushroom.int)
  # Both "iGAP" and "IGAP" should work
  r1 <- suppressMessages(int_convert_format(mushroom.int, to = "iGAP"))
  r2 <- suppressMessages(int_convert_format(mushroom.int, to = "IGAP"))
  expect_equal(nrow(r1), nrow(r2))
  expect_equal(ncol(r1), ncol(r2))
})

test_that("int_convert_format round-trip RSDA -> MM -> iGAP preserves rows", {
  data(mushroom.int)
  mm <- suppressMessages(int_convert_format(mushroom.int, to = "MM"))
  igap <- suppressMessages(int_convert_format(mm, to = "iGAP"))
  expect_equal(nrow(igap), nrow(mushroom.int))
})

test_that("int_convert_format with iGAP auto-detects interval columns", {
  data(face.iGAP)
  result <- suppressMessages(int_convert_format(face.iGAP, to = "MM"))
  expect_s3_class(result, "data.frame")
  # Each interval column splits into min/max, so more columns than original
  expect_true(ncol(result) > ncol(face.iGAP))
})

test_that("int_convert_format with explicit from=iGAP works", {
  data(abalone.iGAP)
  result <- suppressMessages(int_convert_format(abalone.iGAP, to = "MM", from = "iGAP"))
  expect_s3_class(result, "data.frame")
  expect_true(any(grepl("_min$", names(result))))
})

# ---------- MM_to_RSDA -------------------------------------------------------

test_that("MM_to_RSDA returns symbolic_tbl with correct class", {
  data(mushroom.int)
  mm <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  result <- MM_to_RSDA(mm)
  expect_s3_class(result, "symbolic_tbl")
  expect_true(inherits(result, "data.frame"))
})

test_that("MM_to_RSDA complex columns encode min/max correctly", {
  data(mushroom.int)
  mm <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  result <- MM_to_RSDA(mm)
  # Find a complex column
  complex_cols <- which(sapply(result, mode) == "complex")
  expect_true(length(complex_cols) > 0)
  # Verify Re() gives min and Im() gives max
  col <- complex_cols[1]
  col_name <- names(result)[col]
  min_col <- paste0(col_name, "_min")
  max_col <- paste0(col_name, "_max")
  expect_equal(Re(result[[col]]), as.numeric(mm[[min_col]]))
  expect_equal(Im(result[[col]]), as.numeric(mm[[max_col]]))
})

test_that("MM_to_RSDA preserves non-interval columns", {
  # Create a test data.frame with both interval and non-interval columns
  df <- data.frame(
    name = c("a", "b", "c"),
    x_min = c(1, 2, 3),
    x_max = c(4, 5, 6),
    stringsAsFactors = FALSE
  )
  result <- MM_to_RSDA(df)
  expect_true("name" %in% names(result))
  expect_equal(result$name, c("a", "b", "c"))
})

test_that("MM_to_RSDA preserves row count", {
  data(mushroom.int)
  mm <- suppressWarnings(RSDA_to_MM(mushroom.int, RSDA = FALSE))
  result <- MM_to_RSDA(mm)
  expect_equal(nrow(result), nrow(mm))
})

test_that("MM_to_RSDA round-trip preserves data", {
  # Create a known MM data.frame and verify round-trip
  mm <- data.frame(
    name = c("a", "b", "c"),
    x_min = c(1.0, 2.0, 3.0),
    x_max = c(4.0, 5.0, 6.0),
    y_min = c(10.0, 20.0, 30.0),
    y_max = c(40.0, 50.0, 60.0),
    stringsAsFactors = FALSE
  )
  rsda <- MM_to_RSDA(mm)
  mm_back <- suppressWarnings(RSDA_to_MM(rsda, RSDA = FALSE))
  expect_equal(as.numeric(mm_back$x_min), mm$x_min)
  expect_equal(as.numeric(mm_back$x_max), mm$x_max)
  expect_equal(as.numeric(mm_back$y_min), mm$y_min)
  expect_equal(as.numeric(mm_back$y_max), mm$y_max)
  expect_equal(mm_back$name, mm$name)
})

test_that("MM_to_RSDA warns on no _min/_max columns", {
  df <- data.frame(a = 1:3, b = 4:6)
  expect_warning(MM_to_RSDA(df), "no _min/_max columns")
})

# ---------- iGAP_to_RSDA -----------------------------------------------------

test_that("iGAP_to_RSDA returns symbolic_tbl", {
  data(abalone.iGAP)
  result <- iGAP_to_RSDA(abalone.iGAP, 1:7)
  expect_s3_class(result, "symbolic_tbl")
  expect_equal(nrow(result), nrow(abalone.iGAP))
})

test_that("iGAP_to_RSDA has complex columns for interval data", {
  data(abalone.iGAP)
  result <- iGAP_to_RSDA(abalone.iGAP, 1:7)
  complex_cols <- which(sapply(result, mode) == "complex")
  expect_equal(length(complex_cols), 7)
})

test_that("iGAP_to_RSDA round-trip preserves structure", {
  data(abalone.iGAP)
  rsda <- iGAP_to_RSDA(abalone.iGAP, 1:7)
  # Verify structure: 7 complex columns for the 7 interval variables
  complex_cols <- which(sapply(rsda, mode) == "complex")
  expect_equal(length(complex_cols), 7)
  expect_equal(nrow(rsda), nrow(abalone.iGAP))
  # Verify first interval column's min/max match original parsed values
  orig_vals <- strsplit(as.character(abalone.iGAP[[1]]), ",")
  expect_equal(Re(rsda[[1]])[1], as.numeric(orig_vals[[1]][1]))
  expect_equal(Im(rsda[[1]])[1], as.numeric(orig_vals[[1]][2]))
})

test_that("int_convert_format converts iGAP to RSDA with auto-detect", {
  data(abalone.iGAP)
  result <- suppressMessages(int_convert_format(abalone.iGAP, to = "RSDA"))
  expect_s3_class(result, "symbolic_tbl")
  expect_equal(nrow(result), nrow(abalone.iGAP))
})

test_that("int_list_conversions includes MM_to_RSDA and iGAP_to_RSDA", {
  result <- int_list_conversions()
  expect_true("MM_to_RSDA" %in% result$function_name)
  expect_true("iGAP_to_RSDA" %in% result$function_name)
})

test_that("int_list_conversions function_name column includes new functions", {
  result <- int_list_conversions()
  for (fn in result$function_name) {
    expect_true(is.function(get(fn)),
                info = paste(fn, "should be an exported function"))
  }
})

test_that("MM_to_iGAP basic conversion works", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  expect_s3_class(result, "data.frame")
})

test_that("MM_to_iGAP output contains comma-separated values", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  first_col <- result[[1]]
  expect_true(any(grepl(",", first_col)))
})

test_that("MM_to_iGAP round-trip preserves column count", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  expect_equal(ncol(result), ncol(face.iGAP))
})

test_that("MM_to_iGAP round-trip preserves row count", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  expect_equal(nrow(result), nrow(face.iGAP))
})

test_that("MM_to_iGAP round-trip column names match", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  expect_equal(names(result), names(face.iGAP))
})

test_that("MM_to_iGAP with Abalone data works", {
  data(abalone.iGAP)
  abalone_mm <- iGAP_to_MM(abalone.iGAP, 1:7)
  result <- MM_to_iGAP(abalone_mm)
  expect_s3_class(result, "data.frame")
  expect_equal(ncol(result), ncol(abalone.iGAP))
})

test_that("MM_to_iGAP output columns are character type", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  interval_cols <- names(face.iGAP)[1:6]
  for (col in interval_cols) {
    expect_true(is.character(result[[col]]),
                info = paste(col, "should be character"))
  }
})

test_that("MM_to_iGAP round-trip values match original", {
  data(face.iGAP)
  face_mm <- iGAP_to_MM(face.iGAP, 1:6)
  result <- MM_to_iGAP(face_mm)
  for (col in names(face.iGAP)) {
    orig_trimmed <- trimws(as.character(face.iGAP[[col]]))
    result_trimmed <- trimws(as.character(result[[col]]))
    expect_equal(result_trimmed, orig_trimmed,
                 info = paste("Mismatch in column", col))
  }
})

#' @name histogram_stats
#' @title Statistics for Histogram Data
#' @description Functions to compute the mean, variance, covariance, and correlation of histogram-valued data.
#' @param x histogram-valued data object.
#' @param var_name the variable name or the column location.
#' @param var_name1 the variable name or the column location.
#' @param var_name2 the variable name or the column location.
#' @param method method to calculate statistics. One of \code{"BG"} (Bertrand and Goupil, 2000; default),
#'   \code{"BD"} (Billard and Diday, 2006), \code{"B"} (Billard, 2008), or \code{"L2W"} (L2 Wasserstein).
#'   All four methods are available for all four functions.
#' @param ... additional parameters.
#' @return A numeric value or vector for \code{hist_mean} and \code{hist_var}; a single numeric value for \code{hist_cov} and \code{hist_cor}.
#' @details
#' Four functions are provided:
#' \itemize{
#'   \item \code{hist_mean}: Compute the mean of histogram-valued data.
#'   \item \code{hist_var}: Compute the variance of histogram-valued data.
#'   \item \code{hist_cov}: Compute the covariance between two histogram-valued variables.
#'   \item \code{hist_cor}: Compute the correlation between two histogram-valued variables.
#' }
#'
#' Four methods are supported for all functions:
#' \describe{
#'   \item{BG}{Bertrand and Goupil (2000) method. Uses histogram bin boundaries
#'     and probabilities to compute first and second moments.}
#'   \item{BD}{Billard and Diday (2006) method. A signed decomposition using
#'     the sign of each bin's midpoint deviation from the overall mean and a
#'     quadratic form on the bin boundaries.}
#'   \item{B}{Billard (2008) method. Uses cross-products of deviations of the
#'     bin boundaries from the overall mean.}
#'   \item{L2W}{L2 Wasserstein method. Uses optimal-transport (Wasserstein)
#'     distances between the quantile functions of the histogram distributions.}
#' }
#'
#' For the mean, BG, BD, and B return the same value because they share the
#' same first-order moment definition; only L2W uses a different (quantile-based)
#' mean.  For variance, covariance, and correlation, all four methods generally
#' produce different results.
#'
#' For \code{hist_cor}, the BG, BD, and B correlations all use the
#' Bertrand-Goupil standard deviation \eqn{S(Y)} in the denominator, following
#' Irpino and Verde (2015, Eqs. 30--32). Only the L2W method uses its own
#' Wasserstein-based standard deviation in the denominator.
#' @author Po-Wei Chen, Han-Ming Wu
#' @seealso int_mean int_var int_cov int_cor
#' @examples
#' library(HistDAWass)
#' x <- HistDAWass::BLOOD
#' hist_mean(x, var_name = "Cholesterol", method = "BG")
#' hist_mean(x, var_name = "Cholesterol", method = "BD")
#' hist_var(x, var_name = "Cholesterol", method = "BG")
#' hist_var(x, var_name = "Cholesterol", method = "BD")
#' hist_cov(x, var_name1 = "Cholesterol", var_name2 = "Hemoglobin", method = "BG")
#' hist_cor(x, var_name1 = "Cholesterol", var_name2 = "Hemoglobin", method = "BG")
#' @import HistDAWass
#' @export
hist_mean <- function(x, var_name, method = "BG", ...){
  .check_MatH(x, "hist_mean")
  .check_hist_var_name(var_name, x, "hist_mean")
  .check_hist_method(method, c("BG", "BD", "B", "L2W"), "hist_mean")
  object <- x
  if (method == "BG" || method == "BD" || method == "B") {
    ans <- hist_mean_BG(object, var_name)
  }
  if (method == "L2W") {
    ans <- hist_mean_w(object, var_name)
  }
  ans
}


hist_mean_BG <- function(object, var){
  location_var <- which(colnames(object@M) == var)
  nr <- nrow(object@M)
  nc <- ncol(object@M)
  Sample_mean <- c()
  for (i in 1:nr){
    for (j in 1:nc){
      p1 <- object@M[i, j][[1]]@p[2:length(object@M[i, j][[1]]@p)]
      p2 <- object@M[i, j][[1]]@p[1:length(object@M[i, j][[1]]@p) - 1]
      p <- p1 - p2
      m2 <- c()
      for (k in 1:length(p)){
        m1 <- (object@M[i, j][[1]]@x[k] + object@M[i, j][[1]]@x[k + 1])*p[k]
        m2 <- c(m2, m1)
      }
      m3 <- sum(m2)
      Sample_mean <- c(Sample_mean, m3)
    }
  }
  Mean <- matrix(Sample_mean, nrow = nr, byrow = TRUE,
                 dimnames = list(rownames(object@M), colnames(object@M)))
  Sample.means <- apply(Mean, 2, sum)/(2 * nr)
  mean.df <- t(data.frame(Sample.means))
  row.names(mean.df) <- 'mean'
  colnames(mean.df) <- colnames(object@M)
  result <- mean.df[location_var]
  return(result)
}

hist_mean_w <- function(object, var){
  location_var <- which(colnames(object@M) == var)
  Mean <- apply(.MatH_mean(object), 2, mean)
  mean.df <- t(data.frame(Mean))
  row.names(mean.df) <- 'mean'
  colnames(mean.df) <- colnames(object@M)
  result <- mean.df[location_var]
  return(result)
}

#' @rdname histogram_stats
#' @export
hist_var <- function(x, var_name, method = "BG", ...){
  .check_MatH(x, "hist_var")
  .check_hist_var_name(var_name, x, "hist_var")
  .check_hist_method(method, c("BG", "BD", "B", "L2W"), "hist_var")
  object <- x
  if (method == "BG") {
    ans <- hist_var_BG(object, var_name)
  }
  if (method == "BD") {
    ans <- hist_var_BD(object, var_name)
  }
  if (method == "B") {
    ans <- hist_var_B(object, var_name)
  }
  if (method == "L2W") {
    ans <- hist_var_w(object, var_name)
  }
  ans
}

hist_var_BG <- function(object, var){
  location_var <- which(colnames(object@M) == var)
  nr <- nrow(object@M)
  nc <- ncol(object@M)
  b1 <- c()
  b3 <- c()

  for (i in 1:nr){
    for (j in 1:nc){
      p1 <- object@M[i, j][[1]]@p[2:length(object@M[i, j][[1]]@p)]
      p2 <- object@M[i, j][[1]]@p[1:length(object@M[i, j][[1]]@p) - 1]
      p <- p1 - p2
      a <- c()
      a1 <- c()
      for (k in 1:length(p)){
        s <- ((object@M[i, j][[1]]@x[k])^2 + (object@M[i, j][[1]]@x[k + 1])^2 + (object@M[i, j][[1]]@x[k])*(object@M[i, j][[1]]@x[k + 1]))*p[k]
        s1 <- (object@M[i, j][[1]]@x[k] + object@M[i, j][[1]]@x[k + 1])*p[k]
        a <- c(a, s)
        a1 <- c(a1, s1)
      }
      b <- sum(a)
      b1 <- c(b1, b)
      b2 <- sum(a1)
      b3 <- c(b3, b2)
    }
  }

  B <- matrix(b1, nrow = nrow(object@M), byrow = TRUE,
              dimnames = list(rownames(object@M), colnames(object@M)))
  C <- matrix(b3, nrow = nrow(object@M), byrow = TRUE,
              dimnames = list(rownames(object@M), colnames(object@M)))

  squarefun <- function(x){
    x <- sum(x)^2
  }

  S2 <- apply(B, 2, sum)/(3 * nr) - apply(C, 2, squarefun)/(4 * nr^2)
  var.df <- t(data.frame(S2))
  row.names(var.df) <- 'variance'
  colnames(var.df) <- colnames(object@M)
  result <- var.df[location_var]
  return(result)
}


# BD variance: Cov_BD(X, X) — the Billard-Diday covariance of a variable with itself.
hist_var_BD <- function(object, var) {
  location_var <- which(colnames(object@M) == var)
  nr <- nrow(object@M)
  ss <- 0
  for (i in 1:nr) {
    gq <- .hist_get_GQ(object, i, location_var, location_var, var, var)
    pv <- .hist_get_pvars(object, i, location_var, location_var)
    ss <- ss + sum(outer(gq$G1, gq$G2) *
                     outer(pv$pvar1, pv$pvar2) *
                     outer(gq$Q1, gq$Q2)^0.5)
  }
  return(ss / (3 * nr))
}


# B variance: Cov_B(X, X) — the Bertrand covariance of a variable with itself.
hist_var_B <- function(object, var) {
  location_var <- which(colnames(object@M) == var)
  nr <- nrow(object@M)
  ss <- 0
  for (i in 1:nr) {
    qq <- .hist_get_QQ_vals(object, i, location_var, location_var, var, var)
    pv <- .hist_get_pvars(object, i, location_var, location_var)
    ss <- ss + sum((2 * qq$Q1 + qq$Q2 + qq$Q3 + 2 * qq$Q4) *
                     outer(pv$pvar1, pv$pvar2))
  }
  return(ss / (6 * nr))
}


SM2.Wass <- function(object, var){
  location_var <- which(colnames(object@M) == var)
  mean_mat <- .MatH_mean(object)

  Mean.MW <- apply(mean_mat, 2, mean)

  myfun <- function(x){
    sum(x^2)
  }

  SM2.W <- apply(mean_mat, 2, myfun)/nrow(mean_mat) - Mean.MW ^ 2
  SM2.W.df <- t(data.frame(SM2.W))
  colnames(SM2.W.df) <- colnames(object@M)
  rownames(SM2.W.df) <- 'variance'
  result <- SM2.W.df[location_var]
  return(result)
}


SV2.Wass <- function(object, var){
  location_var <- which(colnames(object@M) == var)
  nr <- nrow(object@M)
  nc <- ncol(object@M)
  sd_mat <- .MatH_sd(object)

  myfun <- function(x){
    sum(x^2)
  }

  H <- apply(sd_mat, 2, myfun)/nr
  R_list <- list()

  for(k in 1:nc){
    Correlaiton_table <- matrix(0, nr, nr)
    Sigma_table <- matrix(0, nr, nr)
    R_table <- matrix(0, nr, nr)
    for(i in 1:nr){
      for(j in 1:nr){
        Correlaiton_table[i, j] <- rQQ(object@M[i, k][[1]], object@M[j, k][[1]])
        Sigma_table[i, j] <- sd_mat[i, k] * sd_mat[j, k]
      }
    }
    R_table <- Correlaiton_table * Sigma_table
    dimnames(R_table) <- list(rownames(object@M), rownames(object@M))
    assign(paste("R_table", k, sep = ""), R_table)
    R_list[[k]] <- R_table
  }

  names(R_list) <- paste0("R_table", 1:nc)
  sums <- sapply(R_list, sum)
  SV2 <- t(data.frame(H - sums/(nr^2)))
  colnames(SV2) <- colnames(object@M)
  rownames(SV2) <- 'variance'
  result <- SV2[location_var]
  return(result)
}


hist_var_w <- function(object, var){
  S2.W <- SM2.Wass(object, var) + SV2.Wass(object, var)
  return(S2.W)
}


#' @rdname histogram_stats
#' @export
hist_cov <- function(x, var_name1, var_name2, method = "BG", ...){
  .check_MatH(x, "hist_cov")
  .check_hist_var_name(var_name1, x, "hist_cov")
  .check_hist_var_name(var_name2, x, "hist_cov")
  .check_hist_method(method, c("BG", "BD", "B", "L2W"), "hist_cov")
  object <- x
  var1 <- var_name1
  var2 <- var_name2
  location_var1 <- which(colnames(object@M) == var1)
  location_var2 <- which(colnames(object@M) == var2)
  nr <- nrow(object@M)
  nc <- ncol(object@M)
  mean_mat <- .MatH_mean(object)
  sd_mat <- .MatH_sd(object)

  if (method == 'BG'){
    result <- sum(mean_mat[, location_var1] *
                    mean_mat[, location_var2])/nrow(object@M) -
      hist_mean_BG(object, var1) * hist_mean_BG(object, var2)
    return(result)
  } else if (method == 'BD'){
    ss <- 0
    for (i in 1:nr){
      gq <- .hist_get_GQ(object, i, location_var1, location_var2, var1, var2)
      pv <- .hist_get_pvars(object, i, location_var1, location_var2)
      ss <- ss + sum(outer(gq$G1, gq$G2) *
                       outer(pv$pvar1, pv$pvar2) *
                       outer(gq$Q1, gq$Q2)^0.5)
    }
    return(ss / (3 * nrow(object@M)))
  } else if (method == 'B'){
    ss <- 0
    for (i in 1:nr){
      qq <- .hist_get_QQ_vals(object, i, location_var1, location_var2, var1, var2)
      pv <- .hist_get_pvars(object, i, location_var1, location_var2)
      ss <- ss + sum((2 * qq$Q1 + qq$Q2 + qq$Q3 + 2 * qq$Q4) *
                       outer(pv$pvar1, pv$pvar2))
    }
    return(ss / (6 * nrow(object@M)))
  } else if (method == 'L2W'){
    CM <- sum(mean_mat[, location_var1] *
                mean_mat[, location_var2])/nrow(object@M) -
      hist_mean_w(object, var1) * hist_mean_w(object, var2)

    s1 <- 0
    s2 <- 0
    for (i in 1:nr){
      s1 <- s1 + sum(rQQ(object@M[i, location_var1][[1]], object@M[i, location_var2][[1]]) *
                       sd_mat[i, location_var1] *
                       sd_mat[i, location_var2]) / nrow(object@M)
    }
    for (i in 1:nr){
      for (j in 1:nr){
        s2 <- s2 + sum(rQQ(object@M[i, location_var1][[1]], object@M[j, location_var2][[1]]) *
                         sd_mat[i, location_var1] *
                         sd_mat[j, location_var2]) / nrow(object@M)^2
      }
    }
    CV <- s1 - s2
    result <- CM + CV
    return(result)
  }
}


#' @rdname histogram_stats
#' @export
hist_cor <- function(x, var_name1, var_name2, method = "BG", ...){
  .check_MatH(x, "hist_cor")
  .check_hist_var_name(var_name1, x, "hist_cor")
  .check_hist_var_name(var_name2, x, "hist_cor")
  .check_hist_method(method, c("BG", "BD", "B", "L2W"), "hist_cor")
  object <- x
  var1 <- var_name1
  var2 <- var_name2

  # Per Irpino & Verde (2015) Eqs. 30-32:
  # R_BG, R_BD, and R_B all use the BG standard deviation S(Y) in the denominator.
  # Only L2W uses the Wasserstein variance.
  if (method == "L2W") {
    denom <- sqrt(hist_var_w(object, var1)) * sqrt(hist_var_w(object, var2))
  } else {
    denom <- sqrt(hist_var_BG(object, var1)) * sqrt(hist_var_BG(object, var2))
  }

  result <- hist_cov(object, var1, var2, method = method) / denom
  return(result)
}

test_that("hist_mean BG method works", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  result <- hist_mean(BLOOD, var_name = "Cholesterol", method = "BG")
  expect_true(is.numeric(result))
  expect_true(is.finite(result))
})

test_that("hist_mean L2W method works", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  result <- hist_mean(BLOOD, var_name = "Cholesterol", method = "L2W")
  expect_true(is.numeric(result))
  expect_true(is.finite(result))
})

test_that("hist_var BG method returns non-negative value", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  result <- hist_var(BLOOD, var_name = "Cholesterol", method = "BG")
  expect_true(is.numeric(result))
  expect_true(result >= 0)
})

test_that("hist_var L2W method returns non-negative value", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  result <- hist_var(BLOOD, var_name = "Cholesterol", method = "L2W")
  expect_true(is.numeric(result))
  expect_true(result >= 0)
})

test_that("hist_cov BG method works", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cov(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "BG")
  expect_true(is.numeric(result))
})

test_that("hist_cov BD method works", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cov(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "BD")
  expect_true(is.numeric(result))
})

test_that("hist_cov B method works", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cov(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "B")
  expect_true(is.numeric(result))
})

test_that("hist_cov L2W method works", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cov(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "L2W")
  expect_true(is.numeric(result))
})

test_that("hist_cor BG method returns value in [-1, 1]", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cor(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "BG")
  expect_true(is.numeric(result))
  expect_true(result >= -1 & result <= 1)
})

test_that("hist_cor L2W method returns value in [-1, 1]", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cor(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "L2W")
  expect_true(is.numeric(result))
  expect_true(result >= -1 & result <= 1)
})

test_that("hist_mean BG and L2W both produce results", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  bg <- hist_mean(BLOOD, var_name = "Cholesterol", method = "BG")
  l2w <- hist_mean(BLOOD, var_name = "Cholesterol", method = "L2W")
  expect_true(is.numeric(bg))
  expect_true(is.numeric(l2w))
})

test_that("hist_var for different variables produces results", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  v1 <- hist_var(BLOOD, var_name = var_names[1], method = "BG")
  v2 <- hist_var(BLOOD, var_name = var_names[2], method = "BG")
  expect_true(is.numeric(v1))
  expect_true(is.numeric(v2))
})

test_that("hist_cor BD method returns value in [-1, 1]", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cor(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "BD")
  expect_true(is.numeric(result))
  expect_true(result >= -1.01 & result <= 1.01)
})

test_that("hist_cor B method returns value in [-1, 1]", {
  skip_if_not_installed("HistDAWass")
  skip_on_cran()
  requireNamespace("HistDAWass", quietly = TRUE)
  BLOOD <- HistDAWass::BLOOD
  var_names <- colnames(BLOOD@M)
  skip_if(length(var_names) < 2, "BLOOD needs at least 2 variables")
  result <- hist_cor(BLOOD, var_name1 = var_names[1],
                     var_name2 = var_names[2], method = "B")
  expect_true(is.numeric(result))
  expect_true(result >= -1.01 & result <= 1.01)
})

## ============================================================================
## dataSDA: Datasets for Symbolic Data Analysis
## data.R - Dataset documentation file
## Naming: snake_case + suffix convention:
##   .int   = purely interval-valued
##   .hist  = purely histogram-valued
##   .mix   = mixed symbolic types
##   .distr = distribution-valued
##   .iGAP  = iGAP format
##   .its   = interval time series
## ============================================================================

## ---------------------------------------------------------------------------
## SECTION 1: Interval-valued datasets from external R packages
## ---------------------------------------------------------------------------

#' @name lackinfo.int
#' @title Lack of Information Questionnaire Interval Dataset
#' @description
#' Interval-valued dataset from a lack-of-information questionnaire.
#' Contains biographical data and responses to 5 items measuring perception
#' of lack of information, collected via an interval-valued Likert scale.
#'
#' @details
#' An educational innovation project was carried out for improving
#' teaching-learning processes at the University of Oviedo (Spain)
#' for the 2020/2021 academic year. A total of 50 students answered
#' an online questionnaire about biographical data (sex and age) and
#' their perception of lack of information by selecting the interval
#' that best represents their level of agreement on a scale bounded
#' between 1 (strongly disagree) and 7 (strongly agree).
#'
#' The 5 items measuring perception of lack of information are:
#' \itemize{
#'     \item I1: I receive too little information from my classmates.
#'     \item I2: It is difficult to receive relevant information from my classmates.
#'     \item I3: It is difficult to receive relevant information from the teacher.
#'     \item I4: The amount of information I receive from my classmates is very low.
#'     \item I5: The amount of information I receive from the teacher is very low.
#' }
#'
#' @usage data(lackinfo.int)
#' @format A data frame with 50 observations and 8 variables:
#' \itemize{
#'     \item \code{id}: Identification number.
#'     \item \code{sex}: Sex of the respondent (\code{male} or \code{female}).
#'     \item \code{age}: Respondent's age (in years).
#'     \item \code{item1}: Interval-valued answer to item 1.
#'     \item \code{item2}: Interval-valued answer to item 2.
#'     \item \code{item3}: Interval-valued answer to item 3.
#'     \item \code{item4}: Interval-valued answer to item 4.
#'     \item \code{item5}: Interval-valued answer to item 5.
#' }
#' @examples
#' data(lackinfo.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 50 \cr
#'   \strong{Variables (p)} \tab 8 \cr
#'   \strong{Subject area} \tab Education \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Regression
#' }
#' @keywords datasets interval
#' @source \url{https://CRAN.R-project.org/package=IntervalQuestionStat}
"lackinfo.int"

#' @name ohtemp.int
#' @title Ohio River Basin 30-Year Trimmed Mean Daily Temperatures Interval Dataset
#' @description
#' Interval-valued dataset of 30-year trimmed mean daily temperatures for the
#' Ohio river basin. Intervals are defined by the mean daily maximum and minimum
#' temperatures from January 1, 1988 to December 31, 2018.
#'
#' @usage data(ohtemp.int)
#' @format A data frame with 161 rows and 7 variables:
#' \itemize{
#'     \item \code{ID}: Global Historical Climatological Network (GHCN) station identifier.
#'     \item \code{NAME}: GHCN station name.
#'     \item \code{STATE}: Two-digit state designation.
#'     \item \code{LATITUDE}: Latitude coordinate position.
#'     \item \code{LONGITUDE}: Longitude coordinate position.
#'     \item \code{ELEVATION}: Elevation of the measurement location (meters).
#'     \item \code{TEMPERATURE}: 30-year mean daily temperature (tenths of degrees Celsius).
#' }
#' @examples
#' data(ohtemp.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 161 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression, Spatial analysis
#' }
#' @keywords datasets interval
#' @source \url{https://CRAN.R-project.org/package=intkrige}
"ohtemp.int"

#' @name soccer_bivar.int
#' @title French Soccer Championship Bivariate Interval Dataset
#' @description
#' Interval-valued data for 20 teams from the French premier soccer championship.
#' Contains ranges of Weight (response), Height and Age (explanatory variables).
#'
#' @format A data frame with 20 rows and 3 interval-valued variables:
#' \itemize{
#'     \item \code{y}: Weight (response variable, kg).
#'     \item \code{t1}: Height (explanatory variable, cm).
#'     \item \code{t2}: Age (explanatory variable, years).
#' }
#'
#' @usage data(soccer_bivar.int)
#' @references
#' Lima Neto, E. A., Cordeiro, G. and De Carvalho, F.A.T. (2011).
#' Bivariate symbolic regression models for interval-valued variables.
#' \emph{Journal of Statistical Computation and Simulation}, 81, 1727-1744.
#' @examples
#' data(soccer_bivar.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 20 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Sports \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression
#' }
#' @keywords datasets interval regression
#' @source \url{https://CRAN.R-project.org/package=iRegression}
"soccer_bivar.int"

#' @name cars.int
#' @title Cars Interval Dataset
#' @description
#' Interval-valued data for 27 car models classified into four classes
#' (Utilitarian, Berlina, Sportive, Luxury), described by Price,
#' EngineCapacity, TopSpeed and Acceleration intervals.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 27 observations and 5 variables:
#' \itemize{
#'     \item \code{Price}: Price range (interval).
#'     \item \code{EngCap}: Engine capacity range (cc, interval).
#'     \item \code{TopSpeed}: Top speed range (km/h, interval).
#'     \item \code{Acceleration}: Acceleration range (seconds 0--100 km/h, interval).
#'     \item \code{class}: Car class (Utilitarian, Berlina, Sportive, Luxury; set-valued).
#' }
#'
#' @usage data(cars.int)
#' @references
#' Duarte Silva, A.P., Brito, P., Filzmoser, P. and Dias, J.G. (2021).
#' MAINT.Data: Modelling and Analysing Interval Data in R.
#' \emph{R Journal}, 13(2).
#' @examples
#' data(cars.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 27 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Automotive \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Classification
#' }
#' @keywords datasets interval classification
#' @source \url{https://CRAN.R-project.org/package=MAINT.Data}
"cars.int"

#' @name china_temp.int
#' @title China Meteorological Stations Quarterly Temperature Interval Dataset
#' @description
#' Interval-valued temperature data (Celsius) for 60 Chinese meteorological
#' stations observed over the four quarters of years 1974 to 1988.
#' One outlier observation (YinChuan_1982) has been discarded.
#'
#' @details
#' Originates from the Long-Term Instrumental Climatic Database of the
#' People's Republic of China. Widely used in the SDA literature for
#' demonstrating standardization, clustering, self-organizing maps,
#' MLE and MANOVA.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 899 observations and 5 variables:
#' \itemize{
#'     \item \code{Q1}: Quarter 1 (Jan--Mar) temperature range (tenths of degrees Celsius, interval).
#'     \item \code{Q2}: Quarter 2 (Apr--Jun) temperature range (interval).
#'     \item \code{Q3}: Quarter 3 (Jul--Sep) temperature range (interval).
#'     \item \code{Q4}: Quarter 4 (Oct--Dec) temperature range (interval).
#'     \item \code{GeoReg}: Geographic region classification (factor).
#' }
#'
#' @usage data(china_temp.int)
#' @references
#' Brito, P. and Duarte Silva, A.P. (2012). Modelling interval data with
#' Normal and Skew-Normal distributions. \emph{J. Appl. Stat.}, 39(1), 3-20.
#' @examples
#' data(china_temp.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 899 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
#' @source \url{https://CRAN.R-project.org/package=MAINT.Data}
"china_temp.int"

#' @name loans_by_purpose.int
#' @title Loans by Purpose Interval Dataset
#' @description
#' Interval-valued data for loan characteristics aggregated by their purpose.
#' Original microdata contains 887,383 loan records from Kaggle.
#'
#' @format A data frame with 14 observations and 4 interval-valued variables:
#' \itemize{
#'     \item \code{ln_inc}: Natural logarithm of self-reported annual income.
#'     \item \code{ln_revolbal}: Natural logarithm of total credit revolving balance.
#'     \item \code{open_acc}: Number of open credit lines.
#'     \item \code{total_acc}: Total number of credit lines.
#' }
#'
#' @usage data(loans_by_purpose.int)
#' @examples
#' data(loans_by_purpose.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets interval
#' @source \url{https://CRAN.R-project.org/package=MAINT.Data}
"loans_by_purpose.int"

#' @name nycflights.int
#' @title New York City Flights Interval Dataset
#' @description
#' Interval-valued dataset with 142 units and four interval-valued variables
#' from the nycflights13 package, aggregated by month and carrier.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 142 observations and 5 variables:
#' \itemize{
#'     \item \code{X}: Month-carrier identifier (character).
#'     \item \code{dep_delay}: Departure delay range (minutes, interval).
#'     \item \code{arr_delay}: Arrival delay range (minutes, interval).
#'     \item \code{air_time}: Air time range (minutes, interval).
#'     \item \code{distance}: Distance range (miles, interval).
#' }
#'
#' @usage data(nycflights.int)
#' @references
#' Duarte Silva, A.P., Brito, P., Filzmoser, P. and Dias, J.G. (2021).
#' MAINT.Data: Modelling and Analysing Interval Data in R. \emph{R Journal}, 13(2).
#' @examples
#' data(nycflights.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 142 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Transportation \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression, Descriptive statistics
#' }
#' @keywords datasets interval
#' @source \url{https://CRAN.R-project.org/package=MAINT.Data}
"nycflights.int"

## ---------------------------------------------------------------------------
## SECTION 2: Datasets from Billard and Diday (2006)
## ---------------------------------------------------------------------------

#' @name mushroom.int.mm
#' @title Mushroom Species Dataset (Original Format)
#' @description
#' Interval-valued data for 23 mushroom species of the genus Agaricus
#' with 3 morphological measurements from the Fungi of California Species.
#'
#' @details
#' Classic SDA dataset used for descriptive statistics, histogram
#' construction, and clustering of interval-valued data.
#'
#' @format A data frame with 23 observations and 5 variables:
#' \itemize{
#'     \item \code{Species}: Mushroom species name.
#'     \item \code{Pileus.Cap.Width}: Pileus cap width range (cm).
#'     \item \code{Stipe.Length}: Stipe length range (cm).
#'     \item \code{Stipe.Thickness}: Stipe thickness range (cm).
#'     \item \code{Edibility}: Edibility code (U/Y/N/T).
#' }
#'
#' @usage data(mushroom.int.mm)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.2.
#' @examples
#' data(mushroom.int.mm)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 23 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Biology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets interval
#' @source Billard, L. and Diday, E. (2006), Table 3.2.
"mushroom.int.mm"

#' @name mushroom.int
#' @title Mushroom Species Interval Dataset
#' @description
#' Interval-valued version of the mushroom dataset. See \code{\link{mushroom.int.mm}}.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 23 observations and 5 variables:
#' \itemize{
#'     \item \code{Species}: Mushroom species name (character).
#'     \item \code{Pileus.Cap.Width}: Pileus cap width range (cm, interval).
#'     \item \code{Stipe.Length}: Stipe length range (cm, interval).
#'     \item \code{Stipe.Thickness}: Stipe thickness range (cm, interval).
#'     \item \code{Edibility}: Edibility code (U = Unknown, Y = Yes, N = No, T = Toxic; character).
#' }
#' @usage data(mushroom.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 3.2.
#' @examples
#' data(mushroom.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 23 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Biology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets interval
"mushroom.int"

#' @name age_cholesterol_weight.int
#' @title Age-Cholesterol-Weight Interval Dataset
#' @description
#' Interval-valued dataset of 7 age-group observations with cholesterol and
#' weight measurements. Each observation aggregates individuals in a 10-year
#' age band with interval ranges for cholesterol and weight.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 7 observations and 4 variables:
#' \itemize{
#'     \item \code{Age}: Age range (years, interval).
#'     \item \code{Cholesterol}: Cholesterol level range (mg/dL, interval).
#'     \item \code{Weight}: Weight range (pounds, interval).
#'     \item \code{n}: Number of individuals in the age group (numeric).
#' }
#'
#' @usage data(age_cholesterol_weight.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(age_cholesterol_weight.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 7 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Regression
#' }
#' @keywords datasets interval
"age_cholesterol_weight.int"

#' @name airline_flights.hist
#' @title JFK Airport Airline Flights Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 16 airlines flying into JFK Airport.
#' Six variables (Flight Time, Taxi In, Arrival Delay, Taxi Out,
#' Departure Delay, Weather Delay) recorded as frequency distributions.
#' This is the wide (flat table) format; see \code{\link{airline_flights2.modal}}
#' for the modal-valued version.
#'
#' @format A data frame with 16 observations (Airline1--Airline16) and
#' 17 numeric columns representing 6 histogram variables in wide format:
#' \itemize{
#'     \item \code{Flight Time(<120)}, \code{Flight Time([120, 220])},
#'           \code{Flight Time(>220)}: Flight time distribution (3 bins).
#'     \item \code{Taxi In(<4)}, \code{Taxi In([4, 10])},
#'           \code{Taxi In(>10)}: Taxi-in time distribution (3 bins).
#'     \item \code{Arrival Delay(<0)}, \code{Arrival Delay([0, 60])},
#'           \code{Arrival Delay(>60)}: Arrival delay distribution (3 bins).
#'     \item \code{Taxi Out(<16)}, \code{Taxi Out([16, 30])},
#'           \code{Taxi Out(>30)}: Taxi-out time distribution (3 bins).
#'     \item \code{Departure Delay(<0)}, \code{Departure Delay([0, 60])},
#'           \code{Departure Delay(>60)}: Departure delay distribution (3 bins).
#'     \item \code{Weather Delay(No)}, \code{Weather Delay(Yes)}:
#'           Weather delay distribution (2 bins).
#' }
#'
#' @usage data(airline_flights.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.7.
#' @examples
#' data(airline_flights.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 16 \cr
#'   \strong{Variables (p)} \tab 17 \cr
#'   \strong{Subject area} \tab Transportation \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets histogram
"airline_flights.hist"

#' @name airline_flights2.modal
#' @title JFK Airport Airline Flights Modal-Valued Dataset
#' @description
#' Modal-valued version of the airline flights dataset.
#' See \code{\link{airline_flights.hist}} for the wide-format version.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 16 observations and
#' 6 modal-valued variables:
#' \itemize{
#'     \item \code{FlightTime}: Modal distribution over flight time bins.
#'     \item \code{TaxiIn}: Modal distribution over taxi-in time bins.
#'     \item \code{ArrivalDelay}: Modal distribution over arrival delay bins.
#'     \item \code{TaxiOut}: Modal distribution over taxi-out time bins.
#'     \item \code{DepartureDelay}: Modal distribution over departure delay bins.
#'     \item \code{WeatherDelay}: Modal distribution over weather delay bins.
#' }
#'
#' @usage data(airline_flights2.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.7.
#' @examples
#' data(airline_flights2.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 16 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Transportation \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets modal
"airline_flights2.modal"

#' @name baseball.int
#' @title Baseball Teams Interval Dataset
#' @description
#' Interval-valued data for 19 baseball teams with aggregated player
#' batting statistics and a pattern variable classifying team performance.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 19 observations and 3 variables:
#' \itemize{
#'     \item \code{At_Bats}: Range of at-bats across players (interval).
#'     \item \code{Hits}: Range of hits across players (interval).
#'     \item \code{Pattern}: Team performance pattern code (character).
#' }
#'
#' @usage data(baseball.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(baseball.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 19 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Sports \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets interval
"baseball.int"

#' @name bird.mix
#' @title Bird Species Mixed Symbolic Dataset
#' @description
#' Interval-valued morphological measurements for 20 bird specimens.
#' Despite the \code{.mix} suffix, this dataset contains only
#' interval-valued variables (density and size).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 20 observations and 2 variables:
#' \itemize{
#'     \item \code{Density}: Feather density range (interval).
#'     \item \code{Size}: Body size range (cm, interval).
#' }
#'
#' @usage data(bird.mix)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.5.
#' @examples
#' data(bird.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 20 \cr
#'   \strong{Variables (p)} \tab 2 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets interval
"bird.mix"

#' @name blood_pressure.int
#' @title Blood Pressure Interval Dataset
#' @description
#' Interval-valued blood pressure and pulse rate measurements for
#' 15 patient groups.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 15 observations and
#' 3 interval-valued variables:
#' \itemize{
#'     \item \code{Pulse_Rate}: Pulse rate range (beats per minute, interval).
#'     \item \code{Systolic_Pressure}: Systolic blood pressure range (mmHg, interval).
#'     \item \code{Diastolic_Pressure}: Diastolic blood pressure range (mmHg, interval).
#' }
#'
#' @usage data(blood_pressure.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(blood_pressure.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 15 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Regression
#' }
#' @keywords datasets interval
"blood_pressure.int"

#' @name car.int
#' @title Car Models Interval Dataset
#' @description
#' Interval-valued data for 8 car brands with price and performance
#' specifications. Each brand aggregates multiple models into interval ranges.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 8 observations and 5 variables:
#' \itemize{
#'     \item \code{Car}: Car brand name (character).
#'     \item \code{Price}: Price range (thousands of currency units, interval).
#'     \item \code{Max_Velocity}: Maximum velocity range (km/h, interval).
#'     \item \code{Accn_Time}: Acceleration time range (seconds 0--100 km/h, interval).
#'     \item \code{Cylinder_Capacity}: Engine cylinder capacity range (cc, interval).
#' }
#'
#' @usage data(car.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(car.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 8 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Automotive \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets interval
"car.int"

#' @name crime.modal
#' @title Crime Demographics Dataset
#' @description
#' Modal-valued dataset of 15 gangs described by probability distributions
#' over crime type, gender, and age group. This is the wide (flat table)
#' format; see \code{\link{crime2.modal}} for the modal-valued version.
#'
#' @format A data frame with 15 observations (gang1--gang15) and 7 numeric
#' columns representing 3 modal variables in wide format:
#' \itemize{
#'     \item \code{Crime(violent)}, \code{Crime(non-violent)}, \code{Crime(none)}:
#'           Distribution over crime types (3 bins).
#'     \item \code{Gender(male)}, \code{Gender(female)}:
#'           Distribution over gender (2 bins).
#'     \item \code{Age(<20)}, \code{Age(>=20)}:
#'           Distribution over age groups (2 bins).
#' }
#'
#' @usage data(crime.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(crime.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 15 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Criminology \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets modal
"crime.modal"

#' @name crime2.modal
#' @title Crime Demographics Modal-Valued Dataset
#' @description
#' Modal-valued version of the crime demographics dataset.
#' See \code{\link{crime.modal}} for the wide-format version.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 15 observations and
#' 3 modal-valued variables:
#' \itemize{
#'     \item \code{Crime}: Modal distribution over crime types
#'           (violent, non-violent, none).
#'     \item \code{Gender}: Modal distribution over gender (male, female).
#'     \item \code{Age}: Modal distribution over age groups (<20, >=20).
#' }
#'
#' @usage data(crime2.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(crime2.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 15 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Criminology \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets modal
"crime2.modal"

#' @name finance.int
#' @title Finance Sector Interval Dataset
#' @description
#' Interval-valued data for 14 business sectors described by job-related
#' financial variables (job cost codes, activity codes, budgets).
#' Used for PCA demonstrations.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 14 observations and 7 variables:
#' \itemize{
#'     \item \code{Sector}: Business sector name (character).
#'     \item \code{Job_Cost}: Job cost range (currency units, interval).
#'     \item \code{Job_Code}: Job code range (interval).
#'     \item \code{Activity_Code}: Activity code range (interval).
#'     \item \code{Monthly_Cost}: Monthly cost range (currency units, interval).
#'     \item \code{Annual_Budget}: Annual budget range (currency units, interval).
#'     \item \code{n}: Number of entities in the sector (numeric).
#' }
#'
#' @usage data(finance.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 5.2.
#' @examples
#' data(finance.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval PCA
"finance.int"

#' @name fuel_consumption.modal
#' @title Fuel Consumption by Region Dataset
#' @description
#' Modal-valued dataset describing fuel consumption patterns across 10
#' regions by proportions of heating fuel types (gas, oil, electricity,
#' other) and per-capita expenditure.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations and 3 variables:
#' \itemize{
#'     \item \code{Region}: Region identifier (character).
#'     \item \code{Expenditure}: Per-capita fuel expenditure (numeric).
#'     \item \code{Fuel_Type}: Modal distribution over fuel types
#'           (gas, oil, electric, other).
#' }
#'
#' @usage data(fuel_consumption.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 3.7.
#' @examples
#' data(fuel_consumption.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Energy \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Regression
#' }
#' @keywords datasets modal regression
"fuel_consumption.modal"

#' @name health_insurance.mix
#' @title Health Insurance Mixed Symbolic Dataset
#' @description
#' Classical (microdata) health insurance dataset of 51 individual patient
#' records with 30 variables including demographics, clinical measurements,
#' and diagnostic indicators. This is the raw data underlying the
#' symbolic \code{\link{health_insurance2.modal}} dataset.
#'
#' @format A data frame with 51 observations and 30 variables (Y1--Y30):
#' \itemize{
#'     \item \code{Y1}: City (character).
#'     \item \code{Y2}: Gender (M/F, character).
#'     \item \code{Y3}: Age (integer).
#'     \item \code{Y4}: Sex (M/D, character).
#'     \item \code{Y5}: Marital status (S/M, character).
#'     \item \code{Y6}: Number of dependents (integer).
#'     \item \code{Y7}: Parents alive indicator (integer).
#'     \item \code{Y8}: Number of children (integer).
#'     \item \code{Y9}: Height (cm, integer).
#'     \item \code{Y10}: Weight (pounds, integer).
#'     \item \code{Y11}: Systolic blood pressure (mmHg, integer).
#'     \item \code{Y12}: Diastolic blood pressure (mmHg, integer).
#'     \item \code{Y13}: Cholesterol (mg/dL, integer).
#'     \item \code{Y14}: Cholesterol measure 2 (integer).
#'     \item \code{Y15}: Additional lab measurement (integer).
#'     \item \code{Y16}: Ratio measurement (numeric).
#'     \item \code{Y17}: Lab value (integer).
#'     \item \code{Y18}: Lab value (integer).
#'     \item \code{Y19}: Lab value (integer).
#'     \item \code{Y20}: Lab ratio (numeric).
#'     \item \code{Y21}: Additional lab value (integer).
#'     \item \code{Y22}: Additional lab value (integer).
#'     \item \code{Y23}: Blood chemistry value (numeric).
#'     \item \code{Y24}: Blood chemistry value (numeric).
#'     \item \code{Y25}: Blood chemistry value (numeric).
#'     \item \code{Y26}: Blood chemistry value (numeric).
#'     \item \code{Y27}: Blood chemistry value (numeric).
#'     \item \code{Y28}: Diagnostic indicator (Y/N, character).
#'     \item \code{Y29}: Diagnostic indicator (Y/N, character).
#'     \item \code{Y30}: Count variable (integer).
#' }
#'
#' @usage data(health_insurance.mix)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Tables 2.1-2.2.
#' @examples
#' data(health_insurance.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 51 \cr
#'   \strong{Variables (p)} \tab 30 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Classical (microdata) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Aggregation
#' }
#' @keywords datasets mixed symbolic
"health_insurance.mix"

#' @name health_insurance2.modal
#' @title Health Insurance Modal-Valued Dataset
#' @description
#' Modal-valued symbolic version of the health insurance dataset, aggregated
#' into 6 disease-type-by-gender groups. See \code{\link{health_insurance.mix}}
#' for the underlying microdata.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 6 observations and
#' 6 variables:
#' \itemize{
#'     \item \code{Type Gender}: Disease type and gender label (character).
#'     \item \code{Age}: Modal distribution over age bins.
#'     \item \code{Marital Status}: Modal distribution over marital status (M, S).
#'     \item \code{Parents Alive}: Modal distribution over number of parents alive (0, 1, 2).
#'     \item \code{Weight}: Modal distribution over weight bins (pounds).
#'     \item \code{Cholesterol}: Modal distribution over cholesterol bins (mg/dL).
#' }
#'
#' @usage data(health_insurance2.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.2b.
#' @examples
#' data(health_insurance2.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets modal
"health_insurance2.modal"

#' @name hierarchy
#' @title Hierarchy Dataset
#' @description
#' Classical (microdata) dataset of 20 observations illustrating hierarchical
#' categorical structures with a response variable Y and hierarchical
#' predictors X1--X5. See \code{\link{hierarchy.int}} for the interval-valued
#' version.
#'
#' @format A data frame with 20 observations and 6 variables:
#' \itemize{
#'     \item \code{Y}: Response variable (numeric).
#'     \item \code{X1}: Hierarchy level 1 category (a/b/c, character).
#'     \item \code{X2}: Hierarchy level 2 category (a1/a2, character; NA for non-a).
#'     \item \code{X3}: Hierarchy level 3 category (a11/a12, character; NA for non-a1).
#'     \item \code{X4}: Numeric predictor for group b (numeric; NA for non-b).
#'     \item \code{X5}: Numeric predictor for group c (numeric; NA for non-c).
#' }
#'
#' @usage data(hierarchy)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.15.
#' @examples
#' data(hierarchy)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 20 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Classical (microdata) \cr
#'   \strong{Analytical tasks} \tab Aggregation, Descriptive statistics
#' }
#' @keywords datasets hierarchical
"hierarchy"

#' @name hierarchy.int
#' @title Hierarchy Interval Dataset
#' @description
#' Interval-valued version of the hierarchy dataset. See \code{\link{hierarchy}}
#' for the classical version.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 20 observations and 6 variables:
#' \itemize{
#'     \item \code{Y}: Response variable range (interval).
#'     \item \code{X1}: Hierarchy level 1 category (a/b/c, character).
#'     \item \code{X2}: Hierarchy level 2 category (a1/a2, character; NA for non-a).
#'     \item \code{X3}: Hierarchy level 3 category (a11/a12, character; NA for non-a1).
#'     \item \code{X4}: Predictor range for group b (interval; NA for non-b).
#'     \item \code{X5}: Predictor range for group c (interval; NA for non-c).
#' }
#'
#' @usage data(hierarchy.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.15.
#' @examples
#' data(hierarchy.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 20 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Regression
#' }
#' @keywords datasets interval
"hierarchy.int"

#' @name horses.int
#' @title Horse Breeds Interval Dataset
#' @description
#' Interval-valued data for 8 horse breeds (CES, CMA, PEN, TES, CEN,
#' LES, PES, PAM) described by 6 variables: minimum/maximum weight,
#' minimum/maximum height, cost of mares, cost of fillies.
#'
#' @details
#' Extensively used in SDA for demonstrating divisive clustering, distance
#' computation, hierarchy/pyramid construction, and complete objects.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 8 observations and 7 variables:
#' \itemize{
#'     \item \code{Breed}: Horse breed code (CES, CMA, PEN, TES, CEN, LES, PES, PAM; character).
#'     \item \code{Minimum_Weight}: Minimum weight range (kg, interval).
#'     \item \code{Maximum_Weight}: Maximum weight range (kg, interval).
#'     \item \code{Minimum_Height}: Minimum height range (cm, interval).
#'     \item \code{Maximum_Height}: Maximum height range (cm, interval).
#'     \item \code{Mares_Cost}: Cost of mares range (currency units, interval).
#'     \item \code{Fillies_Cost}: Cost of fillies range (currency units, interval).
#' }
#'
#' @usage data(horses.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 7.14.
#' @examples
#' data(horses.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 8 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
"horses.int"

#' @name occupations.modal
#' @title Occupation Salaries Dataset
#' @description
#' Modal-valued dataset of 9 occupations with gender and salary distributions.
#' This is the wide (flat table) format; see \code{\link{occupations2.modal}} for the
#' modal-valued version.
#'
#' @format A data frame with 9 observations and 11 columns:
#' \itemize{
#'     \item \code{Occupation}: Occupation name (character).
#'     \item \code{Gender(M)}, \code{Gender(F)}: Proportion male/female (2 bins).
#'     \item \code{Salary(1)} through \code{Salary(7)}: Salary distribution
#'           across 7 ordered bins (proportions).
#'     \item \code{n}: Sample size (integer).
#' }
#'
#' @usage data(occupations.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(occupations.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 9 \cr
#'   \strong{Variables (p)} \tab 11 \cr
#'   \strong{Subject area} \tab Sociology \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets modal
"occupations.modal"

#' @name occupations2.modal
#' @title Occupation Salaries Modal-Valued Dataset
#' @description
#' Modal-valued version of the occupation salaries dataset.
#' See \code{\link{occupations.modal}} for the wide-format version.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 9 observations and 4 variables:
#' \itemize{
#'     \item \code{Occupation}: Occupation name (character).
#'     \item \code{Gender}: Modal distribution over gender (Male, Female).
#'     \item \code{Salary}: Modal distribution over 7 ordered salary bins.
#'     \item \code{n}: Sample size (numeric).
#' }
#'
#' @usage data(occupations2.modal)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(occupations2.modal)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 9 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Sociology \cr
#'   \strong{Symbolic format} \tab Modal \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets modal
"occupations2.modal"

#' @name profession.int
#' @title Profession Work Salary Time Interval Dataset
#' @description
#' Interval-valued data for 15 profession entries classified by work type
#' (White Collar / Blue Collar). Each entry describes a specific profession
#' with salary and working duration ranges.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 15 observations and 4 variables:
#' \itemize{
#'     \item \code{Type_of_Work}: Work category (White Collar or Blue Collar, character).
#'     \item \code{Profession}: Profession name (character).
#'     \item \code{Salary}: Salary range (currency units, interval).
#'     \item \code{Duration}: Working duration range (hours per week, interval).
#' }
#'
#' @usage data(profession.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(profession.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 15 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Sociology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Classification
#' }
#' @keywords datasets interval
"profession.int"

#' @name veterinary.int
#' @title Veterinary Interval Dataset
#' @description
#' Interval-valued veterinary dataset of 10 animal specimens described
#' by height and weight ranges. Includes male and female specimens of
#' horses, bears, foxes, cats, and dogs.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations and 3 variables:
#' \itemize{
#'     \item \code{Animal}: Animal type and sex label (e.g., HorseM, BearF; character).
#'     \item \code{Height}: Height range (cm, interval).
#'     \item \code{Weight}: Weight range (kg, interval).
#' }
#'
#' @usage data(veterinary.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' @examples
#' data(veterinary.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets interval
"veterinary.int"

## ---------------------------------------------------------------------------
## SECTION 3: iGAP format datasets
## ---------------------------------------------------------------------------

#' @name abalone.iGAP
#' @title Abalone Dataset (iGAP Format)
#' @description
#' Interval-valued dataset of 24 units from the UCI Abalone dataset,
#' aggregated by sex and age group. iGAP format (comma-separated interval
#' strings). See \code{\link{abalone.int}} for the Min-Max column format.
#'
#' @format A data frame with 24 observations (e.g., F-10-12, M-4-6) and
#' 7 character columns in iGAP format (comma-separated \code{"min, max"} strings):
#' \itemize{
#'     \item \code{Length}: Shell length range.
#'     \item \code{Diameter}: Shell diameter range.
#'     \item \code{Height}: Shell height range.
#'     \item \code{Whole}: Whole weight range.
#'     \item \code{Shucked}: Shucked weight range.
#'     \item \code{Viscera}: Viscera weight range.
#'     \item \code{Shell}: Shell weight range.
#' }
#' Row names encode Sex-AgeGroup (e.g., F-10-12 = Female age 10--12).
#'
#' @usage data(abalone.iGAP)
#' @references
#' Nash, W. J., Sellers, T. L., Talbot, S. R., Cawthorn, A. J., and Ford, W. B.
#' (1994). The Population Biology of Abalone (\emph{Haliotis} species) in
#' Tasmania. Sea Fisheries Division, Technical Report No. 48.
#'
#' UCI Machine Learning Repository.
#' \url{https://archive.ics.uci.edu/ml/datasets/Abalone}
#' @examples
#' data(abalone.iGAP)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 24 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Marine biology \cr
#'   \strong{Symbolic format} \tab Interval (iGAP) \cr
#'   \strong{Analytical tasks} \tab Clustering, Visualization
#' }
#' @keywords datasets interval iGAP
#' @source UCI Machine Learning Repository.
"abalone.iGAP"

#' @name abalone.int
#' @title Abalone Interval Dataset
#' @description
#' Interval-valued dataset of 24 units from the UCI Abalone dataset,
#' aggregated by sex and age group. Min-Max column format (two columns per
#' variable). See \code{\link{abalone.iGAP}} for the iGAP format version.
#'
#' @format A data frame with 24 observations and 14 columns (7 interval variables
#' in \code{_min}/\code{_max} pairs):
#' \itemize{
#'     \item \code{Length_min}, \code{Length_max}: Shell length range.
#'     \item \code{Diameter_min}, \code{Diameter_max}: Shell diameter range.
#'     \item \code{Height_min}, \code{Height_max}: Shell height range.
#'     \item \code{Whole_min}, \code{Whole_max}: Whole weight range.
#'     \item \code{Shucked_min}, \code{Shucked_max}: Shucked weight range.
#'     \item \code{Viscera_min}, \code{Viscera_max}: Viscera weight range.
#'     \item \code{Shell_min}, \code{Shell_max}: Shell weight range.
#' }
#' Row names encode Sex-AgeGroup (e.g., F-10-12 = Female age 10--12).
#'
#' @usage data(abalone.int)
#' @references
#' Nash, W. J., Sellers, T. L., Talbot, S. R., Cawthorn, A. J., and Ford, W. B.
#' (1994). The Population Biology of Abalone (\emph{Haliotis} species) in
#' Tasmania. Sea Fisheries Division, Technical Report No. 48.
#'
#' UCI Machine Learning Repository.
#' \url{https://archive.ics.uci.edu/ml/datasets/Abalone}
#' @examples
#' data(abalone.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 24 \cr
#'   \strong{Variables (p)} \tab 14 \cr
#'   \strong{Subject area} \tab Marine biology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering, Visualization
#' }
#' @keywords datasets interval
#' @source UCI Machine Learning Repository.
"abalone.int"

#' @name face.iGAP
#' @title Face Dataset (iGAP Format)
#' @description
#' Interval-valued facial measurement data for 27 face images (9 individuals
#' x 3 replications) in iGAP format (comma-separated interval strings).
#' Contains 6 distance measurements between facial landmarks.
#'
#' @format A data frame with 27 observations and 6 character columns in iGAP
#' format (comma-separated \code{"min,max"} strings):
#' \itemize{
#'     \item \code{AD}: Distance AD (facial landmark pair).
#'     \item \code{BC}: Distance BC (facial landmark pair).
#'     \item \code{AH}: Distance AH (facial landmark pair).
#'     \item \code{DH}: Distance DH (facial landmark pair).
#'     \item \code{EH}: Distance EH (facial landmark pair).
#'     \item \code{GH}: Distance GH (facial landmark pair).
#' }
#' Row names encode individual and replication (e.g., FRA1, FRA2, FRA3).
#'
#' @usage data(face.iGAP)
#' @references
#' Leroy, B., Chouakria, A., Herlin, I., and Diday, E. (1996). Approche
#' geometrique et classification pour la reconnaissance de visage. In
#' \emph{Reconnaissance des Formes et Intelligence Artificielle (RFIA)},
#' 548--557.
#' @examples
#' data(face.iGAP)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 27 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Biometrics \cr
#'   \strong{Symbolic format} \tab Interval (iGAP) \cr
#'   \strong{Analytical tasks} \tab Classification, Visualization
#' }
#' @keywords datasets interval iGAP
"face.iGAP"


## ===========================================================================
## SECTION 4: NEW DATASETS extracted from reference books
## ===========================================================================

## ---------------------------------------------------------------------------
## 4.1 oils.int
## ---------------------------------------------------------------------------

#' @name oils.int
#' @title Oils and Fats Interval Dataset
#' @description
#' Classic benchmark interval-valued data for 8 oils and fats described
#' by 4 physico-chemical properties. Originally from Ichino (1988).
#'
#' @details
#' The 8 samples are: Linseed oil, Perilla oil, Cottonseed oil, Sesame oil,
#' Camellia oil, Olive oil, Beef tallow, Hog fat. The expected 3-cluster
#' structure is: \{Beef tallow, Hog fat\}, \{Cottonseed, Sesame, Camellia,
#' Olive\}, and \{Linseed, Perilla\}. Widely used for comparing clustering
#' methods and distance measures in symbolic data analysis.
#'
#' @format A data frame with 8 observations and 9 columns (4 interval variables
#' in \code{_l}/\code{_u} Min-Max pairs, plus a label):
#' \itemize{
#'     \item \code{sample}: Oil/fat sample name (character).
#'     \item \code{specific_gravity_l}, \code{specific_gravity_u}: Specific gravity range.
#'     \item \code{freezing_point_l}, \code{freezing_point_u}: Freezing point range (degrees Celsius).
#'     \item \code{iodine_value_l}, \code{iodine_value_u}: Iodine value range.
#'     \item \code{saponification_value_l}, \code{saponification_value_u}: Saponification value range.
#' }
#'
#' @usage data(oils.int)
#' @references
#' Ichino, M. (1988). General metrics for mixed features. \emph{Proc. IEEE
#' Conf. Systems, Man, and Cybernetics}, pp. 494-497.
#'
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley. Table 13.7, p.253.
#' @examples
#' data(oils.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 8 \cr
#'   \strong{Variables (p)} \tab 9 \cr
#'   \strong{Subject area} \tab Chemistry \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
"oils.int"

## ---------------------------------------------------------------------------
## 4.2 teams.int
## ---------------------------------------------------------------------------

#' @name teams.int
#' @title Pickup League Teams Interval Dataset
#' @description
#' Interval-valued data for 5 teams in a local pickup league, classified
#' by season performance. Each team is described by ranges of player age,
#' weight, and speed.
#'
#' @details
#' The symbolic results are more informative than classical midpoint analyses:
#' the Very Good team has homogeneous players, whereas the Poor team has
#' players varying widely in age, weight, and speed. Used for symbolic
#' principal component analysis.
#'
#' @format A data frame with 5 observations and 7 columns (3 interval variables
#' in \code{_l}/\code{_u} Min-Max pairs, plus a label):
#' \itemize{
#'     \item \code{team_type}: Performance category (Very Good, Good, Average, Fair, Poor).
#'     \item \code{age_l}, \code{age_u}: Player age range (years).
#'     \item \code{weight_l}, \code{weight_u}: Player weight range (pounds).
#'     \item \code{speed_l}, \code{speed_u}: Speed range -- time to run 100 yards (seconds).
#' }
#'
#' @usage data(teams.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.24, p.63.
#' @examples
#' data(teams.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 5 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Sports \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval PCA
"teams.int"

## ---------------------------------------------------------------------------
## 4.3 tennis.int
## ---------------------------------------------------------------------------

#' @name tennis.int
#' @title Tennis Court Types Interval Dataset
#' @description
#' Interval-valued data for tennis players aggregated by court type
#' (Hard, Grass, Indoor, Clay) with weight, height, and racket tension.
#'
#' @details
#' Clustering on weight and height separates grass courts from the rest
#' (decision rule: Weight <= 74.75 kg). When all three variables are used,
#' clustering separates by racket tension instead.
#'
#' @format A data frame with 4 observations and 7 columns (3 interval variables
#' in \code{_l}/\code{_u} Min-Max pairs, plus a label):
#' \itemize{
#'     \item \code{court_type}: Type of court (Hard, Grass, Indoor, Clay).
#'     \item \code{player_weight_l}, \code{player_weight_u}: Player weight range (kg).
#'     \item \code{player_height_l}, \code{player_height_u}: Player height range (m).
#'     \item \code{racket_tension_l}, \code{racket_tension_u}: Racket tension range.
#' }
#'
#' @usage data(tennis.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.25, p.64.
#' @examples
#' data(tennis.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 4 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Sports \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
"tennis.int"

## ---------------------------------------------------------------------------
## 4.4 bats.int
## ---------------------------------------------------------------------------

#' @name bats.int
#' @title Bat Species Interval Dataset
#' @description
#' Interval-valued data for 21 bat species described by 4 morphological
#' measurements. Benchmark dataset for matrix visualization.
#'
#' @details
#' Used to demonstrate color coding schemes, the HCT-R2E seriation
#' algorithm, and distance measure comparisons (Gowda-Diday, Hausdorff,
#' City-Block, L1, L2, etc.) for interval data.
#'
#' @format A data frame with 21 observations and 9 columns (4 interval variables
#' in \code{_l}/\code{_u} Min-Max pairs, plus a label):
#' \itemize{
#'     \item \code{species}: Bat species name (character).
#'     \item \code{head_l}, \code{head_u}: Head length range (mm).
#'     \item \code{tail_l}, \code{tail_u}: Tail length range (mm).
#'     \item \code{height_l}, \code{height_u}: Ear height range (cm).
#'     \item \code{forearm_l}, \code{forearm_u}: Forearm length range (mm).
#' }
#'
#' @usage data(bats.int)
#' @references
#' Kao, C.-H. et al. (2014). Exploratory data analysis of interval-valued
#' symbolic data with matrix visualization.
#' \emph{Computational Statistics & Data Analysis}, 79, 14--29.
#' @examples
#' data(bats.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 21 \cr
#'   \strong{Variables (p)} \tab 9 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering, Visualization
#' }
#' @keywords datasets interval clustering visualization
"bats.int"

## ---------------------------------------------------------------------------
## 4.5 credit_card.int
## ---------------------------------------------------------------------------

#' @name credit_card.int
#' @title Credit Card Expenses Interval Dataset
#' @description
#' Interval-valued credit card spending aggregated by person-month.
#' Three individuals' (Jon, Tom, Leigh) monthly expenditures across
#' five categories.
#'
#' @details
#' The original classical dataset (Table 2.3) records individual
#' transactions. The symbolic version (Table 2.4) aggregates into
#' interval-valued observations for each person-month combination.
#'
#' @format A data frame with 6 observations and 11 columns (5 interval variables
#' in \code{_l}/\code{_u} Min-Max pairs, plus a label):
#' \itemize{
#'     \item \code{person_month}: Person-month identifier (e.g., "Jon - January"; character).
#'     \item \code{food_l}, \code{food_u}: Food expenditure range (USD).
#'     \item \code{social_l}, \code{social_u}: Social expenditure range (USD).
#'     \item \code{travel_l}, \code{travel_u}: Travel expenditure range (USD).
#'     \item \code{gas_l}, \code{gas_u}: Gas expenditure range (USD).
#'     \item \code{clothes_l}, \code{clothes_u}: Clothes expenditure range (USD).
#' }
#'
#' @usage data(credit_card.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Tables 2.3-2.4.
#' @examples
#' data(credit_card.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 11 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets interval
"credit_card.int"

## ---------------------------------------------------------------------------
## 4.6 energy_consumption.distr
## ---------------------------------------------------------------------------

#' @name energy_consumption.distr
#' @title US Energy Consumption Distribution-Valued Dataset
#' @description
#' Distribution-valued dataset of energy consumption across US states.
#' Each energy type described by Normal distribution parameters (mean, SD).
#'
#' @details
#' Five types: Petroleum, Natural Gas, Coal, Hydroelectric, Nuclear Power.
#' Values are rescaled consumption from the US Census Bureau (2004).
#'
#' @format A data frame with 5 observations and 3 variables:
#' \itemize{
#'     \item \code{type}: Energy type.
#'     \item \code{mean}: Mean consumption across 50 states.
#'     \item \code{sd}: Standard deviation.
#' }
#'
#' @usage data(energy_consumption.distr)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.8.
#' @examples
#' data(energy_consumption.distr)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 5 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Energy \cr
#'   \strong{Symbolic format} \tab Distribution \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets distribution
"energy_consumption.distr"

## ---------------------------------------------------------------------------
## 4.7 trivial_intervals.int
## ---------------------------------------------------------------------------

#' @name trivial_intervals.int
#' @title Trivial and Non-Trivial Intervals Example Dataset
#' @description
#' Simple 5x3 example illustrating different interval types: full intervals
#' (hyperrectangles), degenerate intervals (lines), and trivial intervals
#' (points). Used for vertices PCA demonstration.
#'
#' @format A data frame with 5 observations (w1--w5) and 6 columns (3 interval
#' variables in \code{_l}/\code{_u} Min-Max pairs):
#' \itemize{
#'     \item \code{y1_l}, \code{y1_u}: First interval variable.
#'     \item \code{y2_l}, \code{y2_u}: Second interval variable.
#'     \item \code{y3_l}, \code{y3_u}: Third interval variable.
#' }
#'
#' @usage data(trivial_intervals.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 5.1, p.146.
#' @examples
#' data(trivial_intervals.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 5 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval PCA
"trivial_intervals.int"

## ---------------------------------------------------------------------------
## 4.8 bird_species.mix
## ---------------------------------------------------------------------------

#' @name bird_species.mix
#' @title Bird Species Mixed Symbolic Dataset
#' @description
#' Symbolic data for 3 bird species (Swallow, Ostrich, Penguin) with
#' interval-valued size, categorical flying, and categorical migration.
#' Foundational SDA example from 600 individual bird observations.
#'
#' @format A data frame with 3 observations (Swallow, Ostrich, Penguin) and
#' 5 variables:
#' \itemize{
#'     \item \code{species}: Species name (character).
#'     \item \code{flying}: Flying ability (Yes/No, character).
#'     \item \code{size_l}, \code{size_u}: Size range (cm, Min-Max pair).
#'     \item \code{migration}: Migratory behavior (TRUE/FALSE, logical).
#' }
#'
#' @usage data(bird_species.mix)
#' @references
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley. Table 1.2, p.6.
#' @examples
#' data(bird_species.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 3 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Mixed (interval, categorical) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets mixed interval categorical
"bird_species.mix"

## ---------------------------------------------------------------------------
## 4.9 temperature_city.int
## ---------------------------------------------------------------------------

#' @name temperature_city.int
#' @title World Cities Monthly Temperature Interval Dataset
#' @description
#' Interval-valued monthly temperatures for major cities worldwide.
#' Benchmark dataset for comparing distance measures (Hausdorff, L2,
#' Wasserstein) in dynamic clustering algorithms.
#'
#' @details
#' Expert partition into 4 classes: Class 1 (tropical/warm), Class 2
#' (temperate European and Asian), Class 3 (Mauritius), Class 4 (Tehran).
#'
#' @format A data frame with 6 observations and 13 columns (6 monthly interval
#' variables in \code{_l}/\code{_u} Min-Max pairs, plus a label). Only
#' January through June are included:
#' \itemize{
#'     \item \code{city}: City name (character).
#'     \item \code{jan_l}, \code{jan_u}: January temperature range (degrees Celsius).
#'     \item \code{feb_l}, \code{feb_u}: February temperature range.
#'     \item \code{mar_l}, \code{mar_u}: March temperature range.
#'     \item \code{apr_l}, \code{apr_u}: April temperature range.
#'     \item \code{may_l}, \code{may_u}: May temperature range.
#'     \item \code{jun_l}, \code{jun_u}: June temperature range.
#' }
#'
#' @usage data(temperature_city.int)
#' @references
#' Verde, R. and Irpino, A. (2008). A new interval data distance based on
#' the Wasserstein metric. \emph{Proc. COMPSTAT 2008}, pp. 705-712.
#' @examples
#' data(temperature_city.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 13 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering distance
"temperature_city.int"

## ---------------------------------------------------------------------------
## 4.10 bird_species_extended.mix
## ---------------------------------------------------------------------------

#' @name bird_species_extended.mix
#' @title Bird Species Extended Mixed Symbolic Dataset
#' @description
#' Three bird species (Geese, Ostrich, Penguin) with interval-valued
#' height, distribution-valued color, and categorical flying/migratory
#' variables.
#'
#' @format A data frame with 3 observations and 6 variables:
#' \itemize{
#'     \item \code{species}: Species name (character).
#'     \item \code{flying}: Flying ability (Yes/No, character).
#'     \item \code{height_l}: Height lower bound (cm, numeric).
#'     \item \code{height_u}: Height upper bound (cm, numeric).
#'     \item \code{color}: Color distribution as weighted set string
#'           (e.g., "\{white, 0.3; black, 0.7\}").
#'     \item \code{migratory}: Migratory behavior (Yes/No, character).
#' }
#'
#' @usage data(bird_species_extended.mix)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.19.
#' @examples
#' data(bird_species_extended.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 3 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Mixed (interval, categorical, distribution) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets mixed interval histogram categorical
"bird_species_extended.mix"

## ---------------------------------------------------------------------------
## 4.11 employment.int
## ---------------------------------------------------------------------------

#' @name employment.int
#' @title European Employment by Gender and Age Interval Dataset
#' @description
#' Interval-valued proportions for 12 sex-age population groups across
#' employment variables (employment type, education, industry sector,
#' occupation, marital status). Used for factorial discriminant analysis.
#'
#' @format A data frame with 12 observations and 20 columns (9 interval variables
#' in \code{_l}/\code{_u} Min-Max pairs, plus a group label and class):
#' \itemize{
#'     \item \code{group}: Sex-age group identifier (character).
#'     \item \code{full_time_l}, \code{full_time_u}: Full-time employment proportion range.
#'     \item \code{part_time_l}, \code{part_time_u}: Part-time employment proportion range.
#'     \item \code{primary_studies_l}, \code{primary_studies_u}: Primary studies proportion range.
#'     \item \code{secondary_studies_l}, \code{secondary_studies_u}: Secondary studies proportion range.
#'     \item \code{uni_studies_l}, \code{uni_studies_u}: University studies proportion range.
#'     \item \code{employee_l}, \code{employee_u}: Employee proportion range.
#'     \item \code{manufacturing_l}, \code{manufacturing_u}: Manufacturing sector proportion range.
#'     \item \code{construction_l}, \code{construction_u}: Construction sector proportion range.
#'     \item \code{wholesale_retail_l}, \code{wholesale_retail_u}: Wholesale/retail proportion range.
#'     \item \code{class}: Group classification (numeric).
#' }
#'
#' @usage data(employment.int)
#' @references
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley. Table 18.1.
#' @examples
#' data(employment.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 12 \cr
#'   \strong{Variables (p)} \tab 20 \cr
#'   \strong{Subject area} \tab Economics \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Discriminant analysis, Classification
#' }
#' @keywords datasets interval discriminant
"employment.int"

## ---------------------------------------------------------------------------
## 4.12 town_services.mix
## ---------------------------------------------------------------------------

#' @name town_services.mix
#' @title Town Services Concatenated Mixed Symbolic Dataset
#' @description
#' Symbolic data for 3 towns (Paris, Lyon, Toulouse) combining school
#' and hospital databases. Contains interval-valued, multi-valued, and
#' modal-valued variables.
#'
#' @format A data frame with 3 observations (Paris, Lyon, Toulouse) and 8 columns:
#' \itemize{
#'     \item \code{town}: Town name (character).
#'     \item \code{no_pupils_l}, \code{no_pupils_u}: Number of pupils range (Min-Max pair).
#'     \item \code{type}: School type (modal, character).
#'     \item \code{level}: Coded level (multi-valued, character).
#'     \item \code{no_beds_l}, \code{no_beds_u}: Number of beds range (Min-Max pair).
#'     \item \code{specialty}: Specialty code (multi-valued, character).
#' }
#'
#' @usage data(town_services.mix)
#' @references
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley. Table 1.21, p.19.
#' @examples
#' data(town_services.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 3 \cr
#'   \strong{Variables (p)} \tab 8 \cr
#'   \strong{Subject area} \tab Public services \cr
#'   \strong{Symbolic format} \tab Mixed (interval, modal, multi-valued) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets mixed interval modal multi-valued
"town_services.mix"

## ---------------------------------------------------------------------------
## 4.13 world_cup.int
## ---------------------------------------------------------------------------

#' @name world_cup.int
#' @title World Cup Soccer Teams Interval Dataset
#' @description
#' Interval-valued data for soccer teams grouped by World Cup qualification
#' status (yes/no). Includes age, weight, height ranges and the covariance
#' between weight and height.
#'
#' @format A data frame with 2 observations and 8 variables:
#' \itemize{
#'     \item \code{world_cup}: Qualification status (yes/no, character).
#'     \item \code{age_l}, \code{age_u}: Player age range (years).
#'     \item \code{weight_l}, \code{weight_u}: Player weight range (kg).
#'     \item \code{height_l}, \code{height_u}: Player height range (meters).
#'     \item \code{cov_weight_height}: Covariance between weight and height (numeric).
#' }
#'
#' @usage data(world_cup.int)
#' @references
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley. Table 1.9, p.13.
#' @examples
#' data(world_cup.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 2 \cr
#'   \strong{Variables (p)} \tab 8 \cr
#'   \strong{Subject area} \tab Sports \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets interval
"world_cup.int"

## ---------------------------------------------------------------------------
## 4.14 mushroom_fuzzy.mix
## ---------------------------------------------------------------------------

#' @name mushroom_fuzzy.mix
#' @title Mushroom Species Fuzzy/Symbolic Dataset
#' @description
#' Extended mushroom data with fuzzy stipe thickness (Small/Average/Large),
#' numerical stipe length, interval cap size, and categorical cap colour
#' for two Amanita species (4 specimens).
#'
#' @format A data frame with 4 observations (Mushroom1--Mushroom4) and 9 variables:
#' \itemize{
#'     \item \code{specimen}: Specimen identifier (character).
#'     \item \code{species}: Species name (character).
#'     \item \code{stipe_thickness}: Stipe thickness measurement (numeric, cm).
#'     \item \code{fuzzy_small}: Fuzzy membership degree for Small (numeric, 0--1).
#'     \item \code{fuzzy_average}: Fuzzy membership degree for Average (numeric, 0--1).
#'     \item \code{fuzzy_large}: Fuzzy membership degree for Large (numeric, 0--1).
#'     \item \code{stipe_length}: Stipe length (numeric, cm).
#'     \item \code{cap_size}: Cap size as interval string (e.g., "24 +/- 1", character).
#'     \item \code{cap_colour}: Cap colour (character).
#' }
#'
#' @usage data(mushroom_fuzzy.mix)
#' @references
#' Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
#' Analysis and the SODAS Software}. Wiley. Tables 1.14-1.16.
#' @examples
#' data(mushroom_fuzzy.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 4 \cr
#'   \strong{Variables (p)} \tab 9 \cr
#'   \strong{Subject area} \tab Biology \cr
#'   \strong{Symbolic format} \tab Fuzzy \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets fuzzy symbolic
"mushroom_fuzzy.mix"

## ---------------------------------------------------------------------------
## 4.15 bank_rates
## ---------------------------------------------------------------------------

#' @name bank_rates
#' @title Bank Interest Rates AR Model Symbolic Dataset
#' @description
#' Symbolic dataset of autoregressive time series models for 4 banks.
#' Each bank is described by AR model order, parameters, and whether
#' parameters are known.
#'
#' @format A data frame with 4 observations (Bank1--Bank4) and 6 variables:
#' \itemize{
#'     \item \code{bank}: Bank identifier (character).
#'     \item \code{order}: AR model order (numeric).
#'     \item \code{phi1}: First AR parameter (numeric; NA if unknown).
#'     \item \code{phi2}: Second AR parameter (numeric; NA if order < 2 or unknown).
#'     \item \code{phi1_known}: Whether phi1 is known (logical).
#'     \item \code{phi2_known}: Whether phi2 is known (logical; NA if order < 2).
#' }
#'
#' @usage data(bank_rates)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.9.
#' @examples
#' data(bank_rates)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 4 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Symbolic (model-valued) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets symbolic model
"bank_rates"

## ---------------------------------------------------------------------------
## 4.16 lung_cancer.hist
## ---------------------------------------------------------------------------

#' @name lung_cancer.hist
#' @title Lung Cancer Treatments by State Histogram-Valued Dataset
#' @description
#' Histogram-valued distribution of lung cancer treatment counts for 2 US
#' states (Massachusetts and New York).
#'
#' @format A data frame with 2 observations and 2 variables:
#' \itemize{
#'     \item \code{state}: State name (character).
#'     \item \code{y30}: Histogram-valued distribution of treatment counts
#'           as a weighted set string (e.g., "\{0, 0.77; 1, 0.08; 2, 0.15\}").
#' }
#'
#' @usage data(lung_cancer.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.20.
#' @examples
#' data(lung_cancer.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 2 \cr
#'   \strong{Variables (p)} \tab 2 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets histogram
"lung_cancer.hist"

## ---------------------------------------------------------------------------
## 4.17 acid_rain.int
## ---------------------------------------------------------------------------

#' @name acid_rain.int
#' @title Acid Rain Pollution Indices Interval Dataset
#' @description
#' Interval-valued acid rain pollution indices for sulphates and nitrates
#' (kg/hectares) for 2 US states (Massachusetts and New York).
#'
#' @format A data frame with 2 observations and 5 variables in Min-Max format:
#' \itemize{
#'     \item \code{state}: State name (character).
#'     \item \code{sulphate_l}, \code{sulphate_u}: Sulphate pollution index range
#'           (kg/hectares).
#'     \item \code{nitrate_l}, \code{nitrate_u}: Nitrate pollution index range
#'           (kg/hectares).
#' }
#'
#' @usage data(acid_rain.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
#' Table 2.21.
#' @examples
#' data(acid_rain.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 2 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Environment \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets interval
"acid_rain.int"

## ---------------------------------------------------------------------------
## SECTION 5: New benchmark datasets from recent SDA papers (2020-2025)
## ---------------------------------------------------------------------------

## ---------------------------------------------------------------------------
## 5.1 freshwater_fish.int
## ---------------------------------------------------------------------------

#' @name freshwater_fish.int
#' @title Freshwater Fish Heavy Metal Bioaccumulation Interval Dataset
#' @description
#' Interval-valued dataset of heavy metal concentrations in organs and tissues
#' of 12 freshwater fish species, grouped into 4 feeding categories (Carnivores,
#' Omnivores, Detritivores, Herbivores). Contains 13 interval-valued variables
#' measuring metal concentrations in organs and organ-to-muscle ratios.
#'
#' @format A data frame with 12 observations and 14 variables:
#' \itemize{
#'     \item \code{body_length}: Body length (cm).
#'     \item \code{body_weight}: Body weight (g).
#'     \item \code{muscle}: Metal concentration in muscle tissue.
#'     \item \code{intestine}: Metal concentration in intestine.
#'     \item \code{stomach}: Metal concentration in stomach.
#'     \item \code{gills}: Metal concentration in gills.
#'     \item \code{liver}: Metal concentration in liver.
#'     \item \code{kidney}: Metal concentration in kidney.
#'     \item \code{liver_muscle_ratio}: Liver-to-muscle concentration ratio.
#'     \item \code{kidney_muscle_ratio}: Kidney-to-muscle concentration ratio.
#'     \item \code{gills_muscle_ratio}: Gills-to-muscle concentration ratio.
#'     \item \code{intestine_muscle_ratio}: Intestine-to-muscle concentration ratio.
#'     \item \code{stomach_muscle_ratio}: Stomach-to-muscle concentration ratio.
#'     \item \code{class}: Feeding category (Carnivores, Omnivores, Detritivores, Herbivores).
#' }
#'
#' @usage data(freshwater_fish.int)
#' @references
#' Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
#' Kernel clustering with automatic variable weighting for interval data.
#' \emph{Neurocomputing}, 617, 128954.
#' @examples
#' data(freshwater_fish.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 12 \cr
#'   \strong{Variables (p)} \tab 14 \cr
#'   \strong{Subject area} \tab Biology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
#' @source \url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
"freshwater_fish.int"

## ---------------------------------------------------------------------------
## 5.2 fungi.int
## ---------------------------------------------------------------------------

#' @name fungi.int
#' @title Fungi Morphological Measurements Interval Dataset
#' @description
#' Interval-valued morphological measurements for 55 fungi specimens from
#' 3 genera (Amanita, Agaricus, Boletus). Contains 5 interval-valued variables
#' describing pileus and stipe dimensions and spore characteristics.
#'
#' @format A data frame with 55 observations and 6 variables:
#' \itemize{
#'     \item \code{pileus_width}: Width of the pileus (cap).
#'     \item \code{stipe_width}: Width of the stipe (stem).
#'     \item \code{stipe_thickness}: Thickness of the stipe.
#'     \item \code{spore_height}: Height of the spores.
#'     \item \code{spore_width}: Width of the spores.
#'     \item \code{class}: Fungus genus (Amanita, Agaricus, Boletus).
#' }
#'
#' @usage data(fungi.int)
#' @references
#' Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
#' Kernel clustering with automatic variable weighting for interval data.
#' \emph{Neurocomputing}, 617, 128954.
#' @examples
#' data(fungi.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 55 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Biology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
#' @source \url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
"fungi.int"

## ---------------------------------------------------------------------------
## 5.3 iris.int
## ---------------------------------------------------------------------------

#' @name iris.int
#' @title Iris Species Interval Dataset
#' @description
#' Interval-valued version of the classic iris dataset, aggregated from
#' Fisher's iris data into 30 interval observations across 3 species
#' (Setosa, Versicolor, Virginica). Each observation represents a group of
#' flowers with ranges for sepal and petal measurements.
#'
#' @format A data frame with 30 observations and 5 variables:
#' \itemize{
#'     \item \code{sepal_length}: Sepal length range (cm).
#'     \item \code{sepal_width}: Sepal width range (cm).
#'     \item \code{petal_length}: Petal length range (cm).
#'     \item \code{petal_width}: Petal width range (cm).
#'     \item \code{class}: Species (Setosa, Versicolor, Virginica).
#' }
#'
#' @usage data(iris.int)
#' @references
#' Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
#' Kernel clustering with automatic variable weighting for interval data.
#' \emph{Neurocomputing}, 617, 128954.
#' @examples
#' data(iris.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 30 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Botany \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
#' @source \url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
"iris.int"

## ---------------------------------------------------------------------------
## 5.4 water_flow.int
## ---------------------------------------------------------------------------

#' @name water_flow.int
#' @title Water Flow Sensor Readings Interval Dataset
#' @description
#' Large interval-valued dataset of water flow sensor readings with 316
#' observations and 47 interval-valued feature variables (IF1-IF48, excluding
#' IF17), classified into 2 groups. Used as a benchmark for interval data
#' clustering with high-dimensional features.
#'
#' @format A data frame with 316 observations and 48 variables:
#' \itemize{
#'     \item \code{if1} through \code{if48} (excluding \code{if17}): 47 interval-valued
#'           sensor feature measurements.
#'     \item \code{class}: Group label (1 or 2).
#' }
#'
#' @usage data(water_flow.int)
#' @references
#' Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
#' Kernel clustering with automatic variable weighting for interval data.
#' \emph{Neurocomputing}, 617, 128954.
#' @examples
#' data(water_flow.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 316 \cr
#'   \strong{Variables (p)} \tab 48 \cr
#'   \strong{Subject area} \tab Engineering \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
#' @source \url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
"water_flow.int"

## ---------------------------------------------------------------------------
## 5.5 wine.int
## ---------------------------------------------------------------------------

#' @name wine.int
#' @title Wine Chemical Properties Interval Dataset
#' @description
#' Interval-valued chemical and physical properties of 33 wine samples
#' classified into 2 groups. Contains 9 interval-valued measurement variables.
#' Used as a benchmark for interval data clustering algorithms.
#'
#' @format A data frame with 33 observations and 10 variables:
#' \itemize{
#'     \item \code{V1} through \code{V9}: Nine interval-valued chemical/physical
#'           property measurements.
#'     \item \code{class}: Wine group (1 or 2).
#' }
#'
#' @usage data(wine.int)
#' @references
#' Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
#' Kernel clustering with automatic variable weighting for interval data.
#' \emph{Neurocomputing}, 617, 128954.
#' @examples
#' data(wine.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 33 \cr
#'   \strong{Variables (p)} \tab 10 \cr
#'   \strong{Subject area} \tab Food science \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering
#' @source \url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
"wine.int"

## ---------------------------------------------------------------------------
## 5.6 car_models.int
## ---------------------------------------------------------------------------

#' @name car_models.int
#' @title Italian Car Models Interval Dataset
#' @description
#' Interval-valued specifications for 33 Italian car models, classified into
#' 4 categories (Utilitaria, Berlina, Ammiraglia, Sportiva). An extended
#' version of the classic cars interval dataset with 8 interval-valued
#' variables including dimensions.
#'
#' @format A data frame with 33 observations and 9 variables:
#' \itemize{
#'     \item \code{price}: Price range (currency units).
#'     \item \code{engine_cc}: Engine displacement range (cc).
#'     \item \code{top_speed}: Top speed range (km/h).
#'     \item \code{acceleration}: Acceleration range (seconds 0-100 km/h).
#'     \item \code{wheelbase}: Wheelbase range (cm).
#'     \item \code{length}: Length range (cm).
#'     \item \code{width}: Width range (cm).
#'     \item \code{height}: Height range (cm).
#'     \item \code{class}: Car category (Utilitaria, Berlina, Ammiraglia, Sportiva).
#' }
#'
#' @usage data(car_models.int)
#' @references
#' Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
#' Kernel clustering with automatic variable weighting for interval data.
#' \emph{Neurocomputing}, 617, 128954.
#' @examples
#' data(car_models.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 33 \cr
#'   \strong{Variables (p)} \tab 9 \cr
#'   \strong{Subject area} \tab Automotive \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering, Classification
#' }
#' @keywords datasets interval clustering
#' @source \url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
"car_models.int"

## ---------------------------------------------------------------------------
## 5.7 hdi_gender.int
## ---------------------------------------------------------------------------

#' @name hdi_gender.int
#' @title Human Development Index and Gender Indicators Interval Dataset
#' @description
#' Interval-valued World Bank gender indicators for 183 countries, with
#' ordinal HDI classification. Contains interval ranges for Women, Business
#' and the Law Index Score and proportion of seats held by women in national
#' parliaments.
#'
#' @format A data frame with 183 observations and 6 variables:
#' \itemize{
#'     \item \code{code}: ISO 3166-1 alpha-3 country code.
#'     \item \code{country}: Country name.
#'     \item \code{hdi}: Human Development Index value (UNDP).
#'     \item \code{women_law_index}: Women, Business and the Law Index Score range.
#'     \item \code{women_parliament}: Proportion of seats held by women in
#'           national parliaments range (\%).
#'     \item \code{hdi_category}: Ordered factor with HDI classification
#'           (Low < Medium < High < Very High).
#' }
#'
#' @usage data(hdi_gender.int)
#' @references
#' Alcacer, A., Barrel, A., Groenen, P. J. F. and Grana, M. (2023).
#' Ordinal classification for interval-valued data and ordinal data.
#' \emph{Expert Systems with Applications}, 238, 121825.
#' @examples
#' data(hdi_gender.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 183 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Socioeconomics \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Classification
#' }
#' @keywords datasets interval ordinal
#' @source \url{https://github.com/aleixalcacer/OCFIVD}
"hdi_gender.int"

## ===========================================================================
## SECTION 6: Datasets extracted from R packages and textbooks (2025 addition)
## ===========================================================================

## ---------------------------------------------------------------------------
## 6.1 cardiological.int
## ---------------------------------------------------------------------------

#' @name cardiological.int
#' @title Cardiological Examination Interval Dataset
#' @description
#' Interval-valued data from cardiological examinations of 44 patients.
#' Each patient is described by 5 interval-valued physiological
#' measurements.
#'
#' @format A data frame with 44 observations and 5 interval-valued variables:
#' \itemize{
#'     \item \code{pulse}: Pulse rate range (beats per minute).
#'     \item \code{systolic}: Systolic blood pressure range (mmHg).
#'     \item \code{diastolic}: Diastolic blood pressure range (mmHg).
#'     \item \code{arterial1}: First arterial measurement range.
#'     \item \code{arterial2}: Second arterial measurement range.
#' }
#'
#' @usage data(cardiological.int)
#' @references
#' Rodriguez, O. (2000). Classification et modeles lineaires en analyse
#' des donnees symboliques. Doctoral Thesis, Universite Paris IX-Dauphine.
#' @examples
#' data(cardiological.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 44 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets interval
#' @source Extracted from RSDA package (\code{cardiologicalv2}).
"cardiological.int"

## ---------------------------------------------------------------------------
## 6.2 prostate.int
## ---------------------------------------------------------------------------

#' @name prostate.int
#' @title Prostate Cancer Clinical Interval Dataset
#' @description
#' Interval-valued clinical measurements for 97 prostate cancer patients
#' (training and test sets combined). Contains 9 interval-valued variables
#' from log-transformed cancer volume, weight, age, and other clinical
#' predictors.
#'
#' @format A data frame with 97 observations and 9 interval-valued variables:
#' \itemize{
#'     \item \code{lcavol}: Log cancer volume range.
#'     \item \code{lweight}: Log prostate weight range.
#'     \item \code{age}: Patient age range.
#'     \item \code{lbph}: Log benign prostatic hyperplasia amount range.
#'     \item \code{svi}: Seminal vesicle invasion range.
#'     \item \code{lcp}: Log capsular penetration range.
#'     \item \code{gleason}: Gleason score range.
#'     \item \code{pgg45}: Percentage Gleason scores 4 or 5 range.
#'     \item \code{lpsa}: Log prostate specific antigen range.
#' }
#'
#' @usage data(prostate.int)
#' @references
#' Stamey, T. et al. (1989). Prostate specific antigen in the diagnosis and
#' treatment of adenocarcinoma of the prostate. II. Radical prostatectomy
#' treated patients. \emph{J. Urology}, 141(5), 1076-1083.
#' @examples
#' data(prostate.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 97 \cr
#'   \strong{Variables (p)} \tab 9 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression
#' }
#' @keywords datasets interval medical
#' @source Extracted from RSDA package (\code{int_prost_train}, \code{int_prost_test}).
"prostate.int"

## ---------------------------------------------------------------------------
## 6.3 uscrime.int
## ---------------------------------------------------------------------------

#' @name uscrime.int
#' @title US Crime Statistics Interval Dataset
#' @description
#' Interval-valued crime statistics for 46 US states, containing 102
#' interval-valued variables covering various crime types and rates.
#' Originally from the RSDA package.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 46 observations and
#' 102 interval-valued variables. Key variables include:
#' \itemize{
#'     \item \code{fold}: Cross-validation fold assignment.
#'     \item \code{population}: Population range.
#'     \item \code{householdsize}: Household size range.
#'     \item \code{racepctblack}, \code{racePctWhite}, \code{racePctAsian},
#'           \code{racePctHisp}: Race percentage ranges.
#'     \item \code{medIncome}, \code{medFamInc}, \code{perCapInc}: Income ranges.
#'     \item \code{PctUnemployed}, \code{PctEmploy}: Employment percentage ranges.
#'     \item \code{ViolentCrimesPerPop}: Violent crimes per population range.
#' }
#' Plus 90 additional interval-valued socio-economic and demographic variables.
#'
#' @usage data(uscrime.int)
#' @references
#' Rodriguez, O. (2000). Classification et modeles lineaires en analyse
#' des donnees symboliques. Doctoral Thesis, Universite Paris IX-Dauphine.
#' @examples
#' data(uscrime.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 46 \cr
#'   \strong{Variables (p)} \tab 102 \cr
#'   \strong{Subject area} \tab Criminology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression, Clustering
#' }
#' @keywords datasets interval crime
#' @source Extracted from RSDA package (\code{uscrime_int}).
"uscrime.int"

## ---------------------------------------------------------------------------
## 6.4 hardwood.hist
## ---------------------------------------------------------------------------

#' @name hardwood.hist
#' @title Hardwood Tree Species Histogram-Valued Dataset
#' @description
#' Histogram-valued climate data for 5 hardwood tree species in the
#' southeastern United States. Each observation represents a species with
#' 4 histogram-valued climate variables.
#'
#' @format A data frame with 5 observations and 4 histogram-valued variables:
#' \itemize{
#'     \item \code{ANNT}: Annual temperature histogram (degrees C).
#'     \item \code{JULT}: July temperature histogram (degrees C).
#'     \item \code{ANNP}: Annual precipitation histogram (mm).
#'     \item \code{MITM}: Moisture index histogram.
#' }
#'
#' @usage data(hardwood.hist)
#' @references
#' Brito, P. (2007). Modelling and Analysing Interval Data.
#' In V. Esposito Vinzi et al. (Eds.), \emph{New Developments in Classification
#' and Data Analysis}, pp. 197-208. Springer.
#' @examples
#' data(hardwood.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 5 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Forestry \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets histogram
#' @source Extracted from RSDA package (\code{hardwoodBrito}).
"hardwood.hist"

## ---------------------------------------------------------------------------
## 6.5 synthetic_clusters.int
## ---------------------------------------------------------------------------

#' @name synthetic_clusters.int
#' @title Synthetic Interval Clusters Dataset
#' @description
#' Synthetic interval-valued dataset with 125 observations in 5 groups of
#' 25 each, described by 6 interval-valued variables and a cluster label.
#' Designed for benchmarking interval data clustering algorithms.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 125 observations and 7 variables:
#' \itemize{
#'     \item \code{V1} through \code{V6}: Six interval-valued variables.
#'     \item \code{class}: Cluster membership (1--5, set-valued).
#' }
#'
#' @usage data(synthetic_clusters.int)
#' @references
#' Dudek, A. and Pelka, M. (2022). \emph{symbolicDA}: Analysis of Symbolic
#' Data. R package.
#' @examples
#' data(synthetic_clusters.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 125 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval clustering synthetic
#' @source Extracted from symbolicDA package (\code{data_symbolic}).
"synthetic_clusters.int"

## ---------------------------------------------------------------------------
## 6.6 environment.mix
## ---------------------------------------------------------------------------

#' @name environment.mix
#' @title EPA Environmental Data Mixed Symbolic Dataset
#' @description
#' Mixed symbolic dataset from the US EPA with 14 state-group observations
#' and 17 variables of mixed types: interval-valued environmental measurements
#' and modal-valued (distributional) categorical variables.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}, of class
#' \code{c("symbolic_tbl", "tbl_df", "tbl", "data.frame")}) with 14
#' observations and 17 variables. The first four are modal-valued
#' (\code{symbolic_modal}) variables and the remaining thirteen are
#' interval-valued (\code{symbolic_interval}) variables. This matches the
#' structure of \code{ggInterval::Environment}.
#' \itemize{
#'     \item \code{URBANICITY}: Modal-valued urbanicity distribution (modal).
#'     \item \code{INCOMELEVEL}: Modal-valued income level distribution (modal).
#'     \item \code{EDUCATION}: Modal-valued education distribution (modal).
#'     \item \code{REGIONDEVELOPME}: Modal-valued regional development distribution (modal).
#'     \item \code{CONTROL}: Environmental control index range (interval).
#'     \item \code{SATISFY}: Satisfaction index range (interval).
#'     \item \code{INDIVIDUAL}: Individual concern index range (interval).
#'     \item \code{WELFARE}: Welfare index range (interval).
#'     \item \code{HUMAN}: Human impact index range (interval).
#'     \item \code{POLITICS}: Political concern index range (interval).
#'     \item \code{BURDEN}: Burden index range (interval).
#'     \item \code{NOISE}: Noise pollution index range (interval).
#'     \item \code{NATURE}: Nature preservation index range (interval).
#'     \item \code{SEASETC}: Seas/coastal index range (interval).
#'     \item \code{MULTI}: Multi-indicator range (interval).
#'     \item \code{WATERWASTE}: Water/waste index range (interval).
#'     \item \code{VEHICLE}: Vehicle emissions index range (interval).
#' }
#'
#' @usage data(environment.mix)
#' @references
#' Jiang, B.-S. and Wu, H.-M. (2025). ggInterval: an R package for
#' visualizing interval-valued data using ggplot2. R package version 0.2.5.
#' \url{https://CRAN.R-project.org/package=ggInterval}
#' @examples
#' data(environment.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 17 \cr
#'   \strong{Subject area} \tab Environment \cr
#'   \strong{Symbolic format} \tab Mixed (interval, modal) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets mixed interval modal
#' @source Extracted from the ggInterval package (\code{Environment}).
"environment.mix"

## ---------------------------------------------------------------------------
## 6.7 weight_age.hist
## ---------------------------------------------------------------------------

#' @name weight_age.hist
#' @title Weight by Age Group Histogram-Valued Dataset
#' @description
#' Histogram-valued weight distributions for 7 age groups (20s through 80s).
#' Each observation represents an age decade with a 7-bin histogram of
#' weight values (pounds).
#'
#' @format A data frame with 7 observations and 1 histogram-valued variable:
#' \itemize{
#'     \item \code{weight}: Histogram-valued weight distribution (pounds).
#' }
#' Row names indicate age groups (20s, 30s, 40s, 50s, 60s, 70s, 80s).
#'
#' @usage data(weight_age.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.10.
#' @examples
#' data(weight_age.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 7 \cr
#'   \strong{Variables (p)} \tab 1 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets histogram
#' @source Billard, L. and Diday, E. (2006), Table 3.10.
"weight_age.hist"

## ---------------------------------------------------------------------------
## 6.8 hospital.hist
## ---------------------------------------------------------------------------

#' @name hospital.hist
#' @title Hospital Costs Histogram-Valued Dataset
#' @description
#' Histogram-valued cost distributions for 15 hospitals. Each observation
#' is a hospital with a 10-bin histogram of patient costs.
#'
#' @format A data frame with 15 observations and 1 histogram-valued variable:
#' \itemize{
#'     \item \code{cost}: Histogram-valued cost distribution (currency units).
#' }
#' Row names are H1 through H15.
#'
#' @usage data(hospital.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.12.
#' @examples
#' data(hospital.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 15 \cr
#'   \strong{Variables (p)} \tab 1 \cr
#'   \strong{Subject area} \tab Healthcare \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets histogram
#' @source Billard, L. and Diday, E. (2006), Table 3.12.
"hospital.hist"

## ---------------------------------------------------------------------------
## 6.9 cholesterol.hist
## ---------------------------------------------------------------------------

#' @name cholesterol.hist
#' @title Cholesterol by Gender and Age Histogram-Valued Dataset
#' @description
#' Histogram-valued cholesterol distributions for 14 gender-age groups
#' (7 female + 7 male age groups from 20s to 80+). Each observation has
#' a 10-bin histogram of cholesterol levels.
#'
#' @format A data frame with 14 observations and 3 variables:
#' \itemize{
#'     \item \code{gender}: Gender (Female or Male).
#'     \item \code{age}: Age group (20s, 30s, 40s, 50s, 60s, 70s, 80+).
#'     \item \code{cholesterol}: Histogram-valued cholesterol distribution.
#' }
#'
#' @usage data(cholesterol.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 4.5.
#' @examples
#' data(cholesterol.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets histogram medical
#' @source Billard, L. and Diday, E. (2006), Table 4.5.
"cholesterol.hist"

## ---------------------------------------------------------------------------
## 6.10 hemoglobin.hist
## ---------------------------------------------------------------------------

#' @name hemoglobin.hist
#' @title Hemoglobin by Gender and Age Histogram-Valued Dataset
#' @description
#' Histogram-valued hemoglobin distributions for 14 gender-age groups
#' (7 female + 7 male age groups from 20s to 80+). Each observation has
#' a 10-bin histogram of hemoglobin levels (g/dL).
#'
#' @format A data frame with 14 observations and 3 variables:
#' \itemize{
#'     \item \code{gender}: Gender (Female or Male).
#'     \item \code{age}: Age group (20s, 30s, 40s, 50s, 60s, 70s, 80+).
#'     \item \code{hemoglobin}: Histogram-valued hemoglobin distribution (g/dL).
#' }
#'
#' @usage data(hemoglobin.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 4.6.
#' @examples
#' data(hemoglobin.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets histogram medical
#' @source Billard, L. and Diday, E. (2006), Table 4.6.
"hemoglobin.hist"

## ---------------------------------------------------------------------------
## 6.11 hematocrit.hist
## ---------------------------------------------------------------------------

#' @name hematocrit.hist
#' @title Hematocrit by Gender and Age Histogram-Valued Dataset
#' @description
#' Histogram-valued hematocrit distributions for 14 gender-age groups
#' (7 female + 7 male age groups from 20s to 80+). Each observation has
#' a 10-bin histogram of hematocrit percentages.
#'
#' @format A data frame with 14 observations and 3 variables:
#' \itemize{
#'     \item \code{gender}: Gender (Female or Male).
#'     \item \code{age}: Age group (20s, 30s, 40s, 50s, 60s, 70s, 80+).
#'     \item \code{hematocrit}: Histogram-valued hematocrit distribution (\%).
#' }
#'
#' @usage data(hematocrit.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 4.14.
#' @examples
#' data(hematocrit.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets histogram medical
#' @source Billard, L. and Diday, E. (2006), Table 4.14.
"hematocrit.hist"

## ---------------------------------------------------------------------------
## 6.12 hematocrit_hemoglobin.hist
## ---------------------------------------------------------------------------

#' @name hematocrit_hemoglobin.hist
#' @title Hematocrit and Hemoglobin Bivariate Histogram-Valued Dataset
#' @description
#' Bivariate histogram-valued dataset with 10 observations, each described
#' by a 2-bin hematocrit histogram and a 2-bin hemoglobin histogram.
#' Used for bivariate symbolic regression demonstrations.
#'
#' @format A data frame with 10 observations and 2 histogram-valued variables:
#' \itemize{
#'     \item \code{hematocrit}: Histogram-valued hematocrit distribution (\%).
#'     \item \code{hemoglobin}: Histogram-valued hemoglobin distribution (g/dL).
#' }
#'
#' @usage data(hematocrit_hemoglobin.hist)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 6.8.
#' @examples
#' data(hematocrit_hemoglobin.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 2 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Regression
#' }
#' @keywords datasets histogram medical regression
#' @source Billard, L. and Diday, E. (2006), Table 6.8.
"hematocrit_hemoglobin.hist"

## ---------------------------------------------------------------------------
## 6.13 energy_usage.distr
## ---------------------------------------------------------------------------

#' @name energy_usage.distr
#' @title Energy Usage Distribution-Valued Dataset
#' @description
#' Distribution-valued dataset for 10 towns (geographic areas) with
#' categorical probability distributions for fuel type and central heating.
#' Each observation has two distribution-valued variables.
#'
#' @format A data frame with 10 observations and 2 distribution-valued variables:
#' \itemize{
#'     \item \code{fuel_type}: Distribution over fuel types
#'           (None, Gas, Oil, Electricity, Coal).
#'     \item \code{central_heating}: Distribution over central heating
#'           (No, Yes).
#' }
#' Row names are Town_1 through Town_10.
#'
#' @usage data(energy_usage.distr)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.7.
#' @examples
#' data(energy_usage.distr)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 2 \cr
#'   \strong{Subject area} \tab Energy \cr
#'   \strong{Symbolic format} \tab Distribution \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets distribution
#' @source Billard, L. and Diday, E. (2006), Table 3.7.
"energy_usage.distr"

## ============================================================================
## SECTION 7: Datasets from Billard & Diday (2020) and R packages (2026 addition)
## ============================================================================

## ---------------------------------------------------------------------------
## 7.1 genome_abundances.int
## ---------------------------------------------------------------------------

#' @name genome_abundances.int
#' @title Genome Dinucleotide Abundance Intervals
#' @description
#' Interval-valued dataset of dinucleotide relative abundances for 14 genome
#' classes. Each class aggregates multiple genomes; the intervals represent
#' the range of observed abundance values within each class for 10 dinucleotide
#' pairs, plus a count variable.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 14 observations
#' (genome classes) and 11 variables:
#' \itemize{
#'     \item \code{CG}: Interval-valued CG dinucleotide relative abundance.
#'     \item \code{GC}: Interval-valued GC dinucleotide relative abundance.
#'     \item \code{TA}: Interval-valued TA dinucleotide relative abundance.
#'     \item \code{AT}: Interval-valued AT dinucleotide relative abundance.
#'     \item \code{CC}: Interval-valued CC dinucleotide relative abundance.
#'     \item \code{AA}: Interval-valued AA dinucleotide relative abundance.
#'     \item \code{AC}: Interval-valued AC dinucleotide relative abundance.
#'     \item \code{AG}: Interval-valued AG dinucleotide relative abundance.
#'     \item \code{CA}: Interval-valued CA dinucleotide relative abundance.
#'     \item \code{GA}: Interval-valued GA dinucleotide relative abundance.
#'     \item \code{n}: Number of genomes in the class (integer).
#' }
#' Row names are Class_1 through Class_14.
#'
#' @usage data(genome_abundances.int)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 3-16.
#' @examples
#' data(genome_abundances.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 11 \cr
#'   \strong{Subject area} \tab Genomics \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets interval genomics
#' @source Billard, L. and Diday, E. (2020), Table 3-16.
"genome_abundances.int"

## ---------------------------------------------------------------------------
## 7.2 china_temp_monthly.int
## ---------------------------------------------------------------------------

#' @name china_temp_monthly.int
#' @title China Monthly Temperature Intervals (15 Stations)
#' @description
#' Interval-valued dataset of monthly temperature ranges for 15 weather
#' stations in China. Each station has 12 monthly temperature intervals
#' (minimum and maximum observed temperatures in degrees Celsius) and
#' an elevation value in meters.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 15 observations
#' (weather stations) and 13 variables:
#' \itemize{
#'     \item \code{January}, \code{February}, \code{March}, \code{April},
#'           \code{May}, \code{June}, \code{July}, \code{August},
#'           \code{September}, \code{October}, \code{November}, \code{December}:
#'           Interval-valued monthly temperature ranges (degrees Celsius).
#'     \item \code{Elevation}: Station elevation above sea level (numeric, meters).
#' }
#' Row names are station names (e.g., BoKeTu, Hailaer, LaSa).
#'
#' @usage data(china_temp_monthly.int)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 7-9.
#' @examples
#' data(china_temp_monthly.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 15 \cr
#'   \strong{Variables (p)} \tab 13 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval temperature climate
#' @source Billard, L. and Diday, E. (2020), Table 7-9.
"china_temp_monthly.int"

## ---------------------------------------------------------------------------
## 7.3 ecoli_routes.int
## ---------------------------------------------------------------------------

#' @name ecoli_routes.int
#' @title E. coli Transport Routes Interval Dataset
#' @description
#' Interval-valued dataset of 9 E. coli transport routes with 5 interval
#' variables representing biochemical pathway measurements.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 9 observations
#' (transport routes) and 5 interval-valued variables:
#' \itemize{
#'     \item \code{Y1} through \code{Y5}: Interval-valued biochemical
#'           pathway measurements.
#' }
#' Row names are Route_1 through Route_9.
#'
#' @usage data(ecoli_routes.int)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 8-10.
#' @examples
#' data(ecoli_routes.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 9 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Biology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval biology
#' @source Billard, L. and Diday, E. (2020), Table 8-10.
"ecoli_routes.int"

## ---------------------------------------------------------------------------
## 7.4 loans_by_risk.int
## ---------------------------------------------------------------------------

#' @name loans_by_risk.int
#' @title Lending Club Loans by Risk Level
#' @description
#' Interval-valued dataset of 35 Lending Club loan groups classified by
#' risk level (A through G, 5 groups each). Each group is described by
#' 4 interval-valued financial variables.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 35 observations
#' and 5 variables:
#' \itemize{
#'     \item \code{log_income}: Interval-valued log annual income.
#'     \item \code{interest_rate}: Interval-valued interest rate (\%).
#'     \item \code{open_accounts}: Interval-valued number of open credit accounts.
#'     \item \code{total_accounts}: Interval-valued total number of credit accounts.
#'     \item \code{risk_level}: Risk grade factor (A, B, C, D, E, F, G).
#' }
#' Row names are A1--A5, B1--B5, ..., G1--G5.
#'
#' @usage data(loans_by_risk.int)
#' @references
#' Brito, P. and Duarte Silva, A.P. (2012). Modelling interval data with
#' Normal and Skew-Normal distributions. \emph{Journal of Applied Statistics},
#' 39(1), 3--20.
#'
#' Original data from the MAINT.Data R package.
#' @examples
#' data(loans_by_risk.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 35 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Classification, Clustering
#' }
#' @keywords datasets interval finance
#' @source MAINT.Data R package (\code{LoansbyRisk_minmax} dataset).
"loans_by_risk.int"

## ---------------------------------------------------------------------------
## 7.5 polish_voivodships.int
## ---------------------------------------------------------------------------

#' @name polish_voivodships.int
#' @title Polish Voivodships Socio-Economic Intervals
#' @description
#' Interval-valued dataset of 18 Polish voivodships (administrative regions)
#' with 9 socio-economic interval variables describing demographic and
#' economic characteristics at the county (powiat) level.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 18 observations
#' (voivodships) and 9 interval-valued variables:
#' \itemize{
#'     \item \code{V1} through \code{V9}: Interval-valued socio-economic
#'           indicators aggregated across counties within each voivodship.
#' }
#' Row names are voivodship names (e.g., Dolnoslaskie, Lubelskie).
#'
#' @usage data(polish_voivodships.int)
#' @references
#' Dudek, A. and Pelka, M. (2022). \emph{symbolicDA: Analysis of Symbolic
#' Data}. R package.
#'
#' Walesiak, M. and Dudek, A. (2020). \emph{clusterSim: Searching for
#' Optimal Clustering Procedure for a Data Set}. R package.
#' @examples
#' data(polish_voivodships.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 18 \cr
#'   \strong{Variables (p)} \tab 9 \cr
#'   \strong{Subject area} \tab Socioeconomics \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets interval socioeconomic
#' @source clusterSim R package (\code{data_pathtinger} dataset).
"polish_voivodships.int"

## ---------------------------------------------------------------------------
## 7.6 iris_species.hist
## ---------------------------------------------------------------------------

#' @name iris_species.hist
#' @title Iris Species Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 3 iris species (Versicolor, Virginica,
#' Setosa) with 4 histogram-valued morphological variables and a species
#' label. Each histogram describes the distribution of measurements
#' within a species.
#'
#' @format A data frame with 3 observations and 5 variables:
#' \itemize{
#'     \item \code{species}: Species name (factor: Versicolor, Virginica, Setosa).
#'     \item \code{sepal_width}: Histogram-valued sepal width distribution.
#'     \item \code{sepal_length}: Histogram-valued sepal length distribution.
#'     \item \code{petal_width}: Histogram-valued petal width distribution.
#'     \item \code{petal_length}: Histogram-valued petal length distribution.
#' }
#' Row names are species names.
#'
#' @usage data(iris_species.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 4-10.
#' @examples
#' data(iris_species.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 3 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Botany \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets histogram iris
#' @source Billard, L. and Diday, E. (2020), Table 4-10.
"iris_species.hist"

## ---------------------------------------------------------------------------
## 7.7 flights_detail.hist
## ---------------------------------------------------------------------------

#' @name flights_detail.hist
#' @title Airline Flights Detailed Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 16 airlines with 5 flight performance
#' histograms. Each histogram has 12 bins describing the distribution
#' of a performance metric across flights for that airline.
#'
#' @format A data frame with 16 observations (airlines) and 5 histogram-valued
#' variables:
#' \itemize{
#'     \item \code{airtime}: Histogram of air time (minutes).
#'     \item \code{taxi_in}: Histogram of taxi-in time (minutes).
#'     \item \code{arrival_delay}: Histogram of arrival delay (minutes).
#'     \item \code{taxi_out}: Histogram of taxi-out time (minutes).
#'     \item \code{departure_delay}: Histogram of departure delay (minutes).
#' }
#' Row names are Airline_1 through Airline_16.
#'
#' @usage data(flights_detail.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 5-1.
#' @examples
#' data(flights_detail.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 16 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Transportation \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets histogram flights
#' @source Billard, L. and Diday, E. (2020), Table 5-1.
"flights_detail.hist"

## ---------------------------------------------------------------------------
## 7.8 cover_types.hist
## ---------------------------------------------------------------------------

#' @name cover_types.hist
#' @title Forest Cover Types Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 7 forest cover types with 4 topographic
#' histogram variables. Each histogram describes the distribution of a
#' terrain feature across locations classified as that cover type.
#'
#' @format A data frame with 7 observations (cover types) and 4
#' histogram-valued variables:
#' \itemize{
#'     \item \code{elevation}: Histogram of elevation values (meters).
#'     \item \code{distance_to_water}: Histogram of horizontal distance
#'           to nearest water source (meters).
#'     \item \code{hillshade}: Histogram of hillshade index values.
#'     \item \code{slope}: Histogram of slope values (degrees).
#' }
#' Row names are CoverType_1 through CoverType_7.
#'
#' @usage data(cover_types.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 7-21.
#' @examples
#' data(cover_types.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 7 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Forestry \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering, Classification
#' }
#' @keywords datasets histogram forestry
#' @source Billard, L. and Diday, E. (2020), Table 7-21.
"cover_types.hist"

## ---------------------------------------------------------------------------
## 7.9 glucose.hist
## ---------------------------------------------------------------------------

#' @name glucose.hist
#' @title Blood Glucose Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 4 regions with a single histogram-valued
#' variable describing the distribution of blood glucose measurements.
#'
#' @format A data frame with 4 observations (regions) and 1 histogram-valued
#' variable:
#' \itemize{
#'     \item \code{glucose}: Histogram of blood glucose levels.
#' }
#' Row names are Region_1 through Region_4.
#'
#' @usage data(glucose.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 4-14.
#' @examples
#' data(glucose.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 4 \cr
#'   \strong{Variables (p)} \tab 1 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets histogram medical
#' @source Billard, L. and Diday, E. (2020), Table 4-14.
"glucose.hist"

## ---------------------------------------------------------------------------
## 7.10 state_income.hist
## ---------------------------------------------------------------------------

#' @name state_income.hist
#' @title State Income Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 6 US states with 4 income distribution
#' histograms. Each histogram describes the distribution of household
#' income within a state.
#'
#' @format A data frame with 6 observations (states) and 4 histogram-valued
#' variables:
#' \itemize{
#'     \item \code{Y1} through \code{Y4}: Histogram-valued income distribution
#'           variables.
#' }
#' Row names are State_1 through State_6.
#'
#' @usage data(state_income.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 7-18.
#' @examples
#' data(state_income.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Economics \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets histogram income
#' @source Billard, L. and Diday, E. (2020), Table 7-18.
"state_income.hist"

## ---------------------------------------------------------------------------
## 7.11 simulated.hist
## ---------------------------------------------------------------------------

#' @name simulated.hist
#' @title Simulated Histogram-Valued Dataset
#' @description
#' Small simulated histogram-valued dataset of 5 observations with 2
#' histogram-valued variables. Useful for testing and demonstrating
#' histogram-valued statistical methods.
#'
#' @format A data frame with 5 observations and 2 histogram-valued variables:
#' \itemize{
#'     \item \code{Y1}: Histogram-valued variable 1.
#'     \item \code{Y2}: Histogram-valued variable 2.
#' }
#' Row names are Obs_1 through Obs_5.
#'
#' @usage data(simulated.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 7-26.
#' @examples
#' data(simulated.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 5 \cr
#'   \strong{Variables (p)} \tab 2 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets histogram simulated
#' @source Billard, L. and Diday, E. (2020), Table 7-26.
"simulated.hist"

## ---------------------------------------------------------------------------
## 7.12 age_pyramids.hist
## ---------------------------------------------------------------------------

#' @name age_pyramids.hist
#' @title World Age Pyramids Histogram-Valued Dataset (2014)
#' @description
#' Histogram-valued dataset of 229 countries with 3 population age pyramid
#' histograms (both sexes, male, female). Each histogram has 21 age bins
#' representing the distribution of the population across age groups.
#'
#' @format A data frame with 229 observations (countries) and 3
#' histogram-valued variables:
#' \itemize{
#'     \item \code{Both.Sexes.Population}: Histogram of total population
#'           by age group.
#'     \item \code{Male.Population}: Histogram of male population by age group.
#'     \item \code{Female.Population}: Histogram of female population by age group.
#' }
#' Row names are country names (e.g., WORLD, Afghanistan, Albania).
#'
#' @usage data(age_pyramids.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: A new metric-based approach.
#' \emph{Advances in Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{Age_Pyramids_2014}).
#' @examples
#' data(age_pyramids.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 229 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Demographics \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets histogram demographics
#' @source HistDAWass R package (\code{Age_Pyramids_2014} dataset).
"age_pyramids.hist"

## ---------------------------------------------------------------------------
## 7.13 ozone.hist
## ---------------------------------------------------------------------------

#' @name ozone.hist
#' @title Ozone Air Quality Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 84 daily observations with 4 weather-related
#' histogram variables. Each histogram has 10 equal-probability (decile) bins
#' summarizing hourly measurements within each day.
#'
#' @format A data frame with 84 observations (days) and 4 histogram-valued
#' variables:
#' \itemize{
#'     \item \code{Ozone.Conc.ppb}: Histogram of ozone concentration (ppb).
#'     \item \code{Temperature.C}: Histogram of temperature (Celsius).
#'     \item \code{Solar.Radiation.WattM2}: Histogram of solar radiation (W/m^2).
#'     \item \code{Wind.Speed.mSec}: Histogram of wind speed (m/s).
#' }
#' Row names are I1 through I84.
#'
#' @usage data(ozone.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: A new metric-based approach.
#' \emph{Advances in Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{OzoneH} dataset),
#' reduced from 100 quantile bins to 10 decile bins.
#' @examples
#' data(ozone.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 84 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Environment \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Regression, Clustering
#' }
#' @keywords datasets histogram weather environment
#' @source HistDAWass R package (\code{OzoneH} dataset).
"ozone.hist"

## ---------------------------------------------------------------------------
## 7.14 french_agriculture.hist
## ---------------------------------------------------------------------------

#' @name french_agriculture.hist
#' @title French Agriculture Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 22 French regions with 4 economic
#' histogram variables related to agricultural production. Each histogram
#' describes the distribution of farm-level values within a region.
#'
#' @format A data frame with 22 observations (French regions) and 4
#' histogram-valued variables:
#' \itemize{
#'     \item \code{Y_TSC}: Histogram of total standard coefficient.
#'     \item \code{X_Wheat}: Histogram of wheat production.
#'     \item \code{X_Pig}: Histogram of pig production.
#'     \item \code{X_Cmilk}: Histogram of cow milk production.
#' }
#' Row names are French region names (e.g., Ile-de-France, Picardie).
#'
#' @usage data(french_agriculture.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: A new metric-based approach.
#' \emph{Advances in Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{Agronomique} dataset).
#' @examples
#' data(french_agriculture.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 22 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Agriculture \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Regression, Clustering
#' }
#' @keywords datasets histogram agriculture economics
#' @source HistDAWass R package (\code{Agronomique} dataset).
"french_agriculture.hist"

## ---------------------------------------------------------------------------
## 7.15 household_characteristics.distr
## ---------------------------------------------------------------------------

#' @name household_characteristics.distr
#' @title Household Characteristics Distribution-Valued Dataset
#' @description
#' Distribution-valued dataset of 12 counties with 3 categorical
#' probability distribution variables describing household fuel type,
#' number of rooms, and household income brackets.
#'
#' @format A data frame with 12 observations (counties) and 3
#' distribution-valued variables:
#' \itemize{
#'     \item \code{fuel_type}: Distribution over fuel types
#'           (gas, electric, oil, wood, none).
#'     \item \code{rooms}: Distribution over room counts
#'           (\{1,2\}, \{3,4,5\}, \{>=6\}).
#'     \item \code{household_income}: Distribution over income brackets
#'           (<10, [10,25), [25,50), [50,75), [75,100), [100,150), [150,200), >=200).
#' }
#' Row names are County_1 through County_12.
#'
#' @usage data(household_characteristics.distr)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 6-1.
#' @examples
#' data(household_characteristics.distr)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 12 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Socioeconomics \cr
#'   \strong{Symbolic format} \tab Distribution \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets distribution household
#' @source Billard, L. and Diday, E. (2020), Table 6-1.
"household_characteristics.distr"

## ---------------------------------------------------------------------------
## 7.16 county_income_gender.hist
## ---------------------------------------------------------------------------

#' @name county_income_gender.hist
#' @title County Income by Gender Histogram-Valued Dataset
#' @description
#' Histogram-valued dataset of 12 counties with gender-stratified income
#' histograms and sample sizes. Each county has a male income histogram,
#' a female income histogram, and the number of respondents in each group.
#'
#' @format A data frame with 12 observations (counties) and 4 variables:
#' \itemize{
#'     \item \code{male_income}: Histogram of male household income
#'           (4 bins from $0 to $100k).
#'     \item \code{female_income}: Histogram of female household income
#'           (4 bins from $0 to $100k).
#'     \item \code{n_males}: Number of male respondents (numeric).
#'     \item \code{n_females}: Number of female respondents (numeric).
#' }
#' Row names are County_1 through County_12.
#'
#' @usage data(county_income_gender.hist)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 6-16.
#' @examples
#' data(county_income_gender.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 12 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Economics \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets histogram income gender
#' @source Billard, L. and Diday, E. (2020), Table 6-16.
"county_income_gender.hist"

## ---------------------------------------------------------------------------
## 7.17 joggers.mix
## ---------------------------------------------------------------------------

#' @name joggers.mix
#' @title Joggers Mixed Symbolic Dataset
#' @description
#' Mixed symbolic dataset of 10 jogger groups with one interval-valued
#' variable (pulse rate) and one histogram-valued variable (running time
#' distribution).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' (jogger groups) and 2 variables:
#' \itemize{
#'     \item \code{pulse_rate}: Interval-valued resting pulse rate range (bpm).
#'     \item \code{running_time}: Histogram-valued distribution of running
#'           times (minutes).
#' }
#' Row names are Group_1 through Group_10.
#'
#' @usage data(joggers.mix)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 2-5.
#' @examples
#' data(joggers.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 2 \cr
#'   \strong{Subject area} \tab Sports \cr
#'   \strong{Symbolic format} \tab Mixed (interval, histogram) \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets mixed interval histogram
#' @source Billard, L. and Diday, E. (2020), Table 2-5.
"joggers.mix"

## ---------------------------------------------------------------------------
## 7.18 census.mix
## ---------------------------------------------------------------------------

#' @name census.mix
#' @title Census Mixed Symbolic Dataset
#' @description
#' Mixed symbolic dataset of 10 census regions combining 6 different symbolic
#' variable types: histograms (age, home value), distributions (gender,
#' tenure), a multi-valued set (fuel), and an interval (income).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' (regions) and 6 variables:
#' \itemize{
#'     \item \code{age}: Histogram-valued age distribution (12 age bins).
#'     \item \code{home_value}: Histogram-valued home value distribution
#'           (7 value bins, in $1000s).
#'     \item \code{gender}: Distribution over gender (male, female).
#'     \item \code{fuel}: Multi-valued set of fuel types used.
#'     \item \code{tenure}: Distribution over housing tenure
#'           (owner, renter, vacant).
#'     \item \code{income}: Interval-valued household income range ($1000s).
#' }
#' Row names are Region_1 through Region_10.
#'
#' @usage data(census.mix)
#' @references
#' Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
#' Symbolic Data}. Wiley, Chichester. Table 7-23.
#' @examples
#' data(census.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 6 \cr
#'   \strong{Subject area} \tab Demographics \cr
#'   \strong{Symbolic format} \tab Mixed (interval, histogram, distribution, multi-valued) \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets mixed interval histogram distribution
#' @source Billard, L. and Diday, E. (2020), Table 7-23.
"census.mix"

## ---------------------------------------------------------------------------
## 7.19 mtcars.mix
## ---------------------------------------------------------------------------

#' @name mtcars.mix
#' @title Motor Trend Cars Mixed Symbolic Dataset
#' @description
#' Mixed symbolic dataset of 5 car groups from the \code{mtcars} data,
#' with 7 interval-valued performance variables and 4 modal-valued
#' categorical variables.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 5 observations
#' (car groups) and 11 variables:
#' \itemize{
#'     \item \code{mpg}: Interval-valued miles per gallon.
#'     \item \code{cyl}: Modal-valued number of cylinders.
#'     \item \code{disp}: Interval-valued displacement (cu.in.).
#'     \item \code{hp}: Interval-valued horsepower.
#'     \item \code{drat}: Interval-valued rear axle ratio.
#'     \item \code{wt}: Interval-valued weight (1000 lbs).
#'     \item \code{qsec}: Interval-valued quarter-mile time (seconds).
#'     \item \code{vs}: Modal-valued engine type (V/S).
#'     \item \code{am}: Modal-valued transmission type (auto/manual).
#'     \item \code{gear}: Modal-valued number of forward gears.
#'     \item \code{carb}: Modal-valued number of carburetors.
#' }
#'
#' @usage data(mtcars.mix)
#' @references
#' Henderson, R. and Velleman, P. (1981). Building multiple regression
#' models interactively. \emph{Biometrics}, 37, 391--411.
#'
#' Original data from the ggInterval R package (\code{mtcars.i} dataset).
#' @examples
#' data(mtcars.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 5 \cr
#'   \strong{Variables (p)} \tab 11 \cr
#'   \strong{Subject area} \tab Automotive \cr
#'   \strong{Symbolic format} \tab Mixed (interval, modal) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets mixed interval modal
#' @source ggInterval R package (\code{mtcars.i} dataset).
"mtcars.mix"

## ---------------------------------------------------------------------------
## SECTION 8: Additional datasets from R packages and SDA literature (2026)
## ---------------------------------------------------------------------------

## ---------------------------------------------------------------------------
## 8.1 utsnow.int
## ---------------------------------------------------------------------------

#' @name utsnow.int
#' @title Utah Snow Load Interval Dataset
#' @description
#' Interval-valued ground snow load data from 415 weather stations in Utah
#' and surrounding states. Each observation is a station with a 50-year
#' ground snow load interval (lower and upper bounds of the prediction
#' interval in kPa) plus the point estimate, geographic coordinates, and
#' elevation.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 415 observations
#' and 5 variables:
#' \itemize{
#'     \item \code{snow_load}: Interval-valued 50-year ground snow load (kPa).
#'     \item \code{point_estimate}: Numeric point estimate (kPa).
#'     \item \code{latitude}: Numeric latitude (degrees).
#'     \item \code{longitude}: Numeric longitude (degrees).
#'     \item \code{elevation}: Numeric elevation (meters).
#' }
#'
#' @usage data(utsnow.int)
#' @references
#' Schmoyer, R. L. (1993). Permutation tests for correlation in regression
#' errors. \emph{Journal of the American Statistical Association}, 89(428),
#' 1507--1516.
#'
#' Bean, B., Sun, Y., and Maguire, M. (2022). Interval-valued kriging models
#' for geostatistical mapping with uncertain inputs.
#'
#' Original data from the intkrige R package (\code{utsnow} dataset).
#' @examples
#' data(utsnow.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 415 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression, Spatial analysis
#' }
#' @keywords datasets interval
#' @source intkrige R package (\code{utsnow} dataset).
"utsnow.int"

## ---------------------------------------------------------------------------
## 8.2 lynne1.int
## ---------------------------------------------------------------------------

#' @name lynne1.int
#' @title Lynne1 Blood Pressure Interval Dataset
#' @description
#' Interval-valued dataset of 10 observations with pulse rate, systolic
#' pressure, and diastolic pressure intervals.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' and 4 variables:
#' \itemize{
#'     \item \code{concept}: Character concept label.
#'     \item \code{Pulse Rate}: Interval-valued pulse rate (beats/min).
#'     \item \code{Systolic Pressure}: Interval-valued systolic pressure (mmHg).
#'     \item \code{Diastolic Pressure}: Interval-valued diastolic pressure (mmHg).
#' }
#'
#' @usage data(lynne1.int)
#' @references
#' Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis: Conceptual
#' Statistics and Data Mining}. Wiley, Chichester.
#'
#' Original data from the RSDA R package (\code{Lynne1} dataset).
#' @examples
#' data(lynne1.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Regression
#' }
#' @keywords datasets interval
#' @source RSDA R package (\code{Lynne1} dataset).
"lynne1.int"

## ---------------------------------------------------------------------------
## 8.3 loans_by_risk_quantile.int
## ---------------------------------------------------------------------------

#' @name loans_by_risk_quantile.int
#' @title Lending Club Loans by Risk Level (Quantile-Based Intervals)
#' @description
#' Interval-valued dataset of 35 Lending Club loan groups stratified by risk
#' level (A1--G5). Intervals represent the 10th to 90th percentile range of
#' each financial variable within each risk subgrade.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 35 observations
#' and 4 variables:
#' \itemize{
#'     \item \code{ln-inc}: Interval-valued log income.
#'     \item \code{int-rate}: Interval-valued interest rate.
#'     \item \code{open-acc}: Interval-valued number of open accounts.
#'     \item \code{total-acc}: Interval-valued total accounts.
#' }
#'
#' @usage data(loans_by_risk_quantile.int)
#' @references
#' Brito, P. and Duarte Silva, A.P. (2012). Modelling interval data with
#' Normal and Skew-Normal distributions. \emph{Journal of Applied Statistics},
#' 39(1), 3--20.
#'
#' Original data from the MAINT.Data R package
#' (\code{LoansbyRiskLvs_qntlDt} dataset).
#' @examples
#' data(loans_by_risk_quantile.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 35 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Classification, Clustering
#' }
#' @keywords datasets interval
#' @source MAINT.Data R package (\code{LoansbyRiskLvs_qntlDt} dataset).
"loans_by_risk_quantile.int"

## ---------------------------------------------------------------------------
## 8.4 judge1.int, judge2.int, judge3.int
## ---------------------------------------------------------------------------

#' @name judge1.int
#' @title Judge 1 Interval-Valued Ratings
#' @description
#' Interval-valued ratings from Judge 1 for 6 regions on 4 variables.
#' From a study of generalized principal component analysis for
#' interval-valued data (GPCSIV).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 6 observations
#' and 4 interval-valued variables (V1--V4).
#'
#' @usage data(judge1.int)
#' @references
#' Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
#' to symbolic histogram variables. \emph{Advances in Data Analysis and
#' Classification}, 6(2), 147--159.
#'
#' Original data from the GPCSIV R package (\code{Judge1} dataset).
#' @examples
#' data(judge1.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval
#' @source GPCSIV R package (\code{Judge1} dataset).
"judge1.int"

#' @name judge2.int
#' @title Judge 2 Interval-Valued Ratings
#' @description
#' Interval-valued ratings from Judge 2 for 6 regions on 4 variables.
#' From a study of generalized principal component analysis for
#' interval-valued data (GPCSIV).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 6 observations
#' and 4 interval-valued variables (V1--V4).
#'
#' @usage data(judge2.int)
#' @references
#' Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
#' to symbolic histogram variables. \emph{Advances in Data Analysis and
#' Classification}, 6(2), 147--159.
#'
#' Original data from the GPCSIV R package (\code{Judge2} dataset).
#' @examples
#' data(judge2.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval
#' @source GPCSIV R package (\code{Judge2} dataset).
"judge2.int"

#' @name judge3.int
#' @title Judge 3 Interval-Valued Ratings
#' @description
#' Interval-valued ratings from Judge 3 for 6 regions on 4 variables.
#' From a study of generalized principal component analysis for
#' interval-valued data (GPCSIV).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 6 observations
#' and 4 interval-valued variables (V1--V4).
#'
#' @usage data(judge3.int)
#' @references
#' Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
#' to symbolic histogram variables. \emph{Advances in Data Analysis and
#' Classification}, 6(2), 147--159.
#'
#' Original data from the GPCSIV R package (\code{Judge3} dataset).
#' @examples
#' data(judge3.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 6 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval
#' @source GPCSIV R package (\code{Judge3} dataset).
"judge3.int"

## ---------------------------------------------------------------------------
## 8.5 video1.int, video2.int, video3.int
## ---------------------------------------------------------------------------

#' @name video1.int
#' @title Video Platform User Engagement Intervals (Dataset 1)
#' @description
#' Interval-valued engagement metrics for 10 user groups on a video
#' platform. Variables represent ranges of visit, watch, like, comment,
#' and share counts.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' and 5 interval-valued variables (V1--V5): number of visits, watches,
#' likes, comments, and shares.
#'
#' @usage data(video1.int)
#' @references
#' Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
#' to symbolic histogram variables. \emph{Advances in Data Analysis and
#' Classification}, 6(2), 147--159.
#'
#' Original data from the GPCSIV R package (\code{video1} dataset).
#' @examples
#' data(video1.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Digital media \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval
#' @source GPCSIV R package (\code{video1} dataset).
"video1.int"

#' @name video2.int
#' @title Video Platform User Engagement Intervals (Dataset 2)
#' @description
#' Interval-valued engagement metrics for 10 user groups on a video
#' platform. Variables represent ranges of visit, watch, like, comment,
#' and share counts.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' and 5 interval-valued variables (V1--V5): number of visits, watches,
#' likes, comments, and shares.
#'
#' @usage data(video2.int)
#' @references
#' Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
#' to symbolic histogram variables. \emph{Advances in Data Analysis and
#' Classification}, 6(2), 147--159.
#'
#' Original data from the GPCSIV R package (\code{video2} dataset).
#' @examples
#' data(video2.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Digital media \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval
#' @source GPCSIV R package (\code{video2} dataset).
"video2.int"

#' @name video3.int
#' @title Video Platform User Engagement Intervals (Dataset 3)
#' @description
#' Interval-valued engagement metrics for 10 user groups on a video
#' platform. Variables represent ranges of visit, watch, like, comment,
#' and share counts.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' and 5 interval-valued variables (V1--V5): number of visits, watches,
#' likes, comments, and shares.
#'
#' @usage data(video3.int)
#' @references
#' Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
#' to symbolic histogram variables. \emph{Advances in Data Analysis and
#' Classification}, 6(2), 147--159.
#'
#' Original data from the GPCSIV R package (\code{video3} dataset).
#' @examples
#' data(video3.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 5 \cr
#'   \strong{Subject area} \tab Digital media \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab PCA
#' }
#' @keywords datasets interval
#' @source GPCSIV R package (\code{video3} dataset).
"video3.int"

## ---------------------------------------------------------------------------
## 8.6 lisbon_air_quality.int
## ---------------------------------------------------------------------------

#' @name lisbon_air_quality.int
#' @title Lisbon Air Quality Daily Interval Dataset
#' @description
#' Interval-valued daily air quality data from the Entrecampos monitoring
#' station in Lisbon, Portugal, covering 2019--2021 (1096 days). Each day's
#' pollutant concentration is represented as a \eqn{[\min, \max]} interval
#' from hourly measurements. Missing days are imputed via linear
#' interpolation.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 1096 observations
#' (daily) and 8 interval-valued pollutant variables:
#' \itemize{
#'     \item \code{so2}: Sulphur dioxide (ug/m3).
#'     \item \code{pm10}: Particulate matter < 10 um (ug/m3).
#'     \item \code{o3}: Ozone (ug/m3).
#'     \item \code{no2}: Nitrogen dioxide (ug/m3).
#'     \item \code{co}: Carbon monoxide (ug/m3).
#'     \item \code{pm25}: Particulate matter < 2.5 um (ug/m3).
#'     \item \code{nox}: Nitrogen oxides (ug/m3).
#'     \item \code{no}: Nitric oxide (ug/m3).
#' }
#'
#' @usage data(lisbon_air_quality.int)
#' @references
#' Dias, S. and Brito, P. (2017). Off the beaten track: A new linear model
#' for interval data. \emph{European Journal of Operational Research},
#' 258(3), 1118--1130.
#'
#' Data from the QualAr Portuguese air quality monitoring network
#' (\samp{https://qualar.apambiente.pt/}).
#' @examples
#' data(lisbon_air_quality.int)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 1096 \cr
#'   \strong{Variables (p)} \tab 8 \cr
#'   \strong{Subject area} \tab Environment \cr
#'   \strong{Symbolic format} \tab Interval \cr
#'   \strong{Analytical tasks} \tab Regression, Time series
#' }
#' @keywords datasets interval
#' @source QualAr, Entrecampos station, Lisbon, Portugal.
"lisbon_air_quality.int"

## ---------------------------------------------------------------------------
## 8.7 polish_cars.mix
## ---------------------------------------------------------------------------

#' @name polish_cars.mix
#' @title Polish Car Models Mixed Symbolic Dataset
#' @description
#' Mixed symbolic dataset of 30 car models sold in Poland, with 9
#' interval-valued technical specification variables and 3 multinomial-valued
#' categorical variables.
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 30 observations
#' and 12 variables:
#' \itemize{
#'     \item \code{price}: Interval-valued price (PLN).
#'     \item \code{body}: Multinomial body types (e.g., hatchback, sedan, combi).
#'     \item \code{wheelbase}: Interval-valued wheelbase (mm).
#'     \item \code{chassis_length}: Interval-valued chassis length (mm).
#'     \item \code{chassis_width}: Interval-valued chassis width (mm).
#'     \item \code{chassis_height}: Interval-valued chassis height (mm).
#'     \item \code{engine_capacity}: Multinomial engine displacement categories (litres).
#'     \item \code{engine_power}: Interval-valued engine power (HP).
#'     \item \code{maximum_speed}: Interval-valued maximum speed (km/h).
#'     \item \code{acceleration}: Interval-valued 0--100 km/h time (seconds).
#'     \item \code{fuel_type}: Multinomial fuel types (petrol, diesel, LPG).
#'     \item \code{fuel_consumption}: Interval-valued fuel consumption (L/100km).
#' }
#'
#' @usage data(polish_cars.mix)
#' @references
#' Dudek, A. and Pelka, M. (2012). \emph{symbolicDA: Analysis of Symbolic
#' Data}. R package.
#' @examples
#' data(polish_cars.mix)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 30 \cr
#'   \strong{Variables (p)} \tab 12 \cr
#'   \strong{Subject area} \tab Automotive \cr
#'   \strong{Symbolic format} \tab Mixed (interval, multinomial) \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets mixed interval multinomial
#' @source symbolicDA R package (\code{cars} dataset).
"polish_cars.mix"

## ---------------------------------------------------------------------------
## 8.8 blood.hist
## ---------------------------------------------------------------------------

#' @name blood.hist
#' @title Blood Test Histogram Dataset
#' @description
#' Histogram-valued blood test results for 14 gender-age groups (e.g.,
#' Female-20, Male-50). Each observation contains histograms for
#' cholesterol, hemoglobin, and hematocrit, represented as multi-bin
#' distributions.
#'
#' @format A data frame with 14 observations and 3 histogram-valued
#' variables:
#' \itemize{
#'     \item \code{Cholesterol}: Histogram of cholesterol levels (mg/dL).
#'     \item \code{Hemoglobin}: Histogram of hemoglobin levels (g/dL).
#'     \item \code{Hematocrit}: Histogram of hematocrit levels (\%).
#' }
#'
#' @usage data(blood.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: a new metric-based approach. \emph{Advances in
#' Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{BLOOD} dataset).
#' @examples
#' data(blood.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 14 \cr
#'   \strong{Variables (p)} \tab 3 \cr
#'   \strong{Subject area} \tab Medical \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics, Clustering
#' }
#' @keywords datasets histogram
#' @source HistDAWass R package (\code{BLOOD} dataset).
"blood.hist"

## ---------------------------------------------------------------------------
## 8.9 china_climate_month.hist
## ---------------------------------------------------------------------------

#' @name china_climate_month.hist
#' @title Chinese Climate Monthly Histogram Dataset
#' @description
#' Histogram-valued monthly climate data for 60 Chinese weather stations.
#' Each station has 14 climate variables measured across 12 months
#' (168 histogram columns total). Histograms are reduced to 10 decile
#' bins from the original HistDAWass distributions.
#'
#' @format A data frame with 60 observations (stations) and 168
#' histogram-valued variables. Variables follow the pattern
#' \code{variable_Month} (e.g., \code{mean.temp_Jan}). The 14 climate
#' variables are: mean pressure, mean temperature, mean max/min
#' temperature, total precipitation, sunshine duration, mean cloud amount,
#' mean relative humidity, snow days, dominant wind direction, mean wind
#' speed, dominant wind frequency, extreme max/min temperature.
#'
#' @usage data(china_climate_month.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: a new metric-based approach. \emph{Advances in
#' Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{China_Month} dataset).
#' @examples
#' data(china_climate_month.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 60 \cr
#'   \strong{Variables (p)} \tab 168 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets histogram
#' @source HistDAWass R package (\code{China_Month} dataset).
"china_climate_month.hist"

## ---------------------------------------------------------------------------
## 8.10 china_climate_season.hist
## ---------------------------------------------------------------------------

#' @name china_climate_season.hist
#' @title Chinese Climate Seasonal Histogram Dataset
#' @description
#' Histogram-valued seasonal climate data for 60 Chinese weather stations.
#' Each station has 14 climate variables measured across 4 seasons
#' (56 histogram columns total). Histograms are reduced to 10 decile
#' bins from the original HistDAWass distributions.
#'
#' @format A data frame with 60 observations (stations) and 56
#' histogram-valued variables. Variables follow the pattern
#' \code{variable_Season} (e.g., \code{mean.temp_Spring}). The 14 climate
#' variables are: mean pressure, mean temperature, mean max/min
#' temperature, total precipitation, sunshine duration, mean cloud amount,
#' mean relative humidity, snow days, dominant wind direction, mean wind
#' speed, dominant wind frequency, extreme max/min temperature.
#'
#' @usage data(china_climate_season.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: a new metric-based approach. \emph{Advances in
#' Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{China_Seas} dataset).
#' @examples
#' data(china_climate_season.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 60 \cr
#'   \strong{Variables (p)} \tab 56 \cr
#'   \strong{Subject area} \tab Climate \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Clustering
#' }
#' @keywords datasets histogram
#' @source HistDAWass R package (\code{China_Seas} dataset).
"china_climate_season.hist"

## ---------------------------------------------------------------------------
## 8.11 exchange_rate_returns.hist
## ---------------------------------------------------------------------------

#' @name exchange_rate_returns.hist
#' @title Exchange Rate Returns Histogram Time Series
#' @description
#' Histogram-valued time series of 108 monthly observations of daily
#' exchange rate returns. Each observation is a histogram distribution
#' of intra-month daily returns.
#'
#' @format A data frame with 108 observations and 1 histogram-valued
#' variable:
#' \itemize{
#'     \item \code{returns}: Histogram of daily exchange rate returns
#'     within each month.
#' }
#'
#' @usage data(exchange_rate_returns.hist)
#' @references
#' Irpino, A. and Verde, R. (2015). Basic statistics for distributional
#' symbolic variables: a new metric-based approach. \emph{Advances in
#' Data Analysis and Classification}, 9(2), 143--175.
#'
#' Original data from the HistDAWass R package (\code{RetHTS} dataset).
#' @examples
#' data(exchange_rate_returns.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 108 \cr
#'   \strong{Variables (p)} \tab 1 \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Histogram \cr
#'   \strong{Analytical tasks} \tab Time series, Descriptive statistics
#' }
#' @keywords datasets histogram
#' @source HistDAWass R package (\code{RetHTS} dataset).
"exchange_rate_returns.hist"

## ---------------------------------------------------------------------------
## 8.12 hierarchy.hist
## ---------------------------------------------------------------------------

#' @name hierarchy.hist
#' @title Hierarchical Symbolic Dataset with Mixed Types
#' @description
#' Mixed symbolic dataset of 10 observations with hierarchical categorical
#' variables, conditional histogram variables, and an interval-valued
#' variable. From Table 6.20 of Billard and Diday (2007).
#'
#' @format A symbolic data frame (\code{symbolic_tbl}) with 10 observations
#' and 7 variables:
#' \itemize{
#'     \item \code{duration_time}: Histogram-valued duration (2-bin).
#'     \item \code{hierarchy_1}: Categorical hierarchy level 1 (a/b/c).
#'     \item \code{hierarchy_2}: Categorical hierarchy level 2 (a1/a2), conditional on hierarchy_1 = a.
#'     \item \code{hierarchy_3}: Categorical hierarchy level 3 (a11/a12), conditional on hierarchy_2 = a1.
#'     \item \code{glucose}: Histogram-valued glucose (2-bin), conditional.
#'     \item \code{pulse_rate}: Histogram-valued pulse rate (2-bin), conditional.
#'     \item \code{cholesterol}: Interval-valued cholesterol level.
#' }
#'
#' @usage data(hierarchy.hist)
#' @references
#' Billard, L. and Diday, E. (2007). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 6.20.
#' @examples
#' data(hierarchy.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 10 \cr
#'   \strong{Variables (p)} \tab 7 \cr
#'   \strong{Subject area} \tab Methodology \cr
#'   \strong{Symbolic format} \tab Mixed (histogram, interval, categorical) \cr
#'   \strong{Analytical tasks} \tab Descriptive statistics
#' }
#' @keywords datasets mixed histogram interval
#' @source Billard, L. and Diday, E. (2007), Table 6.20.
"hierarchy.hist"

## ---------------------------------------------------------------------------
## 8.13 bird_color_taxonomy.hist
## ---------------------------------------------------------------------------

#' @name bird_color_taxonomy.hist
#' @title Bird Color Taxonomy Histogram Dataset
#' @description
#' Mixed symbolic dataset of 20 bird observations with histogram-valued
#' feather density and body size, categorical tone, and distribution-valued
#' shade (fuzzy taxonomy). From Tables 6.9 and 6.14 of Billard and Diday
#' (2007).
#'
#' @format A data frame with 20 observations and 4 variables:
#' \itemize{
#'     \item \code{density}: Histogram-valued feather density (up to 4 bins).
#'     \item \code{size}: Histogram-valued body size (2-bin).
#'     \item \code{tone}: Categorical tone (dark/light).
#'     \item \code{shade}: Distribution-valued shade (purple/red/white/yellow
#'     with fuzzy weights).
#' }
#'
#' @usage data(bird_color_taxonomy.hist)
#' @references
#' Billard, L. and Diday, E. (2007). \emph{Symbolic Data Analysis:
#' Conceptual Statistics and Data Mining}. Wiley, Chichester. Tables 6.9
#' and 6.14.
#' @examples
#' data(bird_color_taxonomy.hist)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 20 \cr
#'   \strong{Variables (p)} \tab 4 \cr
#'   \strong{Subject area} \tab Zoology \cr
#'   \strong{Symbolic format} \tab Mixed (histogram, categorical, distribution) \cr
#'   \strong{Analytical tasks} \tab Clustering, Descriptive statistics
#' }
#' @keywords datasets mixed histogram distribution
#' @source Billard, L. and Diday, E. (2007), Tables 6.9/6.14.
"bird_color_taxonomy.hist"

## ===========================================================================
## SECTION 9: Interval time series (ITS) datasets from financial markets
## ===========================================================================
## These datasets contain daily (or weekly) high/low price intervals for
## financial indices, commodities, exchange rates, and stocks. They are
## widely used as benchmarks in interval time series forecasting research.
## Naming convention: name.its (interval time series)
## ===========================================================================

## ---------------------------------------------------------------------------
## sp500.its - S&P 500 daily high/low (Arroyo et al. 2011)
## ---------------------------------------------------------------------------

#' @name sp500.its
#' @title S&P 500 Daily High/Low Interval Time Series
#' @description
#' Daily high and low prices of the S&P 500 index from January 2, 2004 to
#' December 30, 2005 (504 trading days). This dataset is a benchmark for
#' interval time series forecasting, matching the period used in the
#' foundational work by Arroyo, Gonzalez-Rivera and Mate (2011).
#'
#' @details
#' The S&P 500 is a market-capitalization-weighted index of 500 leading
#' publicly traded companies in the United States. Each observation
#' represents a trading day with the daily low and high prices forming an
#' interval. This dataset has been widely used to evaluate interval-valued
#' autoregressive models, exponential smoothing methods for intervals, and
#' center-and-range forecasting approaches.
#'
#' @usage data(sp500.its)
#' @format A data frame with 504 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low price of the S&P 500 index.
#'     \item \code{high}: Daily high price of the S&P 500 index.
#' }
#'
#' @references
#' Arroyo, J., Gonzalez-Rivera, G. and Mate, C. (2011). Forecasting with
#' interval and histogram data: Some financial applications. In
#' \emph{Handbook of Empirical Economics and Finance}, pp. 247--280.
#' Chapman and Hall/CRC.
#'
#' @source Yahoo Finance, ticker \code{^GSPC}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(sp500.its)
#' head(sp500.its)
#' plot(sp500.its$date, sp500.its$high, type = "l", col = "red",
#'      ylab = "Price", xlab = "Date", main = "S&P 500 Daily High/Low")
#' lines(sp500.its$date, sp500.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 504 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance
"sp500.its"

## ---------------------------------------------------------------------------
## djia.its - Dow Jones Industrial Average daily high/low (Arroyo et al. 2011)
## ---------------------------------------------------------------------------

#' @name djia.its
#' @title Dow Jones Industrial Average Daily High/Low Interval Time Series
#' @description
#' Daily high and low prices of the Dow Jones Industrial Average (DJIA) from
#' January 2, 2004 to December 30, 2005 (504 trading days). This dataset
#' matches the period used in the foundational interval time series work by
#' Arroyo, Gonzalez-Rivera and Mate (2011).
#'
#' @details
#' The DJIA is a price-weighted index of 30 prominent companies listed on
#' stock exchanges in the United States. Each observation represents a
#' trading day with the daily low and high prices forming an interval. This
#' dataset has been used alongside the S&P 500 to compare interval
#' forecasting methods.
#'
#' @usage data(djia.its)
#' @format A data frame with 504 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low price of the DJIA.
#'     \item \code{high}: Daily high price of the DJIA.
#' }
#'
#' @references
#' Arroyo, J., Gonzalez-Rivera, G. and Mate, C. (2011). Forecasting with
#' interval and histogram data: Some financial applications. In
#' \emph{Handbook of Empirical Economics and Finance}, pp. 247--280.
#' Chapman and Hall/CRC.
#'
#' @source Yahoo Finance, ticker \code{^DJI}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(djia.its)
#' head(djia.its)
#' plot(djia.its$date, djia.its$high, type = "l", col = "red",
#'      ylab = "Price", xlab = "Date", main = "DJIA Daily High/Low")
#' lines(djia.its$date, djia.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 504 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance
"djia.its"

## ---------------------------------------------------------------------------
## ibovespa.its - Brazilian IBOVESPA daily high/low (Maciel et al. 2016)
## ---------------------------------------------------------------------------

#' @name ibovespa.its
#' @title IBOVESPA Daily High/Low Interval Time Series
#' @description
#' Daily high and low values of the Brazilian IBOVESPA stock market index
#' from January 3, 2000 to December 28, 2012 (3216 trading days). This
#' dataset matches the period used by Maciel, Ballini and Gomide (2016) for
#' evolving granular analytics for interval time series forecasting.
#'
#' @details
#' The IBOVESPA (Indice Bovespa) is the benchmark index of the Brazilian
#' stock exchange (B3, formerly BM&FBOVESPA). It tracks the performance of
#' the most actively traded stocks on the Sao Paulo stock exchange. The
#' 13-year span of this dataset covers multiple market regimes including the
#' 2008 global financial crisis, making it suitable for evaluating
#' forecasting models under diverse conditions.
#'
#' @usage data(ibovespa.its)
#' @format A data frame with 3216 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low value of the IBOVESPA index.
#'     \item \code{high}: Daily high value of the IBOVESPA index.
#' }
#'
#' @references
#' Maciel, L., Ballini, R. and Gomide, F. (2016). Evolving granular
#' analytics for interval time series forecasting. \emph{Granular Computing},
#' \strong{1}(4), 213--224.
#'
#' @source Yahoo Finance, ticker \code{^BVSP}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(ibovespa.its)
#' head(ibovespa.its)
#' plot(ibovespa.its$date, ibovespa.its$high, type = "l", col = "red",
#'      ylab = "Index Value", xlab = "Date",
#'      main = "IBOVESPA Daily High/Low (2000-2012)")
#' lines(ibovespa.its$date, ibovespa.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 3216 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance
"ibovespa.its"

## ---------------------------------------------------------------------------
## crude_oil_wti.its - WTI Crude Oil Futures daily high/low (Yang et al. 2016)
## ---------------------------------------------------------------------------

#' @name crude_oil_wti.its
#' @title WTI Crude Oil Futures Daily High/Low Interval Time Series
#' @description
#' Daily high and low prices of WTI (West Texas Intermediate) crude oil
#' futures from January 2, 2003 to December 30, 2011 (2261 trading days).
#' This dataset matches the period used by Yang, Han, Hong and Wang (2016)
#' for analyzing crisis impacts on crude oil prices using interval time
#' series modelling.
#'
#' @details
#' WTI crude oil is a benchmark for oil prices in the Americas. This
#' dataset covers a period that includes the 2003 Iraq War, the 2007--2008
#' oil price spike (reaching nearly USD 150/barrel), the 2008 global
#' financial crisis, and the subsequent recovery. The wide variation in
#' price levels and volatility regimes makes this dataset ideal for
#' evaluating interval time series models under structural breaks.
#'
#' @usage data(crude_oil_wti.its)
#' @format A data frame with 2261 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low price (USD per barrel).
#'     \item \code{high}: Daily high price (USD per barrel).
#' }
#'
#' @references
#' Yang, W., Han, A., Hong, Y. and Wang, S. (2016). Analysis of crisis
#' impact on crude oil prices: A new approach with interval time series
#' modelling. \emph{Quantitative Finance}, \strong{16}(12), 1917--1928.
#'
#' @source Yahoo Finance, ticker \code{CL=F}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(crude_oil_wti.its)
#' head(crude_oil_wti.its)
#' plot(crude_oil_wti.its$date, crude_oil_wti.its$high, type = "l",
#'      col = "red", ylab = "Price (USD/barrel)", xlab = "Date",
#'      main = "WTI Crude Oil Daily High/Low (2003-2011)")
#' lines(crude_oil_wti.its$date, crude_oil_wti.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 2261 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance / Commodities \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Structural break analysis
#' }
#' @keywords datasets interval timeseries finance commodities
"crude_oil_wti.its"

## ---------------------------------------------------------------------------
## merval.its - MERVAL Index weekly min/max (de Carvalho & Martos 2022)
## ---------------------------------------------------------------------------

#' @name merval.its
#' @title MERVAL Index Weekly Min/Max Interval Time Series
#' @description
#' Weekly minimum and maximum values of the Argentine MERVAL stock market
#' index from January 4, 2016 to September 28, 2020 (248 weeks). Daily
#' data was downloaded and aggregated to weekly intervals. This dataset
#' matches the period used by de Carvalho and Martos (2022).
#'
#' @details
#' The MERVAL (Mercado de Valores de Buenos Aires) is the main stock market
#' index of the Buenos Aires Stock Exchange. Each observation represents one
#' week, with the weekly low computed as the minimum of daily lows and the
#' weekly high computed as the maximum of daily highs. The date column
#' indicates the Monday (start) of each week. This period covers the
#' Argentine economic crisis and the early COVID-19 pandemic impact.
#'
#' @usage data(merval.its)
#' @format A data frame with 248 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Week start date, Monday (Date class).
#'     \item \code{low}: Weekly minimum of daily low values.
#'     \item \code{high}: Weekly maximum of daily high values.
#' }
#'
#' @references
#' de Carvalho, F. A. T. and Martos, G. (2022). Modeling interval trendlines:
#' Symbolic singular spectrum analysis for interval time series. \emph{Journal
#' of Forecasting}, \strong{41}(1), 167--180.
#'
#' @source Yahoo Finance, ticker \code{^MERV}. Downloaded via the
#' \pkg{quantmod} package and aggregated from daily to weekly.
#' @examples
#' data(merval.its)
#' head(merval.its)
#' plot(merval.its$date, merval.its$high, type = "l", col = "red",
#'      ylab = "Index Value", xlab = "Date",
#'      main = "MERVAL Weekly Min/Max (2016-2020)")
#' lines(merval.its$date, merval.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 248 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval time series (weekly aggregation) \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance
"merval.its"

## ---------------------------------------------------------------------------
## petrobras.its - Petrobras stock daily high/low (Maia et al. 2008)
## ---------------------------------------------------------------------------

#' @name petrobras.its
#' @title Petrobras Stock Daily High/Low Interval Time Series
#' @description
#' Daily high and low stock prices of Petrobras (ADR traded on NYSE) from
#' January 3, 2005 to December 29, 2006 (503 trading days). This dataset
#' matches the period used by Maia, de Carvalho and Ludermir (2008) in
#' their work on forecasting models for interval-valued time series.
#'
#' @details
#' Petrobras (Petroleo Brasileiro S.A.) is the Brazilian multinational
#' petroleum corporation. The ADR (American Depositary Receipt) is traded
#' on the New York Stock Exchange under ticker PBR. Each observation
#' represents a trading day with the daily low and high prices forming an
#' interval. This was one of the first datasets used to demonstrate
#' interval-valued autoregressive (iAR) models.
#'
#' @usage data(petrobras.its)
#' @format A data frame with 503 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low price (USD).
#'     \item \code{high}: Daily high price (USD).
#' }
#'
#' @references
#' Maia, A. L. S., de Carvalho, F. A. T. and Ludermir, T. B. (2008).
#' Forecasting models for interval-valued time series. \emph{Neurocomputing},
#' \strong{71}(16--18), 3344--3352.
#'
#' @source Yahoo Finance, ticker \code{PBR}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(petrobras.its)
#' head(petrobras.its)
#' plot(petrobras.its$date, petrobras.its$high, type = "l", col = "red",
#'      ylab = "Price (USD)", xlab = "Date",
#'      main = "Petrobras Daily High/Low (2005-2006)")
#' lines(petrobras.its$date, petrobras.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 503 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance
"petrobras.its"

## ---------------------------------------------------------------------------
## euro_usd.its - Euro/Dollar exchange rate daily high/low
## (Arroyo, Espinola & Mate 2011)
## ---------------------------------------------------------------------------

#' @name euro_usd.its
#' @title Euro/Dollar Exchange Rate Daily High/Low Interval Time Series
#' @description
#' Daily high and low values of the EUR/USD exchange rate from January 1,
#' 2004 to December 30, 2005 (520 trading days). Inspired by the dataset
#' used by Arroyo, Espinola and Mate (2011) for exponential smoothing
#' methods for interval time series.
#'
#' @details
#' The EUR/USD exchange rate is the most traded currency pair in the world
#' foreign exchange market. Each observation represents a trading day with
#' the daily low and high exchange rates (USD per EUR) forming an interval.
#' Note: the original study by Arroyo et al. (2011) used the period
#' 2002--2003 (519 trading days); this dataset covers 2004--2005 because
#' Yahoo Finance historical data for this ticker is only available from
#' late 2003 onward.
#'
#' @usage data(euro_usd.its)
#' @format A data frame with 520 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low EUR/USD exchange rate.
#'     \item \code{high}: Daily high EUR/USD exchange rate.
#' }
#'
#' @references
#' Arroyo, J., Espinola, R. and Mate, C. (2011). Different approaches to
#' forecast interval time series: A comparison in finance. \emph{Computational
#' Economics}, \strong{37}(2), 169--191.
#'
#' @source Yahoo Finance, ticker \code{EURUSD=X}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(euro_usd.its)
#' head(euro_usd.its)
#' plot(euro_usd.its$date, euro_usd.its$high, type = "l", col = "red",
#'      ylab = "EUR/USD", xlab = "Date",
#'      main = "EUR/USD Daily High/Low (2004-2005)")
#' lines(euro_usd.its$date, euro_usd.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 520 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance / Foreign Exchange \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance exchange
"euro_usd.its"

## ---------------------------------------------------------------------------
## shanghai_stock.its - Shanghai Composite Index daily high/low
## (Yang, Zhang & Wang 2025)
## ---------------------------------------------------------------------------

#' @name shanghai_stock.its
#' @title Shanghai Stock Exchange Composite Index Daily High/Low Interval Time Series
#' @description
#' Daily high and low values of the Shanghai Stock Exchange Composite Index
#' (SSE Composite) from January 2, 2019 to December 30, 2022 (970 trading
#' days). This dataset matches the period used by Yang, Zhang and Wang
#' (2025) for interval time series forecasting.
#'
#' @details
#' The SSE Composite Index is the most commonly used indicator to reflect
#' the performance of the Shanghai Stock Exchange. It tracks all stocks
#' (A-shares and B-shares) listed on the exchange. This dataset covers a
#' period that includes the COVID-19 pandemic and its market impacts,
#' providing a rich testbed for evaluating interval forecasting models
#' under extreme volatility.
#'
#' @usage data(shanghai_stock.its)
#' @format A data frame with 970 observations and 3 variables:
#' \itemize{
#'     \item \code{date}: Trading date (Date class).
#'     \item \code{low}: Daily low value of the SSE Composite Index.
#'     \item \code{high}: Daily high value of the SSE Composite Index.
#' }
#'
#' @references
#' Yang, W., Zhang, S. and Wang, S. (2025). On smooth transition interval
#' autoregressive models. \emph{Journal of Forecasting}, \strong{44}(2),
#' 310--332.
#'
#' @source Yahoo Finance, ticker \code{000001.SS}. Downloaded via the
#' \pkg{quantmod} package.
#' @examples
#' data(shanghai_stock.its)
#' head(shanghai_stock.its)
#' plot(shanghai_stock.its$date, shanghai_stock.its$high, type = "l",
#'      col = "red", ylab = "Index Value", xlab = "Date",
#'      main = "Shanghai Composite Daily High/Low (2019-2022)")
#' lines(shanghai_stock.its$date, shanghai_stock.its$low, col = "blue")
#' legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 970 \cr
#'   \strong{Variables (p)} \tab 3 (date, low, high) \cr
#'   \strong{Subject area} \tab Finance \cr
#'   \strong{Symbolic format} \tab Interval time series \cr
#'   \strong{Analytical tasks} \tab Forecasting, Time series analysis
#' }
#' @keywords datasets interval timeseries finance
"shanghai_stock.its"

## ---------------------------------------------------------------------------
## 9.9 irish_wind.its - Irish wind speed monthly interval (Teles & Brito 2015)
## ---------------------------------------------------------------------------

#' @name irish_wind.its
#' @title Irish Wind Speed Monthly Interval Time Series
#' @description
#' Monthly interval-valued wind speed data at 5 meteorological stations in
#' Ireland from January 1961 to December 1978 (216 months). For each month
#' and station, the interval is defined as [minimum daily average wind speed,
#' maximum daily average wind speed] across all days in that month.
#'
#' @details
#' The original data contains daily average wind speeds (in knots) at 12
#' synoptic meteorological stations in the Republic of Ireland, collected by
#' the Irish Meteorological Service. This is the classic Haslett and Raftery
#' (1989) dataset, one of the most widely used benchmarks in spatial
#' statistics. Following the approach of Teles and Brito (2015), the raw
#' daily data is aggregated to monthly intervals for 5 selected stations:
#' Birr (BIR), Dublin Airport (DUB), Kilkenny (KIL), Shannon Airport (SHA),
#' and Valentia Observatory (VAL). Each monthly interval captures the range
#' of daily wind variability within that month.
#'
#' @usage data(irish_wind.its)
#' @format A data frame with 216 observations and 11 columns (5 interval
#' variables in \code{_l}/\code{_u} Min-Max pairs, plus a date):
#' \itemize{
#'     \item \code{date}: First day of the month (Date class).
#'     \item \code{BIR_l}, \code{BIR_u}: Monthly [min, max] daily wind speed
#'     at Birr (knots).
#'     \item \code{DUB_l}, \code{DUB_u}: Monthly [min, max] daily wind speed
#'     at Dublin Airport (knots).
#'     \item \code{KIL_l}, \code{KIL_u}: Monthly [min, max] daily wind speed
#'     at Kilkenny (knots).
#'     \item \code{SHA_l}, \code{SHA_u}: Monthly [min, max] daily wind speed
#'     at Shannon Airport (knots).
#'     \item \code{VAL_l}, \code{VAL_u}: Monthly [min, max] daily wind speed
#'     at Valentia Observatory (knots).
#' }
#'
#' @references
#' Haslett, J. and Raftery, A. E. (1989). Space-time modelling with
#' long-memory dependence: Assessing Ireland's wind power resource.
#' \emph{Journal of the Royal Statistical Society, Series C (Applied
#' Statistics)}, \strong{38}(1), 1--50.
#'
#' Teles, P. and Brito, P. (2015). Modeling interval time series with
#' space-time processes. \emph{Communications in Statistics -- Theory and
#' Methods}, \strong{44}(17), 3599--3619.
#'
#' @source Derived from the \code{wind} dataset in the \pkg{gstat} R
#' package (originally from Haslett and Raftery, 1989). Daily data
#' aggregated to monthly intervals.
#' @examples
#' data(irish_wind.its)
#' head(irish_wind.its)
#' # Plot Valentia Observatory wind speed interval
#' plot(irish_wind.its$date, irish_wind.its$VAL_u, type = "l", col = "red",
#'      ylab = "Wind speed (knots)", xlab = "Date",
#'      main = "Valentia Observatory Monthly Wind Speed Interval")
#' lines(irish_wind.its$date, irish_wind.its$VAL_l, col = "blue")
#' legend("topright", c("Max", "Min"), col = c("red", "blue"), lty = 1)
#' @section Metadata:
#' \tabular{ll}{
#'   \strong{Sample size (n)} \tab 216 \cr
#'   \strong{Variables (p)} \tab 11 \cr
#'   \strong{Subject area} \tab Meteorology \cr
#'   \strong{Symbolic format} \tab Interval time series (multivariate) \cr
#'   \strong{Analytical tasks} \tab Space-time modelling, Forecasting,
#'   Clustering
#' }
#' @keywords datasets interval timeseries meteorology spatial
"irish_wind.its"

#' @name interval_robust
#' @title Robust Statistics for Interval Data
#' @description Functions to compute robust statistics for interval-valued data.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name the variable name or the column location (multiple variables are allowed).
#' @param method methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.
#' @param trim the fraction (0 to 0.5) of observations to be trimmed from each end.
#' @param ... additional parameters
#' @return A numeric matrix
#' @details
#' These functions provide robust alternatives to standard statistics:
#' \itemize{
#'   \item \code{int_trimmed_mean}: Mean after trimming extreme values
#'   \item \code{int_winsorized_mean}: Mean after winsorizing extreme values
#'   \item \code{int_trimmed_var}: Variance after trimming extreme values
#'   \item \code{int_winsorized_var}: Variance after winsorizing extreme values
#' }
#' 
#' Trimming vs Winsorizing:
#' \itemize{
#'   \item Trimming: Remove extreme values
#'   \item Winsorizing: Replace extreme values with less extreme values
#' }
#' @author Han-Ming Wu
#' @seealso int_mean int_var int_trimmed_mean
#' @examples
#' data(mushroom.int)
#' 
#' # Trimmed mean (10% from each end)
#' int_trimmed_mean(mushroom.int, var_name = "Pileus.Cap.Width", trim = 0.1)
#' 
#' # Winsorized mean
#' int_winsorized_mean(mushroom.int, var_name = 2:3, trim = 0.05, method = "CM")
#' 
#' # Trimmed variance
#' int_trimmed_var(mushroom.int, var_name = c("Stipe.Length"), trim = 0.1)
#' @importFrom stats quantile
#' @export
int_trimmed_mean <- function(x, var_name, trim = 0.1, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_trimmed_mean")
  .check_var_name(var_name, x, "int_trimmed_mean")
  .check_interval_method(method, "int_trimmed_mean")
  
  if (trim < 0 || trim >= 0.5) {
    stop("trim must be between 0 and 0.5", call. = FALSE)
  }
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  tmean_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  compute_trimmed_mean <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- mean(X_tmp, trim = trim)
    } else {
      x <- apply(X_tmp, 2, mean, trim = trim)
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    tmean_tmp[idx, ] <- compute_trimmed_mean(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    tmean_tmp[6, ] <- tmean_tmp[7, ] <- tmean_tmp[8, ] <- compute_trimmed_mean(X_tmp)
  }
  
  tmean_output <- matrix(tmean_tmp[at, ], 
                         nrow = length(method), 
                         ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(tmean_output) <- colnames(x)[var_name]
  } else {
    colnames(tmean_output) <- var_name
  }
  rownames(tmean_output) <- options[at]
  
  tmean_output
}


#' @rdname interval_robust
#' @export
int_winsorized_mean <- function(x, var_name, trim = 0.1, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_winsorized_mean")
  .check_var_name(var_name, x, "int_winsorized_mean")
  .check_interval_method(method, "int_winsorized_mean")
  
  if (trim < 0 || trim >= 0.5) {
    stop("trim must be between 0 and 0.5", call. = FALSE)
  }
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  wmean_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  
  # Winsorize function
  winsorize <- function(x, trim) {
    n <- length(x)
    lower_idx <- floor(n * trim) + 1
    upper_idx <- n - floor(n * trim)
    
    x_sorted <- sort(x)
    lower_val <- x_sorted[lower_idx]
    upper_val <- x_sorted[upper_idx]
    
    x[x < lower_val] <- lower_val
    x[x > upper_val] <- upper_val
    x
  }
  
  compute_winsorized_mean <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- mean(winsorize(X_tmp, trim))
    } else {
      x <- apply(X_tmp, 2, function(col) mean(winsorize(col, trim)))
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    wmean_tmp[idx, ] <- compute_winsorized_mean(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    X_tmp <- if (!is.null(transforms$CM)) transforms$CM else Interval_to_Center(idata)
    wmean_tmp[6, ] <- wmean_tmp[7, ] <- wmean_tmp[8, ] <- compute_winsorized_mean(X_tmp)
  }
  
  wmean_output <- matrix(wmean_tmp[at, ], 
                         nrow = length(method), 
                         ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(wmean_output) <- colnames(x)[var_name]
  } else {
    colnames(wmean_output) <- var_name
  }
  rownames(wmean_output) <- options[at]
  
  wmean_output
}


#' @rdname interval_robust
#' @export
int_trimmed_var <- function(x, var_name, trim = 0.1, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_trimmed_var")
  .check_var_name(var_name, x, "int_trimmed_var")
  .check_interval_method(method, "int_trimmed_var")
  
  if (trim < 0 || trim >= 0.5) {
    stop("trim must be between 0 and 0.5", call. = FALSE)
  }
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  tvar_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  
  # Compute variance after trimming
  compute_trimmed_var <- function(X_tmp) {
    if (length(var_name) == 1) {
      n_obs <- length(X_tmp)
      n_trim <- floor(n_obs * trim)
      x_sorted <- sort(X_tmp)
      x_trimmed <- x_sorted[(n_trim + 1):(n_obs - n_trim)]
      x <- var(x_trimmed)
    } else {
      x <- numeric(ncol(X_tmp))
      for (j in 1:ncol(X_tmp)) {
        n_obs <- nrow(X_tmp)
        n_trim <- floor(n_obs * trim)
        x_sorted <- sort(X_tmp[, j])
        x_trimmed <- x_sorted[(n_trim + 1):(n_obs - n_trim)]
        x[j] <- var(x_trimmed)
      }
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    tvar_tmp[idx, ] <- compute_trimmed_var(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    # For these methods, use trimmed variance formula similar to regular variance
    n_trim <- floor(n * trim)
    n_effective <- n - 2 * n_trim
    
    ans <- numeric(length(var_name))
    names(ans) <- var_name
    for (i in var_name) {
      # Get sorted intervals
      centers <- (idata[, i, 1] + idata[, i, 2]) / 2
      order_idx <- order(centers)
      idata_sorted <- idata[order_idx, i, ]
      
      # Trim
      idata_trimmed <- idata_sorted[(n_trim + 1):(n - n_trim), ]
      
      a <- sum(idata_trimmed[, 2]^2 + idata_trimmed[, 1] * idata_trimmed[, 2] + 
               idata_trimmed[, 1]^2)
      b <- (sum(idata_trimmed[, 1] + idata_trimmed[, 2]))^2
      ans[i] <- a / (3 * n_effective) - b / (4 * n_effective^2)
    }
    
    if (at[6]) tvar_tmp[6, ] <- ans
    if (at[7]) tvar_tmp[7, ] <- ans
    if (at[8]) tvar_tmp[8, ] <- ans
  }
  
  tvar_output <- matrix(tvar_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(tvar_output) <- colnames(x)[var_name]
  } else {
    colnames(tvar_output) <- var_name
  }
  rownames(tvar_output) <- options[at]
  
  tvar_output
}


#' @rdname interval_robust
#' @export
int_winsorized_var <- function(x, var_name, trim = 0.1, method = "CM", ...) {
  .check_symbolic_tbl(x, "int_winsorized_var")
  .check_var_name(var_name, x, "int_winsorized_var")
  .check_interval_method(method, "int_winsorized_var")
  
  if (trim < 0 || trim >= 0.5) {
    stop("trim must be between 0 and 0.5", call. = FALSE)
  }
  
  options <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  at <- options %in% method
  wvar_tmp <- matrix(0, nrow = length(options), ncol = length(var_name))
  
  idata <- symbolic_tbl_to_idata(x[, var_name])
  n <- nrow(idata)
  
  # Winsorize function
  winsorize <- function(x, trim) {
    n <- length(x)
    lower_idx <- floor(n * trim) + 1
    upper_idx <- n - floor(n * trim)
    
    x_sorted <- sort(x)
    lower_val <- x_sorted[lower_idx]
    upper_val <- x_sorted[upper_idx]
    
    x[x < lower_val] <- lower_val
    x[x > upper_val] <- upper_val
    x
  }
  
  compute_winsorized_var <- function(X_tmp) {
    if (length(var_name) == 1) {
      x <- var(winsorize(X_tmp, trim))
    } else {
      x <- apply(X_tmp, 2, function(col) var(winsorize(col, trim)))
    }
    x
  }
  
  transforms <- .get_interval_transforms(idata, at)
  for (nm in names(transforms)) {
    idx <- which(options == nm)
    wvar_tmp[idx, ] <- compute_winsorized_var(transforms[[nm]])
  }
  
  if (at[6] | at[7] | at[8]) {  # EJD, GQ, SPT
    # Winsorize the intervals
    ans <- numeric(length(var_name))
    names(ans) <- var_name
    
    for (i in var_name) {
      # Winsorize based on centers
      centers <- (idata[, i, 1] + idata[, i, 2]) / 2
      wins_centers <- winsorize(centers, trim)
      
      # Find which values were winsorized
      order_idx <- order(centers)
      n_trim <- floor(n * trim)
      
      lower_val <- sort(centers)[n_trim + 1]
      upper_val <- sort(centers)[n - n_trim]
      
      idata_wins <- idata[, i, ]
      # Replace winsorized intervals
      idata_wins[centers < lower_val, ] <- idata[order_idx[n_trim + 1], i, ]
      idata_wins[centers > upper_val, ] <- idata[order_idx[n - n_trim], i, ]
      
      a <- sum(idata_wins[, 2]^2 + idata_wins[, 1] * idata_wins[, 2] + 
               idata_wins[, 1]^2)
      b <- (sum(idata_wins[, 1] + idata_wins[, 2]))^2
      ans[i] <- a / (3 * n) - b / (4 * n^2)
    }
    
    if (at[6]) wvar_tmp[6, ] <- ans
    if (at[7]) wvar_tmp[7, ] <- ans
    if (at[8]) wvar_tmp[8, ] <- ans
  }
  
  wvar_output <- matrix(wvar_tmp[at, ], 
                        nrow = length(method), 
                        ncol = length(var_name))
  
  if (is.numeric(var_name)) {
    colnames(wvar_output) <- colnames(x)[var_name]
  } else {
    colnames(wvar_output) <- var_name
  }
  rownames(wvar_output) <- options[at]
  
  wvar_output
}

# Internal validation helpers (not exported)

.check_data_frame <- function(data, fn_name) {
  if (is.null(data)) {
    stop(fn_name, ": 'data' must not be NULL.", call. = FALSE)
  }
  if (!is.data.frame(data)) {
    stop(fn_name, ": 'data' must be a data.frame, not ", class(data)[1], ".", call. = FALSE)
  }
}

.check_location <- function(location, ncol_data, fn_name) {
  if (is.null(location)) {
    stop(fn_name, ": 'location' must not be NULL.", call. = FALSE)
  }
  if (!is.numeric(location)) {
    stop(fn_name, ": 'location' must be numeric, not ", class(location)[1], ".", call. = FALSE)
  }
  if (any(location < 1) || any(location > ncol_data)) {
    stop(fn_name, ": 'location' values must be between 1 and ", ncol_data, ".", call. = FALSE)
  }
}

.check_symbolic_tbl <- function(x, fn_name) {
  if (is.null(x)) {
    stop(fn_name, ": 'x' must not be NULL.", call. = FALSE)
  }
  if (!inherits(x, "symbolic_tbl")) {
    stop(fn_name, ": 'x' must be a symbolic_tbl object, not ", class(x)[1], ".", call. = FALSE)
  }
}

.check_var_name <- function(var_name, x, fn_name) {
  if (is.character(var_name)) {
    missing_vars <- setdiff(var_name, colnames(x))
    if (length(missing_vars) > 0) {
      stop(fn_name, ": variable(s) not found in data: ",
           paste(missing_vars, collapse = ", "), ".", call. = FALSE)
    }
  } else if (is.numeric(var_name)) {
    if (any(var_name < 1) || any(var_name > ncol(x))) {
      stop(fn_name, ": 'var_name' indices must be between 1 and ", ncol(x), ".", call. = FALSE)
    }
  }
}

.check_interval_method <- function(method, fn_name) {
  valid <- c("CM", "VM", "QM", "SE", "FV", "EJD", "GQ", "SPT")
  bad <- setdiff(method, valid)
  if (length(bad) > 0) {
    warning(fn_name, ": unknown method(s) ignored: ",
            paste(bad, collapse = ", "),
            ". Valid methods: ", paste(valid, collapse = ", "), ".",
            call. = FALSE)
  }
}

.check_MatH <- function(x, fn_name) {
  if (is.null(x)) {
    stop(fn_name, ": 'x' must not be NULL.", call. = FALSE)
  }
  if (!inherits(x, "MatH")) {
    stop(fn_name, ": 'x' must be a MatH object (from HistDAWass), not ", class(x)[1], ".", call. = FALSE)
  }
}

.check_hist_var_name <- function(var_name, x, fn_name) {
  if (!is.character(var_name)) {
    stop(fn_name, ": 'var_name' must be a character string.", call. = FALSE)
  }
  available <- colnames(x@M)
  missing_vars <- setdiff(var_name, available)
  if (length(missing_vars) > 0) {
    stop(fn_name, ": variable(s) not found in data: ",
         paste(missing_vars, collapse = ", "), ".", call. = FALSE)
  }
}

.check_hist_method <- function(method, valid, fn_name) {
  bad <- setdiff(method, valid)
  if (length(bad) > 0) {
    warning(fn_name, ": unknown method(s) ignored: ",
            paste(bad, collapse = ", "),
            ". Valid methods: ", paste(valid, collapse = ", "), ".",
            call. = FALSE)
  }
}

.check_file_path <- function(file, fn_name) {
  if (is.null(file) || !is.character(file) || length(file) != 1 || nchar(file) == 0) {
    stop(fn_name, ": 'file' must be a non-empty character string.", call. = FALSE)
  }
}

.check_file_exists <- function(file, fn_name) {
  if (!file.exists(file)) {
    stop(fn_name, ": file not found: '", file, "'.", call. = FALSE)
  }
}

.check_logical <- function(arg, arg_name, fn_name) {
  if (!is.logical(arg) || length(arg) != 1 || is.na(arg)) {
    stop(fn_name, ": '", arg_name, "' must be TRUE or FALSE.", call. = FALSE)
  }
}

#' @name interval_similarity
#' @title Similarity Measures for Interval Data
#' @description Functions to compute similarity measures between interval-valued observations.
#' @param x interval-valued data with symbolic_tbl class.
#' @param var_name1 the first variable name or column location.
#' @param var_name2 the second variable name or column location.
#' @param method similarity method for int_similarity_matrix: "jaccard", "dice", or "overlap".
#' @param ... additional parameters
#' @return A numeric matrix or value
#' @details
#' These functions compute various similarity measures:
#' \itemize{
#'   \item \code{int_jaccard}: Jaccard similarity coefficient
#'   \item \code{int_dice}: Dice similarity coefficient
#'   \item \code{int_cosine}: Cosine similarity
#'   \item \code{int_overlap_coefficient}: Overlap coefficient
#'   \item \code{int_tanimoto}: Tanimoto coefficient (generalized Jaccard)
#'   \item \code{int_similarity_matrix}: Pairwise similarity matrix across all observations
#' }
#' 
#' All similarity measures range from 0 (no similarity) to 1 (perfect similarity).
#' @author Han-Ming Wu
#' @seealso int_dist int_cor int_jaccard
#' @examples
#' data(mushroom.int)
#' 
#' # Jaccard similarity
#' int_jaccard(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")
#' 
#' # Dice coefficient
#' int_dice(mushroom.int, 2, 3)
#' 
#' # Cosine similarity
#' int_cosine(mushroom.int, 
#'            var_name1 = c("Pileus.Cap.Width"), 
#'            var_name2 = c("Stipe.Length", "Stipe.Thickness"))
#' 
#' # Overlap coefficient
#' int_overlap_coefficient(mushroom.int, 2, 3:4)
#'
#' # Tanimoto coefficient
#' int_tanimoto(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")
#'
#' # Similarity matrix across all observations
#' int_similarity_matrix(mushroom.int, method = "jaccard")
#' @export
int_jaccard <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_jaccard")
  .check_var_name(var_name1, x, "int_jaccard")
  .check_var_name(var_name2, x, "int_jaccard")
  
  # Resolve names
  if (is.numeric(var_name1)) {
    name1 <- colnames(x)[var_name1]
  } else {
    name1 <- var_name1
  }
  if (is.numeric(var_name2)) {
    name2 <- colnames(x)[var_name2]
  } else {
    name2 <- var_name2
  }

  all_vars <- unique(c(name1, name2))
  idata <- symbolic_tbl_to_idata(x[, all_vars])
  n <- nrow(idata)

  data1 <- idata[, name1, , drop = FALSE]
  data2 <- idata[, name2, , drop = FALSE]

  # Calculate Jaccard: |A ∩ B| / |A ∪ B|
  jaccard_mat <- matrix(0, nrow = n, ncol = length(name1) * length(name2))
  col_idx <- 1
  col_names <- character(length(name1) * length(name2))
  
  for (i in seq_along(name1)) {
    for (j in seq_along(name2)) {
      for (k in 1:n) {
        # Intersection
        int_start <- max(data1[k, i, 1], data2[k, j, 1])
        int_end <- min(data1[k, i, 2], data2[k, j, 2])
        intersection <- max(0, int_end - int_start)
        
        # Union
        union_start <- min(data1[k, i, 1], data2[k, j, 1])
        union_end <- max(data1[k, i, 2], data2[k, j, 2])
        union <- union_end - union_start
        
        if (union > 0) {
          jaccard_mat[k, col_idx] <- intersection / union
        } else {
          jaccard_mat[k, col_idx] <- 0
        }
      }
      col_names[col_idx] <- paste0(name1[i], "_", name2[j])
      col_idx <- col_idx + 1
    }
  }
  
  rownames(jaccard_mat) <- rownames(x)
  colnames(jaccard_mat) <- col_names
  
  jaccard_mat
}


#' @rdname interval_similarity
#' @export
int_dice <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_dice")
  .check_var_name(var_name1, x, "int_dice")
  .check_var_name(var_name2, x, "int_dice")
  
  # Resolve names
  if (is.numeric(var_name1)) {
    name1 <- colnames(x)[var_name1]
  } else {
    name1 <- var_name1
  }
  if (is.numeric(var_name2)) {
    name2 <- colnames(x)[var_name2]
  } else {
    name2 <- var_name2
  }

  all_vars <- unique(c(name1, name2))
  idata <- symbolic_tbl_to_idata(x[, all_vars])
  n <- nrow(idata)

  data1 <- idata[, name1, , drop = FALSE]
  data2 <- idata[, name2, , drop = FALSE]

  # Calculate Dice: 2|A ∩ B| / (|A| + |B|)
  dice_mat <- matrix(0, nrow = n, ncol = length(name1) * length(name2))
  col_idx <- 1
  col_names <- character(length(name1) * length(name2))
  
  for (i in seq_along(name1)) {
    for (j in seq_along(name2)) {
      for (k in 1:n) {
        # Intersection
        int_start <- max(data1[k, i, 1], data2[k, j, 1])
        int_end <- min(data1[k, i, 2], data2[k, j, 2])
        intersection <- max(0, int_end - int_start)
        
        # Sizes
        size_a <- data1[k, i, 2] - data1[k, i, 1]
        size_b <- data2[k, j, 2] - data2[k, j, 1]
        
        if ((size_a + size_b) > 0) {
          dice_mat[k, col_idx] <- 2 * intersection / (size_a + size_b)
        } else {
          dice_mat[k, col_idx] <- 0
        }
      }
      col_names[col_idx] <- paste0(name1[i], "_", name2[j])
      col_idx <- col_idx + 1
    }
  }
  
  rownames(dice_mat) <- rownames(x)
  colnames(dice_mat) <- col_names
  
  dice_mat
}


#' @rdname interval_similarity
#' @export
int_cosine <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_cosine")
  .check_var_name(var_name1, x, "int_cosine")
  .check_var_name(var_name2, x, "int_cosine")
  
  # Resolve names
  if (is.numeric(var_name1)) {
    name1 <- colnames(x)[var_name1]
  } else {
    name1 <- var_name1
  }
  if (is.numeric(var_name2)) {
    name2 <- colnames(x)[var_name2]
  } else {
    name2 <- var_name2
  }

  all_vars <- unique(c(name1, name2))
  idata <- symbolic_tbl_to_idata(x[, all_vars])
  n <- nrow(idata)

  data1 <- idata[, name1, , drop = FALSE]
  data2 <- idata[, name2, , drop = FALSE]

  # Calculate Cosine similarity on centers
  cosine_mat <- matrix(0, nrow = n, ncol = length(name1) * length(name2))
  col_idx <- 1
  col_names <- character(length(name1) * length(name2))
  
  for (i in seq_along(name1)) {
    for (j in seq_along(name2)) {
      # Get centers for all observations
      centers1 <- (data1[, i, 1] + data1[, i, 2]) / 2
      centers2 <- (data2[, j, 1] + data2[, j, 2]) / 2
      
      # Cosine similarity: (A · B) / (||A|| * ||B||)
      dot_product <- sum(centers1 * centers2)
      norm1 <- sqrt(sum(centers1^2))
      norm2 <- sqrt(sum(centers2^2))
      
      if (norm1 > 0 && norm2 > 0) {
        # Return single value for the pair
        cosine_val <- dot_product / (norm1 * norm2)
        cosine_mat[, col_idx] <- cosine_val
      } else {
        cosine_mat[, col_idx] <- 0
      }
      
      col_names[col_idx] <- paste0(name1[i], "_", name2[j])
      col_idx <- col_idx + 1
    }
  }
  
  # Return as single row matrix (one value per variable pair)
  cosine_output <- matrix(cosine_mat[1, ], nrow = 1)
  colnames(cosine_output) <- col_names
  rownames(cosine_output) <- "Cosine"
  
  cosine_output
}


#' @rdname interval_similarity
#' @export
int_overlap_coefficient <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_overlap_coefficient")
  .check_var_name(var_name1, x, "int_overlap_coefficient")
  .check_var_name(var_name2, x, "int_overlap_coefficient")
  
  # Resolve names
  if (is.numeric(var_name1)) {
    name1 <- colnames(x)[var_name1]
  } else {
    name1 <- var_name1
  }
  if (is.numeric(var_name2)) {
    name2 <- colnames(x)[var_name2]
  } else {
    name2 <- var_name2
  }

  all_vars <- unique(c(name1, name2))
  idata <- symbolic_tbl_to_idata(x[, all_vars])
  n <- nrow(idata)

  data1 <- idata[, name1, , drop = FALSE]
  data2 <- idata[, name2, , drop = FALSE]

  # Calculate Overlap: |A ∩ B| / min(|A|, |B|)
  overlap_mat <- matrix(0, nrow = n, ncol = length(name1) * length(name2))
  col_idx <- 1
  col_names <- character(length(name1) * length(name2))
  
  for (i in seq_along(name1)) {
    for (j in seq_along(name2)) {
      for (k in 1:n) {
        # Intersection
        int_start <- max(data1[k, i, 1], data2[k, j, 1])
        int_end <- min(data1[k, i, 2], data2[k, j, 2])
        intersection <- max(0, int_end - int_start)
        
        # Sizes
        size_a <- data1[k, i, 2] - data1[k, i, 1]
        size_b <- data2[k, j, 2] - data2[k, j, 1]
        min_size <- min(size_a, size_b)
        
        if (min_size > 0) {
          overlap_mat[k, col_idx] <- intersection / min_size
        } else {
          overlap_mat[k, col_idx] <- 0
        }
      }
      col_names[col_idx] <- paste0(name1[i], "_", name2[j])
      col_idx <- col_idx + 1
    }
  }
  
  rownames(overlap_mat) <- rownames(x)
  colnames(overlap_mat) <- col_names
  
  overlap_mat
}


#' @rdname interval_similarity
#' @export
int_tanimoto <- function(x, var_name1, var_name2, ...) {
  .check_symbolic_tbl(x, "int_tanimoto")
  .check_var_name(var_name1, x, "int_tanimoto")
  .check_var_name(var_name2, x, "int_tanimoto")
  
  # Tanimoto is equivalent to Jaccard for intervals
  int_jaccard(x, var_name1, var_name2, ...)
}


#' @rdname interval_similarity
#' @export
int_similarity_matrix <- function(x, method = "jaccard", ...) {
  .check_symbolic_tbl(x, "int_similarity_matrix")
  
  method <- match.arg(method, c("jaccard", "dice", "overlap"))
  
  # Only use interval (complex-mode) columns
  int_cols <- which(sapply(x, mode) == "complex")
  idata <- symbolic_tbl_to_idata(x[, int_cols])
  n <- nrow(idata)
  p <- ncol(idata)
  
  # Create similarity matrix
  sim_mat <- matrix(1, nrow = n, ncol = n)
  
  sim_func <- switch(method,
                     jaccard = .compute_jaccard_sim,
                     dice = .compute_dice_sim,
                     overlap = .compute_overlap_sim)
  
  for (i in 1:(n-1)) {
    for (j in (i+1):n) {
      sim <- 0
      for (k in 1:p) {
        # Average similarity across all variables
        sim <- sim + sim_func(idata[i, k, ], idata[j, k, ])
      }
      sim <- sim / p
      sim_mat[i, j] <- sim_mat[j, i] <- sim
    }
  }
  
  rownames(sim_mat) <- colnames(sim_mat) <- rownames(x)
  sim_mat
}


# Helper functions for similarity computation
.compute_jaccard_sim <- function(int1, int2) {
  int_start <- max(int1[1], int2[1])
  int_end <- min(int1[2], int2[2])
  intersection <- max(0, int_end - int_start)
  
  union_start <- min(int1[1], int2[1])
  union_end <- max(int1[2], int2[2])
  union <- union_end - union_start
  
  if (union > 0) intersection / union else 0
}


.compute_dice_sim <- function(int1, int2) {
  int_start <- max(int1[1], int2[1])
  int_end <- min(int1[2], int2[2])
  intersection <- max(0, int_end - int_start)
  
  size_a <- int1[2] - int1[1]
  size_b <- int2[2] - int2[1]
  
  if ((size_a + size_b) > 0) 2 * intersection / (size_a + size_b) else 0
}


.compute_overlap_sim <- function(int1, int2) {
  int_start <- max(int1[1], int2[1])
  int_end <- min(int1[2], int2[2])
  intersection <- max(0, int_end - int_start)
  
  size_a <- int1[2] - int1[1]
  size_b <- int2[2] - int2[1]
  min_size <- min(size_a, size_b)
  
  if (min_size > 0) intersection / min_size else 0
}

dataset_names <- c(
  "abalone.int", "abalone.iGAP", "face.iGAP",
  "mushroom.int.mm", "mushroom.int",
  "bird.mix", "baseball.int", "blood_pressure.int",
  "car.int", "cars.int", "china_temp.int",
  "finance.int", "hierarchy.int", "horses.int",
  "lackinfo.int", "loans_by_purpose.int",
  "nycflights.int", "ohtemp.int", "profession.int",
  "soccer_bivar.int", "veterinary.int",
  "age_cholesterol_weight.int",
  "hierarchy",
  "fuel_consumption.modal", "health_insurance.mix", "health_insurance2.modal",
  "airline_flights.hist", "airline_flights2.modal",
  "crime.modal", "crime2.modal",
  "occupations.modal", "occupations2.modal"
)

test_that("all datasets can be loaded", {
  for (ds in dataset_names) {
    data(list = ds, package = "dataSDA")
    expect_true(exists(ds), info = paste("Dataset", ds, "could not be loaded"))
  }
})

test_that("all datasets have rows and columns > 0", {
  for (ds in dataset_names) {
    data(list = ds, package = "dataSDA")
    obj <- get(ds)
    expect_true(nrow(obj) > 0, info = paste(ds, "has 0 rows"))
    expect_true(ncol(obj) > 0, info = paste(ds, "has 0 columns"))
  }
})

int_datasets <- c(
  "mushroom.int", "baseball.int", "blood_pressure.int",
  "car.int", "cars.int", "china_temp.int",
  "finance.int", "hierarchy.int", "horses.int",
  "lackinfo.int", "loans_by_purpose.int",
  "nycflights.int", "ohtemp.int", "profession.int",
  "soccer_bivar.int", "veterinary.int",
  "age_cholesterol_weight.int"
)

test_that(".int datasets have symbolic_tbl class", {
  for (ds in int_datasets) {
    data(list = ds, package = "dataSDA")
    obj <- get(ds)
    expect_true("symbolic_tbl" %in% class(obj),
                info = paste(ds, "should have symbolic_tbl class"))
  }
})

igap_datasets <- c("abalone.iGAP", "face.iGAP")

test_that(".iGAP datasets are data.frames", {
  for (ds in igap_datasets) {
    data(list = ds, package = "dataSDA")
    obj <- get(ds)
    expect_true(is.data.frame(obj),
                info = paste(ds, "should be a data.frame"))
  }
})

test_that("mushroom.int.mm dataset is a data.frame", {
  data(mushroom.int.mm)
  expect_s3_class(mushroom.int.mm, "data.frame")
  expect_true(nrow(mushroom.int.mm) > 0)
})

test_that("mushroom.int has complex columns", {
  data(mushroom.int)
  modes <- sapply(mushroom.int, mode)
  expect_true(any(modes == "complex"),
              info = "mushroom.int should have complex-mode columns for intervals")
})

test_that("abalone.iGAP contains comma-separated interval values", {
  data(abalone.iGAP)
  first_col <- abalone.iGAP[[1]]
  expect_true(any(grepl(",", first_col)),
              info = "iGAP data should contain comma-separated values")
})

test_that("face.iGAP contains comma-separated interval values", {
  data(face.iGAP)
  first_col <- face.iGAP[[1]]
  expect_true(any(grepl(",", first_col)),
              info = "iGAP data should contain comma-separated values")
})

test_that("bird.mix has expected structure", {
  data(bird.mix)
  expect_true(ncol(bird.mix) >= 2)
  expect_true(nrow(bird.mix) >= 2)
})

test_that("abalone.int dataset is a data.frame with positive dimensions", {
  data(abalone.int)
  expect_true(is.data.frame(abalone.int))
  expect_true(nrow(abalone.int) > 0)
  expect_true(ncol(abalone.int) > 0)
})

test_that("hierarchy dataset loads correctly", {
  data(hierarchy)
  expect_true(is.data.frame(hierarchy))
  expect_true(nrow(hierarchy) > 0)
})

test_that("fuel_consumption.modal dataset loads correctly", {
  data(fuel_consumption.modal)
  expect_true(is.data.frame(fuel_consumption.modal))
  expect_true(nrow(fuel_consumption.modal) > 0)
})

test_that("crime.modal dataset loads correctly", {
  data(crime.modal)
  expect_true(is.data.frame(crime.modal))
  expect_true(nrow(crime.modal) > 0)
})

test_that("occupations.modal dataset loads correctly", {
  data(occupations.modal)
  expect_true(is.data.frame(occupations.modal))
  expect_true(nrow(occupations.modal) > 0)
})

---
title: "Introduction to dataSDA"
author: "Po-Wei Chen"
date: "2023-06-13"
output: 
  rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to dataSDA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, include = FALSE}
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(warning = FALSE)
knitr::opts_chunk$set(message = FALSE)
knitr::opts_chunk$set(out.width = "100%")
knitr::opts_chunk$set(fig.align = 'center')
library(knitr)
library(dataSDA)
library(RSDA)
```

## Example of Interval-Valued Data
### Built-in Data

```{r}
data(mushroom)
head(mushroom)
```

### Set variable format

Changes the format of the set variables in the data to conform to the RSDA format.

+ `data` = the conventional dataframe
+ `location` = the location of the set variable in the data
+ `var` = the name of the set variable in the data

```{r}
mushroom.set <- set_variable_format(data = mushroom, location = 8, var = "Species")
head(mushroom.set)
```

### RSDA format

Changes the format of the data to conform to RSDA format.

+ `data` = the conventional dataframe
+ `sym_type1` = the labels I means an interval variable, and S means set variable in location
+ `location` = the location of the sym_type in the data
+ `sym_type2` = the labels I means an interval variable, and S means set variable in var
+ `var` = the name of the symbolic variable in the data
 
```{r}
mushroom.tmp <- RSDA_format(data = mushroom.set, sym_type1 = c("I", "S"),
                            location = c(25, 31), sym_type2 = c("S", "I", "I"),
                            var = c("Species", "Stipe.Length_min", "Stipe.Thickness_min"))
head(mushroom.tmp)
```

### Clean the column names

Clean up variable names to conform to the RSDA format.

+ `data` = the conventional dataframe

```{r}
mushroom.clean <- clean_colnames(data = mushroom.tmp)
head(mushroom.clean)
```

### Write a symbolic data table from a CSV data file

```{r, eval = FALSE}
write_csv_table(data = mushroom.clean, file = 'mushroom_interval.csv')
```

### Read the symbolic data table and check the format

```{r}
mushroom.int <- read.sym.table(file = 'mushroom_interval.csv', header = T, sep = ';', dec = '.', row.names = 1)
head(mushroom.int)
```

## Example of iGAP format Data
### Built-in Data

```{r}
data(Abalone.iGAP)
head(Abalone.iGAP)
```

### iGAP to MM

To convert iGAP files to CSV files.

+ `data` = the conventional dataframe(iGAP format)
+ `location` = the location of the symbolic variable in the data

```{r}
Abalone <- iGAP_to_MM(Abalone.iGAP, c(1, 2, 3, 4, 5, 6, 7))
head(Abalone)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{profession.int}
\alias{profession.int}
\title{Profession Work Salary Time Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 15 observations and 4 variables:
\itemize{
    \item \code{Type_of_Work}: Work category (White Collar or Blue Collar, character).
    \item \code{Profession}: Profession name (character).
    \item \code{Salary}: Salary range (currency units, interval).
    \item \code{Duration}: Working duration range (hours per week, interval).
}
}
\usage{
data(profession.int)
}
\description{
Interval-valued data for 15 profession entries classified by work type
(White Collar / Blue Collar). Each entry describes a specific profession
with salary and working duration ranges.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 15 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Sociology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Classification
}
}

\examples{
data(profession.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{baseball.int}
\alias{baseball.int}
\title{Baseball Teams Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 19 observations and 3 variables:
\itemize{
    \item \code{At_Bats}: Range of at-bats across players (interval).
    \item \code{Hits}: Range of hits across players (interval).
    \item \code{Pattern}: Team performance pattern code (character).
}
}
\usage{
data(baseball.int)
}
\description{
Interval-valued data for 19 baseball teams with aggregated player
batting statistics and a pattern variable classifying team performance.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 19 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Sports \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(baseball.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{china_climate_season.hist}
\alias{china_climate_season.hist}
\title{Chinese Climate Seasonal Histogram Dataset}
\format{
A data frame with 60 observations (stations) and 56
histogram-valued variables. Variables follow the pattern
\code{variable_Season} (e.g., \code{mean.temp_Spring}). The 14 climate
variables are: mean pressure, mean temperature, mean max/min
temperature, total precipitation, sunshine duration, mean cloud amount,
mean relative humidity, snow days, dominant wind direction, mean wind
speed, dominant wind frequency, extreme max/min temperature.
}
\source{
HistDAWass R package (\code{China_Seas} dataset).
}
\usage{
data(china_climate_season.hist)
}
\description{
Histogram-valued seasonal climate data for 60 Chinese weather stations.
Each station has 14 climate variables measured across 4 seasons
(56 histogram columns total). Histograms are reduced to 10 decile
bins from the original HistDAWass distributions.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 60 \cr
  \strong{Variables (p)} \tab 56 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(china_climate_season.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: a new metric-based approach. \emph{Advances in
Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{China_Seas} dataset).
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ohtemp.int}
\alias{ohtemp.int}
\title{Ohio River Basin 30-Year Trimmed Mean Daily Temperatures Interval Dataset}
\format{
A data frame with 161 rows and 7 variables:
\itemize{
    \item \code{ID}: Global Historical Climatological Network (GHCN) station identifier.
    \item \code{NAME}: GHCN station name.
    \item \code{STATE}: Two-digit state designation.
    \item \code{LATITUDE}: Latitude coordinate position.
    \item \code{LONGITUDE}: Longitude coordinate position.
    \item \code{ELEVATION}: Elevation of the measurement location (meters).
    \item \code{TEMPERATURE}: 30-year mean daily temperature (tenths of degrees Celsius).
}
}
\source{
\url{https://CRAN.R-project.org/package=intkrige}
}
\usage{
data(ohtemp.int)
}
\description{
Interval-valued dataset of 30-year trimmed mean daily temperatures for the
Ohio river basin. Intervals are defined by the mean daily maximum and minimum
temperatures from January 1, 1988 to December 31, 2018.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 161 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression, Spatial analysis
}
}

\examples{
data(ohtemp.int)
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lynne1.int}
\alias{lynne1.int}
\title{Lynne1 Blood Pressure Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations
and 4 variables:
\itemize{
    \item \code{concept}: Character concept label.
    \item \code{Pulse Rate}: Interval-valued pulse rate (beats/min).
    \item \code{Systolic Pressure}: Interval-valued systolic pressure (mmHg).
    \item \code{Diastolic Pressure}: Interval-valued diastolic pressure (mmHg).
}
}
\source{
RSDA R package (\code{Lynne1} dataset).
}
\usage{
data(lynne1.int)
}
\description{
Interval-valued dataset of 10 observations with pulse rate, systolic
pressure, and diastolic pressure intervals.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Regression
}
}

\examples{
data(lynne1.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis: Conceptual
Statistics and Data Mining}. Wiley, Chichester.

Original data from the RSDA R package (\code{Lynne1} dataset).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fungi.int}
\alias{fungi.int}
\title{Fungi Morphological Measurements Interval Dataset}
\format{
A data frame with 55 observations and 6 variables:
\itemize{
    \item \code{pileus_width}: Width of the pileus (cap).
    \item \code{stipe_width}: Width of the stipe (stem).
    \item \code{stipe_thickness}: Thickness of the stipe.
    \item \code{spore_height}: Height of the spores.
    \item \code{spore_width}: Width of the spores.
    \item \code{class}: Fungus genus (Amanita, Agaricus, Boletus).
}
}
\source{
\url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
}
\usage{
data(fungi.int)
}
\description{
Interval-valued morphological measurements for 55 fungi specimens from
3 genera (Amanita, Agaricus, Boletus). Contains 5 interval-valued variables
describing pileus and stipe dimensions and spore characteristics.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 55 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Biology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(fungi.int)
}
\references{
Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
Kernel clustering with automatic variable weighting for interval data.
\emph{Neurocomputing}, 617, 128954.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{airline_flights.hist}
\alias{airline_flights.hist}
\title{JFK Airport Airline Flights Histogram-Valued Dataset}
\format{
A data frame with 16 observations (Airline1--Airline16) and
17 numeric columns representing 6 histogram variables in wide format:
\itemize{
    \item \code{Flight Time(<120)}, \code{Flight Time([120, 220])},
          \code{Flight Time(>220)}: Flight time distribution (3 bins).
    \item \code{Taxi In(<4)}, \code{Taxi In([4, 10])},
          \code{Taxi In(>10)}: Taxi-in time distribution (3 bins).
    \item \code{Arrival Delay(<0)}, \code{Arrival Delay([0, 60])},
          \code{Arrival Delay(>60)}: Arrival delay distribution (3 bins).
    \item \code{Taxi Out(<16)}, \code{Taxi Out([16, 30])},
          \code{Taxi Out(>30)}: Taxi-out time distribution (3 bins).
    \item \code{Departure Delay(<0)}, \code{Departure Delay([0, 60])},
          \code{Departure Delay(>60)}: Departure delay distribution (3 bins).
    \item \code{Weather Delay(No)}, \code{Weather Delay(Yes)}:
          Weather delay distribution (2 bins).
}
}
\usage{
data(airline_flights.hist)
}
\description{
Histogram-valued dataset of 16 airlines flying into JFK Airport.
Six variables (Flight Time, Taxi In, Arrival Delay, Taxi Out,
Departure Delay, Weather Delay) recorded as frequency distributions.
This is the wide (flat table) format; see \code{\link{airline_flights2.modal}}
for the modal-valued version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 16 \cr
  \strong{Variables (p)} \tab 17 \cr
  \strong{Subject area} \tab Transportation \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(airline_flights.hist)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.7.
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{energy_consumption.distr}
\alias{energy_consumption.distr}
\title{US Energy Consumption Distribution-Valued Dataset}
\format{
A data frame with 5 observations and 3 variables:
\itemize{
    \item \code{type}: Energy type.
    \item \code{mean}: Mean consumption across 50 states.
    \item \code{sd}: Standard deviation.
}
}
\usage{
data(energy_consumption.distr)
}
\description{
Distribution-valued dataset of energy consumption across US states.
Each energy type described by Normal distribution parameters (mean, SD).
}
\details{
Five types: Petroleum, Natural Gas, Coal, Hydroelectric, Nuclear Power.
Values are rescaled consumption from the US Census Bureau (2004).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 5 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Energy \cr
  \strong{Symbolic format} \tab Distribution \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(energy_consumption.distr)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.8.
}
\keyword{datasets}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{car_models.int}
\alias{car_models.int}
\title{Italian Car Models Interval Dataset}
\format{
A data frame with 33 observations and 9 variables:
\itemize{
    \item \code{price}: Price range (currency units).
    \item \code{engine_cc}: Engine displacement range (cc).
    \item \code{top_speed}: Top speed range (km/h).
    \item \code{acceleration}: Acceleration range (seconds 0-100 km/h).
    \item \code{wheelbase}: Wheelbase range (cm).
    \item \code{length}: Length range (cm).
    \item \code{width}: Width range (cm).
    \item \code{height}: Height range (cm).
    \item \code{class}: Car category (Utilitaria, Berlina, Ammiraglia, Sportiva).
}
}
\source{
\url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
}
\usage{
data(car_models.int)
}
\description{
Interval-valued specifications for 33 Italian car models, classified into
4 categories (Utilitaria, Berlina, Ammiraglia, Sportiva). An extended
version of the classic cars interval dataset with 8 interval-valued
variables including dimensions.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 33 \cr
  \strong{Variables (p)} \tab 9 \cr
  \strong{Subject area} \tab Automotive \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering, Classification
}
}

\examples{
data(car_models.int)
}
\references{
Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
Kernel clustering with automatic variable weighting for interval data.
\emph{Neurocomputing}, 617, 128954.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hierarchy.int}
\alias{hierarchy.int}
\title{Hierarchy Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 20 observations and 6 variables:
\itemize{
    \item \code{Y}: Response variable range (interval).
    \item \code{X1}: Hierarchy level 1 category (a/b/c, character).
    \item \code{X2}: Hierarchy level 2 category (a1/a2, character; NA for non-a).
    \item \code{X3}: Hierarchy level 3 category (a11/a12, character; NA for non-a1).
    \item \code{X4}: Predictor range for group b (interval; NA for non-b).
    \item \code{X5}: Predictor range for group c (interval; NA for non-c).
}
}
\usage{
data(hierarchy.int)
}
\description{
Interval-valued version of the hierarchy dataset. See \code{\link{hierarchy}}
for the classical version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 20 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Methodology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Regression
}
}

\examples{
data(hierarchy.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.15.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hdi_gender.int}
\alias{hdi_gender.int}
\title{Human Development Index and Gender Indicators Interval Dataset}
\format{
A data frame with 183 observations and 6 variables:
\itemize{
    \item \code{code}: ISO 3166-1 alpha-3 country code.
    \item \code{country}: Country name.
    \item \code{hdi}: Human Development Index value (UNDP).
    \item \code{women_law_index}: Women, Business and the Law Index Score range.
    \item \code{women_parliament}: Proportion of seats held by women in
          national parliaments range (\%).
    \item \code{hdi_category}: Ordered factor with HDI classification
          (Low < Medium < High < Very High).
}
}
\source{
\url{https://github.com/aleixalcacer/OCFIVD}
}
\usage{
data(hdi_gender.int)
}
\description{
Interval-valued World Bank gender indicators for 183 countries, with
ordinal HDI classification. Contains interval ranges for Women, Business
and the Law Index Score and proportion of seats held by women in national
parliaments.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 183 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Socioeconomics \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Classification
}
}

\examples{
data(hdi_gender.int)
}
\references{
Alcacer, A., Barrel, A., Groenen, P. J. F. and Grana, M. (2023).
Ordinal classification for interval-valued data and ordinal data.
\emph{Expert Systems with Applications}, 238, 121825.
}
\keyword{datasets}
\keyword{interval}
\keyword{ordinal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{glucose.hist}
\alias{glucose.hist}
\title{Blood Glucose Histogram-Valued Dataset}
\format{
A data frame with 4 observations (regions) and 1 histogram-valued
variable:
\itemize{
    \item \code{glucose}: Histogram of blood glucose levels.
}
Row names are Region_1 through Region_4.
}
\source{
Billard, L. and Diday, E. (2020), Table 4-14.
}
\usage{
data(glucose.hist)
}
\description{
Histogram-valued dataset of 4 regions with a single histogram-valued
variable describing the distribution of blood glucose measurements.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 4 \cr
  \strong{Variables (p)} \tab 1 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(glucose.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 4-14.
}
\keyword{datasets}
\keyword{histogram}
\keyword{medical}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{census.mix}
\alias{census.mix}
\title{Census Mixed Symbolic Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations
(regions) and 6 variables:
\itemize{
    \item \code{age}: Histogram-valued age distribution (12 age bins).
    \item \code{home_value}: Histogram-valued home value distribution
          (7 value bins, in $1000s).
    \item \code{gender}: Distribution over gender (male, female).
    \item \code{fuel}: Multi-valued set of fuel types used.
    \item \code{tenure}: Distribution over housing tenure
          (owner, renter, vacant).
    \item \code{income}: Interval-valued household income range ($1000s).
}
Row names are Region_1 through Region_10.
}
\source{
Billard, L. and Diday, E. (2020), Table 7-23.
}
\usage{
data(census.mix)
}
\description{
Mixed symbolic dataset of 10 census regions combining 6 different symbolic
variable types: histograms (age, home value), distributions (gender,
tenure), a multi-valued set (fuel), and an interval (income).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Demographics \cr
  \strong{Symbolic format} \tab Mixed (interval, histogram, distribution, multi-valued) \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(census.mix)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 7-23.
}
\keyword{datasets}
\keyword{distribution}
\keyword{histogram}
\keyword{interval}
\keyword{mixed}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cardiological.int}
\alias{cardiological.int}
\title{Cardiological Examination Interval Dataset}
\format{
A data frame with 44 observations and 5 interval-valued variables:
\itemize{
    \item \code{pulse}: Pulse rate range (beats per minute).
    \item \code{systolic}: Systolic blood pressure range (mmHg).
    \item \code{diastolic}: Diastolic blood pressure range (mmHg).
    \item \code{arterial1}: First arterial measurement range.
    \item \code{arterial2}: Second arterial measurement range.
}
}
\source{
Extracted from RSDA package (\code{cardiologicalv2}).
}
\usage{
data(cardiological.int)
}
\description{
Interval-valued data from cardiological examinations of 44 patients.
Each patient is described by 5 interval-valued physiological
measurements.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 44 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(cardiological.int)
}
\references{
Rodriguez, O. (2000). Classification et modeles lineaires en analyse
des donnees symboliques. Doctoral Thesis, Universite Paris IX-Dauphine.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mushroom_fuzzy.mix}
\alias{mushroom_fuzzy.mix}
\title{Mushroom Species Fuzzy/Symbolic Dataset}
\format{
A data frame with 4 observations (Mushroom1--Mushroom4) and 9 variables:
\itemize{
    \item \code{specimen}: Specimen identifier (character).
    \item \code{species}: Species name (character).
    \item \code{stipe_thickness}: Stipe thickness measurement (numeric, cm).
    \item \code{fuzzy_small}: Fuzzy membership degree for Small (numeric, 0--1).
    \item \code{fuzzy_average}: Fuzzy membership degree for Average (numeric, 0--1).
    \item \code{fuzzy_large}: Fuzzy membership degree for Large (numeric, 0--1).
    \item \code{stipe_length}: Stipe length (numeric, cm).
    \item \code{cap_size}: Cap size as interval string (e.g., "24 +/- 1", character).
    \item \code{cap_colour}: Cap colour (character).
}
}
\usage{
data(mushroom_fuzzy.mix)
}
\description{
Extended mushroom data with fuzzy stipe thickness (Small/Average/Large),
numerical stipe length, interval cap size, and categorical cap colour
for two Amanita species (4 specimens).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 4 \cr
  \strong{Variables (p)} \tab 9 \cr
  \strong{Subject area} \tab Biology \cr
  \strong{Symbolic format} \tab Fuzzy \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(mushroom_fuzzy.mix)
}
\references{
Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley. Tables 1.14-1.16.
}
\keyword{datasets}
\keyword{fuzzy}
\keyword{symbolic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{oils.int}
\alias{oils.int}
\title{Oils and Fats Interval Dataset}
\format{
A data frame with 8 observations and 9 columns (4 interval variables
in \code{_l}/\code{_u} Min-Max pairs, plus a label):
\itemize{
    \item \code{sample}: Oil/fat sample name (character).
    \item \code{specific_gravity_l}, \code{specific_gravity_u}: Specific gravity range.
    \item \code{freezing_point_l}, \code{freezing_point_u}: Freezing point range (degrees Celsius).
    \item \code{iodine_value_l}, \code{iodine_value_u}: Iodine value range.
    \item \code{saponification_value_l}, \code{saponification_value_u}: Saponification value range.
}
}
\usage{
data(oils.int)
}
\description{
Classic benchmark interval-valued data for 8 oils and fats described
by 4 physico-chemical properties. Originally from Ichino (1988).
}
\details{
The 8 samples are: Linseed oil, Perilla oil, Cottonseed oil, Sesame oil,
Camellia oil, Olive oil, Beef tallow, Hog fat. The expected 3-cluster
structure is: \{Beef tallow, Hog fat\}, \{Cottonseed, Sesame, Camellia,
Olive\}, and \{Linseed, Perilla\}. Widely used for comparing clustering
methods and distance measures in symbolic data analysis.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 8 \cr
  \strong{Variables (p)} \tab 9 \cr
  \strong{Subject area} \tab Chemistry \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(oils.int)
}
\references{
Ichino, M. (1988). General metrics for mixed features. \emph{Proc. IEEE
Conf. Systems, Man, and Cybernetics}, pp. 494-497.

Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley. Table 13.7, p.253.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{petrobras.its}
\alias{petrobras.its}
\title{Petrobras Stock Daily High/Low Interval Time Series}
\format{
A data frame with 503 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low price (USD).
    \item \code{high}: Daily high price (USD).
}
}
\source{
Yahoo Finance, ticker \code{PBR}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(petrobras.its)
}
\description{
Daily high and low stock prices of Petrobras (ADR traded on NYSE) from
January 3, 2005 to December 29, 2006 (503 trading days). This dataset
matches the period used by Maia, de Carvalho and Ludermir (2008) in
their work on forecasting models for interval-valued time series.
}
\details{
Petrobras (Petroleo Brasileiro S.A.) is the Brazilian multinational
petroleum corporation. The ADR (American Depositary Receipt) is traded
on the New York Stock Exchange under ticker PBR. Each observation
represents a trading day with the daily low and high prices forming an
interval. This was one of the first datasets used to demonstrate
interval-valued autoregressive (iAR) models.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 503 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(petrobras.its)
head(petrobras.its)
plot(petrobras.its$date, petrobras.its$high, type = "l", col = "red",
     ylab = "Price (USD)", xlab = "Date",
     main = "Petrobras Daily High/Low (2005-2006)")
lines(petrobras.its$date, petrobras.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Maia, A. L. S., de Carvalho, F. A. T. and Ludermir, T. B. (2008).
Forecasting models for interval-valued time series. \emph{Neurocomputing},
\strong{71}(16--18), 3344--3352.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{china_climate_month.hist}
\alias{china_climate_month.hist}
\title{Chinese Climate Monthly Histogram Dataset}
\format{
A data frame with 60 observations (stations) and 168
histogram-valued variables. Variables follow the pattern
\code{variable_Month} (e.g., \code{mean.temp_Jan}). The 14 climate
variables are: mean pressure, mean temperature, mean max/min
temperature, total precipitation, sunshine duration, mean cloud amount,
mean relative humidity, snow days, dominant wind direction, mean wind
speed, dominant wind frequency, extreme max/min temperature.
}
\source{
HistDAWass R package (\code{China_Month} dataset).
}
\usage{
data(china_climate_month.hist)
}
\description{
Histogram-valued monthly climate data for 60 Chinese weather stations.
Each station has 14 climate variables measured across 12 months
(168 histogram columns total). Histograms are reduced to 10 decile
bins from the original HistDAWass distributions.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 60 \cr
  \strong{Variables (p)} \tab 168 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(china_climate_month.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: a new metric-based approach. \emph{Advances in
Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{China_Month} dataset).
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_robust.R
\name{interval_robust}
\alias{interval_robust}
\alias{int_trimmed_mean}
\alias{int_winsorized_mean}
\alias{int_trimmed_var}
\alias{int_winsorized_var}
\title{Robust Statistics for Interval Data}
\usage{
int_trimmed_mean(x, var_name, trim = 0.1, method = "CM", ...)

int_winsorized_mean(x, var_name, trim = 0.1, method = "CM", ...)

int_trimmed_var(x, var_name, trim = 0.1, method = "CM", ...)

int_winsorized_var(x, var_name, trim = 0.1, method = "CM", ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name}{the variable name or the column location (multiple variables are allowed).}

\item{trim}{the fraction (0 to 0.5) of observations to be trimmed from each end.}

\item{method}{methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.}

\item{...}{additional parameters}
}
\value{
A numeric matrix
}
\description{
Functions to compute robust statistics for interval-valued data.
}
\details{
These functions provide robust alternatives to standard statistics:
\itemize{
  \item \code{int_trimmed_mean}: Mean after trimming extreme values
  \item \code{int_winsorized_mean}: Mean after winsorizing extreme values
  \item \code{int_trimmed_var}: Variance after trimming extreme values
  \item \code{int_winsorized_var}: Variance after winsorizing extreme values
}

Trimming vs Winsorizing:
\itemize{
  \item Trimming: Remove extreme values
  \item Winsorizing: Replace extreme values with less extreme values
}
}
\examples{
data(mushroom.int)

# Trimmed mean (10\% from each end)
int_trimmed_mean(mushroom.int, var_name = "Pileus.Cap.Width", trim = 0.1)

# Winsorized mean
int_winsorized_mean(mushroom.int, var_name = 2:3, trim = 0.05, method = "CM")

# Trimmed variance
int_trimmed_var(mushroom.int, var_name = c("Stipe.Length"), trim = 0.1)
}
\seealso{
int_mean int_var int_trimmed_mean
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bats.int}
\alias{bats.int}
\title{Bat Species Interval Dataset}
\format{
A data frame with 21 observations and 9 columns (4 interval variables
in \code{_l}/\code{_u} Min-Max pairs, plus a label):
\itemize{
    \item \code{species}: Bat species name (character).
    \item \code{head_l}, \code{head_u}: Head length range (mm).
    \item \code{tail_l}, \code{tail_u}: Tail length range (mm).
    \item \code{height_l}, \code{height_u}: Ear height range (cm).
    \item \code{forearm_l}, \code{forearm_u}: Forearm length range (mm).
}
}
\usage{
data(bats.int)
}
\description{
Interval-valued data for 21 bat species described by 4 morphological
measurements. Benchmark dataset for matrix visualization.
}
\details{
Used to demonstrate color coding schemes, the HCT-R2E seriation
algorithm, and distance measure comparisons (Gowda-Diday, Hausdorff,
City-Block, L1, L2, etc.) for interval data.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 21 \cr
  \strong{Variables (p)} \tab 9 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering, Visualization
}
}

\examples{
data(bats.int)
}
\references{
Kao, C.-H. et al. (2014). Exploratory data analysis of interval-valued
symbolic data with matrix visualization.
\emph{Computational Statistics & Data Analysis}, 79, 14--29.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}
\keyword{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{ARRAY_to_RSDA}
\alias{ARRAY_to_RSDA}
\title{ARRAY to RSDA}
\usage{
ARRAY_to_RSDA(data)
}
\arguments{
\item{data}{A numeric array of dimension \code{[n, p, 2]} where
\code{[,,1]} stores minima and \code{[,,2]} stores maxima.}
}
\value{
A symbolic_tbl with p symbolic_interval columns.
}
\description{
Convert a 3-dimensional array \code{[n, p, 2]} to RSDA format
(symbolic_tbl with symbolic_interval columns).
}
\examples{
x <- array(NA, dim = c(4, 3, 2))
x[,,1] <- matrix(c(1,2,3,4, 5,6,7,8, 9,10,11,12), nrow = 4)
x[,,2] <- matrix(c(3,5,6,7, 8,9,10,12, 13,15,16,18), nrow = 4)
dimnames(x) <- list(paste0("obs_", 1:4), c("V1","V2","V3"), c("min","max"))
rsda <- ARRAY_to_RSDA(x)
rsda
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{int_detect_format}
\alias{int_detect_format}
\title{Detect Interval Data Format}
\usage{
int_detect_format(x)
}
\arguments{
\item{x}{interval data in unknown format}
}
\value{
A character string indicating the detected format: "RSDA", "MM", "iGAP", "ARRAY", "SODAS", or "unknown"
}
\description{
Automatically detect the format of interval data.
}
\details{
Detection rules:
\itemize{
  \item \code{RSDA}: has class "symbolic_tbl" and contains complex columns
  \item \code{MM}: data.frame with paired "_min" and "_max" columns
  \item \code{iGAP}: data.frame with columns containing comma-separated values (e.g., "1.2,3.4")
  \item \code{ARRAY}: a 3-dimensional array with \code{dim[3] = 2} (min/max slices)
  \item \code{SODAS}: character string ending with ".xml" (file path)
  \item \code{SDS}: alias for SODAS
}
}
\examples{
data(mushroom.int)
int_detect_format(mushroom.int)  # Should return "RSDA"

data(abalone.iGAP)
int_detect_format(abalone.iGAP)  # Should return "iGAP"

# ARRAY format
x <- array(1:24, dim = c(4, 3, 2))
int_detect_format(x)  # Should return "ARRAY"
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{airline_flights2.modal}
\alias{airline_flights2.modal}
\title{JFK Airport Airline Flights Modal-Valued Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 16 observations and
6 modal-valued variables:
\itemize{
    \item \code{FlightTime}: Modal distribution over flight time bins.
    \item \code{TaxiIn}: Modal distribution over taxi-in time bins.
    \item \code{ArrivalDelay}: Modal distribution over arrival delay bins.
    \item \code{TaxiOut}: Modal distribution over taxi-out time bins.
    \item \code{DepartureDelay}: Modal distribution over departure delay bins.
    \item \code{WeatherDelay}: Modal distribution over weather delay bins.
}
}
\usage{
data(airline_flights2.modal)
}
\description{
Modal-valued version of the airline flights dataset.
See \code{\link{airline_flights.hist}} for the wide-format version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 16 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Transportation \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(airline_flights2.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.7.
}
\keyword{datasets}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_utils.R
\name{check_zero_width_intervals}
\alias{check_zero_width_intervals}
\title{Check for Zero-Width Intervals}
\usage{
check_zero_width_intervals(data, tol = 0, warn = TRUE)
}
\arguments{
\item{data}{Interval-valued data, in one of two accepted formats:
\itemize{
  \item a \code{data.frame} in MM format with paired \code{_min}/\code{_max}
  columns, or
  \item a \code{symbolic_tbl} (RSDA format) with \code{symbolic_interval}
  columns. Non-interval columns (e.g. set/modal variables) are ignored.
}}

\item{tol}{Non-negative numeric tolerance. An interval is flagged when
\code{abs(max - min) <= tol}. Defaults to \code{0} (exact \code{min == max}).}

\item{warn}{Logical; if \code{TRUE} (default) a single warning naming the
affected variables is emitted when zero-width intervals are found.}
}
\value{
Invisibly, a logical scalar: \code{TRUE} if any zero-width interval
is present, otherwise \code{FALSE}. The returned value carries two attributes:
\code{"flagged"}, a logical \code{[n, p]} matrix marking each zero-width cell
(rows = concepts, columns = interval variables), and \code{"variables"}, the
names of variables containing at least one zero-width interval.
}
\description{
Checks whether interval-valued data contains any zero-width
intervals (\code{min == max}). Such degenerate intervals break downstream
tools that divide by interval width (e.g.
\code{ggInterval::ggInterval_indexImage()}). The input may be supplied either
in MM format (a \code{data.frame} with paired \code{_min}/\code{_max} columns)
or in RSDA format (a \code{symbolic_tbl} with \code{symbolic_interval}
columns).
}
\examples{
# MM format (paired _min/_max columns)
data(mushroom.int.mm)
check_zero_width_intervals(mushroom.int.mm)

# RSDA format (symbolic_tbl)
data(mushroom.int)
check_zero_width_intervals(mushroom.int)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{joggers.mix}
\alias{joggers.mix}
\title{Joggers Mixed Symbolic Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations
(jogger groups) and 2 variables:
\itemize{
    \item \code{pulse_rate}: Interval-valued resting pulse rate range (bpm).
    \item \code{running_time}: Histogram-valued distribution of running
          times (minutes).
}
Row names are Group_1 through Group_10.
}
\source{
Billard, L. and Diday, E. (2020), Table 2-5.
}
\usage{
data(joggers.mix)
}
\description{
Mixed symbolic dataset of 10 jogger groups with one interval-valued
variable (pulse rate) and one histogram-valued variable (running time
distribution).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 2 \cr
  \strong{Subject area} \tab Sports \cr
  \strong{Symbolic format} \tab Mixed (interval, histogram) \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(joggers.mix)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 2-5.
}
\keyword{datasets}
\keyword{histogram}
\keyword{interval}
\keyword{mixed}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{prostate.int}
\alias{prostate.int}
\title{Prostate Cancer Clinical Interval Dataset}
\format{
A data frame with 97 observations and 9 interval-valued variables:
\itemize{
    \item \code{lcavol}: Log cancer volume range.
    \item \code{lweight}: Log prostate weight range.
    \item \code{age}: Patient age range.
    \item \code{lbph}: Log benign prostatic hyperplasia amount range.
    \item \code{svi}: Seminal vesicle invasion range.
    \item \code{lcp}: Log capsular penetration range.
    \item \code{gleason}: Gleason score range.
    \item \code{pgg45}: Percentage Gleason scores 4 or 5 range.
    \item \code{lpsa}: Log prostate specific antigen range.
}
}
\source{
Extracted from RSDA package (\code{int_prost_train}, \code{int_prost_test}).
}
\usage{
data(prostate.int)
}
\description{
Interval-valued clinical measurements for 97 prostate cancer patients
(training and test sets combined). Contains 9 interval-valued variables
from log-transformed cancer volume, weight, age, and other clinical
predictors.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 97 \cr
  \strong{Variables (p)} \tab 9 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression
}
}

\examples{
data(prostate.int)
}
\references{
Stamey, T. et al. (1989). Prostate specific antigen in the diagnosis and
treatment of adenocarcinoma of the prostate. II. Radical prostatectomy
treated patients. \emph{J. Urology}, 141(5), 1076-1083.
}
\keyword{datasets}
\keyword{interval}
\keyword{medical}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cars.int}
\alias{cars.int}
\title{Cars Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 27 observations and 5 variables:
\itemize{
    \item \code{Price}: Price range (interval).
    \item \code{EngCap}: Engine capacity range (cc, interval).
    \item \code{TopSpeed}: Top speed range (km/h, interval).
    \item \code{Acceleration}: Acceleration range (seconds 0--100 km/h, interval).
    \item \code{class}: Car class (Utilitarian, Berlina, Sportive, Luxury; set-valued).
}
}
\source{
\url{https://CRAN.R-project.org/package=MAINT.Data}
}
\usage{
data(cars.int)
}
\description{
Interval-valued data for 27 car models classified into four classes
(Utilitarian, Berlina, Sportive, Luxury), described by Price,
EngineCapacity, TopSpeed and Acceleration intervals.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 27 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Automotive \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Classification
}
}

\examples{
data(cars.int)
}
\references{
Duarte Silva, A.P., Brito, P., Filzmoser, P. and Dias, J.G. (2021).
MAINT.Data: Modelling and Analysing Interval Data in R.
\emph{R Journal}, 13(2).
}
\keyword{classification}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{RSDA_to_iGAP}
\alias{RSDA_to_iGAP}
\title{RSDA to iGAP}
\usage{
RSDA_to_iGAP(data)
}
\arguments{
\item{data}{The RSDA format with interval dataframe.}
}
\value{
Return a dataframe with the iGAP format.
}
\description{
To convert RSDA format interval dataframe to iGAP format.
}
\examples{
data(mushroom.int)
RSDA_to_iGAP(mushroom.int)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_shape.R
\name{interval_shape}
\alias{interval_shape}
\alias{int_skewness}
\alias{int_kurtosis}
\alias{int_symmetry}
\alias{int_tailedness}
\title{Distribution Shape Measures for Interval Data}
\usage{
int_skewness(x, var_name, method = "CM", ...)

int_kurtosis(x, var_name, method = "CM", ...)

int_symmetry(x, var_name, method = "CM", ...)

int_tailedness(x, var_name, method = "CM", ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name}{the variable name or the column location (multiple variables are allowed).}

\item{method}{methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.}

\item{...}{additional parameters}
}
\value{
A numeric matrix
}
\description{
Functions to compute shape statistics (skewness, kurtosis) for interval-valued data.
}
\details{
These functions measure distribution shape:
\itemize{
  \item \code{int_skewness}: Measure of asymmetry (skewness)
  \item \code{int_kurtosis}: Measure of tail heaviness (kurtosis)
  \item \code{int_symmetry}: Symmetry coefficient
  \item \code{int_tailedness}: Tailedness measure (alias for excess kurtosis)
}

Skewness interpretation:
\itemize{
  \item = 0: Symmetric distribution
  \item > 0: Right-skewed (positive skew)
  \item < 0: Left-skewed (negative skew)
}

Kurtosis interpretation (excess kurtosis):
\itemize{
  \item = 0: Normal distribution (mesokurtic)
  \item > 0: Heavy tails (leptokurtic)
  \item < 0: Light tails (platykurtic)
}
}
\examples{
data(mushroom.int)

# Calculate skewness
int_skewness(mushroom.int, var_name = "Pileus.Cap.Width")
int_skewness(mushroom.int, var_name = 2:3, method = c("CM", "EJD"))

# Calculate kurtosis
int_kurtosis(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))

# Check symmetry
int_symmetry(mushroom.int, var_name = 2:4, method = "CM")

# Check tailedness
int_tailedness(mushroom.int, var_name = "Pileus.Cap.Width", method = "CM")
}
\seealso{
int_mean int_var int_skewness int_kurtosis
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bird.mix}
\alias{bird.mix}
\title{Bird Species Mixed Symbolic Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 20 observations and 2 variables:
\itemize{
    \item \code{Density}: Feather density range (interval).
    \item \code{Size}: Body size range (cm, interval).
}
}
\usage{
data(bird.mix)
}
\description{
Interval-valued morphological measurements for 20 bird specimens.
Despite the \code{.mix} suffix, this dataset contains only
interval-valued variables (density and size).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 20 \cr
  \strong{Variables (p)} \tab 2 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(bird.mix)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.5.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{water_flow.int}
\alias{water_flow.int}
\title{Water Flow Sensor Readings Interval Dataset}
\format{
A data frame with 316 observations and 48 variables:
\itemize{
    \item \code{if1} through \code{if48} (excluding \code{if17}): 47 interval-valued
          sensor feature measurements.
    \item \code{class}: Group label (1 or 2).
}
}
\source{
\url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
}
\usage{
data(water_flow.int)
}
\description{
Large interval-valued dataset of water flow sensor readings with 316
observations and 47 interval-valued feature variables (IF1-IF48, excluding
IF17), classified into 2 groups. Used as a benchmark for interval data
clustering with high-dimensional features.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 316 \cr
  \strong{Variables (p)} \tab 48 \cr
  \strong{Subject area} \tab Engineering \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(water_flow.int)
}
\references{
Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
Kernel clustering with automatic variable weighting for interval data.
\emph{Neurocomputing}, 617, 128954.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hematocrit_hemoglobin.hist}
\alias{hematocrit_hemoglobin.hist}
\title{Hematocrit and Hemoglobin Bivariate Histogram-Valued Dataset}
\format{
A data frame with 10 observations and 2 histogram-valued variables:
\itemize{
    \item \code{hematocrit}: Histogram-valued hematocrit distribution (\%).
    \item \code{hemoglobin}: Histogram-valued hemoglobin distribution (g/dL).
}
}
\source{
Billard, L. and Diday, E. (2006), Table 6.8.
}
\usage{
data(hematocrit_hemoglobin.hist)
}
\description{
Bivariate histogram-valued dataset with 10 observations, each described
by a 2-bin hematocrit histogram and a 2-bin hemoglobin histogram.
Used for bivariate symbolic regression demonstrations.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 2 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Regression
}
}

\examples{
data(hematocrit_hemoglobin.hist)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 6.8.
}
\keyword{datasets}
\keyword{histogram}
\keyword{medical}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{blood.hist}
\alias{blood.hist}
\title{Blood Test Histogram Dataset}
\format{
A data frame with 14 observations and 3 histogram-valued
variables:
\itemize{
    \item \code{Cholesterol}: Histogram of cholesterol levels (mg/dL).
    \item \code{Hemoglobin}: Histogram of hemoglobin levels (g/dL).
    \item \code{Hematocrit}: Histogram of hematocrit levels (\%).
}
}
\source{
HistDAWass R package (\code{BLOOD} dataset).
}
\usage{
data(blood.hist)
}
\description{
Histogram-valued blood test results for 14 gender-age groups (e.g.,
Female-20, Male-50). Each observation contains histograms for
cholesterol, hemoglobin, and hematocrit, represented as multi-bin
distributions.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 14 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(blood.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: a new metric-based approach. \emph{Advances in
Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{BLOOD} dataset).
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ozone.hist}
\alias{ozone.hist}
\title{Ozone Air Quality Histogram-Valued Dataset}
\format{
A data frame with 84 observations (days) and 4 histogram-valued
variables:
\itemize{
    \item \code{Ozone.Conc.ppb}: Histogram of ozone concentration (ppb).
    \item \code{Temperature.C}: Histogram of temperature (Celsius).
    \item \code{Solar.Radiation.WattM2}: Histogram of solar radiation (W/m^2).
    \item \code{Wind.Speed.mSec}: Histogram of wind speed (m/s).
}
Row names are I1 through I84.
}
\source{
HistDAWass R package (\code{OzoneH} dataset).
}
\usage{
data(ozone.hist)
}
\description{
Histogram-valued dataset of 84 daily observations with 4 weather-related
histogram variables. Each histogram has 10 equal-probability (decile) bins
summarizing hourly measurements within each day.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 84 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Environment \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Regression, Clustering
}
}

\examples{
data(ozone.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: A new metric-based approach.
\emph{Advances in Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{OzoneH} dataset),
reduced from 100 quantile bins to 10 decile bins.
}
\keyword{datasets}
\keyword{environment}
\keyword{histogram}
\keyword{weather}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{shanghai_stock.its}
\alias{shanghai_stock.its}
\title{Shanghai Stock Exchange Composite Index Daily High/Low Interval Time Series}
\format{
A data frame with 970 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low value of the SSE Composite Index.
    \item \code{high}: Daily high value of the SSE Composite Index.
}
}
\source{
Yahoo Finance, ticker \code{000001.SS}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(shanghai_stock.its)
}
\description{
Daily high and low values of the Shanghai Stock Exchange Composite Index
(SSE Composite) from January 2, 2019 to December 30, 2022 (970 trading
days). This dataset matches the period used by Yang, Zhang and Wang
(2025) for interval time series forecasting.
}
\details{
The SSE Composite Index is the most commonly used indicator to reflect
the performance of the Shanghai Stock Exchange. It tracks all stocks
(A-shares and B-shares) listed on the exchange. This dataset covers a
period that includes the COVID-19 pandemic and its market impacts,
providing a rich testbed for evaluating interval forecasting models
under extreme volatility.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 970 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(shanghai_stock.its)
head(shanghai_stock.its)
plot(shanghai_stock.its$date, shanghai_stock.its$high, type = "l",
     col = "red", ylab = "Index Value", xlab = "Date",
     main = "Shanghai Composite Daily High/Low (2019-2022)")
lines(shanghai_stock.its$date, shanghai_stock.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Yang, W., Zhang, S. and Wang, S. (2025). On smooth transition interval
autoregressive models. \emph{Journal of Forecasting}, \strong{44}(2),
310--332.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{weight_age.hist}
\alias{weight_age.hist}
\title{Weight by Age Group Histogram-Valued Dataset}
\format{
A data frame with 7 observations and 1 histogram-valued variable:
\itemize{
    \item \code{weight}: Histogram-valued weight distribution (pounds).
}
Row names indicate age groups (20s, 30s, 40s, 50s, 60s, 70s, 80s).
}
\source{
Billard, L. and Diday, E. (2006), Table 3.10.
}
\usage{
data(weight_age.hist)
}
\description{
Histogram-valued weight distributions for 7 age groups (20s through 80s).
Each observation represents an age decade with a 7-bin histogram of
weight values (pounds).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 7 \cr
  \strong{Variables (p)} \tab 1 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(weight_age.hist)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.10.
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{crime2.modal}
\alias{crime2.modal}
\title{Crime Demographics Modal-Valued Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 15 observations and
3 modal-valued variables:
\itemize{
    \item \code{Crime}: Modal distribution over crime types
          (violent, non-violent, none).
    \item \code{Gender}: Modal distribution over gender (male, female).
    \item \code{Age}: Modal distribution over age groups (<20, >=20).
}
}
\usage{
data(crime2.modal)
}
\description{
Modal-valued version of the crime demographics dataset.
See \code{\link{crime.modal}} for the wide-format version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 15 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Criminology \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(crime2.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{loans_by_risk.int}
\alias{loans_by_risk.int}
\title{Lending Club Loans by Risk Level}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 35 observations
and 5 variables:
\itemize{
    \item \code{log_income}: Interval-valued log annual income.
    \item \code{interest_rate}: Interval-valued interest rate (\%).
    \item \code{open_accounts}: Interval-valued number of open credit accounts.
    \item \code{total_accounts}: Interval-valued total number of credit accounts.
    \item \code{risk_level}: Risk grade factor (A, B, C, D, E, F, G).
}
Row names are A1--A5, B1--B5, ..., G1--G5.
}
\source{
MAINT.Data R package (\code{LoansbyRisk_minmax} dataset).
}
\usage{
data(loans_by_risk.int)
}
\description{
Interval-valued dataset of 35 Lending Club loan groups classified by
risk level (A through G, 5 groups each). Each group is described by
4 interval-valued financial variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 35 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Classification, Clustering
}
}

\examples{
data(loans_by_risk.int)
}
\references{
Brito, P. and Duarte Silva, A.P. (2012). Modelling interval data with
Normal and Skew-Normal distributions. \emph{Journal of Applied Statistics},
39(1), 3--20.

Original data from the MAINT.Data R package.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{credit_card.int}
\alias{credit_card.int}
\title{Credit Card Expenses Interval Dataset}
\format{
A data frame with 6 observations and 11 columns (5 interval variables
in \code{_l}/\code{_u} Min-Max pairs, plus a label):
\itemize{
    \item \code{person_month}: Person-month identifier (e.g., "Jon - January"; character).
    \item \code{food_l}, \code{food_u}: Food expenditure range (USD).
    \item \code{social_l}, \code{social_u}: Social expenditure range (USD).
    \item \code{travel_l}, \code{travel_u}: Travel expenditure range (USD).
    \item \code{gas_l}, \code{gas_u}: Gas expenditure range (USD).
    \item \code{clothes_l}, \code{clothes_u}: Clothes expenditure range (USD).
}
}
\usage{
data(credit_card.int)
}
\description{
Interval-valued credit card spending aggregated by person-month.
Three individuals' (Jon, Tom, Leigh) monthly expenditures across
five categories.
}
\details{
The original classical dataset (Table 2.3) records individual
transactions. The symbolic version (Table 2.4) aggregates into
interval-valued observations for each person-month combination.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 6 \cr
  \strong{Variables (p)} \tab 11 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(credit_card.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Tables 2.3-2.4.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{veterinary.int}
\alias{veterinary.int}
\title{Veterinary Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations and 3 variables:
\itemize{
    \item \code{Animal}: Animal type and sex label (e.g., HorseM, BearF; character).
    \item \code{Height}: Height range (cm, interval).
    \item \code{Weight}: Weight range (kg, interval).
}
}
\usage{
data(veterinary.int)
}
\description{
Interval-valued veterinary dataset of 10 animal specimens described
by height and weight ranges. Includes male and female specimens of
horses, bears, foxes, cats, and dogs.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(veterinary.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{blood_pressure.int}
\alias{blood_pressure.int}
\title{Blood Pressure Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 15 observations and
3 interval-valued variables:
\itemize{
    \item \code{Pulse_Rate}: Pulse rate range (beats per minute, interval).
    \item \code{Systolic_Pressure}: Systolic blood pressure range (mmHg, interval).
    \item \code{Diastolic_Pressure}: Diastolic blood pressure range (mmHg, interval).
}
}
\usage{
data(blood_pressure.int)
}
\description{
Interval-valued blood pressure and pulse rate measurements for
15 patient groups.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 15 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Regression
}
}

\examples{
data(blood_pressure.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{fuel_consumption.modal}
\alias{fuel_consumption.modal}
\title{Fuel Consumption by Region Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations and 3 variables:
\itemize{
    \item \code{Region}: Region identifier (character).
    \item \code{Expenditure}: Per-capita fuel expenditure (numeric).
    \item \code{Fuel_Type}: Modal distribution over fuel types
          (gas, oil, electric, other).
}
}
\usage{
data(fuel_consumption.modal)
}
\description{
Modal-valued dataset describing fuel consumption patterns across 10
regions by proportions of heating fuel types (gas, oil, electricity,
other) and per-capita expenditure.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Energy \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Regression
}
}

\examples{
data(fuel_consumption.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 3.7.
}
\keyword{datasets}
\keyword{modal}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{loans_by_purpose.int}
\alias{loans_by_purpose.int}
\title{Loans by Purpose Interval Dataset}
\format{
A data frame with 14 observations and 4 interval-valued variables:
\itemize{
    \item \code{ln_inc}: Natural logarithm of self-reported annual income.
    \item \code{ln_revolbal}: Natural logarithm of total credit revolving balance.
    \item \code{open_acc}: Number of open credit lines.
    \item \code{total_acc}: Total number of credit lines.
}
}
\source{
\url{https://CRAN.R-project.org/package=MAINT.Data}
}
\usage{
data(loans_by_purpose.int)
}
\description{
Interval-valued data for loan characteristics aggregated by their purpose.
Original microdata contains 887,383 loan records from Kaggle.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 14 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(loans_by_purpose.int)
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{video1.int}
\alias{video1.int}
\title{Video Platform User Engagement Intervals (Dataset 1)}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations
and 5 interval-valued variables (V1--V5): number of visits, watches,
likes, comments, and shares.
}
\source{
GPCSIV R package (\code{video1} dataset).
}
\usage{
data(video1.int)
}
\description{
Interval-valued engagement metrics for 10 user groups on a video
platform. Variables represent ranges of visit, watch, like, comment,
and share counts.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Digital media \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab PCA
}
}

\examples{
data(video1.int)
}
\references{
Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
to symbolic histogram variables. \emph{Advances in Data Analysis and
Classification}, 6(2), 147--159.

Original data from the GPCSIV R package (\code{video1} dataset).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{world_cup.int}
\alias{world_cup.int}
\title{World Cup Soccer Teams Interval Dataset}
\format{
A data frame with 2 observations and 8 variables:
\itemize{
    \item \code{world_cup}: Qualification status (yes/no, character).
    \item \code{age_l}, \code{age_u}: Player age range (years).
    \item \code{weight_l}, \code{weight_u}: Player weight range (kg).
    \item \code{height_l}, \code{height_u}: Player height range (meters).
    \item \code{cov_weight_height}: Covariance between weight and height (numeric).
}
}
\usage{
data(world_cup.int)
}
\description{
Interval-valued data for soccer teams grouped by World Cup qualification
status (yes/no). Includes age, weight, height ranges and the covariance
between weight and height.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 2 \cr
  \strong{Variables (p)} \tab 8 \cr
  \strong{Subject area} \tab Sports \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(world_cup.int)
}
\references{
Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley. Table 1.9, p.13.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{write_symbolic_csv}
\alias{write_symbolic_csv}
\title{Write Symbolic Data to a CSV File}
\usage{
write_symbolic_csv(
  x,
  file,
  sep = ",",
  row.names = TRUE,
  na = "NA",
  quote = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{data.frame}, \code{symbolic_tbl}, or other tabular object
containing symbolic data.}

\item{file}{Path to the output CSV file.}

\item{sep}{Field separator character. Default \code{","}.}

\item{row.names}{Logical or character.  If \code{TRUE} (the default),
row names are written as the first column.}

\item{na}{Character string to use for missing values.  Default \code{"NA"}.}

\item{quote}{Logical; should character and factor columns be quoted?
Default \code{TRUE}.}

\item{\dots}{Additional arguments passed to \code{\link[utils]{write.table}}.}
}
\value{
Invisibly returns the data frame that was written (after any
  conversion).
}
\description{
Writes a symbolic data object (interval, histogram, modal, or any
data frame) to a CSV file.  Interval data stored in RSDA format
(\code{symbolic_tbl} with complex columns) is automatically converted to
MM format (paired \code{_min}/\code{_max} columns) before writing.
}
\details{
\code{write_symbolic_csv} handles every tabular symbolic type stored in
\pkg{dataSDA}:
\itemize{
  \item \strong{Interval (RSDA)}: \code{symbolic_tbl} objects with complex
    interval columns are converted to MM format before writing.
  \item \strong{Interval (MM)}: Data frames with \code{_min}/\code{_max}
    columns are written directly.
  \item \strong{Histogram / Modal / Other}: Plain data frames are written
    directly.
}
The output is a standard CSV that can be read back with
\code{\link{read_symbolic_csv}}.
}
\examples{
# Interval data (RSDA symbolic_tbl)
data(mushroom.int)
tmp <- tempfile(fileext = ".csv")
write_symbolic_csv(mushroom.int, tmp)
cat(readLines(tmp, n = 3), sep = "\n")

# Histogram data
data(airline_flights.hist)
tmp2 <- tempfile(fileext = ".csv")
write_symbolic_csv(airline_flights.hist, tmp2)
cat(readLines(tmp2, n = 3), sep = "\n")
}
\seealso{
\code{\link{read_symbolic_csv}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{iris.int}
\alias{iris.int}
\title{Iris Species Interval Dataset}
\format{
A data frame with 30 observations and 5 variables:
\itemize{
    \item \code{sepal_length}: Sepal length range (cm).
    \item \code{sepal_width}: Sepal width range (cm).
    \item \code{petal_length}: Petal length range (cm).
    \item \code{petal_width}: Petal width range (cm).
    \item \code{class}: Species (Setosa, Versicolor, Virginica).
}
}
\source{
\url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
}
\usage{
data(iris.int)
}
\description{
Interval-valued version of the classic iris dataset, aggregated from
Fisher's iris data into 30 interval observations across 3 species
(Setosa, Versicolor, Virginica). Each observation represents a group of
flowers with ranges for sepal and petal measurements.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 30 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Botany \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(iris.int)
}
\references{
Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
Kernel clustering with automatic variable weighting for interval data.
\emph{Neurocomputing}, 617, 128954.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_utils.R
\name{set_variable_format}
\alias{set_variable_format}
\title{Set Variable Format}
\usage{
set_variable_format(data, location = NULL, var = NULL)
}
\arguments{
\item{data}{A conventional data.}

\item{location}{The location of the set variable in the data.}

\item{var}{The name of the set variable in the data.}
}
\value{
Return a dataframe in which a set variable is converted to one-hot encoding.
}
\description{
This function changes the format of the set variables
in the data to conform to the RSDA format.
}
\examples{
data("mushroom.int.mm")
mushroom.set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{int_convert_format}
\alias{int_convert_format}
\title{Convert Interval Data Format}
\usage{
int_convert_format(x, to = "MM", from = NULL, ...)
}
\arguments{
\item{x}{interval data in one of the supported formats}

\item{to}{target format: "MM", "iGAP", "RSDA", "ARRAY", "SODAS" (default: "MM")}

\item{from}{source format (optional): "MM", "iGAP", "RSDA", "ARRAY", "SODAS". If NULL, will auto-detect.}

\item{...}{additional parameters passed to specific conversion functions}
}
\value{
Interval data in the target format
}
\description{
Automatically detect the format of interval data and convert it to the target format.
}
\details{
This function provides a unified interface for all interval format conversions.
It automatically detects the source format (unless specified) and applies the
appropriate conversion function.

Supported conversions:
\itemize{
  \item RSDA ??? MM, iGAP, ARRAY
  \item MM ??? iGAP, RSDA, ARRAY
  \item iGAP ??? MM, RSDA, ARRAY
  \item ARRAY ??? RSDA, MM, iGAP
  \item SODAS ??? MM, iGAP, ARRAY
}
}
\examples{
# Auto-detect and convert to MM
data(mushroom.int)
data_mm <- int_convert_format(mushroom.int, to = "MM")

# Explicitly specify source format
data(abalone.iGAP)
data_mm <- int_convert_format(abalone.iGAP, from = "iGAP", to = "MM")

# Convert MM to iGAP
data_igap <- int_convert_format(data_mm, to = "iGAP")

 # Convert multiple datasets to MM
datasets <- list(mushroom.int, abalone.int, car.int)
mm_datasets <- lapply(datasets, int_convert_format, to = "MM")

# Check what conversions are available
int_list_conversions()
}
\seealso{
int_detect_format int_list_conversions RSDA_to_MM RSDA_to_ARRAY
  MM_to_RSDA MM_to_ARRAY ARRAY_to_RSDA ARRAY_to_MM ARRAY_to_iGAP
  iGAP_to_MM iGAP_to_RSDA iGAP_to_ARRAY MM_to_iGAP
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{djia.its}
\alias{djia.its}
\title{Dow Jones Industrial Average Daily High/Low Interval Time Series}
\format{
A data frame with 504 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low price of the DJIA.
    \item \code{high}: Daily high price of the DJIA.
}
}
\source{
Yahoo Finance, ticker \code{^DJI}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(djia.its)
}
\description{
Daily high and low prices of the Dow Jones Industrial Average (DJIA) from
January 2, 2004 to December 30, 2005 (504 trading days). This dataset
matches the period used in the foundational interval time series work by
Arroyo, Gonzalez-Rivera and Mate (2011).
}
\details{
The DJIA is a price-weighted index of 30 prominent companies listed on
stock exchanges in the United States. Each observation represents a
trading day with the daily low and high prices forming an interval. This
dataset has been used alongside the S&P 500 to compare interval
forecasting methods.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 504 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(djia.its)
head(djia.its)
plot(djia.its$date, djia.its$high, type = "l", col = "red",
     ylab = "Price", xlab = "Date", main = "DJIA Daily High/Low")
lines(djia.its$date, djia.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Arroyo, J., Gonzalez-Rivera, G. and Mate, C. (2011). Forecasting with
interval and histogram data: Some financial applications. In
\emph{Handbook of Empirical Economics and Finance}, pp. 247--280.
Chapman and Hall/CRC.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{judge1.int}
\alias{judge1.int}
\title{Judge 1 Interval-Valued Ratings}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 6 observations
and 4 interval-valued variables (V1--V4).
}
\source{
GPCSIV R package (\code{Judge1} dataset).
}
\usage{
data(judge1.int)
}
\description{
Interval-valued ratings from Judge 1 for 6 regions on 4 variables.
From a study of generalized principal component analysis for
interval-valued data (GPCSIV).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 6 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Methodology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab PCA
}
}

\examples{
data(judge1.int)
}
\references{
Makosso-Kallyth, S. and Diday, E. (2012). Adaptation of interval PCA
to symbolic histogram variables. \emph{Advances in Data Analysis and
Classification}, 6(2), 147--159.

Original data from the GPCSIV R package (\code{Judge1} dataset).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{occupations2.modal}
\alias{occupations2.modal}
\title{Occupation Salaries Modal-Valued Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 9 observations and 4 variables:
\itemize{
    \item \code{Occupation}: Occupation name (character).
    \item \code{Gender}: Modal distribution over gender (Male, Female).
    \item \code{Salary}: Modal distribution over 7 ordered salary bins.
    \item \code{n}: Sample size (numeric).
}
}
\usage{
data(occupations2.modal)
}
\description{
Modal-valued version of the occupation salaries dataset.
See \code{\link{occupations.modal}} for the wide-format version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 9 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Sociology \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(occupations2.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{teams.int}
\alias{teams.int}
\title{Pickup League Teams Interval Dataset}
\format{
A data frame with 5 observations and 7 columns (3 interval variables
in \code{_l}/\code{_u} Min-Max pairs, plus a label):
\itemize{
    \item \code{team_type}: Performance category (Very Good, Good, Average, Fair, Poor).
    \item \code{age_l}, \code{age_u}: Player age range (years).
    \item \code{weight_l}, \code{weight_u}: Player weight range (pounds).
    \item \code{speed_l}, \code{speed_u}: Speed range -- time to run 100 yards (seconds).
}
}
\usage{
data(teams.int)
}
\description{
Interval-valued data for 5 teams in a local pickup league, classified
by season performance. Each team is described by ranges of player age,
weight, and speed.
}
\details{
The symbolic results are more informative than classical midpoint analyses:
the Very Good team has homogeneous players, whereas the Poor team has
players varying widely in age, weight, and speed. Used for symbolic
principal component analysis.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 5 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Sports \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab PCA
}
}

\examples{
data(teams.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.24, p.63.
}
\keyword{PCA}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exchange_rate_returns.hist}
\alias{exchange_rate_returns.hist}
\title{Exchange Rate Returns Histogram Time Series}
\format{
A data frame with 108 observations and 1 histogram-valued
variable:
\itemize{
    \item \code{returns}: Histogram of daily exchange rate returns
    within each month.
}
}
\source{
HistDAWass R package (\code{RetHTS} dataset).
}
\usage{
data(exchange_rate_returns.hist)
}
\description{
Histogram-valued time series of 108 monthly observations of daily
exchange rate returns. Each observation is a histogram distribution
of intra-month daily returns.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 108 \cr
  \strong{Variables (p)} \tab 1 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Time series, Descriptive statistics
}
}

\examples{
data(exchange_rate_returns.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: a new metric-based approach. \emph{Advances in
Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{RetHTS} dataset).
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{temperature_city.int}
\alias{temperature_city.int}
\title{World Cities Monthly Temperature Interval Dataset}
\format{
A data frame with 6 observations and 13 columns (6 monthly interval
variables in \code{_l}/\code{_u} Min-Max pairs, plus a label). Only
January through June are included:
\itemize{
    \item \code{city}: City name (character).
    \item \code{jan_l}, \code{jan_u}: January temperature range (degrees Celsius).
    \item \code{feb_l}, \code{feb_u}: February temperature range.
    \item \code{mar_l}, \code{mar_u}: March temperature range.
    \item \code{apr_l}, \code{apr_u}: April temperature range.
    \item \code{may_l}, \code{may_u}: May temperature range.
    \item \code{jun_l}, \code{jun_u}: June temperature range.
}
}
\usage{
data(temperature_city.int)
}
\description{
Interval-valued monthly temperatures for major cities worldwide.
Benchmark dataset for comparing distance measures (Hausdorff, L2,
Wasserstein) in dynamic clustering algorithms.
}
\details{
Expert partition into 4 classes: Class 1 (tropical/warm), Class 2
(temperate European and Asian), Class 3 (Mauritius), Class 4 (Tehran).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 6 \cr
  \strong{Variables (p)} \tab 13 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(temperature_city.int)
}
\references{
Verde, R. and Irpino, A. (2008). A new interval data distance based on
the Wasserstein metric. \emph{Proc. COMPSTAT 2008}, pp. 705-712.
}
\keyword{clustering}
\keyword{datasets}
\keyword{distance}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{energy_usage.distr}
\alias{energy_usage.distr}
\title{Energy Usage Distribution-Valued Dataset}
\format{
A data frame with 10 observations and 2 distribution-valued variables:
\itemize{
    \item \code{fuel_type}: Distribution over fuel types
          (None, Gas, Oil, Electricity, Coal).
    \item \code{central_heating}: Distribution over central heating
          (No, Yes).
}
Row names are Town_1 through Town_10.
}
\source{
Billard, L. and Diday, E. (2006), Table 3.7.
}
\usage{
data(energy_usage.distr)
}
\description{
Distribution-valued dataset for 10 towns (geographic areas) with
categorical probability distributions for fuel type and central heating.
Each observation has two distribution-valued variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 2 \cr
  \strong{Subject area} \tab Energy \cr
  \strong{Symbolic format} \tab Distribution \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(energy_usage.distr)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.7.
}
\keyword{datasets}
\keyword{distribution}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tennis.int}
\alias{tennis.int}
\title{Tennis Court Types Interval Dataset}
\format{
A data frame with 4 observations and 7 columns (3 interval variables
in \code{_l}/\code{_u} Min-Max pairs, plus a label):
\itemize{
    \item \code{court_type}: Type of court (Hard, Grass, Indoor, Clay).
    \item \code{player_weight_l}, \code{player_weight_u}: Player weight range (kg).
    \item \code{player_height_l}, \code{player_height_u}: Player height range (m).
    \item \code{racket_tension_l}, \code{racket_tension_u}: Racket tension range.
}
}
\usage{
data(tennis.int)
}
\description{
Interval-valued data for tennis players aggregated by court type
(Hard, Grass, Indoor, Clay) with weight, height, and racket tension.
}
\details{
Clustering on weight and height separates grass courts from the rest
(decision rule: Weight <= 74.75 kg). When all three variables are used,
clustering separates by racket tension instead.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 4 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Sports \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(tennis.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.25, p.64.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{face.iGAP}
\alias{face.iGAP}
\title{Face Dataset (iGAP Format)}
\format{
A data frame with 27 observations and 6 character columns in iGAP
format (comma-separated \code{"min,max"} strings):
\itemize{
    \item \code{AD}: Distance AD (facial landmark pair).
    \item \code{BC}: Distance BC (facial landmark pair).
    \item \code{AH}: Distance AH (facial landmark pair).
    \item \code{DH}: Distance DH (facial landmark pair).
    \item \code{EH}: Distance EH (facial landmark pair).
    \item \code{GH}: Distance GH (facial landmark pair).
}
Row names encode individual and replication (e.g., FRA1, FRA2, FRA3).
}
\usage{
data(face.iGAP)
}
\description{
Interval-valued facial measurement data for 27 face images (9 individuals
x 3 replications) in iGAP format (comma-separated interval strings).
Contains 6 distance measurements between facial landmarks.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 27 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Biometrics \cr
  \strong{Symbolic format} \tab Interval (iGAP) \cr
  \strong{Analytical tasks} \tab Classification, Visualization
}
}

\examples{
data(face.iGAP)
}
\references{
Leroy, B., Chouakria, A., Herlin, I., and Diday, E. (1996). Approche
geometrique et classification pour la reconnaissance de visage. In
\emph{Reconnaissance des Formes et Intelligence Artificielle (RFIA)},
548--557.
}
\keyword{datasets}
\keyword{iGAP}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mushroom.int}
\alias{mushroom.int}
\title{Mushroom Species Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 23 observations and 5 variables:
\itemize{
    \item \code{Species}: Mushroom species name (character).
    \item \code{Pileus.Cap.Width}: Pileus cap width range (cm, interval).
    \item \code{Stipe.Length}: Stipe length range (cm, interval).
    \item \code{Stipe.Thickness}: Stipe thickness range (cm, interval).
    \item \code{Edibility}: Edibility code (U = Unknown, Y = Yes, N = No, T = Toxic; character).
}
}
\usage{
data(mushroom.int)
}
\description{
Interval-valued version of the mushroom dataset. See \code{\link{mushroom.int.mm}}.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 23 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Biology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(mushroom.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 3.2.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{iGAP_to_ARRAY}
\alias{iGAP_to_ARRAY}
\title{iGAP to ARRAY}
\usage{
iGAP_to_ARRAY(data, location = NULL)
}
\arguments{
\item{data}{A data.frame in iGAP format.}

\item{location}{Integer vector specifying which columns contain
comma-separated interval values.}
}
\value{
A numeric array of dimension \code{[n, p, 2]} with dimnames.
}
\description{
Convert iGAP format to a 3-dimensional array \code{[n, p, 2]}.
}
\examples{
data(abalone.iGAP)
arr <- iGAP_to_ARRAY(abalone.iGAP, 1:7)
dim(arr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{iGAP_to_RSDA}
\alias{iGAP_to_RSDA}
\title{iGAP to RSDA}
\usage{
iGAP_to_RSDA(data, location = NULL)
}
\arguments{
\item{data}{The dataframe with the iGAP format.}

\item{location}{The location of the symbolic variable in the data.}
}
\value{
Return a symbolic_tbl dataframe with complex-encoded interval columns.
}
\description{
To convert iGAP format interval dataframe to RSDA format (symbolic_tbl).
}
\examples{
data(abalone.iGAP)
rsda <- iGAP_to_RSDA(abalone.iGAP, 1:7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lackinfo.int}
\alias{lackinfo.int}
\title{Lack of Information Questionnaire Interval Dataset}
\format{
A data frame with 50 observations and 8 variables:
\itemize{
    \item \code{id}: Identification number.
    \item \code{sex}: Sex of the respondent (\code{male} or \code{female}).
    \item \code{age}: Respondent's age (in years).
    \item \code{item1}: Interval-valued answer to item 1.
    \item \code{item2}: Interval-valued answer to item 2.
    \item \code{item3}: Interval-valued answer to item 3.
    \item \code{item4}: Interval-valued answer to item 4.
    \item \code{item5}: Interval-valued answer to item 5.
}
}
\source{
\url{https://CRAN.R-project.org/package=IntervalQuestionStat}
}
\usage{
data(lackinfo.int)
}
\description{
Interval-valued dataset from a lack-of-information questionnaire.
Contains biographical data and responses to 5 items measuring perception
of lack of information, collected via an interval-valued Likert scale.
}
\details{
An educational innovation project was carried out for improving
teaching-learning processes at the University of Oviedo (Spain)
for the 2020/2021 academic year. A total of 50 students answered
an online questionnaire about biographical data (sex and age) and
their perception of lack of information by selecting the interval
that best represents their level of agreement on a scale bounded
between 1 (strongly disagree) and 7 (strongly agree).

The 5 items measuring perception of lack of information are:
\itemize{
    \item I1: I receive too little information from my classmates.
    \item I2: It is difficult to receive relevant information from my classmates.
    \item I3: It is difficult to receive relevant information from the teacher.
    \item I4: The amount of information I receive from my classmates is very low.
    \item I5: The amount of information I receive from the teacher is very low.
}
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 50 \cr
  \strong{Variables (p)} \tab 8 \cr
  \strong{Subject area} \tab Education \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Regression
}
}

\examples{
data(lackinfo.int)
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histogram_stats.R
\name{histogram_stats}
\alias{histogram_stats}
\alias{hist_mean}
\alias{hist_var}
\alias{hist_cov}
\alias{hist_cor}
\title{Statistics for Histogram Data}
\usage{
hist_mean(x, var_name, method = "BG", ...)

hist_var(x, var_name, method = "BG", ...)

hist_cov(x, var_name1, var_name2, method = "BG", ...)

hist_cor(x, var_name1, var_name2, method = "BG", ...)
}
\arguments{
\item{x}{histogram-valued data object.}

\item{var_name}{the variable name or the column location.}

\item{method}{method to calculate statistics. One of \code{"BG"} (Bertrand and Goupil, 2000; default),
\code{"BD"} (Billard and Diday, 2006), \code{"B"} (Billard, 2008), or \code{"L2W"} (L2 Wasserstein).
All four methods are available for all four functions.}

\item{...}{additional parameters.}

\item{var_name1}{the variable name or the column location.}

\item{var_name2}{the variable name or the column location.}
}
\value{
A numeric value or vector for \code{hist_mean} and \code{hist_var}; a single numeric value for \code{hist_cov} and \code{hist_cor}.
}
\description{
Functions to compute the mean, variance, covariance, and correlation of histogram-valued data.
}
\details{
Four functions are provided:
\itemize{
  \item \code{hist_mean}: Compute the mean of histogram-valued data.
  \item \code{hist_var}: Compute the variance of histogram-valued data.
  \item \code{hist_cov}: Compute the covariance between two histogram-valued variables.
  \item \code{hist_cor}: Compute the correlation between two histogram-valued variables.
}

Four methods are supported for all functions:
\describe{
  \item{BG}{Bertrand and Goupil (2000) method. Uses histogram bin boundaries
    and probabilities to compute first and second moments.}
  \item{BD}{Billard and Diday (2006) method. A signed decomposition using
    the sign of each bin's midpoint deviation from the overall mean and a
    quadratic form on the bin boundaries.}
  \item{B}{Billard (2008) method. Uses cross-products of deviations of the
    bin boundaries from the overall mean.}
  \item{L2W}{L2 Wasserstein method. Uses optimal-transport (Wasserstein)
    distances between the quantile functions of the histogram distributions.}
}

For the mean, BG, BD, and B return the same value because they share the
same first-order moment definition; only L2W uses a different (quantile-based)
mean.  For variance, covariance, and correlation, all four methods generally
produce different results.

For \code{hist_cor}, the BG, BD, and B correlations all use the
Bertrand-Goupil standard deviation \eqn{S(Y)} in the denominator, following
Irpino and Verde (2015, Eqs. 30--32). Only the L2W method uses its own
Wasserstein-based standard deviation in the denominator.
}
\examples{
library(HistDAWass)
x <- HistDAWass::BLOOD
hist_mean(x, var_name = "Cholesterol", method = "BG")
hist_mean(x, var_name = "Cholesterol", method = "BD")
hist_var(x, var_name = "Cholesterol", method = "BG")
hist_var(x, var_name = "Cholesterol", method = "BD")
hist_cov(x, var_name1 = "Cholesterol", var_name2 = "Hemoglobin", method = "BG")
hist_cor(x, var_name1 = "Cholesterol", var_name2 = "Hemoglobin", method = "BG")
}
\seealso{
int_mean int_var int_cov int_cor
}
\author{
Po-Wei Chen, Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{crime.modal}
\alias{crime.modal}
\title{Crime Demographics Dataset}
\format{
A data frame with 15 observations (gang1--gang15) and 7 numeric
columns representing 3 modal variables in wide format:
\itemize{
    \item \code{Crime(violent)}, \code{Crime(non-violent)}, \code{Crime(none)}:
          Distribution over crime types (3 bins).
    \item \code{Gender(male)}, \code{Gender(female)}:
          Distribution over gender (2 bins).
    \item \code{Age(<20)}, \code{Age(>=20)}:
          Distribution over age groups (2 bins).
}
}
\usage{
data(crime.modal)
}
\description{
Modal-valued dataset of 15 gangs described by probability distributions
over crime type, gender, and age group. This is the wide (flat table)
format; see \code{\link{crime2.modal}} for the modal-valued version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 15 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Criminology \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(crime.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{age_cholesterol_weight.int}
\alias{age_cholesterol_weight.int}
\title{Age-Cholesterol-Weight Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 7 observations and 4 variables:
\itemize{
    \item \code{Age}: Age range (years, interval).
    \item \code{Cholesterol}: Cholesterol level range (mg/dL, interval).
    \item \code{Weight}: Weight range (pounds, interval).
    \item \code{n}: Number of individuals in the age group (numeric).
}
}
\usage{
data(age_cholesterol_weight.int)
}
\description{
Interval-valued dataset of 7 age-group observations with cholesterol and
weight measurements. Each observation aggregates individuals in a 10-year
age band with interval ranges for cholesterol and weight.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 7 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Regression
}
}

\examples{
data(age_cholesterol_weight.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{merval.its}
\alias{merval.its}
\title{MERVAL Index Weekly Min/Max Interval Time Series}
\format{
A data frame with 248 observations and 3 variables:
\itemize{
    \item \code{date}: Week start date, Monday (Date class).
    \item \code{low}: Weekly minimum of daily low values.
    \item \code{high}: Weekly maximum of daily high values.
}
}
\source{
Yahoo Finance, ticker \code{^MERV}. Downloaded via the
\pkg{quantmod} package and aggregated from daily to weekly.
}
\usage{
data(merval.its)
}
\description{
Weekly minimum and maximum values of the Argentine MERVAL stock market
index from January 4, 2016 to September 28, 2020 (248 weeks). Daily
data was downloaded and aggregated to weekly intervals. This dataset
matches the period used by de Carvalho and Martos (2022).
}
\details{
The MERVAL (Mercado de Valores de Buenos Aires) is the main stock market
index of the Buenos Aires Stock Exchange. Each observation represents one
week, with the weekly low computed as the minimum of daily lows and the
weekly high computed as the maximum of daily highs. The date column
indicates the Monday (start) of each week. This period covers the
Argentine economic crisis and the early COVID-19 pandemic impact.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 248 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval time series (weekly aggregation) \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(merval.its)
head(merval.its)
plot(merval.its$date, merval.its$high, type = "l", col = "red",
     ylab = "Index Value", xlab = "Date",
     main = "MERVAL Weekly Min/Max (2016-2020)")
lines(merval.its$date, merval.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
de Carvalho, F. A. T. and Martos, G. (2022). Modeling interval trendlines:
Symbolic singular spectrum analysis for interval time series. \emph{Journal
of Forecasting}, \strong{41}(1), 167--180.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bird_species.mix}
\alias{bird_species.mix}
\title{Bird Species Mixed Symbolic Dataset}
\format{
A data frame with 3 observations (Swallow, Ostrich, Penguin) and
5 variables:
\itemize{
    \item \code{species}: Species name (character).
    \item \code{flying}: Flying ability (Yes/No, character).
    \item \code{size_l}, \code{size_u}: Size range (cm, Min-Max pair).
    \item \code{migration}: Migratory behavior (TRUE/FALSE, logical).
}
}
\usage{
data(bird_species.mix)
}
\description{
Symbolic data for 3 bird species (Swallow, Ostrich, Penguin) with
interval-valued size, categorical flying, and categorical migration.
Foundational SDA example from 600 individual bird observations.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 3 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Mixed (interval, categorical) \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(bird_species.mix)
}
\references{
Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley. Table 1.2, p.6.
}
\keyword{categorical}
\keyword{datasets}
\keyword{interval}
\keyword{mixed}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{SODAS_to_MM}
\alias{SODAS_to_MM}
\title{SODAS to MM}
\usage{
SODAS_to_MM(XMLPath)
}
\arguments{
\item{XMLPath}{Disk path where the SODAS *.XML file is.}
}
\value{
Return a dataframe with the MM format.
}
\description{
To convert SODAS format interval dataframe to the MM format.
}
\examples{
\dontrun{
# Read from a SODAS XML file:
abalone <- SODAS_to_MM("C:/Users/user/AppData/abalone.xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hematocrit.hist}
\alias{hematocrit.hist}
\title{Hematocrit by Gender and Age Histogram-Valued Dataset}
\format{
A data frame with 14 observations and 3 variables:
\itemize{
    \item \code{gender}: Gender (Female or Male).
    \item \code{age}: Age group (20s, 30s, 40s, 50s, 60s, 70s, 80+).
    \item \code{hematocrit}: Histogram-valued hematocrit distribution (\%).
}
}
\source{
Billard, L. and Diday, E. (2006), Table 4.14.
}
\usage{
data(hematocrit.hist)
}
\description{
Histogram-valued hematocrit distributions for 14 gender-age groups
(7 female + 7 male age groups from 20s to 80+). Each observation has
a 10-bin histogram of hematocrit percentages.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 14 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(hematocrit.hist)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 4.14.
}
\keyword{datasets}
\keyword{histogram}
\keyword{medical}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{freshwater_fish.int}
\alias{freshwater_fish.int}
\title{Freshwater Fish Heavy Metal Bioaccumulation Interval Dataset}
\format{
A data frame with 12 observations and 14 variables:
\itemize{
    \item \code{body_length}: Body length (cm).
    \item \code{body_weight}: Body weight (g).
    \item \code{muscle}: Metal concentration in muscle tissue.
    \item \code{intestine}: Metal concentration in intestine.
    \item \code{stomach}: Metal concentration in stomach.
    \item \code{gills}: Metal concentration in gills.
    \item \code{liver}: Metal concentration in liver.
    \item \code{kidney}: Metal concentration in kidney.
    \item \code{liver_muscle_ratio}: Liver-to-muscle concentration ratio.
    \item \code{kidney_muscle_ratio}: Kidney-to-muscle concentration ratio.
    \item \code{gills_muscle_ratio}: Gills-to-muscle concentration ratio.
    \item \code{intestine_muscle_ratio}: Intestine-to-muscle concentration ratio.
    \item \code{stomach_muscle_ratio}: Stomach-to-muscle concentration ratio.
    \item \code{class}: Feeding category (Carnivores, Omnivores, Detritivores, Herbivores).
}
}
\source{
\url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
}
\usage{
data(freshwater_fish.int)
}
\description{
Interval-valued dataset of heavy metal concentrations in organs and tissues
of 12 freshwater fish species, grouped into 4 feeding categories (Carnivores,
Omnivores, Detritivores, Herbivores). Contains 13 interval-valued variables
measuring metal concentrations in organs and organ-to-muscle ratios.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 12 \cr
  \strong{Variables (p)} \tab 14 \cr
  \strong{Subject area} \tab Biology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(freshwater_fish.int)
}
\references{
Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
Kernel clustering with automatic variable weighting for interval data.
\emph{Neurocomputing}, 617, 128954.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sp500.its}
\alias{sp500.its}
\title{S&P 500 Daily High/Low Interval Time Series}
\format{
A data frame with 504 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low price of the S&P 500 index.
    \item \code{high}: Daily high price of the S&P 500 index.
}
}
\source{
Yahoo Finance, ticker \code{^GSPC}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(sp500.its)
}
\description{
Daily high and low prices of the S&P 500 index from January 2, 2004 to
December 30, 2005 (504 trading days). This dataset is a benchmark for
interval time series forecasting, matching the period used in the
foundational work by Arroyo, Gonzalez-Rivera and Mate (2011).
}
\details{
The S&P 500 is a market-capitalization-weighted index of 500 leading
publicly traded companies in the United States. Each observation
represents a trading day with the daily low and high prices forming an
interval. This dataset has been widely used to evaluate interval-valued
autoregressive models, exponential smoothing methods for intervals, and
center-and-range forecasting approaches.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 504 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(sp500.its)
head(sp500.its)
plot(sp500.its$date, sp500.its$high, type = "l", col = "red",
     ylab = "Price", xlab = "Date", main = "S&P 500 Daily High/Low")
lines(sp500.its$date, sp500.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Arroyo, J., Gonzalez-Rivera, G. and Mate, C. (2011). Forecasting with
interval and histogram data: Some financial applications. In
\emph{Handbook of Empirical Economics and Finance}, pp. 247--280.
Chapman and Hall/CRC.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{county_income_gender.hist}
\alias{county_income_gender.hist}
\title{County Income by Gender Histogram-Valued Dataset}
\format{
A data frame with 12 observations (counties) and 4 variables:
\itemize{
    \item \code{male_income}: Histogram of male household income
          (4 bins from $0 to $100k).
    \item \code{female_income}: Histogram of female household income
          (4 bins from $0 to $100k).
    \item \code{n_males}: Number of male respondents (numeric).
    \item \code{n_females}: Number of female respondents (numeric).
}
Row names are County_1 through County_12.
}
\source{
Billard, L. and Diday, E. (2020), Table 6-16.
}
\usage{
data(county_income_gender.hist)
}
\description{
Histogram-valued dataset of 12 counties with gender-stratified income
histograms and sample sizes. Each county has a male income histogram,
a female income histogram, and the number of respondents in each group.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 12 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Economics \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(county_income_gender.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 6-16.
}
\keyword{datasets}
\keyword{gender}
\keyword{histogram}
\keyword{income}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{polish_cars.mix}
\alias{polish_cars.mix}
\title{Polish Car Models Mixed Symbolic Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 30 observations
and 12 variables:
\itemize{
    \item \code{price}: Interval-valued price (PLN).
    \item \code{body}: Multinomial body types (e.g., hatchback, sedan, combi).
    \item \code{wheelbase}: Interval-valued wheelbase (mm).
    \item \code{chassis_length}: Interval-valued chassis length (mm).
    \item \code{chassis_width}: Interval-valued chassis width (mm).
    \item \code{chassis_height}: Interval-valued chassis height (mm).
    \item \code{engine_capacity}: Multinomial engine displacement categories (litres).
    \item \code{engine_power}: Interval-valued engine power (HP).
    \item \code{maximum_speed}: Interval-valued maximum speed (km/h).
    \item \code{acceleration}: Interval-valued 0--100 km/h time (seconds).
    \item \code{fuel_type}: Multinomial fuel types (petrol, diesel, LPG).
    \item \code{fuel_consumption}: Interval-valued fuel consumption (L/100km).
}
}
\source{
symbolicDA R package (\code{cars} dataset).
}
\usage{
data(polish_cars.mix)
}
\description{
Mixed symbolic dataset of 30 car models sold in Poland, with 9
interval-valued technical specification variables and 3 multinomial-valued
categorical variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 30 \cr
  \strong{Variables (p)} \tab 12 \cr
  \strong{Subject area} \tab Automotive \cr
  \strong{Symbolic format} \tab Mixed (interval, multinomial) \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(polish_cars.mix)
}
\references{
Dudek, A. and Pelka, M. (2012). \emph{symbolicDA: Analysis of Symbolic
Data}. R package.
}
\keyword{datasets}
\keyword{interval}
\keyword{mixed}
\keyword{multinomial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mtcars.mix}
\alias{mtcars.mix}
\title{Motor Trend Cars Mixed Symbolic Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 5 observations
(car groups) and 11 variables:
\itemize{
    \item \code{mpg}: Interval-valued miles per gallon.
    \item \code{cyl}: Modal-valued number of cylinders.
    \item \code{disp}: Interval-valued displacement (cu.in.).
    \item \code{hp}: Interval-valued horsepower.
    \item \code{drat}: Interval-valued rear axle ratio.
    \item \code{wt}: Interval-valued weight (1000 lbs).
    \item \code{qsec}: Interval-valued quarter-mile time (seconds).
    \item \code{vs}: Modal-valued engine type (V/S).
    \item \code{am}: Modal-valued transmission type (auto/manual).
    \item \code{gear}: Modal-valued number of forward gears.
    \item \code{carb}: Modal-valued number of carburetors.
}
}
\source{
ggInterval R package (\code{mtcars.i} dataset).
}
\usage{
data(mtcars.mix)
}
\description{
Mixed symbolic dataset of 5 car groups from the \code{mtcars} data,
with 7 interval-valued performance variables and 4 modal-valued
categorical variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 5 \cr
  \strong{Variables (p)} \tab 11 \cr
  \strong{Subject area} \tab Automotive \cr
  \strong{Symbolic format} \tab Mixed (interval, modal) \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(mtcars.mix)
}
\references{
Henderson, R. and Velleman, P. (1981). Building multiple regression
models interactively. \emph{Biometrics}, 37, 391--411.

Original data from the ggInterval R package (\code{mtcars.i} dataset).
}
\keyword{datasets}
\keyword{interval}
\keyword{mixed}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{MM_to_RSDA}
\alias{MM_to_RSDA}
\title{MM to RSDA}
\usage{
MM_to_RSDA(data)
}
\arguments{
\item{data}{The dataframe with the MM format (paired _min/_max columns).}
}
\value{
Return a symbolic_tbl dataframe with complex-encoded interval columns.
}
\description{
To convert MM format interval dataframe to RSDA format (symbolic_tbl).
}
\examples{
data(mushroom.int)
mm <- RSDA_to_MM(mushroom.int, RSDA = FALSE)
rsda <- MM_to_RSDA(mm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lisbon_air_quality.int}
\alias{lisbon_air_quality.int}
\title{Lisbon Air Quality Daily Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 1096 observations
(daily) and 8 interval-valued pollutant variables:
\itemize{
    \item \code{so2}: Sulphur dioxide (ug/m3).
    \item \code{pm10}: Particulate matter < 10 um (ug/m3).
    \item \code{o3}: Ozone (ug/m3).
    \item \code{no2}: Nitrogen dioxide (ug/m3).
    \item \code{co}: Carbon monoxide (ug/m3).
    \item \code{pm25}: Particulate matter < 2.5 um (ug/m3).
    \item \code{nox}: Nitrogen oxides (ug/m3).
    \item \code{no}: Nitric oxide (ug/m3).
}
}
\source{
QualAr, Entrecampos station, Lisbon, Portugal.
}
\usage{
data(lisbon_air_quality.int)
}
\description{
Interval-valued daily air quality data from the Entrecampos monitoring
station in Lisbon, Portugal, covering 2019--2021 (1096 days). Each day's
pollutant concentration is represented as a \eqn{[\min, \max]} interval
from hourly measurements. Missing days are imputed via linear
interpolation.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 1096 \cr
  \strong{Variables (p)} \tab 8 \cr
  \strong{Subject area} \tab Environment \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression, Time series
}
}

\examples{
data(lisbon_air_quality.int)
}
\references{
Dias, S. and Brito, P. (2017). Off the beaten track: A new linear model
for interval data. \emph{European Journal of Operational Research},
258(3), 1118--1130.

Data from the QualAr Portuguese air quality monitoring network
(\samp{https://qualar.apambiente.pt/}).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{int_list_conversions}
\alias{int_list_conversions}
\title{List Available Format Conversions}
\usage{
int_list_conversions(from = NULL, to = NULL)
}
\arguments{
\item{from}{source format (optional): "RSDA", "MM", "iGAP", "ARRAY", "SODAS"}

\item{to}{target format (optional): "RSDA", "MM", "iGAP", "ARRAY", "SODAS"}
}
\value{
A data.frame showing available conversions
}
\description{
List all available format conversion functions.
}
\examples{
# List all conversions
int_list_conversions()

# List conversions from RSDA
int_list_conversions(from = "RSDA")

# List conversions to MM
int_list_conversions(to = "MM")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{RSDA_to_ARRAY}
\alias{RSDA_to_ARRAY}
\title{RSDA to ARRAY}
\usage{
RSDA_to_ARRAY(data)
}
\arguments{
\item{data}{A symbolic_tbl with interval columns.}
}
\value{
A numeric array of dimension \code{[n, p, 2]} with dimnames.
Only interval (symbolic_interval) columns are included.
}
\description{
Convert RSDA format (symbolic_tbl) to a 3-dimensional array
\code{[n, p, 2]} where slice \code{[,,1]} contains the minima and
slice \code{[,,2]} contains the maxima.
}
\examples{
data(mushroom.int)
arr <- RSDA_to_ARRAY(mushroom.int)
dim(arr)  # [23, 3, 2]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{read_symbolic_csv}
\alias{read_symbolic_csv}
\title{Read a Symbolic Data CSV File}
\usage{
read_symbolic_csv(
  file,
  sep = ",",
  header = TRUE,
  row.names = NULL,
  stringsAsFactors = FALSE,
  na.strings = c("", "NA"),
  symbolic_type = NULL,
  ...
)
}
\arguments{
\item{file}{Path to the CSV file to read.}

\item{sep}{Field separator character. Default \code{","}.}

\item{header}{Logical; does the first row contain column names?
Default \code{TRUE}.}

\item{row.names}{Column number or character string giving row names.
Passed to \code{\link[utils]{read.table}}.  Default \code{NULL} (automatic).}

\item{stringsAsFactors}{Logical; should character columns be converted to
factors? Default \code{FALSE}.}

\item{na.strings}{Character vector of strings to interpret as \code{NA}.
Default \code{c("", "NA")}.}

\item{symbolic_type}{Optional character string to override automatic type
detection.  One of \code{"interval"}, \code{"histogram"}, \code{"modal"},
or \code{"other"}.  When \code{NULL} (the default) the type is detected
automatically.}

\item{\dots}{Additional arguments passed to \code{\link[utils]{read.table}}.}
}
\value{
A \code{data.frame}.  Interval data is returned in MM format
  (paired \code{_min}/\code{_max} columns).  All other symbolic types are
  returned as plain data frames.
}
\description{
Reads an external CSV file containing symbolic data, automatically detects
whether the data is interval-valued (min/max pairs or comma-separated),
histogram-valued, modal-valued, or another symbolic type, and returns an
appropriate R object.
}
\details{
The detection heuristic works as follows:
\enumerate{
  \item \strong{Interval (MM)}: If the file contains paired
    \code{_min}/\code{_max} columns the data is returned as-is (MM format).
  \item \strong{Interval (iGAP)}: If one or more character columns contain
    comma-separated numeric pairs (e.g., \code{"1.2,3.4"}) they are
    expanded into \code{_min}/\code{_max} column pairs and the result is
    returned in MM format.
  \item \strong{Histogram / Modal}: If columns follow a \code{VarName(bin)}
    naming pattern (e.g., \code{Crime(violent)}) and the proportions within
    each variable group sum to approximately 1, the data is classified as
    histogram or modal.  It is returned as a plain \code{data.frame}.
  \item \strong{Other}: If none of the above patterns match, the data is
    returned as a plain \code{data.frame}.
}
}
\examples{
# Write then read back an interval dataset
data(mushroom.int.mm)
tmp <- tempfile(fileext = ".csv")
write_symbolic_csv(mushroom.int.mm, tmp)
df <- read_symbolic_csv(tmp)
head(df)

# Write then read back a histogram dataset
data(airline_flights.hist)
tmp2 <- tempfile(fileext = ".csv")
write_symbolic_csv(airline_flights.hist, tmp2)
df2 <- read_symbolic_csv(tmp2)
head(df2)
}
\seealso{
\code{\link{write_symbolic_csv}}, \code{\link{int_detect_format}},
  \code{\link{int_convert_format}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{nycflights.int}
\alias{nycflights.int}
\title{New York City Flights Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 142 observations and 5 variables:
\itemize{
    \item \code{X}: Month-carrier identifier (character).
    \item \code{dep_delay}: Departure delay range (minutes, interval).
    \item \code{arr_delay}: Arrival delay range (minutes, interval).
    \item \code{air_time}: Air time range (minutes, interval).
    \item \code{distance}: Distance range (miles, interval).
}
}
\source{
\url{https://CRAN.R-project.org/package=MAINT.Data}
}
\usage{
data(nycflights.int)
}
\description{
Interval-valued dataset with 142 units and four interval-valued variables
from the nycflights13 package, aggregated by month and carrier.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 142 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Transportation \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression, Descriptive statistics
}
}

\examples{
data(nycflights.int)
}
\references{
Duarte Silva, A.P., Brito, P., Filzmoser, P. and Dias, J.G. (2021).
MAINT.Data: Modelling and Analysing Interval Data in R. \emph{R Journal}, 13(2).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{euro_usd.its}
\alias{euro_usd.its}
\title{Euro/Dollar Exchange Rate Daily High/Low Interval Time Series}
\format{
A data frame with 520 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low EUR/USD exchange rate.
    \item \code{high}: Daily high EUR/USD exchange rate.
}
}
\source{
Yahoo Finance, ticker \code{EURUSD=X}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(euro_usd.its)
}
\description{
Daily high and low values of the EUR/USD exchange rate from January 1,
2004 to December 30, 2005 (520 trading days). Inspired by the dataset
used by Arroyo, Espinola and Mate (2011) for exponential smoothing
methods for interval time series.
}
\details{
The EUR/USD exchange rate is the most traded currency pair in the world
foreign exchange market. Each observation represents a trading day with
the daily low and high exchange rates (USD per EUR) forming an interval.
Note: the original study by Arroyo et al. (2011) used the period
2002--2003 (519 trading days); this dataset covers 2004--2005 because
Yahoo Finance historical data for this ticker is only available from
late 2003 onward.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 520 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance / Foreign Exchange \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(euro_usd.its)
head(euro_usd.its)
plot(euro_usd.its$date, euro_usd.its$high, type = "l", col = "red",
     ylab = "EUR/USD", xlab = "Date",
     main = "EUR/USD Daily High/Low (2004-2005)")
lines(euro_usd.its$date, euro_usd.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Arroyo, J., Espinola, R. and Mate, C. (2011). Different approaches to
forecast interval time series: A comparison in finance. \emph{Computational
Economics}, \strong{37}(2), 169--191.
}
\keyword{datasets}
\keyword{exchange}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{RSDA_to_MM}
\alias{RSDA_to_MM}
\title{RSDA to MM}
\usage{
RSDA_to_MM(data, RSDA = TRUE)
}
\arguments{
\item{data}{The RSDA format with interval dataframe.}

\item{RSDA}{Whether to load the RSDA package.}
}
\value{
Return a dataframe with the MM format.
}
\description{
To convert RSDA format interval dataframe to MM format.
}
\examples{
data(mushroom.int)
RSDA_to_MM(mushroom.int, RSDA = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{health_insurance.mix}
\alias{health_insurance.mix}
\title{Health Insurance Mixed Symbolic Dataset}
\format{
A data frame with 51 observations and 30 variables (Y1--Y30):
\itemize{
    \item \code{Y1}: City (character).
    \item \code{Y2}: Gender (M/F, character).
    \item \code{Y3}: Age (integer).
    \item \code{Y4}: Sex (M/D, character).
    \item \code{Y5}: Marital status (S/M, character).
    \item \code{Y6}: Number of dependents (integer).
    \item \code{Y7}: Parents alive indicator (integer).
    \item \code{Y8}: Number of children (integer).
    \item \code{Y9}: Height (cm, integer).
    \item \code{Y10}: Weight (pounds, integer).
    \item \code{Y11}: Systolic blood pressure (mmHg, integer).
    \item \code{Y12}: Diastolic blood pressure (mmHg, integer).
    \item \code{Y13}: Cholesterol (mg/dL, integer).
    \item \code{Y14}: Cholesterol measure 2 (integer).
    \item \code{Y15}: Additional lab measurement (integer).
    \item \code{Y16}: Ratio measurement (numeric).
    \item \code{Y17}: Lab value (integer).
    \item \code{Y18}: Lab value (integer).
    \item \code{Y19}: Lab value (integer).
    \item \code{Y20}: Lab ratio (numeric).
    \item \code{Y21}: Additional lab value (integer).
    \item \code{Y22}: Additional lab value (integer).
    \item \code{Y23}: Blood chemistry value (numeric).
    \item \code{Y24}: Blood chemistry value (numeric).
    \item \code{Y25}: Blood chemistry value (numeric).
    \item \code{Y26}: Blood chemistry value (numeric).
    \item \code{Y27}: Blood chemistry value (numeric).
    \item \code{Y28}: Diagnostic indicator (Y/N, character).
    \item \code{Y29}: Diagnostic indicator (Y/N, character).
    \item \code{Y30}: Count variable (integer).
}
}
\usage{
data(health_insurance.mix)
}
\description{
Classical (microdata) health insurance dataset of 51 individual patient
records with 30 variables including demographics, clinical measurements,
and diagnostic indicators. This is the raw data underlying the
symbolic \code{\link{health_insurance2.modal}} dataset.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 51 \cr
  \strong{Variables (p)} \tab 30 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Classical (microdata) \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Aggregation
}
}

\examples{
data(health_insurance.mix)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Tables 2.1-2.2.
}
\keyword{datasets}
\keyword{mixed}
\keyword{symbolic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{horses.int}
\alias{horses.int}
\title{Horse Breeds Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 8 observations and 7 variables:
\itemize{
    \item \code{Breed}: Horse breed code (CES, CMA, PEN, TES, CEN, LES, PES, PAM; character).
    \item \code{Minimum_Weight}: Minimum weight range (kg, interval).
    \item \code{Maximum_Weight}: Maximum weight range (kg, interval).
    \item \code{Minimum_Height}: Minimum height range (cm, interval).
    \item \code{Maximum_Height}: Maximum height range (cm, interval).
    \item \code{Mares_Cost}: Cost of mares range (currency units, interval).
    \item \code{Fillies_Cost}: Cost of fillies range (currency units, interval).
}
}
\usage{
data(horses.int)
}
\description{
Interval-valued data for 8 horse breeds (CES, CMA, PEN, TES, CEN,
LES, PES, PAM) described by 6 variables: minimum/maximum weight,
minimum/maximum height, cost of mares, cost of fillies.
}
\details{
Extensively used in SDA for demonstrating divisive clustering, distance
computation, hierarchy/pyramid construction, and complete objects.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 8 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(horses.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 7.14.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{synthetic_clusters.int}
\alias{synthetic_clusters.int}
\title{Synthetic Interval Clusters Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 125 observations and 7 variables:
\itemize{
    \item \code{V1} through \code{V6}: Six interval-valued variables.
    \item \code{class}: Cluster membership (1--5, set-valued).
}
}
\source{
Extracted from symbolicDA package (\code{data_symbolic}).
}
\usage{
data(synthetic_clusters.int)
}
\description{
Synthetic interval-valued dataset with 125 observations in 5 groups of
25 each, described by 6 interval-valued variables and a cluster label.
Designed for benchmarking interval data clustering algorithms.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 125 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Methodology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(synthetic_clusters.int)
}
\references{
Dudek, A. and Pelka, M. (2022). \emph{symbolicDA}: Analysis of Symbolic
Data. R package.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}
\keyword{synthetic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_similarity.R
\name{interval_similarity}
\alias{interval_similarity}
\alias{int_jaccard}
\alias{int_dice}
\alias{int_cosine}
\alias{int_overlap_coefficient}
\alias{int_tanimoto}
\alias{int_similarity_matrix}
\title{Similarity Measures for Interval Data}
\usage{
int_jaccard(x, var_name1, var_name2, ...)

int_dice(x, var_name1, var_name2, ...)

int_cosine(x, var_name1, var_name2, ...)

int_overlap_coefficient(x, var_name1, var_name2, ...)

int_tanimoto(x, var_name1, var_name2, ...)

int_similarity_matrix(x, method = "jaccard", ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name1}{the first variable name or column location.}

\item{var_name2}{the second variable name or column location.}

\item{...}{additional parameters}

\item{method}{similarity method for int_similarity_matrix: "jaccard", "dice", or "overlap".}
}
\value{
A numeric matrix or value
}
\description{
Functions to compute similarity measures between interval-valued observations.
}
\details{
These functions compute various similarity measures:
\itemize{
  \item \code{int_jaccard}: Jaccard similarity coefficient
  \item \code{int_dice}: Dice similarity coefficient
  \item \code{int_cosine}: Cosine similarity
  \item \code{int_overlap_coefficient}: Overlap coefficient
  \item \code{int_tanimoto}: Tanimoto coefficient (generalized Jaccard)
  \item \code{int_similarity_matrix}: Pairwise similarity matrix across all observations
}

All similarity measures range from 0 (no similarity) to 1 (perfect similarity).
}
\examples{
data(mushroom.int)

# Jaccard similarity
int_jaccard(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")

# Dice coefficient
int_dice(mushroom.int, 2, 3)

# Cosine similarity
int_cosine(mushroom.int, 
           var_name1 = c("Pileus.Cap.Width"), 
           var_name2 = c("Stipe.Length", "Stipe.Thickness"))

# Overlap coefficient
int_overlap_coefficient(mushroom.int, 2, 3:4)

# Tanimoto coefficient
int_tanimoto(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")

# Similarity matrix across all observations
int_similarity_matrix(mushroom.int, method = "jaccard")
}
\seealso{
int_dist int_cor int_jaccard
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{search_data}
\alias{search_data}
\title{Search Datasets}
\usage{
search_data(...)
}
\arguments{
\item{...}{Filter expressions. Each argument is a comparison expression
evaluated against the dataset metadata. Supported columns:
\describe{
  \item{\code{n}}{Sample size (numeric). Operators: \code{==, >, <, >=, <=}.}
  \item{\code{p}}{Number of variables (numeric). Operators: \code{==, >, <, >=, <=}.}
  \item{\code{subject}}{Subject area (character). Case-insensitive partial match with \code{==}.
    Areas: Agriculture, Automotive, Biology, Biometrics, Botany, Chemistry, Climate,
    Criminology, Demographics, Digital media, Economics, Education, Energy,
    Engineering, Environment, Finance, Food science, Forestry, Genomics,
    Healthcare, Marine biology, Medical, Methodology, Public services,
    Socioeconomics, Sociology, Sports, Transportation, Zoology.}
  \item{\code{type}}{Symbolic format (character). Exact match with \code{==}.
    Types correspond to the dataset name suffix:
    \code{"int"} (interval), \code{"hist"} (histogram),
    \code{"mix"} (mixed), \code{"distr"} (distribution),
    \code{"its"} (interval time series), \code{"modal"} (modal),
    \code{"iGAP"} (interval in iGAP format).}
  \item{\code{task}}{Analytical tasks (character). Case-insensitive partial match with \code{==}.
    Tasks: Clustering, Classification, Regression, PCA, Descriptive statistics,
    Discriminant analysis, Visualization, Spatial analysis, Time series, Aggregation.}
  \item{\code{tag}}{Keywords (character). Case-insensitive partial match with \code{==}.
    Use \code{tag == "all"} to list all datasets.}
  \item{\code{book}}{Book reference short name (character). Case-insensitive partial match
    with \code{==}. Available books:
    \code{SDA_2006} (Billard & Diday, 2006),
    \code{CMD_2020} (Billard & Diday, 2020),
    \code{SODAS_2008} (Diday & Noirhomme-Fraiture, 2008).}
}}
}
\value{
A data frame with one row per matching dataset and the following
columns: \code{name}, \code{n}, \code{p}, \code{subject}, \code{type},
\code{task}, \code{tag}, \code{book}.
}
\description{
Search and filter the dataSDA dataset catalog by metadata
criteria including sample size, number of variables, subject area,
symbolic format, analytical tasks, keywords, and book reference.
}
\details{
For character columns (\code{subject}, \code{type}, \code{task}, \code{tag},
\code{book}), the \code{==} operator performs a case-insensitive substring
match (using \code{grepl}). The \code{type} column uses short suffix-based
labels that match the dataset name suffix (e.g., \code{type == "int"}
matches all \code{.int} datasets).

For numeric columns (\code{n}, \code{p}), standard comparison operators
are used with exact semantics.

When no arguments are provided, or when \code{tag == "all"} is used,
all datasets are returned.
}
\examples{
# List all datasets
search_data()

# Filter by symbolic format (suffix-based)
search_data(type == "hist")

# Filter by analytical task and size
search_data(task == "Regression", n > 10)

# Filter by book reference
search_data(book == "SDA_2006")

# Combine multiple filters
search_data(type == "int", task == "Clustering", subject == "Biology")

# Filter by size range
search_data(n >= 20, n <= 100, p < 10)

}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester.

Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley.

Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_stats.R
\name{interval_stats}
\alias{interval_stats}
\alias{int_mean}
\alias{int_var}
\alias{int_cov}
\alias{int_cor}
\title{Statistics for Interval Data}
\usage{
int_mean(x, var_name, method = "CM", ...)

int_var(x, var_name, method = "CM", ...)

int_cov(x, var_name1, var_name2, method = "CM", ...)

int_cor(x, var_name1, var_name2, method = "CM", ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name}{the variable name or the column location (multiple variables are allowed).}

\item{method}{methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.}

\item{...}{additional parameters}

\item{var_name1}{the variable name or the column location (multiple variables are allowed).}

\item{var_name2}{the variable name or the column location (multiple variables are allowed).}
}
\value{
A numeric matrix for \code{int_mean} and \code{int_var} (methods x variables);
  a named list of covariance/correlation matrices for \code{int_cov} and \code{int_cor}
  (one matrix per method).
}
\description{
Functions to compute the mean, variance, covariance, and correlation of interval-valued data.
}
\details{
Available methods (applicable to all four functions):
\itemize{
  \item \code{CM}: Center Method — uses midpoints (a + b) / 2
  \item \code{VM}: Vertices Method — uses all 2^p vertex combinations
  \item \code{QM}: Quantiles Method — uses equally spaced quantile points
  \item \code{SE}: Set Expansion — uses endpoints only (quantiles with m = 1)
  \item \code{FV}: Fitted Values — uses linear regression fitted values
  \item \code{EJD}: Empirical Joint Distribution
  \item \code{GQ}: Symbolic Covariance method (Billard and Diday, 2006)
  \item \code{SPT}: Total Sum of Products (Billard, 2008)
}
}
\examples{
data(mushroom.int)
int_mean(mushroom.int, var_name = "Pileus.Cap.Width")
int_mean(mushroom.int, var_name = 2:3)

var_name <- c("Stipe.Length", "Stipe.Thickness")
method <- c("CM", "FV", "EJD")
int_mean(mushroom.int, var_name, method)
int_var(mushroom.int, var_name, method)

var_name1 <- "Pileus.Cap.Width"
var_name2 <- c("Stipe.Length", "Stipe.Thickness")
method <- c("CM", "VM", "EJD", "GQ", "SPT")
int_cov(mushroom.int, var_name1, var_name2, method)
int_cor(mushroom.int, var_name1, var_name2, method)
}
\seealso{
int_mean int_var int_cov int_cor
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{soccer_bivar.int}
\alias{soccer_bivar.int}
\title{French Soccer Championship Bivariate Interval Dataset}
\format{
A data frame with 20 rows and 3 interval-valued variables:
\itemize{
    \item \code{y}: Weight (response variable, kg).
    \item \code{t1}: Height (explanatory variable, cm).
    \item \code{t2}: Age (explanatory variable, years).
}
}
\source{
\url{https://CRAN.R-project.org/package=iRegression}
}
\usage{
data(soccer_bivar.int)
}
\description{
Interval-valued data for 20 teams from the French premier soccer championship.
Contains ranges of Weight (response), Height and Age (explanatory variables).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 20 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Sports \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression
}
}

\examples{
data(soccer_bivar.int)
}
\references{
Lima Neto, E. A., Cordeiro, G. and De Carvalho, F.A.T. (2011).
Bivariate symbolic regression models for interval-valued variables.
\emph{Journal of Statistical Computation and Simulation}, 81, 1727-1744.
}
\keyword{datasets}
\keyword{interval}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_geometry.R
\name{interval_geometry}
\alias{interval_geometry}
\alias{int_width}
\alias{int_radius}
\alias{int_center}
\alias{int_overlap}
\alias{int_containment}
\alias{int_midrange}
\title{Geometric Properties of Interval Data}
\usage{
int_width(x, var_name, ...)

int_radius(x, var_name, ...)

int_center(x, var_name, ...)

int_overlap(x, var_name1, var_name2, ...)

int_containment(x, var_name1, var_name2, ...)

int_midrange(x, var_name, ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name}{the variable name or the column location (multiple variables are allowed).}

\item{...}{additional parameters}

\item{var_name1}{the first variable name or column location.}

\item{var_name2}{the second variable name or column location.}
}
\value{
A numeric matrix or value
}
\description{
Functions to compute geometric characteristics of interval-valued data.
}
\details{
These functions compute basic geometric properties:
\itemize{
  \item \code{int_width}: Width of each interval (upper - lower)
  \item \code{int_radius}: Radius of each interval (width / 2)
  \item \code{int_center}: Center point of each interval ((lower + upper) / 2)
  \item \code{int_overlap}: Overlap measure between two interval variables
  \item \code{int_containment}: Check if one interval contains another
  \item \code{int_midrange}: Half-range of each interval ((upper - lower) / 2)
}
}
\examples{
data(mushroom.int)

# Calculate interval widths
int_width(mushroom.int, var_name = "Pileus.Cap.Width")
int_width(mushroom.int, var_name = 2:3)

# Calculate interval radius
int_radius(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))

# Get interval centers
int_center(mushroom.int, var_name = 2:4)

# Measure overlap between two variables
int_overlap(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")

# Check containment
int_containment(mushroom.int, "Pileus.Cap.Width", "Stipe.Length")

# Calculate midrange
int_midrange(mushroom.int, var_name = 2:3)
}
\seealso{
int_width int_radius int_center int_overlap
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cover_types.hist}
\alias{cover_types.hist}
\title{Forest Cover Types Histogram-Valued Dataset}
\format{
A data frame with 7 observations (cover types) and 4
histogram-valued variables:
\itemize{
    \item \code{elevation}: Histogram of elevation values (meters).
    \item \code{distance_to_water}: Histogram of horizontal distance
          to nearest water source (meters).
    \item \code{hillshade}: Histogram of hillshade index values.
    \item \code{slope}: Histogram of slope values (degrees).
}
Row names are CoverType_1 through CoverType_7.
}
\source{
Billard, L. and Diday, E. (2020), Table 7-21.
}
\usage{
data(cover_types.hist)
}
\description{
Histogram-valued dataset of 7 forest cover types with 4 topographic
histogram variables. Each histogram describes the distribution of a
terrain feature across locations classified as that cover type.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 7 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Forestry \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering, Classification
}
}

\examples{
data(cover_types.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 7-21.
}
\keyword{datasets}
\keyword{forestry}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{health_insurance2.modal}
\alias{health_insurance2.modal}
\title{Health Insurance Modal-Valued Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 6 observations and
6 variables:
\itemize{
    \item \code{Type Gender}: Disease type and gender label (character).
    \item \code{Age}: Modal distribution over age bins.
    \item \code{Marital Status}: Modal distribution over marital status (M, S).
    \item \code{Parents Alive}: Modal distribution over number of parents alive (0, 1, 2).
    \item \code{Weight}: Modal distribution over weight bins (pounds).
    \item \code{Cholesterol}: Modal distribution over cholesterol bins (mg/dL).
}
}
\usage{
data(health_insurance2.modal)
}
\description{
Modal-valued symbolic version of the health insurance dataset, aggregated
into 6 disease-type-by-gender groups. See \code{\link{health_insurance.mix}}
for the underlying microdata.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 6 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(health_insurance2.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 2.2b.
}
\keyword{datasets}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_utils.R
\name{RSDA_format}
\alias{RSDA_format}
\title{RSDA Format}
\usage{
RSDA_format(data, sym_type1 = NULL, location = NULL, sym_type2 = NULL, var = NULL)
}
\arguments{
\item{data}{A conventional data.}

\item{sym_type1}{The labels I means an interval variable and $S means set variable.}

\item{location}{The location of the sym_type in the data.}

\item{sym_type2}{The labels I means an interval variable and $S means set variable.}

\item{var}{The name of the symbolic variable in the data.}
}
\value{
Return a dataframe with a label added to the previous column of symbolic variable.
}
\description{
This function changes the format of the data to conform to RSDA format.
}
\examples{
data("mushroom.int.mm")
mushroom.set <- set_variable_format(data = mushroom.int.mm, location = 8, var = "Species")
mushroom.tmp <- RSDA_format(data = mushroom.set, sym_type1 = c("I", "S"),
                            location = c(25, 31), sym_type2 = c("S", "I", "I"),
                            var = c("Species", "Stipe.Length_min", "Stipe.Thickness_min"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{iGAP_to_MM}
\alias{iGAP_to_MM}
\title{iGAP to MM}
\usage{
iGAP_to_MM(data, location = NULL)
}
\arguments{
\item{data}{The dataframe with the iGAP format.}

\item{location}{The location of the symbolic variable in the data.}
}
\value{
Return a dataframe with the MM format.
}
\description{
To convert iGAP format to MM format.
}
\examples{
data(abalone.iGAP)
abalone <- iGAP_to_MM(abalone.iGAP, 1:7)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{employment.int}
\alias{employment.int}
\title{European Employment by Gender and Age Interval Dataset}
\format{
A data frame with 12 observations and 20 columns (9 interval variables
in \code{_l}/\code{_u} Min-Max pairs, plus a group label and class):
\itemize{
    \item \code{group}: Sex-age group identifier (character).
    \item \code{full_time_l}, \code{full_time_u}: Full-time employment proportion range.
    \item \code{part_time_l}, \code{part_time_u}: Part-time employment proportion range.
    \item \code{primary_studies_l}, \code{primary_studies_u}: Primary studies proportion range.
    \item \code{secondary_studies_l}, \code{secondary_studies_u}: Secondary studies proportion range.
    \item \code{uni_studies_l}, \code{uni_studies_u}: University studies proportion range.
    \item \code{employee_l}, \code{employee_u}: Employee proportion range.
    \item \code{manufacturing_l}, \code{manufacturing_u}: Manufacturing sector proportion range.
    \item \code{construction_l}, \code{construction_u}: Construction sector proportion range.
    \item \code{wholesale_retail_l}, \code{wholesale_retail_u}: Wholesale/retail proportion range.
    \item \code{class}: Group classification (numeric).
}
}
\usage{
data(employment.int)
}
\description{
Interval-valued proportions for 12 sex-age population groups across
employment variables (employment type, education, industry sector,
occupation, marital status). Used for factorial discriminant analysis.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 12 \cr
  \strong{Variables (p)} \tab 20 \cr
  \strong{Subject area} \tab Economics \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Discriminant analysis, Classification
}
}

\examples{
data(employment.int)
}
\references{
Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley. Table 18.1.
}
\keyword{datasets}
\keyword{discriminant}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{MM_to_iGAP}
\alias{MM_to_iGAP}
\title{MM to iGAP}
\usage{
MM_to_iGAP(data)
}
\arguments{
\item{data}{The dataframe with the MM format.}
}
\value{
Return a dataframe with the iGAP format.
}
\description{
To convert MM format to iGAP format.
}
\examples{
data(face.iGAP)
face <- iGAP_to_MM(face.iGAP, 1:6)
MM_to_iGAP(face)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{acid_rain.int}
\alias{acid_rain.int}
\title{Acid Rain Pollution Indices Interval Dataset}
\format{
A data frame with 2 observations and 5 variables in Min-Max format:
\itemize{
    \item \code{state}: State name (character).
    \item \code{sulphate_l}, \code{sulphate_u}: Sulphate pollution index range
          (kg/hectares).
    \item \code{nitrate_l}, \code{nitrate_u}: Nitrate pollution index range
          (kg/hectares).
}
}
\usage{
data(acid_rain.int)
}
\description{
Interval-valued acid rain pollution indices for sulphates and nitrates
(kg/hectares) for 2 US states (Massachusetts and New York).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 2 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Environment \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(acid_rain.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.21.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{french_agriculture.hist}
\alias{french_agriculture.hist}
\title{French Agriculture Histogram-Valued Dataset}
\format{
A data frame with 22 observations (French regions) and 4
histogram-valued variables:
\itemize{
    \item \code{Y_TSC}: Histogram of total standard coefficient.
    \item \code{X_Wheat}: Histogram of wheat production.
    \item \code{X_Pig}: Histogram of pig production.
    \item \code{X_Cmilk}: Histogram of cow milk production.
}
Row names are French region names (e.g., Ile-de-France, Picardie).
}
\source{
HistDAWass R package (\code{Agronomique} dataset).
}
\usage{
data(french_agriculture.hist)
}
\description{
Histogram-valued dataset of 22 French regions with 4 economic
histogram variables related to agricultural production. Each histogram
describes the distribution of farm-level values within a region.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 22 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Agriculture \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Regression, Clustering
}
}

\examples{
data(french_agriculture.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: A new metric-based approach.
\emph{Advances in Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{Agronomique} dataset).
}
\keyword{agriculture}
\keyword{datasets}
\keyword{economics}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ecoli_routes.int}
\alias{ecoli_routes.int}
\title{E. coli Transport Routes Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 9 observations
(transport routes) and 5 interval-valued variables:
\itemize{
    \item \code{Y1} through \code{Y5}: Interval-valued biochemical
          pathway measurements.
}
Row names are Route_1 through Route_9.
}
\source{
Billard, L. and Diday, E. (2020), Table 8-10.
}
\usage{
data(ecoli_routes.int)
}
\description{
Interval-valued dataset of 9 E. coli transport routes with 5 interval
variables representing biochemical pathway measurements.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 9 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Biology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(ecoli_routes.int)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 8-10.
}
\keyword{biology}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{genome_abundances.int}
\alias{genome_abundances.int}
\title{Genome Dinucleotide Abundance Intervals}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 14 observations
(genome classes) and 11 variables:
\itemize{
    \item \code{CG}: Interval-valued CG dinucleotide relative abundance.
    \item \code{GC}: Interval-valued GC dinucleotide relative abundance.
    \item \code{TA}: Interval-valued TA dinucleotide relative abundance.
    \item \code{AT}: Interval-valued AT dinucleotide relative abundance.
    \item \code{CC}: Interval-valued CC dinucleotide relative abundance.
    \item \code{AA}: Interval-valued AA dinucleotide relative abundance.
    \item \code{AC}: Interval-valued AC dinucleotide relative abundance.
    \item \code{AG}: Interval-valued AG dinucleotide relative abundance.
    \item \code{CA}: Interval-valued CA dinucleotide relative abundance.
    \item \code{GA}: Interval-valued GA dinucleotide relative abundance.
    \item \code{n}: Number of genomes in the class (integer).
}
Row names are Class_1 through Class_14.
}
\source{
Billard, L. and Diday, E. (2020), Table 3-16.
}
\usage{
data(genome_abundances.int)
}
\description{
Interval-valued dataset of dinucleotide relative abundances for 14 genome
classes. Each class aggregates multiple genomes; the intervals represent
the range of observed abundance values within each class for 10 dinucleotide
pairs, plus a count variable.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 14 \cr
  \strong{Variables (p)} \tab 11 \cr
  \strong{Subject area} \tab Genomics \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(genome_abundances.int)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 3-16.
}
\keyword{datasets}
\keyword{genomics}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hardwood.hist}
\alias{hardwood.hist}
\title{Hardwood Tree Species Histogram-Valued Dataset}
\format{
A data frame with 5 observations and 4 histogram-valued variables:
\itemize{
    \item \code{ANNT}: Annual temperature histogram (degrees C).
    \item \code{JULT}: July temperature histogram (degrees C).
    \item \code{ANNP}: Annual precipitation histogram (mm).
    \item \code{MITM}: Moisture index histogram.
}
}
\source{
Extracted from RSDA package (\code{hardwoodBrito}).
}
\usage{
data(hardwood.hist)
}
\description{
Histogram-valued climate data for 5 hardwood tree species in the
southeastern United States. Each observation represents a species with
4 histogram-valued climate variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 5 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Forestry \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(hardwood.hist)
}
\references{
Brito, P. (2007). Modelling and Analysing Interval Data.
In V. Esposito Vinzi et al. (Eds.), \emph{New Developments in Classification
and Data Analysis}, pp. 197-208. Springer.
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_utils.R
\name{interval_utils}
\alias{interval_utils}
\title{Internal Utility Functions for Interval Data}
\description{
Internal functions for interval data transformation.
  These are used by the exported interval statistics functions
  (\code{\link{int_mean}}, \code{\link{int_var}}, \code{\link{int_cov}},
  \code{\link{int_cor}}) and are not intended to be called directly.
}
\details{
Internal Utility Functions for Interval Data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_dist.R
\name{interval_distance}
\alias{interval_distance}
\alias{int_dist}
\alias{int_dist_matrix}
\alias{int_pairwise_dist}
\alias{int_dist_all}
\title{Distance Measures for Interval Data}
\usage{
int_dist(x, method = "euclidean", gamma = 0.5, q = 1, p = 2, ...)

int_dist_matrix(x, method = "euclidean", gamma = 0.5, q = 1, p = 2, ...)

int_pairwise_dist(x, var_name1, var_name2, method = "euclidean", ...)

int_dist_all(x, gamma = 0.5, q = 1)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class, or an array of dimension [n, p, 2]}

\item{method}{distance method: "GD", "IY", "L1", "L2", "CB", "HD", "EHD", "nEHD", "snEHD", "TD", "WD", "euclidean", "hausdorff", "manhattan", "city_block", "minkowski", "wasserstein", "ichino", "de_carvalho"}

\item{gamma}{parameter for the Ichino-Yaguchi distance, 0 <= gamma <= 0.5 (default: 0.5)}

\item{q}{parameter for the Ichino-Yaguchi distance (Minkowski exponent) (default: 1)}

\item{p}{power parameter for Minkowski distance (default: 2)}

\item{...}{additional parameters}

\item{var_name1}{first variable name or column location}

\item{var_name2}{second variable name or column location}
}
\value{
A distance matrix (class 'dist') or numeric vector
}
\description{
Functions to compute various distance measures between interval-valued observations.

\code{int_dist_all} computes all available distance measures at once.
}
\details{
Available distance methods:
\itemize{
  \item \code{GD}: Gowda-Diday distance (Gowda & Diday, 1991)
  \item \code{IY}: Ichino-Yaguchi distance (Ichino, 1988)
  \item \code{L1}: L1 (midpoint Manhattan) distance
  \item \code{L2}: L2 (Euclidean midpoint) distance
  \item \code{CB}: City-Block distance (Souza & de Carvalho, 2004)
  \item \code{HD}: Hausdorff distance (Chavent & Lechevallier, 2002)
  \item \code{EHD}: Euclidean Hausdorff distance
  \item \code{nEHD}: Normalized Euclidean Hausdorff distance
  \item \code{snEHD}: Span Normalized Euclidean Hausdorff distance
  \item \code{TD}: Tran-Duckstein distance (Tran & Duckstein, 2002)
  \item \code{WD}: L2-Wasserstein distance (Verde & Irpino, 2008)
  \item \code{euclidean}: Euclidean distance on interval centers (same as L2)
  \item \code{hausdorff}: Hausdorff distance (same as HD)
  \item \code{manhattan}: Manhattan distance (same as L1)
  \item \code{city_block}: City-block distance (same as CB)
  \item \code{minkowski}: Minkowski distance with parameter p
  \item \code{wasserstein}: Wasserstein distance (same as WD)
  \item \code{ichino}: Ichino-Yaguchi distance (simplified version)
  \item \code{de_carvalho}: De Carvalho distance
}
}
\examples{
# Using symbolic_tbl format
data(mushroom.int)
d1 <- int_dist(mushroom.int[, 3:4], method = "euclidean")
d2 <- int_dist(mushroom.int[, 3:4], method = "hausdorff")
d3 <- int_dist(mushroom.int[, 3:4], method = "GD")

# Using array format: 4 concepts, 3 variables
x <- array(NA, dim = c(4, 3, 2))
x[,,1] <- matrix(c(1,2,3,4, 5,6,7,8, 9,10,11,12), nrow=4)
x[,,2] <- matrix(c(3,5,6,7, 8,9,10,12, 13,15,16,18), nrow=4)
d4 <- int_dist(x, method = "snEHD")
d5 <- int_dist(x, method = "IY", gamma = 0.3)
}
\references{
Gowda, K. C., & Diday, E. (1991). Symbolic clustering using a new dissimilarity measure. 
\emph{Pattern Recognition}, 24(6), 567-578.

Ichino, M. (1988). General metrics for mixed features. 
\emph{Systems and Computers in Japan}, 19(2), 37-50.

Chavent, M., & Lechevallier, Y. (2002). Dynamical clustering of interval data. 
In \emph{Classification, Clustering and Data Analysis} (pp. 53-60). Springer.

Tran, L., & Duckstein, L. (2002). Comparison of fuzzy numbers using a fuzzy distance measure.
\emph{Fuzzy Sets and Systems}, 130, 331-341.

Verde, R., & Irpino, A. (2008). A new interval data distance based on the Wasserstein metric.

Kao, C.-H. et al. (2014). Exploratory data analysis of interval-valued symbolic data with
matrix visualization. \emph{Computational Statistics & Data Analysis}, 79, 14--29.
}
\seealso{
int_dist_matrix int_dist_all int_pairwise_dist
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{aggregate_to_symbolic}
\alias{aggregate_to_symbolic}
\title{Aggregate Tabular Data to Symbolic Data}
\usage{
aggregate_to_symbolic(x, type = "int", group_by = "kmeans",
  stratify_var = NULL, K = 5, interval = "range",
  quantile_probs = c(0.05, 0.95), bins = 10, nK = NULL,
  zero_width = c("keep", "remove", "regenerate", "adjust"), epsilon = 1e-07)
}
\arguments{
\item{x}{A data.frame with n rows and p columns. May contain non-numeric
columns used for grouping or stratification; only numeric columns are
aggregated.}

\item{type}{Output symbolic type: \code{"int"} for interval data or
\code{"hist"} for histogram data.}

\item{group_by}{Grouping mechanism. One of:
\describe{
  \item{\code{"kmeans"}}{Partition the data into \code{K} groups using
    k-means clustering.}
  \item{\code{"hclust"}}{Partition the data into \code{K} groups using
    hierarchical clustering.}
  \item{\code{"resampling"}}{Generate \code{K} concepts by randomly
    sampling \code{nK} observations with replacement, repeated \code{K}
    times.}
  \item{A column name or column index}{Use the specified categorical
    variable to define groups.}
}}

\item{stratify_var}{Optional column name or index for a stratification
variable. When provided, grouping and aggregation are performed
independently within each level. Default is \code{NULL}.}

\item{K}{Number of groups for clustering (\code{group_by = "kmeans"} or
\code{"hclust"}) or resampling (\code{group_by = "resampling"}).
Ignored when \code{group_by} is a variable. Default is 5.}

\item{interval}{Interval construction method when \code{type = "int"}:
\code{"range"} uses min/max; \code{"quantile"} uses quantiles given by
\code{quantile_probs}. Default is \code{"range"}.}

\item{quantile_probs}{Numeric vector of length 2 giving the lower and upper
quantile probabilities for \code{interval = "quantile"}.
Default is \code{c(0.05, 0.95)}.}

\item{bins}{Number of histogram bins when \code{type = "hist"}.
Default is 10.}

\item{nK}{Number of observations to sample per group when
\code{group_by = "resampling"}. Default is \code{floor(n / K)}.}

\item{zero_width}{How to handle zero-width intervals (\code{min == max})
produced when \code{type = "int"}. Such degenerate intervals break
downstream tools that divide by interval width (e.g.
\code{ggInterval::ggInterval_indexImage()}). One of:
\describe{
  \item{\code{"keep"}}{(default) Leave the aggregated output unchanged;
    zero-width intervals are returned as-is and no action is taken. Use
    \code{\link{check_zero_width_intervals}} to screen the result.}
  \item{\code{"remove"}}{Drop every concept (row) that contains
    at least one zero-width interval.}
  \item{\code{"regenerate"}}{Re-run the aggregation (re-clustering or
    re-sampling) until no zero-width interval remains. Only effective for
    stochastic \code{group_by} (\code{"kmeans"}, \code{"resampling"});
    for deterministic grouping (a variable or \code{"hclust"}) the result
    cannot change, so an error is raised suggesting another option.}
  \item{\code{"adjust"}}{Add a small amount \code{epsilon} to the upper
    endpoint of each zero-width interval.}
}
Ignored when \code{type = "hist"}.}

\item{epsilon}{Positive amount added to the upper endpoint of each
zero-width interval when \code{zero_width = "adjust"}. Default is
\code{1e-07}.}
}
\value{
\itemize{
  \item For \code{type = "int"}: a \code{symbolic_tbl} (RSDA format) with
    a label column followed by \code{symbolic_interval} columns for each
    numeric variable (K rows, 1 + p columns).
  \item For \code{type = "hist"}: a \code{\link[HistDAWass]{MatH}} object
    (K rows by p columns of histogram-valued data).
}
}
\description{
Aggregate tabular numerical data (n by p) into interval-valued
or histogram-valued symbolic data (K by p) based on a grouping mechanism.
}
\details{
The function aggregates classical tabular data into symbolic data by:
\enumerate{
  \item Partitioning observations into groups via \code{group_by}
    (clustering, resampling, or a categorical variable).
  \item Within each group, summarizing each numeric variable as an
    interval (min/max or quantiles) or a histogram.
}

When \code{stratify_var} is provided, grouping and aggregation are performed
within each level of the stratification variable. Label values are prefixed
by the stratum name (e.g., \code{"setosa.cluster_1"}).

For \code{type = "hist"}, bin boundaries are computed from the global data
range to ensure comparability across groups.

Non-numeric columns (other than those used for grouping or stratification)
are silently excluded from aggregation.
}
\examples{
# Group by a categorical variable -> interval data
res1 <- aggregate_to_symbolic(iris, type = "int", group_by = "Species")
res1

# K-means clustering -> interval data
res2 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
                               group_by = "kmeans", K = 3)

# Quantile-based intervals
res3 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
                               group_by = "kmeans", K = 3,
                               interval = "quantile",
                               quantile_probs = c(0.1, 0.9))

# Resampling -> interval data
set.seed(42)
res4 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
                               group_by = "resampling", K = 5, nK = 30)

# Histogram aggregation
res5 <- aggregate_to_symbolic(iris, type = "hist",
                               group_by = "Species", bins = 5)

# Hierarchical clustering -> interval data
res6 <- aggregate_to_symbolic(iris[, 1:4], type = "int",
                               group_by = "hclust", K = 3)

# Stratified aggregation
res7 <- aggregate_to_symbolic(iris, type = "int",
                               group_by = "kmeans", K = 2,
                               stratify_var = "Species")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{simulated.hist}
\alias{simulated.hist}
\title{Simulated Histogram-Valued Dataset}
\format{
A data frame with 5 observations and 2 histogram-valued variables:
\itemize{
    \item \code{Y1}: Histogram-valued variable 1.
    \item \code{Y2}: Histogram-valued variable 2.
}
Row names are Obs_1 through Obs_5.
}
\source{
Billard, L. and Diday, E. (2020), Table 7-26.
}
\usage{
data(simulated.hist)
}
\description{
Small simulated histogram-valued dataset of 5 observations with 2
histogram-valued variables. Useful for testing and demonstrating
histogram-valued statistical methods.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 5 \cr
  \strong{Variables (p)} \tab 2 \cr
  \strong{Subject area} \tab Methodology \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(simulated.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 7-26.
}
\keyword{datasets}
\keyword{histogram}
\keyword{simulated}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{trivial_intervals.int}
\alias{trivial_intervals.int}
\title{Trivial and Non-Trivial Intervals Example Dataset}
\format{
A data frame with 5 observations (w1--w5) and 6 columns (3 interval
variables in \code{_l}/\code{_u} Min-Max pairs):
\itemize{
    \item \code{y1_l}, \code{y1_u}: First interval variable.
    \item \code{y2_l}, \code{y2_u}: Second interval variable.
    \item \code{y3_l}, \code{y3_u}: Third interval variable.
}
}
\usage{
data(trivial_intervals.int)
}
\description{
Simple 5x3 example illustrating different interval types: full intervals
(hyperrectangles), degenerate intervals (lines), and trivial intervals
(points). Used for vertices PCA demonstration.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 5 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Methodology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab PCA
}
}

\examples{
data(trivial_intervals.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 5.1, p.146.
}
\keyword{PCA}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bank_rates}
\alias{bank_rates}
\title{Bank Interest Rates AR Model Symbolic Dataset}
\format{
A data frame with 4 observations (Bank1--Bank4) and 6 variables:
\itemize{
    \item \code{bank}: Bank identifier (character).
    \item \code{order}: AR model order (numeric).
    \item \code{phi1}: First AR parameter (numeric; NA if unknown).
    \item \code{phi2}: Second AR parameter (numeric; NA if order < 2 or unknown).
    \item \code{phi1_known}: Whether phi1 is known (logical).
    \item \code{phi2_known}: Whether phi2 is known (logical; NA if order < 2).
}
}
\usage{
data(bank_rates)
}
\description{
Symbolic dataset of autoregressive time series models for 4 banks.
Each bank is described by AR model order, parameters, and whether
parameters are known.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 4 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Symbolic (model-valued) \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(bank_rates)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.9.
}
\keyword{datasets}
\keyword{model}
\keyword{symbolic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ibovespa.its}
\alias{ibovespa.its}
\title{IBOVESPA Daily High/Low Interval Time Series}
\format{
A data frame with 3216 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low value of the IBOVESPA index.
    \item \code{high}: Daily high value of the IBOVESPA index.
}
}
\source{
Yahoo Finance, ticker \code{^BVSP}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(ibovespa.its)
}
\description{
Daily high and low values of the Brazilian IBOVESPA stock market index
from January 3, 2000 to December 28, 2012 (3216 trading days). This
dataset matches the period used by Maciel, Ballini and Gomide (2016) for
evolving granular analytics for interval time series forecasting.
}
\details{
The IBOVESPA (Indice Bovespa) is the benchmark index of the Brazilian
stock exchange (B3, formerly BM&FBOVESPA). It tracks the performance of
the most actively traded stocks on the Sao Paulo stock exchange. The
13-year span of this dataset covers multiple market regimes including the
2008 global financial crisis, making it suitable for evaluating
forecasting models under diverse conditions.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 3216 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Time series analysis
}
}

\examples{
data(ibovespa.its)
head(ibovespa.its)
plot(ibovespa.its$date, ibovespa.its$high, type = "l", col = "red",
     ylab = "Index Value", xlab = "Date",
     main = "IBOVESPA Daily High/Low (2000-2012)")
lines(ibovespa.its$date, ibovespa.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Maciel, L., Ballini, R. and Gomide, F. (2016). Evolving granular
analytics for interval time series forecasting. \emph{Granular Computing},
\strong{1}(4), 213--224.
}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hierarchy.hist}
\alias{hierarchy.hist}
\title{Hierarchical Symbolic Dataset with Mixed Types}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 10 observations
and 7 variables:
\itemize{
    \item \code{duration_time}: Histogram-valued duration (2-bin).
    \item \code{hierarchy_1}: Categorical hierarchy level 1 (a/b/c).
    \item \code{hierarchy_2}: Categorical hierarchy level 2 (a1/a2), conditional on hierarchy_1 = a.
    \item \code{hierarchy_3}: Categorical hierarchy level 3 (a11/a12), conditional on hierarchy_2 = a1.
    \item \code{glucose}: Histogram-valued glucose (2-bin), conditional.
    \item \code{pulse_rate}: Histogram-valued pulse rate (2-bin), conditional.
    \item \code{cholesterol}: Interval-valued cholesterol level.
}
}
\source{
Billard, L. and Diday, E. (2007), Table 6.20.
}
\usage{
data(hierarchy.hist)
}
\description{
Mixed symbolic dataset of 10 observations with hierarchical categorical
variables, conditional histogram variables, and an interval-valued
variable. From Table 6.20 of Billard and Diday (2007).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 10 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Methodology \cr
  \strong{Symbolic format} \tab Mixed (histogram, interval, categorical) \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(hierarchy.hist)
}
\references{
Billard, L. and Diday, E. (2007). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 6.20.
}
\keyword{datasets}
\keyword{histogram}
\keyword{interval}
\keyword{mixed}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bird_color_taxonomy.hist}
\alias{bird_color_taxonomy.hist}
\title{Bird Color Taxonomy Histogram Dataset}
\format{
A data frame with 20 observations and 4 variables:
\itemize{
    \item \code{density}: Histogram-valued feather density (up to 4 bins).
    \item \code{size}: Histogram-valued body size (2-bin).
    \item \code{tone}: Categorical tone (dark/light).
    \item \code{shade}: Distribution-valued shade (purple/red/white/yellow
    with fuzzy weights).
}
}
\source{
Billard, L. and Diday, E. (2007), Tables 6.9/6.14.
}
\usage{
data(bird_color_taxonomy.hist)
}
\description{
Mixed symbolic dataset of 20 bird observations with histogram-valued
feather density and body size, categorical tone, and distribution-valued
shade (fuzzy taxonomy). From Tables 6.9 and 6.14 of Billard and Diday
(2007).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 20 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Mixed (histogram, categorical, distribution) \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(bird_color_taxonomy.hist)
}
\references{
Billard, L. and Diday, E. (2007). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Tables 6.9
and 6.14.
}
\keyword{datasets}
\keyword{distribution}
\keyword{histogram}
\keyword{mixed}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{china_temp_monthly.int}
\alias{china_temp_monthly.int}
\title{China Monthly Temperature Intervals (15 Stations)}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 15 observations
(weather stations) and 13 variables:
\itemize{
    \item \code{January}, \code{February}, \code{March}, \code{April},
          \code{May}, \code{June}, \code{July}, \code{August},
          \code{September}, \code{October}, \code{November}, \code{December}:
          Interval-valued monthly temperature ranges (degrees Celsius).
    \item \code{Elevation}: Station elevation above sea level (numeric, meters).
}
Row names are station names (e.g., BoKeTu, Hailaer, LaSa).
}
\source{
Billard, L. and Diday, E. (2020), Table 7-9.
}
\usage{
data(china_temp_monthly.int)
}
\description{
Interval-valued dataset of monthly temperature ranges for 15 weather
stations in China. Each station has 12 monthly temperature intervals
(minimum and maximum observed temperatures in degrees Celsius) and
an elevation value in meters.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 15 \cr
  \strong{Variables (p)} \tab 13 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(china_temp_monthly.int)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 7-9.
}
\keyword{climate}
\keyword{datasets}
\keyword{interval}
\keyword{temperature}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{environment.mix}
\alias{environment.mix}
\title{EPA Environmental Data Mixed Symbolic Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}, of class
\code{c("symbolic_tbl", "tbl_df", "tbl", "data.frame")}) with 14
observations and 17 variables. The first four are modal-valued
(\code{symbolic_modal}) variables and the remaining thirteen are
interval-valued (\code{symbolic_interval}) variables. This matches the
structure of \code{ggInterval::Environment}.
\itemize{
    \item \code{URBANICITY}: Modal-valued urbanicity distribution (modal).
    \item \code{INCOMELEVEL}: Modal-valued income level distribution (modal).
    \item \code{EDUCATION}: Modal-valued education distribution (modal).
    \item \code{REGIONDEVELOPME}: Modal-valued regional development distribution (modal).
    \item \code{CONTROL}: Environmental control index range (interval).
    \item \code{SATISFY}: Satisfaction index range (interval).
    \item \code{INDIVIDUAL}: Individual concern index range (interval).
    \item \code{WELFARE}: Welfare index range (interval).
    \item \code{HUMAN}: Human impact index range (interval).
    \item \code{POLITICS}: Political concern index range (interval).
    \item \code{BURDEN}: Burden index range (interval).
    \item \code{NOISE}: Noise pollution index range (interval).
    \item \code{NATURE}: Nature preservation index range (interval).
    \item \code{SEASETC}: Seas/coastal index range (interval).
    \item \code{MULTI}: Multi-indicator range (interval).
    \item \code{WATERWASTE}: Water/waste index range (interval).
    \item \code{VEHICLE}: Vehicle emissions index range (interval).
}
}
\source{
Extracted from the ggInterval package (\code{Environment}).
}
\usage{
data(environment.mix)
}
\description{
Mixed symbolic dataset from the US EPA with 14 state-group observations
and 17 variables of mixed types: interval-valued environmental measurements
and modal-valued (distributional) categorical variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 14 \cr
  \strong{Variables (p)} \tab 17 \cr
  \strong{Subject area} \tab Environment \cr
  \strong{Symbolic format} \tab Mixed (interval, modal) \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(environment.mix)
}
\references{
Jiang, B.-S. and Wu, H.-M. (2025). ggInterval: an R package for
visualizing interval-valued data using ggplot2. R package version 0.2.5.
\url{https://CRAN.R-project.org/package=ggInterval}
}
\keyword{datasets}
\keyword{interval}
\keyword{mixed}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{iris_species.hist}
\alias{iris_species.hist}
\title{Iris Species Histogram-Valued Dataset}
\format{
A data frame with 3 observations and 5 variables:
\itemize{
    \item \code{species}: Species name (factor: Versicolor, Virginica, Setosa).
    \item \code{sepal_width}: Histogram-valued sepal width distribution.
    \item \code{sepal_length}: Histogram-valued sepal length distribution.
    \item \code{petal_width}: Histogram-valued petal width distribution.
    \item \code{petal_length}: Histogram-valued petal length distribution.
}
Row names are species names.
}
\source{
Billard, L. and Diday, E. (2020), Table 4-10.
}
\usage{
data(iris_species.hist)
}
\description{
Histogram-valued dataset of 3 iris species (Versicolor, Virginica,
Setosa) with 4 histogram-valued morphological variables and a species
label. Each histogram describes the distribution of measurements
within a species.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 3 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Botany \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(iris_species.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 4-10.
}
\keyword{datasets}
\keyword{histogram}
\keyword{iris}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{abalone.int}
\alias{abalone.int}
\title{Abalone Interval Dataset}
\format{
A data frame with 24 observations and 14 columns (7 interval variables
in \code{_min}/\code{_max} pairs):
\itemize{
    \item \code{Length_min}, \code{Length_max}: Shell length range.
    \item \code{Diameter_min}, \code{Diameter_max}: Shell diameter range.
    \item \code{Height_min}, \code{Height_max}: Shell height range.
    \item \code{Whole_min}, \code{Whole_max}: Whole weight range.
    \item \code{Shucked_min}, \code{Shucked_max}: Shucked weight range.
    \item \code{Viscera_min}, \code{Viscera_max}: Viscera weight range.
    \item \code{Shell_min}, \code{Shell_max}: Shell weight range.
}
Row names encode Sex-AgeGroup (e.g., F-10-12 = Female age 10--12).
}
\source{
UCI Machine Learning Repository.
}
\usage{
data(abalone.int)
}
\description{
Interval-valued dataset of 24 units from the UCI Abalone dataset,
aggregated by sex and age group. Min-Max column format (two columns per
variable). See \code{\link{abalone.iGAP}} for the iGAP format version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 24 \cr
  \strong{Variables (p)} \tab 14 \cr
  \strong{Subject area} \tab Marine biology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering, Visualization
}
}

\examples{
data(abalone.int)
}
\references{
Nash, W. J., Sellers, T. L., Talbot, S. R., Cawthorn, A. J., and Ford, W. B.
(1994). The Population Biology of Abalone (\emph{Haliotis} species) in
Tasmania. Sea Fisheries Division, Technical Report No. 48.

UCI Machine Learning Repository.
\url{https://archive.ics.uci.edu/ml/datasets/Abalone}
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{crude_oil_wti.its}
\alias{crude_oil_wti.its}
\title{WTI Crude Oil Futures Daily High/Low Interval Time Series}
\format{
A data frame with 2261 observations and 3 variables:
\itemize{
    \item \code{date}: Trading date (Date class).
    \item \code{low}: Daily low price (USD per barrel).
    \item \code{high}: Daily high price (USD per barrel).
}
}
\source{
Yahoo Finance, ticker \code{CL=F}. Downloaded via the
\pkg{quantmod} package.
}
\usage{
data(crude_oil_wti.its)
}
\description{
Daily high and low prices of WTI (West Texas Intermediate) crude oil
futures from January 2, 2003 to December 30, 2011 (2261 trading days).
This dataset matches the period used by Yang, Han, Hong and Wang (2016)
for analyzing crisis impacts on crude oil prices using interval time
series modelling.
}
\details{
WTI crude oil is a benchmark for oil prices in the Americas. This
dataset covers a period that includes the 2003 Iraq War, the 2007--2008
oil price spike (reaching nearly USD 150/barrel), the 2008 global
financial crisis, and the subsequent recovery. The wide variation in
price levels and volatility regimes makes this dataset ideal for
evaluating interval time series models under structural breaks.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 2261 \cr
  \strong{Variables (p)} \tab 3 (date, low, high) \cr
  \strong{Subject area} \tab Finance / Commodities \cr
  \strong{Symbolic format} \tab Interval time series \cr
  \strong{Analytical tasks} \tab Forecasting, Structural break analysis
}
}

\examples{
data(crude_oil_wti.its)
head(crude_oil_wti.its)
plot(crude_oil_wti.its$date, crude_oil_wti.its$high, type = "l",
     col = "red", ylab = "Price (USD/barrel)", xlab = "Date",
     main = "WTI Crude Oil Daily High/Low (2003-2011)")
lines(crude_oil_wti.its$date, crude_oil_wti.its$low, col = "blue")
legend("topleft", c("High", "Low"), col = c("red", "blue"), lty = 1)
}
\references{
Yang, W., Han, A., Hong, Y. and Wang, S. (2016). Analysis of crisis
impact on crude oil prices: A new approach with interval time series
modelling. \emph{Quantitative Finance}, \strong{16}(12), 1917--1928.
}
\keyword{commodities}
\keyword{datasets}
\keyword{finance}
\keyword{interval}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{utsnow.int}
\alias{utsnow.int}
\title{Utah Snow Load Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 415 observations
and 5 variables:
\itemize{
    \item \code{snow_load}: Interval-valued 50-year ground snow load (kPa).
    \item \code{point_estimate}: Numeric point estimate (kPa).
    \item \code{latitude}: Numeric latitude (degrees).
    \item \code{longitude}: Numeric longitude (degrees).
    \item \code{elevation}: Numeric elevation (meters).
}
}
\source{
intkrige R package (\code{utsnow} dataset).
}
\usage{
data(utsnow.int)
}
\description{
Interval-valued ground snow load data from 415 weather stations in Utah
and surrounding states. Each observation is a station with a 50-year
ground snow load interval (lower and upper bounds of the prediction
interval in kPa) plus the point estimate, geographic coordinates, and
elevation.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 415 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression, Spatial analysis
}
}

\examples{
data(utsnow.int)
}
\references{
Schmoyer, R. L. (1993). Permutation tests for correlation in regression
errors. \emph{Journal of the American Statistical Association}, 89(428),
1507--1516.

Bean, B., Sun, Y., and Maguire, M. (2022). Interval-valued kriging models
for geostatistical mapping with uncertain inputs.

Original data from the intkrige R package (\code{utsnow} dataset).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{polish_voivodships.int}
\alias{polish_voivodships.int}
\title{Polish Voivodships Socio-Economic Intervals}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 18 observations
(voivodships) and 9 interval-valued variables:
\itemize{
    \item \code{V1} through \code{V9}: Interval-valued socio-economic
          indicators aggregated across counties within each voivodship.
}
Row names are voivodship names (e.g., Dolnoslaskie, Lubelskie).
}
\source{
clusterSim R package (\code{data_pathtinger} dataset).
}
\usage{
data(polish_voivodships.int)
}
\description{
Interval-valued dataset of 18 Polish voivodships (administrative regions)
with 9 socio-economic interval variables describing demographic and
economic characteristics at the county (powiat) level.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 18 \cr
  \strong{Variables (p)} \tab 9 \cr
  \strong{Subject area} \tab Socioeconomics \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(polish_voivodships.int)
}
\references{
Dudek, A. and Pelka, M. (2022). \emph{symbolicDA: Analysis of Symbolic
Data}. R package.

Walesiak, M. and Dudek, A. (2020). \emph{clusterSim: Searching for
Optimal Clustering Procedure for a Data Set}. R package.
}
\keyword{datasets}
\keyword{interval}
\keyword{socioeconomic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{finance.int}
\alias{finance.int}
\title{Finance Sector Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 14 observations and 7 variables:
\itemize{
    \item \code{Sector}: Business sector name (character).
    \item \code{Job_Cost}: Job cost range (currency units, interval).
    \item \code{Job_Code}: Job code range (interval).
    \item \code{Activity_Code}: Activity code range (interval).
    \item \code{Monthly_Cost}: Monthly cost range (currency units, interval).
    \item \code{Annual_Budget}: Annual budget range (currency units, interval).
    \item \code{n}: Number of entities in the sector (numeric).
}
}
\usage{
data(finance.int)
}
\description{
Interval-valued data for 14 business sectors described by job-related
financial variables (job cost codes, activity codes, budgets).
Used for PCA demonstrations.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 14 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab PCA
}
}

\examples{
data(finance.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley. Table 5.2.
}
\keyword{PCA}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{wine.int}
\alias{wine.int}
\title{Wine Chemical Properties Interval Dataset}
\format{
A data frame with 33 observations and 10 variables:
\itemize{
    \item \code{V1} through \code{V9}: Nine interval-valued chemical/physical
          property measurements.
    \item \code{class}: Wine group (1 or 2).
}
}
\source{
\url{https://github.com/Natandradesa/Kernel-Clustering-for-Interval-Data}
}
\usage{
data(wine.int)
}
\description{
Interval-valued chemical and physical properties of 33 wine samples
classified into 2 groups. Contains 9 interval-valued measurement variables.
Used as a benchmark for interval data clustering algorithms.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 33 \cr
  \strong{Variables (p)} \tab 10 \cr
  \strong{Subject area} \tab Food science \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(wine.int)
}
\references{
Andrade, N. A., de Carvalho, F. A. T. and Pimentel, B. A. (2025).
Kernel clustering with automatic variable weighting for interval data.
\emph{Neurocomputing}, 617, 128954.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{town_services.mix}
\alias{town_services.mix}
\title{Town Services Concatenated Mixed Symbolic Dataset}
\format{
A data frame with 3 observations (Paris, Lyon, Toulouse) and 8 columns:
\itemize{
    \item \code{town}: Town name (character).
    \item \code{no_pupils_l}, \code{no_pupils_u}: Number of pupils range (Min-Max pair).
    \item \code{type}: School type (modal, character).
    \item \code{level}: Coded level (multi-valued, character).
    \item \code{no_beds_l}, \code{no_beds_u}: Number of beds range (Min-Max pair).
    \item \code{specialty}: Specialty code (multi-valued, character).
}
}
\usage{
data(town_services.mix)
}
\description{
Symbolic data for 3 towns (Paris, Lyon, Toulouse) combining school
and hospital databases. Contains interval-valued, multi-valued, and
modal-valued variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 3 \cr
  \strong{Variables (p)} \tab 8 \cr
  \strong{Subject area} \tab Public services \cr
  \strong{Symbolic format} \tab Mixed (interval, modal, multi-valued) \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(town_services.mix)
}
\references{
Diday, E. and Noirhomme-Fraiture, M. (Eds.) (2008). \emph{Symbolic Data
Analysis and the SODAS Software}. Wiley. Table 1.21, p.19.
}
\keyword{datasets}
\keyword{interval}
\keyword{mixed}
\keyword{modal}
\keyword{multi-valued}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{to_all_interval_formats}
\alias{to_all_interval_formats}
\title{Convert Interval Data to All Supported Formats}
\usage{
to_all_interval_formats(x, ...)
}
\arguments{
\item{x}{Interval data in one of the supported formats:
\code{"RSDA"}, \code{"MM"}, \code{"iGAP"}, \code{"ARRAY"},
\code{"SODAS"}, or \code{"SDS"}.}

\item{...}{Additional arguments passed to conversion functions (e.g.,
\code{location} for iGAP input).}
}
\value{
A named list with six slots:
\describe{
  \item{\code{RSDA}}{A \code{symbolic_tbl} with complex-encoded
    \code{symbolic_interval} columns.}
  \item{\code{MM}}{A \code{data.frame} with paired \code{_min}/\code{_max}
    columns.}
  \item{\code{iGAP}}{A \code{data.frame} with comma-separated
    \code{"min,max"} character values.}
  \item{\code{ARRAY}}{A three-dimensional numeric \code{array} of dimension
    \code{[n, p, 2]} where \code{[,,1]} stores minima and \code{[,,2]}
    stores maxima.}
  \item{\code{SODAS}}{\code{NULL} unless the input is a SODAS XML file path,
    in which case it stores the original path.}
  \item{\code{SDS}}{\code{NULL} unless the input is a SODAS/SDS XML file path
    (alias for SODAS).}
}
}
\description{
Convert interval data from any recognized format to all six
supported interval data formats and return the results as a named list.
This is useful for inspecting and comparing how the same interval data is
represented across different formats.
}
\details{
Six interval data formats are supported in this package.  Each format
stores the same information -- lower and upper bounds for every variable of
every observation -- but differs in its structure and origin:

\describe{
  \item{\strong{RSDA}}{
    A \code{symbolic_tbl} object (class
    \code{c("symbolic_tbl", "tbl_df", "tbl", "data.frame")}) where each
    interval variable is a complex column (\code{symbolic_interval}):
    \code{Re()} gives the minimum and \code{Im()} gives the maximum.
    This is the native format of the \pkg{RSDA} package
    (Billard & Diday, 2006; Rodriguez, 2024).
  }
  \item{\strong{MM (Min-Max)}}{
    A plain \code{data.frame} where each interval variable is represented by
    two numeric columns named \code{<var>_min} and \code{<var>_max}.
    This is a widely used general-purpose representation.
  }
  \item{\strong{iGAP}}{
    A \code{data.frame} where each interval variable is stored as a character
    column with comma-separated values \code{"min,max"}.
    This is the format used by the \pkg{iGAP} software (Correia, 2009).
  }
  \item{\strong{ARRAY}}{
    A three-dimensional numeric \code{array} of size \code{[n, p, 2]}.
    The first slice \code{[,,1]} contains all minima and the second slice
    \code{[,,2]} contains all maxima.  Dimnames encode observation labels,
    variable names, and \code{c("min", "max")}.  This format is convenient
    for matrix-based computations.
  }
  \item{\strong{SODAS}}{
    An XML file on disk produced by the SODAS software (Diday & Noirhomme,
    2008).  In R, SODAS data is referenced by its file path and read via
    \code{RSDA::SODAS.to.RSDA()}.  Since SODAS is a file-based format, it
    cannot be generated from in-memory data.
  }
  \item{\strong{SDS}}{
    An alias for SODAS.  Both refer to the same XML-based format.
  }
}
}
\examples{
data(car.int)
result <- to_all_interval_formats(car.int)
names(result)

# RSDA format (symbolic_tbl)
result$RSDA

# MM format (data.frame with _min/_max columns)
head(result$MM)

# iGAP format (data.frame with comma-separated values)
head(result$iGAP)

# ARRAY format (3D array)
dim(result$ARRAY)
result$ARRAY[1:3, , 1]  # minima
result$ARRAY[1:3, , 2]  # maxima

# SODAS/SDS slots are NULL (file-based format)
result$SODAS
result$SDS
}
\references{
Billard, L. and Diday, E. (2006).
\emph{Symbolic Data Analysis: Conceptual Statistics and Data Mining}.
Wiley.

Rodriguez, O. (2024).
\emph{RSDA: R to Symbolic Data Analysis}.
R package, \url{https://CRAN.R-project.org/package=RSDA}.

Correia, M. (2009).
\emph{Interval GARCH and Aggregation of Predictions}.

Diday, E. and Noirhomme-Fraiture, M. (2008).
\emph{Symbolic Data Analysis and the SODAS Software}.
Wiley.
}
\seealso{
\code{\link{int_detect_format}}, \code{\link{int_convert_format}},
  \code{\link{int_list_conversions}}
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{age_pyramids.hist}
\alias{age_pyramids.hist}
\title{World Age Pyramids Histogram-Valued Dataset (2014)}
\format{
A data frame with 229 observations (countries) and 3
histogram-valued variables:
\itemize{
    \item \code{Both.Sexes.Population}: Histogram of total population
          by age group.
    \item \code{Male.Population}: Histogram of male population by age group.
    \item \code{Female.Population}: Histogram of female population by age group.
}
Row names are country names (e.g., WORLD, Afghanistan, Albania).
}
\source{
HistDAWass R package (\code{Age_Pyramids_2014} dataset).
}
\usage{
data(age_pyramids.hist)
}
\description{
Histogram-valued dataset of 229 countries with 3 population age pyramid
histograms (both sexes, male, female). Each histogram has 21 age bins
representing the distribution of the population across age groups.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 229 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Demographics \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(age_pyramids.hist)
}
\references{
Irpino, A. and Verde, R. (2015). Basic statistics for distributional
symbolic variables: A new metric-based approach.
\emph{Advances in Data Analysis and Classification}, 9(2), 143--175.

Original data from the HistDAWass R package (\code{Age_Pyramids_2014}).
}
\keyword{datasets}
\keyword{demographics}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{household_characteristics.distr}
\alias{household_characteristics.distr}
\title{Household Characteristics Distribution-Valued Dataset}
\format{
A data frame with 12 observations (counties) and 3
distribution-valued variables:
\itemize{
    \item \code{fuel_type}: Distribution over fuel types
          (gas, electric, oil, wood, none).
    \item \code{rooms}: Distribution over room counts
          (\{1,2\}, \{3,4,5\}, \{>=6\}).
    \item \code{household_income}: Distribution over income brackets
          (<10, [10,25), [25,50), [50,75), [75,100), [100,150), [150,200), >=200).
}
Row names are County_1 through County_12.
}
\source{
Billard, L. and Diday, E. (2020), Table 6-1.
}
\usage{
data(household_characteristics.distr)
}
\description{
Distribution-valued dataset of 12 counties with 3 categorical
probability distribution variables describing household fuel type,
number of rooms, and household income brackets.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 12 \cr
  \strong{Variables (p)} \tab 3 \cr
  \strong{Subject area} \tab Socioeconomics \cr
  \strong{Symbolic format} \tab Distribution \cr
  \strong{Analytical tasks} \tab Clustering, Descriptive statistics
}
}

\examples{
data(household_characteristics.distr)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 6-1.
}
\keyword{datasets}
\keyword{distribution}
\keyword{household}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{occupations.modal}
\alias{occupations.modal}
\title{Occupation Salaries Dataset}
\format{
A data frame with 9 observations and 11 columns:
\itemize{
    \item \code{Occupation}: Occupation name (character).
    \item \code{Gender(M)}, \code{Gender(F)}: Proportion male/female (2 bins).
    \item \code{Salary(1)} through \code{Salary(7)}: Salary distribution
          across 7 ordered bins (proportions).
    \item \code{n}: Sample size (integer).
}
}
\usage{
data(occupations.modal)
}
\description{
Modal-valued dataset of 9 occupations with gender and salary distributions.
This is the wide (flat table) format; see \code{\link{occupations2.modal}} for the
modal-valued version.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 9 \cr
  \strong{Variables (p)} \tab 11 \cr
  \strong{Subject area} \tab Sociology \cr
  \strong{Symbolic format} \tab Modal \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(occupations.modal)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{modal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{SODAS_to_ARRAY}
\alias{SODAS_to_ARRAY}
\title{SODAS to ARRAY}
\usage{
SODAS_to_ARRAY(XMLPath)
}
\arguments{
\item{XMLPath}{Disk path where the SODAS \code{*.XML} file is.}
}
\value{
A numeric array of dimension \code{[n, p, 2]} with dimnames.
}
\description{
Convert SODAS format (XML file) to a 3-dimensional array
\code{[n, p, 2]}.
}
\examples{
\dontrun{
arr <- SODAS_to_ARRAY("C:/Users/user/AppData/abalone.xml")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{state_income.hist}
\alias{state_income.hist}
\title{State Income Histogram-Valued Dataset}
\format{
A data frame with 6 observations (states) and 4 histogram-valued
variables:
\itemize{
    \item \code{Y1} through \code{Y4}: Histogram-valued income distribution
          variables.
}
Row names are State_1 through State_6.
}
\source{
Billard, L. and Diday, E. (2020), Table 7-18.
}
\usage{
data(state_income.hist)
}
\description{
Histogram-valued dataset of 6 US states with 4 income distribution
histograms. Each histogram describes the distribution of household
income within a state.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 6 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Economics \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(state_income.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 7-18.
}
\keyword{datasets}
\keyword{histogram}
\keyword{income}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{car.int}
\alias{car.int}
\title{Car Models Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 8 observations and 5 variables:
\itemize{
    \item \code{Car}: Car brand name (character).
    \item \code{Price}: Price range (thousands of currency units, interval).
    \item \code{Max_Velocity}: Maximum velocity range (km/h, interval).
    \item \code{Accn_Time}: Acceleration time range (seconds 0--100 km/h, interval).
    \item \code{Cylinder_Capacity}: Engine cylinder capacity range (cc, interval).
}
}
\usage{
data(car.int)
}
\description{
Interval-valued data for 8 car brands with price and performance
specifications. Each brand aggregates multiple models into interval ranges.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 8 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Automotive \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(car.int)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{bird_species_extended.mix}
\alias{bird_species_extended.mix}
\title{Bird Species Extended Mixed Symbolic Dataset}
\format{
A data frame with 3 observations and 6 variables:
\itemize{
    \item \code{species}: Species name (character).
    \item \code{flying}: Flying ability (Yes/No, character).
    \item \code{height_l}: Height lower bound (cm, numeric).
    \item \code{height_u}: Height upper bound (cm, numeric).
    \item \code{color}: Color distribution as weighted set string
          (e.g., "\{white, 0.3; black, 0.7\}").
    \item \code{migratory}: Migratory behavior (Yes/No, character).
}
}
\usage{
data(bird_species_extended.mix)
}
\description{
Three bird species (Geese, Ostrich, Penguin) with interval-valued
height, distribution-valued color, and categorical flying/migratory
variables.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 3 \cr
  \strong{Variables (p)} \tab 6 \cr
  \strong{Subject area} \tab Zoology \cr
  \strong{Symbolic format} \tab Mixed (interval, categorical, distribution) \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(bird_species_extended.mix)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.19.
}
\keyword{categorical}
\keyword{datasets}
\keyword{histogram}
\keyword{interval}
\keyword{mixed}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_uncertainty.R
\name{interval_uncertainty}
\alias{interval_uncertainty}
\alias{int_entropy}
\alias{int_cv}
\alias{int_dispersion}
\alias{int_imprecision}
\alias{int_granularity}
\alias{int_uniformity}
\alias{int_information_content}
\title{Uncertainty and Variability Measures for Interval Data}
\usage{
int_entropy(x, var_name, method = "CM", base = 2, ...)

int_cv(x, var_name, method = "CM", ...)

int_dispersion(x, var_name, method = "CM", ...)

int_imprecision(x, var_name, ...)

int_granularity(x, var_name, ...)

int_uniformity(x, var_name, ...)

int_information_content(x, var_name, method = "CM", ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name}{the variable name or the column location (multiple variables are allowed).}

\item{method}{methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.}

\item{base}{logarithm base for entropy calculation (default: 2)}

\item{...}{additional parameters}
}
\value{
A numeric matrix or value
}
\description{
Functions to compute uncertainty and variability measures for interval-valued data.
}
\details{
These functions measure uncertainty and variability:
\itemize{
  \item \code{int_entropy}: Shannon entropy (information content)
  \item \code{int_cv}: Coefficient of variation (CV = SD / Mean)
  \item \code{int_dispersion}: General dispersion index
  \item \code{int_imprecision}: Imprecision based on interval width
  \item \code{int_granularity}: Variability in interval sizes
  \item \code{int_uniformity}: Uniformity of interval widths (inverse of granularity)
  \item \code{int_information_content}: Normalized entropy (entropy / log2(n))
}
}
\examples{
data(mushroom.int)

# Calculate entropy
int_entropy(mushroom.int, var_name = "Pileus.Cap.Width")

# Coefficient of variation
int_cv(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"), method = c("CM", "EJD"))

# Measure imprecision
int_imprecision(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))

# Dispersion index
int_dispersion(mushroom.int, var_name = "Pileus.Cap.Width", method = "CM")

# Check data granularity
int_granularity(mushroom.int, var_name = 2:4)

# Check uniformity
int_uniformity(mushroom.int, var_name = 2:3)

# Information content
int_information_content(mushroom.int, var_name = "Stipe.Length", method = "CM")
}
\seealso{
int_var int_entropy int_cv
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_position.R
\name{interval_position}
\alias{interval_position}
\alias{int_median}
\alias{int_quantile}
\alias{int_range}
\alias{int_iqr}
\alias{int_mad}
\alias{int_mode}
\title{Position and Scale Measures for Interval Data}
\usage{
int_median(x, var_name, method = "CM", ...)

int_quantile(x, var_name, probs = c(0.25, 0.5, 0.75), method = "CM", ...)

int_range(x, var_name, method = "CM", ...)

int_iqr(x, var_name, method = "CM", ...)

int_mad(x, var_name, method = "CM", ...)

int_mode(x, var_name, method = "CM", breaks = 30, ...)
}
\arguments{
\item{x}{interval-valued data with symbolic_tbl class.}

\item{var_name}{the variable name or the column location (multiple variables are allowed).}

\item{method}{methods to calculate statistics: CM (default), VM, QM, SE, FV, EJD, GQ, SPT.}

\item{...}{additional parameters}

\item{probs}{numeric vector of probabilities with values in [0,1].}

\item{breaks}{number of histogram breaks for mode estimation (default: 30).}
}
\value{
A numeric matrix or value
}
\description{
Functions to compute position and scale statistics for interval-valued data.
}
\details{
These functions provide position and scale measures:
\itemize{
  \item \code{int_median}: Median of interval data
  \item \code{int_quantile}: Quantiles of interval data
  \item \code{int_range}: Range (max - min) of interval data
  \item \code{int_iqr}: Interquartile range (Q3 - Q1)
  \item \code{int_mad}: Median absolute deviation
  \item \code{int_mode}: Mode of interval data (estimated via histogram)
}
}
\examples{
data(mushroom.int)

# Calculate median
int_median(mushroom.int, var_name = "Pileus.Cap.Width")
int_median(mushroom.int, var_name = 2:3, method = c("CM", "EJD"))

# Calculate quantiles
int_quantile(mushroom.int, var_name = 2, probs = c(0.25, 0.5, 0.75))

# Calculate interquartile range
int_iqr(mushroom.int, var_name = c("Stipe.Length", "Stipe.Thickness"))

# Calculate range
int_range(mushroom.int, var_name = "Pileus.Cap.Width")

# Calculate MAD
int_mad(mushroom.int, var_name = 2:3, method = "CM")

# Estimate mode
int_mode(mushroom.int, var_name = "Stipe.Length", method = "CM")
}
\seealso{
int_mean int_var int_median int_quantile
}
\author{
Han-Ming Wu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{china_temp.int}
\alias{china_temp.int}
\title{China Meteorological Stations Quarterly Temperature Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 899 observations and 5 variables:
\itemize{
    \item \code{Q1}: Quarter 1 (Jan--Mar) temperature range (tenths of degrees Celsius, interval).
    \item \code{Q2}: Quarter 2 (Apr--Jun) temperature range (interval).
    \item \code{Q3}: Quarter 3 (Jul--Sep) temperature range (interval).
    \item \code{Q4}: Quarter 4 (Oct--Dec) temperature range (interval).
    \item \code{GeoReg}: Geographic region classification (factor).
}
}
\source{
\url{https://CRAN.R-project.org/package=MAINT.Data}
}
\usage{
data(china_temp.int)
}
\description{
Interval-valued temperature data (Celsius) for 60 Chinese meteorological
stations observed over the four quarters of years 1974 to 1988.
One outlier observation (YinChuan_1982) has been discarded.
}
\details{
Originates from the Long-Term Instrumental Climatic Database of the
People's Republic of China. Widely used in the SDA literature for
demonstrating standardization, clustering, self-organizing maps,
MLE and MANOVA.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 899 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Climate \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(china_temp.int)
}
\references{
Brito, P. and Duarte Silva, A.P. (2012). Modelling interval data with
Normal and Skew-Normal distributions. \emph{J. Appl. Stat.}, 39(1), 3-20.
}
\keyword{clustering}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{abalone.iGAP}
\alias{abalone.iGAP}
\title{Abalone Dataset (iGAP Format)}
\format{
A data frame with 24 observations (e.g., F-10-12, M-4-6) and
7 character columns in iGAP format (comma-separated \code{"min, max"} strings):
\itemize{
    \item \code{Length}: Shell length range.
    \item \code{Diameter}: Shell diameter range.
    \item \code{Height}: Shell height range.
    \item \code{Whole}: Whole weight range.
    \item \code{Shucked}: Shucked weight range.
    \item \code{Viscera}: Viscera weight range.
    \item \code{Shell}: Shell weight range.
}
Row names encode Sex-AgeGroup (e.g., F-10-12 = Female age 10--12).
}
\source{
UCI Machine Learning Repository.
}
\usage{
data(abalone.iGAP)
}
\description{
Interval-valued dataset of 24 units from the UCI Abalone dataset,
aggregated by sex and age group. iGAP format (comma-separated interval
strings). See \code{\link{abalone.int}} for the Min-Max column format.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 24 \cr
  \strong{Variables (p)} \tab 7 \cr
  \strong{Subject area} \tab Marine biology \cr
  \strong{Symbolic format} \tab Interval (iGAP) \cr
  \strong{Analytical tasks} \tab Clustering, Visualization
}
}

\examples{
data(abalone.iGAP)
}
\references{
Nash, W. J., Sellers, T. L., Talbot, S. R., Cawthorn, A. J., and Ford, W. B.
(1994). The Population Biology of Abalone (\emph{Haliotis} species) in
Tasmania. Sea Fisheries Division, Technical Report No. 48.

UCI Machine Learning Repository.
\url{https://archive.ics.uci.edu/ml/datasets/Abalone}
}
\keyword{datasets}
\keyword{iGAP}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{lung_cancer.hist}
\alias{lung_cancer.hist}
\title{Lung Cancer Treatments by State Histogram-Valued Dataset}
\format{
A data frame with 2 observations and 2 variables:
\itemize{
    \item \code{state}: State name (character).
    \item \code{y30}: Histogram-valued distribution of treatment counts
          as a weighted set string (e.g., "\{0, 0.77; 1, 0.08; 2, 0.15\}").
}
}
\usage{
data(lung_cancer.hist)
}
\description{
Histogram-valued distribution of lung cancer treatment counts for 2 US
states (Massachusetts and New York).
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 2 \cr
  \strong{Variables (p)} \tab 2 \cr
  \strong{Subject area} \tab Medical \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Descriptive statistics
}
}

\examples{
data(lung_cancer.hist)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis}. Wiley.
Table 2.20.
}
\keyword{datasets}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{flights_detail.hist}
\alias{flights_detail.hist}
\title{Airline Flights Detailed Histogram-Valued Dataset}
\format{
A data frame with 16 observations (airlines) and 5 histogram-valued
variables:
\itemize{
    \item \code{airtime}: Histogram of air time (minutes).
    \item \code{taxi_in}: Histogram of taxi-in time (minutes).
    \item \code{arrival_delay}: Histogram of arrival delay (minutes).
    \item \code{taxi_out}: Histogram of taxi-out time (minutes).
    \item \code{departure_delay}: Histogram of departure delay (minutes).
}
Row names are Airline_1 through Airline_16.
}
\source{
Billard, L. and Diday, E. (2020), Table 5-1.
}
\usage{
data(flights_detail.hist)
}
\description{
Histogram-valued dataset of 16 airlines with 5 flight performance
histograms. Each histogram has 12 bins describing the distribution
of a performance metric across flights for that airline.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 16 \cr
  \strong{Variables (p)} \tab 5 \cr
  \strong{Subject area} \tab Transportation \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Clustering
}
}

\examples{
data(flights_detail.hist)
}
\references{
Billard, L. and Diday, E. (2020). \emph{Clustering Methodology for
Symbolic Data}. Wiley, Chichester. Table 5-1.
}
\keyword{datasets}
\keyword{flights}
\keyword{histogram}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_format_conversions.R
\name{MM_to_ARRAY}
\alias{MM_to_ARRAY}
\title{MM to ARRAY}
\usage{
MM_to_ARRAY(data)
}
\arguments{
\item{data}{A data.frame in MM format with paired \code{_min} and \code{_max}
columns.}
}
\value{
A numeric array of dimension \code{[n, p, 2]} with dimnames.
Non-interval columns are excluded.
}
\description{
Convert MM format (paired \code{_min}/\code{_max} columns) to a
3-dimensional array \code{[n, p, 2]}.
}
\examples{
data(mushroom.int)
mm <- RSDA_to_MM(mushroom.int, RSDA = FALSE)
arr <- MM_to_ARRAY(mm)
dim(arr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{loans_by_risk_quantile.int}
\alias{loans_by_risk_quantile.int}
\title{Lending Club Loans by Risk Level (Quantile-Based Intervals)}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 35 observations
and 4 variables:
\itemize{
    \item \code{ln-inc}: Interval-valued log income.
    \item \code{int-rate}: Interval-valued interest rate.
    \item \code{open-acc}: Interval-valued number of open accounts.
    \item \code{total-acc}: Interval-valued total accounts.
}
}
\source{
MAINT.Data R package (\code{LoansbyRiskLvs_qntlDt} dataset).
}
\usage{
data(loans_by_risk_quantile.int)
}
\description{
Interval-valued dataset of 35 Lending Club loan groups stratified by risk
level (A1--G5). Intervals represent the 10th to 90th percentile range of
each financial variable within each risk subgrade.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 35 \cr
  \strong{Variables (p)} \tab 4 \cr
  \strong{Subject area} \tab Finance \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Classification, Clustering
}
}

\examples{
data(loans_by_risk_quantile.int)
}
\references{
Brito, P. and Duarte Silva, A.P. (2012). Modelling interval data with
Normal and Skew-Normal distributions. \emph{Journal of Applied Statistics},
39(1), 3--20.

Original data from the MAINT.Data R package
(\code{LoansbyRiskLvs_qntlDt} dataset).
}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval_utils.R
\name{clean_colnames}
\alias{clean_colnames}
\title{clean_colnames}
\usage{
clean_colnames(data)
}
\arguments{
\item{data}{The conventional data.}
}
\value{
Data after cleaning variable names.
}
\description{
This function is used to clean up variable names to conform to the RSDA format.
}
\examples{
data(mushroom.int.mm)
mushroom.clean <- clean_colnames(data = mushroom.int.mm)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{irish_wind.its}
\alias{irish_wind.its}
\title{Irish Wind Speed Monthly Interval Time Series}
\format{
A data frame with 216 observations and 11 columns (5 interval
variables in \code{_l}/\code{_u} Min-Max pairs, plus a date):
\itemize{
    \item \code{date}: First day of the month (Date class).
    \item \code{BIR_l}, \code{BIR_u}: Monthly [min, max] daily wind speed
    at Birr (knots).
    \item \code{DUB_l}, \code{DUB_u}: Monthly [min, max] daily wind speed
    at Dublin Airport (knots).
    \item \code{KIL_l}, \code{KIL_u}: Monthly [min, max] daily wind speed
    at Kilkenny (knots).
    \item \code{SHA_l}, \code{SHA_u}: Monthly [min, max] daily wind speed
    at Shannon Airport (knots).
    \item \code{VAL_l}, \code{VAL_u}: Monthly [min, max] daily wind speed
    at Valentia Observatory (knots).
}
}
\source{
Derived from the \code{wind} dataset in the \pkg{gstat} R
package (originally from Haslett and Raftery, 1989). Daily data
aggregated to monthly intervals.
}
\usage{
data(irish_wind.its)
}
\description{
Monthly interval-valued wind speed data at 5 meteorological stations in
Ireland from January 1961 to December 1978 (216 months). For each month
and station, the interval is defined as [minimum daily average wind speed,
maximum daily average wind speed] across all days in that month.
}
\details{
The original data contains daily average wind speeds (in knots) at 12
synoptic meteorological stations in the Republic of Ireland, collected by
the Irish Meteorological Service. This is the classic Haslett and Raftery
(1989) dataset, one of the most widely used benchmarks in spatial
statistics. Following the approach of Teles and Brito (2015), the raw
daily data is aggregated to monthly intervals for 5 selected stations:
Birr (BIR), Dublin Airport (DUB), Kilkenny (KIL), Shannon Airport (SHA),
and Valentia Observatory (VAL). Each monthly interval captures the range
of daily wind variability within that month.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 216 \cr
  \strong{Variables (p)} \tab 11 \cr
  \strong{Subject area} \tab Meteorology \cr
  \strong{Symbolic format} \tab Interval time series (multivariate) \cr
  \strong{Analytical tasks} \tab Space-time modelling, Forecasting,
  Clustering
}
}

\examples{
data(irish_wind.its)
head(irish_wind.its)
# Plot Valentia Observatory wind speed interval
plot(irish_wind.its$date, irish_wind.its$VAL_u, type = "l", col = "red",
     ylab = "Wind speed (knots)", xlab = "Date",
     main = "Valentia Observatory Monthly Wind Speed Interval")
lines(irish_wind.its$date, irish_wind.its$VAL_l, col = "blue")
legend("topright", c("Max", "Min"), col = c("red", "blue"), lty = 1)
}
\references{
Haslett, J. and Raftery, A. E. (1989). Space-time modelling with
long-memory dependence: Assessing Ireland's wind power resource.
\emph{Journal of the Royal Statistical Society, Series C (Applied
Statistics)}, \strong{38}(1), 1--50.

Teles, P. and Brito, P. (2015). Modeling interval time series with
space-time processes. \emph{Communications in Statistics -- Theory and
Methods}, \strong{44}(17), 3599--3619.
}
\keyword{datasets}
\keyword{interval}
\keyword{meteorology}
\keyword{spatial}
\keyword{timeseries}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{uscrime.int}
\alias{uscrime.int}
\title{US Crime Statistics Interval Dataset}
\format{
A symbolic data frame (\code{symbolic_tbl}) with 46 observations and
102 interval-valued variables. Key variables include:
\itemize{
    \item \code{fold}: Cross-validation fold assignment.
    \item \code{population}: Population range.
    \item \code{householdsize}: Household size range.
    \item \code{racepctblack}, \code{racePctWhite}, \code{racePctAsian},
          \code{racePctHisp}: Race percentage ranges.
    \item \code{medIncome}, \code{medFamInc}, \code{perCapInc}: Income ranges.
    \item \code{PctUnemployed}, \code{PctEmploy}: Employment percentage ranges.
    \item \code{ViolentCrimesPerPop}: Violent crimes per population range.
}
Plus 90 additional interval-valued socio-economic and demographic variables.
}
\source{
Extracted from RSDA package (\code{uscrime_int}).
}
\usage{
data(uscrime.int)
}
\description{
Interval-valued crime statistics for 46 US states, containing 102
interval-valued variables covering various crime types and rates.
Originally from the RSDA package.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 46 \cr
  \strong{Variables (p)} \tab 102 \cr
  \strong{Subject area} \tab Criminology \cr
  \strong{Symbolic format} \tab Interval \cr
  \strong{Analytical tasks} \tab Regression, Clustering
}
}

\examples{
data(uscrime.int)
}
\references{
Rodriguez, O. (2000). Classification et modeles lineaires en analyse
des donnees symboliques. Doctoral Thesis, Universite Paris IX-Dauphine.
}
\keyword{crime}
\keyword{datasets}
\keyword{interval}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hospital.hist}
\alias{hospital.hist}
\title{Hospital Costs Histogram-Valued Dataset}
\format{
A data frame with 15 observations and 1 histogram-valued variable:
\itemize{
    \item \code{cost}: Histogram-valued cost distribution (currency units).
}
Row names are H1 through H15.
}
\source{
Billard, L. and Diday, E. (2006), Table 3.12.
}
\usage{
data(hospital.hist)
}
\description{
Histogram-valued cost distributions for 15 hospitals. Each observation
is a hospital with a 10-bin histogram of patient costs.
}
\section{Metadata}{

\tabular{ll}{
  \strong{Sample size (n)} \tab 15 \cr
  \strong{Variables (p)} \tab 1 \cr
  \strong{Subject area} \tab Healthcare \cr
  \strong{Symbolic format} \tab Histogram \cr
  \strong{Analytical tasks} \tab Descriptive statistics, Clustering
}
}

\examples{
data(hospital.hist)
}
\references{
Billard, L. and Diday, E. (2006). \emph{Symbolic Data Analysis:
Conceptual Statistics and Data Mining}. Wiley, Chichester. Table 3.12.
}
\keyword{datasets}
\keyword{histogram}
